@@ -11,6 +11,7 @@ import synapse.exc as s_exc
 import synapse.common as s_common
 
 import synapse.lib.coro as s_coro
+import synapse.lib.cache as s_cache
 import synapse.lib.types as s_types
 import synapse.lib.dyndeps as s_dyndeps
 import synapse.lib.grammar as s_grammar
@@ -19,6 +20,8 @@ logger = logging.getLogger(__name__)
 
 hexre = regex.compile('^[0-9a-z]+$')
 
+PREFIX_CACHE_SIZE = 1000
+
 class TagProp:
 
     def __init__(self, model, name, tdef, info):
@@ -27,6 +30,7 @@ class TagProp:
         self.info = info
         self.tdef = tdef
         self.model = model
+        self.locked = False
 
         self.utf8 = name.encode()
         self.nenc = name.encode() + b'\x00'
@@ -106,18 +110,33 @@ class Prop:
             self.isext = name.startswith('._')
         self.isform = False     # for quick Prop()/Form() detection
 
+        self.delperms = [('node', 'prop', 'del', self.full)]
+        self.setperms = [('node', 'prop', 'set', self.full)]
+
+        if form is not None:
+            self.setperms.append(('node', 'prop', 'set', form.name, self.name))
+            self.delperms.append(('node', 'prop', 'del', form.name, self.name))
+
+        self.setperms.reverse()  # Make them in precedence order
+        self.delperms.reverse()  # Make them in precedence order
+
         self.form = form
         self.type = None
         self.typedef = typedef
 
+        self.alts = None
         self.locked = False
         self.deprecated = self.info.get('deprecated', False)
 
         self.type = self.modl.getTypeClone(typedef)
+        self.typehash = self.type.typehash
+
+        if self.type.isarray:
+            self.arraytypehash = self.type.arraytype.typehash
 
         if form is not None:
             form.setProp(name, self)
-            self.modl.propsbytype[self.type.name].append(self)
+            self.modl.propsbytype[self.type.name][self.full] = self
 
         if self.deprecated or self.type.deprecated:
             async def depfunc(node, oldv):
@@ -195,6 +214,8 @@ class Prop:
 
     def pack(self):
         info = {
+            'name': self.name,
+            'full': self.full,
             'type': self.typedef,
             'stortype': self.type.stortype,
         }
@@ -230,6 +251,18 @@ class Prop:
 
         return (buid, {'props': pnorms, 'ndef': ndef})
 
+    def getAlts(self):
+        '''
+        Return a list of Prop instances that are considered
+        alternative locations for our property value, including
+        self.
+        '''
+        if self.alts is None:
+            self.alts = [self]
+            for name in self.info.get('alts', ()):
+                self.alts.append(self.form.reqProp(name))
+        return self.alts
+
 class Form:
     '''
     The Form class implements data model logic for a node form.
@@ -247,10 +280,18 @@ class Form:
         self.onadds = []
         self.ondels = []
 
+        self.addperm = ('node', 'add', self.name)
+        self.delperm = ('node', 'del', self.name)
+
         self.type = modl.types.get(name)
         if self.type is None:
             raise s_exc.NoSuchType(name=name)
 
+        self.typehash = self.type.typehash
+
+        if self.type.isarray:
+            self.arraytypehash = self.type.arraytype.typehash
+
         self.form = self
 
         self.props = {}     # name: Prop()
@@ -312,11 +353,16 @@ class Form:
                 'prop': [],
                 'ndef': [],
                 'array': [],
+                'ndefarray': [],
             }
 
             for name, prop in self.props.items():
 
                 if isinstance(prop.type, s_types.Array):
+                    if isinstance(prop.type.arraytype, s_types.Ndef):
+                        self.refsout['ndefarray'].append(name)
+                        continue
+
                     typename = prop.type.arraytype.name
                     if self.modl.forms.get(typename) is not None:
                         self.refsout['array'].append((name, typename))
@@ -399,9 +445,19 @@ class Form:
         '''
         return self.props.get(name)
 
+    def reqProp(self, name):
+        prop = self.props.get(name)
+        if prop is not None:
+            return prop
+
+        full = f'{self.name}:{name}'
+        mesg = f'No property named {full}.'
+        raise s_exc.NoSuchProp(mesg=mesg, name=full)
+
     def pack(self):
         props = {p.name: p.pack() for p in self.props.values()}
         info = {
+            'name': self.name,
             'props': props,
             'stortype': self.type.stortype,
         }
@@ -412,33 +468,51 @@ class Form:
         propdefs = [p.getPropDef() for p in self.props.values() if not p.isuniv]
         return (self.name, self.info, propdefs)
 
+class Edge:
+
+    def __init__(self, modl, edgetype, edgeinfo):
+        self.modl = modl
+        self.edgetype = edgetype
+        self.edgeinfo = edgeinfo
+
+    def pack(self):
+        return (self.edgetype, self.edgeinfo)
+
 class Model:
     '''
     The data model used by a Cortex hypergraph.
     '''
-    def __init__(self):
+    def __init__(self, core=None):
 
+        self.core = core
         self.types = {}  # name: Type()
         self.forms = {}  # name: Form()
         self.props = {}  # (form,name): Prop() and full: Prop()
+        self.edges = {}  # (n1form, verb, n2form): Edge()
         self.ifaces = {}  # name: <ifdef>
         self.tagprops = {}  # name: TagProp()
         self.formabbr = {}  # name: [Form(), ... ]
         self.modeldefs = []
 
         self.univs = {}
+        self.allunivs = collections.defaultdict(list)
 
-        self.propsbytype = collections.defaultdict(list)  # name: Prop()
-        self.arraysbytype = collections.defaultdict(list)
-        # TODO use this for <nodes> -> foo:iface
+        self.propsbytype = collections.defaultdict(dict)  # name: Prop()
+        self.arraysbytype = collections.defaultdict(dict)
+        self.ifaceprops = collections.defaultdict(list)
         self.formsbyiface = collections.defaultdict(list)
+        self.edgesbyn1 = collections.defaultdict(set)
+        self.edgesbyn2 = collections.defaultdict(set)
+
+        self.formprefixcache = s_cache.LruDict(PREFIX_CACHE_SIZE)
 
         self._type_pends = collections.defaultdict(list)
         self._modeldef = {
             'ctors': [],
             'types': [],
             'forms': [],
-            'univs': []
+            'univs': [],
+            'edges': [],
         }
 
         # add the primitive base types
@@ -543,31 +617,113 @@ class Model:
         self.addUnivProp('seen', ('ival', {}), {
             'doc': 'The time interval for first/last observation of the node.',
         })
-        self.addUnivProp('created', ('time', {}), {
+        self.addUnivProp('created', ('time', {'ismin': True}), {
             'ro': True,
             'doc': 'The time the node was created in the cortex.',
         })
 
-        self.addIface('taxonomy', {
-            'props': (
-                ('title', ('str', {}), {'doc': 'A brief title of the definition.'}),
-                ('summary', ('str', {}), {'doc': 'A summary of the definition.', 'disp': {'hint': 'text'}}),
-                ('sort', ('int', {}), {'doc': 'A display sort order for siblings.', }),
-                ('base', ('taxon', {}), {'ro': True, 'doc': 'The base taxon.', }),
-                ('depth', ('int', {}), {'ro': True, 'doc': 'The depth indexed from 0.', }),
-                ('parent', ('$self', {}), {'ro': True, 'doc': 'The taxonomy parent.', }),
-            ),
-        })
-
     def getPropsByType(self, name):
-        props = self.propsbytype.get(name, ())
+        props = self.propsbytype.get(name)
+        if props is None:
+            return ()
         # TODO order props based on score...
-        return props
+        return list(props.values())
+
+    def getArrayPropsByType(self, name):
+        props = self.arraysbytype.get(name)
+        if props is None:
+            return ()
+        return list(props.values())
 
     def getProps(self):
         return [pobj for pname, pobj in self.props.items()
                 if not (isinstance(pname, tuple))]
 
+    def getFormsByPrefix(self, prefix):
+        forms = self.formprefixcache.get(prefix)
+        if forms is not None:
+            return forms
+
+        forms = []
+        for form in self.forms:
+            if form.startswith(prefix):
+                forms.append(form)
+
+        if forms:
+            forms.sort()
+            self.formprefixcache[prefix] = forms
+        return forms
+
+    def reqProp(self, name, extra=None):
+        prop = self.prop(name)
+        if prop is not None:
+            return prop
+
+        exc = s_exc.NoSuchProp.init(name)
+        if extra is not None:
+            exc = extra(exc)
+
+        raise exc
+
+    def reqUniv(self, name):
+        prop = self.univ(name)
+        if prop is not None:
+            return prop
+
+        mesg = f'No universal property named {name}.'
+        raise s_exc.NoSuchUniv(mesg=mesg, name=name)
+
+    def reqTagProp(self, name):
+        prop = self.getTagProp(name)
+        if prop is not None:
+            return prop
+
+        mesg = f'No tag property named {name}.'
+        raise s_exc.NoSuchTagProp(mesg=mesg, name=name)
+
+    def reqFormsByPrefix(self, prefix, extra=None):
+        forms = self.getFormsByPrefix(prefix)
+        if not forms:
+            mesg = f'No forms match prefix {prefix}.'
+            exc = s_exc.NoSuchForm(name=prefix, mesg=mesg)
+            if extra is not None:
+                exc = extra(exc)
+            raise exc
+
+        return forms
+
+    def reqFormsByLook(self, name, extra=None):
+        if (form := self.form(name)) is not None:
+            return (form.name,)
+
+        if (forms := self.formsbyiface.get(name)) is not None:
+            return forms
+
+        if name.endswith('*'):
+            return self.reqFormsByPrefix(name[:-1], extra=extra)
+
+        exc = s_exc.NoSuchForm.init(name)
+        if extra is not None:
+            exc = extra(exc)
+
+        raise exc
+
+    def reqPropsByLook(self, name, extra=None):
+        if (forms := self.formsbyiface.get(name)) is not None:
+            return forms
+
+        if (props := self.ifaceprops.get(name)) is not None:
+            return props
+
+        if name.endswith('*'):
+            return self.reqFormsByPrefix(name[:-1], extra=extra)
+
+        exc = s_exc.NoSuchProp.init(name)
+        if extra is not None:
+            exc = extra(exc)
+
+        raise exc
+
     def getTypeClone(self, typedef):
 
         base = self.types.get(typedef[0])
@@ -587,12 +743,14 @@ class Model:
         mdef['univs'] = [u.getPropDef() for u in self.univs.values()]
         mdef['tagprops'] = [t.getTagPropDef() for t in self.tagprops.values()]
         mdef['interfaces'] = list(self.ifaces.items())
+        mdef['edges'] = [e.pack() for e in self.edges.values()]
         return [('all', mdef)]
 
     def getModelDict(self):
         retn = {
             'types': {},
             'forms': {},
+            'edges': [],
             'univs': {},
             'tagprops': {},
             'interfaces': self.ifaces.copy()
@@ -610,6 +768,9 @@ class Model:
         for pobj in self.tagprops.values():
             retn['tagprops'][pobj.name] = pobj.pack()
 
+        for eobj in self.edges.values():
+            retn['edges'].append(eobj.pack())
+
         return retn
 
     def addDataModels(self, mods):
@@ -692,7 +853,56 @@ class Model:
         for _, mdef in mods:
 
             for formname, forminfo, propdefs in mdef.get('forms', ()):
-                self.addForm(formname, forminfo, propdefs)
+                self.addForm(formname, forminfo, propdefs, checks=False)
+
+        # now we can load edge definitions...
+        for _, mdef in mods:
+            for etype, einfo in mdef.get('edges', ()):
+                self.addEdge(etype, einfo)
+
+        # now we can check the forms display settings...
+        for form in self.forms.values():
+            self._checkFormDisplay(form)
+
+    def addEdge(self, edgetype, edgeinfo):
+
+        n1form, verb, n2form = edgetype
+
+        if n1form is not None:
+            self._reqFormName(n1form)
+
+        if n2form is not None:
+            self._reqFormName(n2form)
+
+        if not isinstance(verb, str):
+            mesg = f'Edge definition verb must be a string: {edgetype}.'
+            raise s_exc.BadArg(mesg=mesg)
+
+        if self.edges.get(edgetype) is not None:
+            mesg = f'Duplicate edge declared: {edgetype}.'
+            raise s_exc.BadArg(mesg=mesg)
+
+        edge = Edge(self, edgetype, edgeinfo)
+
+        self.edges[edgetype] = edge
+        self.edgesbyn1[n1form].add(edge)
+        self.edgesbyn2[n2form].add(edge)
+
+    def delEdge(self, edgetype):
+        if self.edges.get(edgetype) is None:
+            return
+
+        n1form, verb, n2form = edgetype
+
+        self.edges.pop(edgetype, None)
+        self.edgesbyn1[n1form].discard(edgetype)
+        self.edgesbyn2[n2form].discard(edgetype)
+
+    def _reqFormName(self, name):
+        form = self.forms.get(name)
+        if form is None:
+            raise s_exc.NoSuchForm.init(name)
+        return form
 
     def addType(self, typename, basename, typeopts, typeinfo):
         base = self.types.get(basename)
@@ -703,13 +913,13 @@ class Model:
 
         if newtype.deprecated and typeinfo.get('custom'):
             mesg = f'The type {typename} is based on a deprecated type {newtype.name} which ' \
-                   f'which which will be removed in 3.0.0.'
+                   f'will be removed in 3.0.0.'
             logger.warning(mesg)
 
         self.types[typename] = newtype
         self._modeldef['types'].append(newtype.getTypeDef())
 
-    def addForm(self, formname, forminfo, propdefs):
+    def addForm(self, formname, forminfo, propdefs, checks=True):
 
         if not s_grammar.isFormName(formname):
             mesg = f'Invalid form name {formname}'
@@ -725,7 +935,7 @@ class Model:
         self.props[formname] = form
 
         if isinstance(form.type, s_types.Array):
-            self.arraysbytype[form.type.arraytype.name].append(form)
+            self.arraysbytype[form.type.arraytype.name][form.name] = form
 
         for univname, typedef, univinfo in (u.getPropDef() for u in self.univs.values()):
             self._addFormUniv(form, univname, typedef, univinfo)
@@ -743,23 +953,76 @@ class Model:
         for ifname in form.type.info.get('interfaces', ()):
             self._addFormIface(form, ifname)
 
+        if checks:
+            self._checkFormDisplay(form)
+
+        self.formprefixcache.clear()
+
         return form
 
+    def _checkFormDisplay(self, form):
+
+        formtype = self.types.get(form.full)
+
+        display = formtype.info.get('display')
+        if display is None:
+            return
+
+        for column in display.get('columns', ()):
+            coltype = column.get('type')
+            colopts = column.get('opts')
+
+            if coltype == 'prop':
+                curf = form
+                propname = colopts.get('name')
+                parts = propname.split('::')
+
+                for partname in parts:
+                    prop = curf.prop(partname)
+                    if prop is None:
+                        mesg = (f'Form {form.name} defines prop column {propname}'
+                               f' but {curf.full} has no property named {partname}.')
+                        raise s_exc.BadFormDef(mesg=mesg)
+
+                    curf = self.form(prop.type.name)
+
+            else:
+                mesg = f'Form {form.name} defines column with invalid type ({coltype}).'
+                raise s_exc.BadFormDef(mesg=mesg)
+
     def delForm(self, formname):
 
         form = self.forms.get(formname)
         if form is None:
             return
 
-        if isinstance(form.type, s_types.Array):
-            self.arraysbytype[form.type.arraytype.name].remove(form)
+        ifaceprops = set()
+        for iface in form.ifaces.values():
+            for prop in iface.get('props', ()):
+                ifaceprops.add(prop[0])
 
-        for ifname in form.ifaces.keys():
-            self.formsbyiface[ifname].remove(form)
+        formprops = []
+        for propname, prop in form.props.items():
+            if prop.univ is not None or propname in ifaceprops:
+                continue
+            formprops.append(prop)
+
+        if formprops:
+            propnames = ', '.join(prop.name for prop in formprops)
+            mesg = f'Form has extended properties: {propnames}'
+            raise s_exc.CantDelForm(mesg=mesg)
+
+        if isinstance(form.type, s_types.Array):
+            self.arraysbytype[form.type.arraytype.name].pop(form.name, None)
+
+        for ifname in form.type.info.get('interfaces', ()):
+            self._delFormIface(form, ifname)
 
         self.forms.pop(formname, None)
         self.props.pop(formname, None)
 
+        self.formprefixcache.clear()
+
     def addIface(self, name, info):
         # TODO should we add some meta-props here for queries?
         self.ifaces[name] = info
@@ -771,20 +1034,36 @@ class Model:
             return
 
         if self.propsbytype.get(typename):
-            raise s_exc.CantDelType(name=typename)
+            mesg = f'Cannot delete type {typename} as it is still in use by properties.'
+            raise s_exc.CantDelType(mesg=mesg, name=typename)
+
+        for _type in self.types.values():
+            if typename in _type.info['bases']:
+                mesg = f'Cannot delete type {typename} as it is still in use by other types.'
+                raise s_exc.CantDelType(mesg=mesg, name=typename)
+
+            if _type.isarray and _type.arraytype.name == typename:
+                mesg = f'Cannot delete type {typename} as it is still in use by array types.'
+                raise s_exc.CantDelType(mesg=mesg, name=typename)
 
         self.types.pop(typename, None)
         self.propsbytype.pop(typename, None)
+        self.arraysbytype.pop(typename, None)
 
     def _addFormUniv(self, form, name, tdef, info):
 
+        univ = self.reqUniv(name)
+
         prop = Prop(self, form, name, tdef, info)
+        prop.locked = univ.locked
 
         full = f'{form.name}{name}'
 
         self.props[full] = prop
         self.props[(form.name, name)] = prop
 
+        self.allunivs[name].append(prop)
+
     def addUnivProp(self, name, tdef, info):
 
         base = '.' + name
@@ -798,13 +1077,18 @@ class Model:
         self.props[base] = univ
         self.univs[base] = univ
 
+        self.allunivs[base].append(univ)
+
         for form in self.forms.values():
-            self._addFormUniv(form, base, tdef, info)
+            prop = self._addFormUniv(form, base, tdef, info)
+
+    def getAllUnivs(self, name):
+        return list(self.allunivs.get(name, ()))
 
     def addFormProp(self, formname, propname, tdef, info):
         form = self.forms.get(formname)
         if form is None:
-            raise s_exc.NoSuchForm(name=formname)
+            raise s_exc.NoSuchForm.init(formname)
         return self._addFormProp(form, propname, tdef, info)
 
     def _addFormProp(self, form, name, tdef, info):
@@ -813,30 +1097,112 @@ class Model:
 
         # index the array item types
         if isinstance(prop.type, s_types.Array):
-            self.arraysbytype[prop.type.arraytype.name].append(prop)
+            self.arraysbytype[prop.type.arraytype.name][prop.full] = prop
 
         self.props[prop.full] = prop
         return prop
 
-    def _addFormIface(self, form, name):
+    def _prepFormIface(self, form, iface):
+
+        template = iface.get('template', {})
+        template.update(form.type.info.get('template', {}))
+
+        def convert(item):
+
+            if isinstance(item, str):
+
+                if item == '$self':
+                    return form.name
+
+                item = s_common.format(item, **template)
+
+                # warn but do not blow up. there may be extended model elements
+                # with {}s which are not used for templates...
+                if item.find('{') != -1: # pragma: no cover
+                    logger.warning(f'Missing template specifier in: {item}')
+
+                return item
+
+            if isinstance(item, dict):
+                return {convert(k): convert(v) for (k, v) in item.items()}
+
+            if isinstance(item, (list, tuple)):
+                return tuple([convert(v) for v in item])
+
+            return item
+
+        return convert(iface)
+
+    def _addFormIface(self, form, name, subifaces=None):
 
         iface = self.ifaces.get(name)
 
         if iface is None:
-            mesg = f'Form {form.name} depends on non-existant interface: {name}'
+            mesg = f'Form {form.name} depends on non-existent interface: {name}'
             raise s_exc.NoSuchName(mesg=mesg)
 
+        if iface.get('deprecated'):
+            mesg = f'Form {form.name} depends on deprecated interface {name} which will be removed in 3.0.0'
+            logger.warning(mesg)
+
+        iface = self._prepFormIface(form, iface)
+
         for propname, typedef, propinfo in iface.get('props', ()):
-            if typedef[0] == '$self':
-                typedef = (form.name, typedef[1])
-            self._addFormProp(form, propname, typedef, propinfo)
 
-        # TODO use this to allow storm: +foo:iface
+            # allow form props to take precedence
+            if form.prop(propname) is not None:
+                continue
+
+            prop = self._addFormProp(form, propname, typedef, propinfo)
+            self.ifaceprops[f'{name}:{propname}'].append(prop.full)
+
+            if subifaces is not None:
+                for subi in subifaces:
+                    self.ifaceprops[f'{subi}:{propname}'].append(prop.full)
+
         form.ifaces[name] = iface
-        self.formsbyiface[name].append(form)
+        self.formsbyiface[name].append(form.name)
 
-        for ifname in iface.get('interfaces', ()):
-            self._addFormIface(form, ifname)
+        if (ifaces := iface.get('interfaces')) is not None:
+            if subifaces is None:
+                subifaces = []
+            else:
+                subifaces = list(subifaces)
+
+            subifaces.append(name)
+
+            for ifname in ifaces:
+                self._addFormIface(form, ifname, subifaces=subifaces)
+
+    def _delFormIface(self, form, name, subifaces=None):
+
+        if (iface := self.ifaces.get(name)) is None:
+            return
+
+        iface = self._prepFormIface(form, iface)
+
+        for propname, typedef, propinfo in iface.get('props', ()):
+            fullprop = f'{form.name}:{propname}'
+            self.delFormProp(form.name, propname)
+            self.ifaceprops[f'{name}:{propname}'].remove(fullprop)
+
+            if subifaces is not None:
+                for subi in subifaces:
+                    self.ifaceprops[f'{subi}:{propname}'].remove(fullprop)
+
+        form.ifaces.pop(name, None)
+        self.formsbyiface[name].remove(form.name)
+
+        if (ifaces := iface.get('interfaces')) is not None:
+            if subifaces is None:
+                subifaces = []
+            else:
+                subifaces = list(subifaces)
+
+            subifaces.append(name)
+
+            for ifname in ifaces:
+                self._delFormIface(form, ifname, subifaces=subifaces)
 
     def delTagProp(self, name):
         return self.tagprops.pop(name)
@@ -862,7 +1228,7 @@ class Model:
 
         form = self.forms.get(formname)
         if form is None:
-            raise s_exc.NoSuchForm(name=formname)
+            raise s_exc.NoSuchForm.init(formname)
 
         prop = form.delProp(propname)
         if prop is None:
@@ -871,12 +1237,12 @@ class Model:
             raise s_exc.NoSuchProp(mesg=mesg, name=name)
 
         if isinstance(prop.type, s_types.Array):
-            self.arraysbytype[prop.type.arraytype.name].remove(prop)
+            self.arraysbytype[prop.type.arraytype.name].pop(prop.full, None)
 
         self.props.pop(prop.full, None)
         self.props.pop((form.name, prop.name), None)
 
-        self.propsbytype[prop.type.name].remove(prop)
+        self.propsbytype[prop.type.name].pop(prop.full, None)
 
     def delUnivProp(self, propname):
 
@@ -887,6 +1253,7 @@ class Model:
             raise s_exc.NoSuchUniv(name=propname)
 
         self.univs.pop(univname, None)
+        self.allunivs.pop(univname, None)
 
         for form in self.forms.values():
             self.delFormProp(form.name, univname)
@@ -911,8 +1278,19 @@ class Model:
     def form(self, name):
         return self.forms.get(name)
 
+    def reqForm(self, name):
+        form = self.forms.get(name)
+        if form is not None:
+            return form
+
+        mesg = f'No form named {name}.'
+        raise s_exc.NoSuchForm(mesg=mesg, name=name)
+
     def univ(self, name):
         return self.univs.get(name)
 
     def tagprop(self, name):
         return self.tagprops.get(name)
+
+    def edge(self, edgetype):
+        return self.edges.get(edgetype)
