@@ -14,6 +14,15 @@ class SynErr(Exception):
         displ = ' '.join(['%s=%r' % (p, v) for (p, v) in props])
         return '%s: %s' % (self.__class__.__name__, displ)
 
+    def _setExcMesg(self):
+        '''Should be called when self.errinfo is modified.'''
+        self.args = (self._getExcMsg(),)
+
+    def __setstate__(self, state):
+        '''Pickle support.'''
+        super(SynErr, self).__setstate__(state)
+        self._setExcMesg()
+
     def items(self):
         return {k: v for k, v in self.errinfo.items()}
 
@@ -31,12 +40,33 @@ class SynErr(Exception):
         '''
         return self.errinfo.get(name, defv)
 
+    def set(self, name, valu):
+        '''
+        Set a value in the errinfo dict.
+        '''
+        self.errinfo[name] = valu
+        self._setExcMesg()
+
+    def setdefault(self, name, valu):
+        '''
+        Set a value in errinfo dict if it is not already set.
+        '''
+        if name in self.errinfo:
+            return
+        self.errinfo[name] = valu
+        self._setExcMesg()
+
 class StormRaise(SynErr):
-    def __init__(self, name, mesg, info):
-        info['mesg'] = mesg
-        info['errname'] = name
-        SynErr.__init__(self, **info)
-        self.errname = name
+    '''
+    This represents a user provided exception inside of a Storm runtime. It requires a errname key.
+    '''
+    def __init__(self, *args, **info):
+        SynErr.__init__(self, *args, **info)
+        name = info.get('errname')
+        if name is not None:
+            self.errname = name
+        else:
+            raise BadArg(mesg='StormRaise must have a key errname provided')
 
 class AuthDeny(SynErr): pass
 
@@ -47,6 +77,7 @@ class BackupAlreadyRunning(SynErr):
 class StormPkgRequires(SynErr): pass
 class StormPkgConflicts(SynErr): pass
 
+class BadName(SynErr): pass
 class BadPkgDef(SynErr): pass
 class BadCmdName(SynErr): pass
 class BadCmprValu(SynErr): pass
@@ -75,14 +106,20 @@ class BadFormDef(SynErr): pass
 class BadHivePath(SynErr): pass
 class BadLiftValu(SynErr): pass
 class BadPropDef(SynErr): pass
+class BadEdgeDef(SynErr): pass
 class BadTypeDef(SynErr): pass
 class BadTypeValu(SynErr): pass
 class BadJsonText(SynErr): pass
+class BadDataValu(SynErr):
+    '''Cannot process the data as intended.'''
+    pass
 
 class BadArg(SynErr):
     ''' Improper function arguments '''
     pass
 
+class BadState(SynErr): pass
+
 class BadFileExt(SynErr): pass
 class BadIndxValu(SynErr): pass
 class BadMesgVers(SynErr): pass
@@ -104,12 +141,15 @@ class BadTag(SynErr): pass
 class BadTime(SynErr): pass
 class BadUrl(SynErr): pass
 
+class TypeMismatch(SynErr): pass
+
 class CantDelCmd(SynErr): pass
 class CantDelNode(SynErr): pass
 class CantDelForm(SynErr): pass
 class CantDelProp(SynErr): pass
 class CantDelType(SynErr): pass
 class CantDelUniv(SynErr): pass
+class CantDelView(SynErr): pass
 class CantMergeView(SynErr): pass
 class CantRevLayer(SynErr): pass
 class CliFini(SynErr):
@@ -128,6 +168,12 @@ class BadEccExchange(CryptoErr):
     ''' Raised when there is an issue doing a ECC Key Exchange '''
     pass
 
+class BadCertBytes(SynErr):
+    '''Raised by certdir when the certificate fails to load.'''
+
+class BadCertVerify(SynErr):
+    '''Raised by certdir when there is a failure to verify a certificate context.'''
+
 class PathExists(SynErr): pass
 class DataAlreadyExists(SynErr):
     '''
@@ -147,6 +193,22 @@ class DupTagPropName(SynErr): pass
 class DupUserName(SynErr): pass
 class DupStormSvc(SynErr): pass
 
+class DupTypeName(SynErr):
+    @classmethod
+    def init(cls, name, mesg=None):
+        if mesg is None:
+            mesg = f'Type already exists: {name}.'
+        return DupTypeName(mesg=mesg, name=name)
+
+class DupEdgeType(SynErr):
+
+    @classmethod
+    def init(cls, edge, mesg=None):
+        if mesg is None:
+            (n1form, verb, n2form) = edge
+            mesg = f'Edge already exists: {n1form} -({verb})> {n2form}.'
+        return DupEdgeType(mesg=mesg, n1form=n1form, verb=verb, n2form=n2form)
+
 class FileExists(SynErr): pass
 
 class InconsistentStorage(SynErr):
@@ -166,6 +228,8 @@ class LinkErr(SynErr): pass
 class LinkBadCert(LinkErr): pass
 class LinkShutDown(LinkErr): pass
 
+class LowSpace(SynErr): pass
+
 class NoCertKey(SynErr):
     ''' Raised when a Cert object requires a RSA Private Key to perform an operation and the key is not present.  '''
     pass
@@ -176,6 +240,42 @@ class ModAlreadyLoaded(SynErr): pass
 class MustBeJsonSafe(SynErr): pass
 class NotMsgpackSafe(SynErr): pass
 
+class NoSuchForm(SynErr):
+
+    # new convention where exceptions with an init() class
+    # method may be called to enforce normalization of message
+    # contents based on variables that are relevant to the
+    # exception type...
+    @classmethod
+    def init(cls, name, mesg=None):
+        if mesg is None:
+            mesg = f'No form named {name}.'
+        return NoSuchForm(mesg=mesg, name=name)
+
+class NoSuchType(SynErr):
+    @classmethod
+    def init(cls, name, mesg=None):
+        if mesg is None:
+            mesg = f'No type named {name}.'
+        return NoSuchType(mesg=mesg, name=name)
+
+class NoSuchProp(SynErr):
+
+    @classmethod
+    def init(cls, name, mesg=None):
+        if mesg is None:
+            mesg = f'No property named {name}.'
+        return NoSuchProp(mesg=mesg, name=name)
+
+class NoSuchEdge(SynErr):
+
+    @classmethod
+    def init(cls, edge, mesg=None):
+        if mesg is None:
+            (n1form, verb, n2form) = edge
+            mesg = f'No edge defined for {n1form} -({verb})> {n2form}.'
+        return NoSuchEdge(mesg=mesg, n1form=n1form, verb=verb, n2form=n2form)
+
 class NoSuchAbrv(SynErr): pass
 class NoSuchAct(SynErr): pass
 class NoSuchAuthGate(SynErr): pass
@@ -189,7 +289,6 @@ class NoSuchDir(SynErr): pass
 class NoSuchDyn(SynErr): pass
 class NoSuchEncoder(SynErr): pass
 class NoSuchFile(SynErr): pass
-class NoSuchForm(SynErr): pass
 class NoSuchFunc(SynErr): pass
 class NoSuchIden(SynErr): pass
 class NoSuchImpl(SynErr): pass
@@ -202,20 +301,14 @@ class NoSuchObj(SynErr): pass
 class NoSuchOpt(SynErr): pass
 class NoSuchPath(SynErr): pass
 class NoSuchPivot(SynErr): pass
-class NoSuchProp(SynErr): pass
 class NoSuchUniv(SynErr): pass
 class NoSuchRole(SynErr): pass
-class NoSuchType(SynErr): pass
 class NoSuchUser(SynErr): pass
 class NoSuchVar(SynErr): pass
 class NoSuchView(SynErr): pass
 class NoSuchTagProp(SynErr): pass
 class NoSuchStormSvc(SynErr): pass
 
-class NoSuchStor(SynErr):
-    def __init__(self, name):
-        SynErr.__init__(self, mesg=f'No storage type found named {name!r}', name=name)
-
 class NotANumberCompared(SynErr): pass
 
 class ParserExit(SynErr):
@@ -256,7 +349,6 @@ class StepTimeout(SynErr):
 class StormRuntimeError(SynErr): pass
 class StormVarListError(StormRuntimeError): pass
 
-class TeleRedir(SynErr): pass
 class FatalErr(SynErr):
     '''
     Raised when a fatal error has occurred which an application cannot recover from.
@@ -264,5 +356,3 @@ class FatalErr(SynErr):
     pass
 
 class LmdbLock(SynErr): pass
-
-proxy_admin_mesg = 'Specifying a proxy to the HTTP library requires admin privileges.'
