@@ -39,3 +39,11 @@ day = hour * 24
 week = day * 7
 month = day * 30
 year = day * 365
+
+# function specific constants shared across multiple files
+layer_pdef_csize = 1_000
+layer_pdef_qsize = 10_000
+layer_pdef_csize_max = 1_000
+layer_pdef_qsize_max = 10_000
+assert layer_pdef_csize <= layer_pdef_csize_max
+assert layer_pdef_qsize <= layer_pdef_qsize_max
