@@ -91,7 +91,7 @@ class Hive(s_nexus.Pusher, s_telepath.Aware):
     An optionally persistent atomically accessed tree which implements
     primitives for use in making distributed/clustered services.
     '''
-    async def __anit__(self, conf=None, nexsroot=None):
+    async def __anit__(self, conf=None, nexsroot=None, cell=None):
 
         await s_nexus.Pusher.__anit__(self, 'hive', nexsroot=nexsroot)
 
@@ -100,6 +100,7 @@ class Hive(s_nexus.Pusher, s_telepath.Aware):
         if conf is None:
             conf = {}
 
+        self.cell = cell
         self.conf = conf
         self.nodes = {}  # full=Node()
 
@@ -340,6 +341,18 @@ class Hive(s_nexus.Pusher, s_telepath.Aware):
 
     @s_nexus.Pusher.onPush('hive:set')
     async def _set(self, full, valu):
+        if self.cell is not None:
+            if full[0] == 'auth':
+                if len(full) == 5:
+                    _, _, iden, dtyp, name = full
+                    if dtyp == 'vars':
+                        await self.cell.auth._hndlsetUserVarValu(iden, name, valu)
+                    elif dtyp == 'profile':
+                        await self.cell.auth._hndlsetUserProfileValu(iden, name, valu)
+
+            elif full[0] == 'cellvers':
+                await self.cell.setCellVers(full[-1], valu, nexs=False)
+
         node = await self._getHiveNode(full)
 
         oldv = node.valu
@@ -377,6 +390,14 @@ class Hive(s_nexus.Pusher, s_telepath.Aware):
     @s_nexus.Pusher.onPush('hive:pop')
     async def _pop(self, full):
 
+        if self.cell is not None and full[0] == 'auth':
+            if len(full) == 5:
+                _, _, iden, dtyp, name = full
+                if dtyp == 'vars':
+                    await self.cell.auth._hndlpopUserVarValu(iden, name)
+                elif dtyp == 'profile':
+                    await self.cell.auth._hndlpopUserProfileValu(iden, name)
+
         node = self.nodes.get(full)
         if node is None:
             return
@@ -419,7 +440,7 @@ class Hive(s_nexus.Pusher, s_telepath.Aware):
         # passwd None always fails...
         passwd = info.get('passwd')
         if not await user.tryPasswd(passwd):
-            raise s_exc.AuthDeny(mesg='Invalid password', user=user.name)
+            raise s_exc.AuthDeny(mesg='Invalid password', user=user.iden, username=user.name)
 
         return await HiveApi.anit(self, user)
 
@@ -434,10 +455,10 @@ class Hive(s_nexus.Pusher, s_telepath.Aware):
 
 class SlabHive(Hive):
 
-    async def __anit__(self, slab, db=None, conf=None, nexsroot=None):
+    async def __anit__(self, slab, db=None, conf=None, nexsroot=None, cell=None):
         self.db = db
         self.slab = slab
-        await Hive.__anit__(self, conf=conf, nexsroot=nexsroot)
+        await Hive.__anit__(self, conf=conf, nexsroot=nexsroot, cell=cell)
         self.slab.onfini(self.fini)
 
     async def _storLoadHive(self):
@@ -473,12 +494,15 @@ class HiveApi(s_base.Base):
         self.onfini(self._onHapiFini)
 
     async def loadHiveTree(self, tree, path=(), trim=False):
+        s_common.deprecated('HiveApi.loadHiveTree', curv='2.167.0')
         return await self.hive.loadHiveTree(tree, path=path, trim=trim)
 
     async def saveHiveTree(self, path=()):
+        s_common.deprecated('HiveApi.saveHiveTree', curv='2.167.0')
         return await self.hive.saveHiveTree(path=path)
 
     async def treeAndSync(self, path, iden):
+        s_common.deprecated('HiveApi.treeAndSync', curv='2.167.0')
 
         node = await self.hive.open(path)
 
@@ -510,16 +534,22 @@ class HiveApi(s_base.Base):
         return
 
     async def setAndSync(self, path, valu, iden, nexs=False):
+        s_common.deprecated('HiveApi.setAndSync', curv='2.167.0')
+
         valu = await self.hive.set(path, valu, nexs=nexs)
         await self.msgq.put(('hive:sync', {'iden': iden}))
         return valu
 
     async def addAndSync(self, path, valu, iden):
+        s_common.deprecated('HiveApi.addAndSync', curv='2.167.0')
+
         valu = await self.hive.add(path, valu)
         await self.msgq.put(('hive:sync', {'iden': iden}))
         return valu
 
     async def popAndSync(self, path, iden, nexs=False):
+        s_common.deprecated('HiveApi.popAndSync', curv='2.167.0')
+
         valu = await self.hive.pop(path, nexs=nexs)
         await self.msgq.put(('hive:sync', {'iden': iden}))
         return valu
@@ -531,9 +561,11 @@ class HiveApi(s_base.Base):
         self.msgq.put_nowait(mesg)
 
     async def get(self, full):
+        s_common.deprecated('HiveApi.get', curv='2.167.0')
         return await self.hive.get(full)
 
     async def edits(self):
+        s_common.deprecated('HiveApi.edits', curv='2.167.0')
 
         while not self.isfini:
 
@@ -689,9 +721,13 @@ class HiveDict(s_base.Base):
 
         return node.valu
 
-    async def set(self, name, valu):
+    async def set(self, name, valu, nexs=None):
+
+        if nexs is None:
+            nexs = self.nexs
+
         full = self.node.full + (name,)
-        return await self.hive.set(full, valu, nexs=self.nexs)
+        return await self.hive.set(full, valu, nexs=nexs)
 
     def setdefault(self, name, valu):
         self.defs[name] = valu
