@@ -1,4 +1,7 @@
 import logging
+import dataclasses
+
+from typing import Optional, Union
 
 import synapse.exc as s_exc
 import synapse.common as s_common
@@ -8,6 +11,8 @@ import synapse.lib.cache as s_cache
 import synapse.lib.nexus as s_nexus
 import synapse.lib.config as s_config
 
+import synapse.lib.crypto.passwd as s_passwd
+
 logger = logging.getLogger(__name__)
 
 reqValidRules = s_config.getJsValidator({
@@ -23,11 +28,58 @@ reqValidRules = s_config.getJsValidator({
     }
 })
 
-def getShadow(passwd):
+def getShadow(passwd):  # pragma: no cover
+    '''This API is deprecated.'''
+    s_common.deprecated('hiveauth.getShadow()', curv='2.110.0')
     salt = s_common.guid()
     hashed = s_common.guid((salt, passwd))
     return (salt, hashed)
 
+def textFromRule(rule):
+    text = '.'.join(rule[1])
+    if not rule[0]:
+        text = '!' + text
+    return text
+
+@dataclasses.dataclass(slots=True)
+class _allowedReason:
+    value: Union[bool | None]
+    default: bool = False
+    isadmin: bool = False
+    islocked: bool = False
+    gateiden: Union[str | None] = None
+    roleiden: Union[str | None] = None
+    rolename: Union[str | None] = None
+    rule: tuple = ()
+
+    @property
+    def mesg(self):
+        if self.islocked:
+            return 'The user is locked.'
+        if self.default:
+            return 'No matching rule found.'
+
+        if self.isadmin:
+            if self.gateiden:
+                return f'The user is an admin of auth gate {self.gateiden}.'
+            return 'The user is a global admin.'
+
+        if self.rule:
+            rt = textFromRule((self.value, self.rule))
+            if self.gateiden:
+                if self.roleiden:
+                    m = f'Matched role rule ({rt}) for role {self.rolename} on gate {self.gateiden}.'
+                else:
+                    m = f'Matched user rule ({rt}) on gate {self.gateiden}.'
+            else:
+                if self.roleiden:
+                    m = f'Matched role rule ({rt}) for role {self.rolename}.'
+                else:
+                    m = f'Matched user rule ({rt}).'
+            return m
+
+        return 'No matching rule found.'
+
 class Auth(s_nexus.Pusher):
     '''
     Auth is a user authentication and authorization stored in a Hive.  Users
@@ -78,7 +130,7 @@ class Auth(s_nexus.Pusher):
 
     '''
 
-    async def __anit__(self, node, nexsroot=None, seed=None):
+    async def __anit__(self, node, nexsroot=None, seed=None, maxusers=0):
         '''
         Args:
             node (HiveNode): The root of the persistent storage for auth
@@ -92,6 +144,8 @@ class Auth(s_nexus.Pusher):
         if seed is None:
             seed = s_common.guid()
 
+        self.maxusers = maxusers
+
         self.usersbyiden = {}
         self.rolesbyiden = {}
         self.usersbyname = {}
@@ -159,7 +213,7 @@ class Auth(s_nexus.Pusher):
         user = self.user(iden)
         if user is None:
             mesg = f'No user with iden {iden}.'
-            raise s_exc.NoSuchUser(mesg=mesg)
+            raise s_exc.NoSuchUser(mesg=mesg, user=iden)
         return user
 
     async def reqRole(self, iden):
@@ -174,7 +228,7 @@ class Auth(s_nexus.Pusher):
         user = await self.getUserByName(name)
         if user is None:
             mesg = f'No user named {name}.'
-            raise s_exc.NoSuchUser(mesg=mesg)
+            raise s_exc.NoSuchUser(mesg=mesg, username=name)
         return user
 
     async def reqUserByNameOrIden(self, name):
@@ -223,7 +277,6 @@ class Auth(s_nexus.Pusher):
 
         return user
 
-    @s_nexus.Pusher.onPushAuto('user:name')
     async def setUserName(self, iden, name):
         if not isinstance(name, str):
             raise s_exc.BadArg(mesg='setUserName() name must be a string')
@@ -242,7 +295,12 @@ class Auth(s_nexus.Pusher):
         user.name = name
         await user.node.set(name)
 
-    @s_nexus.Pusher.onPushAuto('role:name')
+        beheld = {
+            'iden': iden,
+            'valu': name,
+        }
+        await self.feedBeholder('user:name', beheld)
+
     async def setRoleName(self, iden, name):
         if not isinstance(name, str):
             raise s_exc.BadArg(mesg='setRoleName() name must be a string')
@@ -265,8 +323,28 @@ class Auth(s_nexus.Pusher):
         role.name = name
         await role.node.set(name)
 
-    @s_nexus.Pusher.onPushAuto('user:info')
-    async def setUserInfo(self, iden, name, valu, gateiden=None):
+        beheld = {
+            'iden': iden,
+            'valu': name,
+        }
+        await self.feedBeholder('role:name', beheld)
+
+    async def feedBeholder(self, evnt, info, gateiden=None, logged=True):
+        if self.nexsroot and self.nexsroot.started and logged:
+            behold = {
+                'event': evnt,
+                'offset': await self.nexsroot.index(),
+                'info': info
+            }
+
+            if gateiden:
+                gate = self.getAuthGate(gateiden)
+                if gate:
+                    behold['gates'] = [gate.pack()]
+
+            await self.fire('cell:beholder', **behold)
+
+    async def setUserInfo(self, iden, name, valu, gateiden=None, logged=True, mesg=None):
 
         user = await self.reqUser(iden)
 
@@ -274,13 +352,25 @@ class Auth(s_nexus.Pusher):
         if gateiden is not None:
             info = await user.genGateInfo(gateiden)
 
+        if name in ('locked', 'archived') and not valu:
+            self.checkUserLimit()
+
         await info.set(name, valu)
 
+        if mesg is None:
+            mesg = {
+                'iden': iden,
+                'name': name,
+            }
+            if name != 'passwd':
+                mesg['valu'] = valu
+
+        await self.feedBeholder('user:info', mesg, gateiden=gateiden, logged=logged)
+
         # since any user info *may* effect auth
         user.clearAuthCache()
 
-    @s_nexus.Pusher.onPushAuto('role:info')
-    async def setRoleInfo(self, iden, name, valu, gateiden=None):
+    async def setRoleInfo(self, iden, name, valu, gateiden=None, logged=True, mesg=None):
         role = await self.reqRole(iden)
 
         info = role.info
@@ -288,6 +378,15 @@ class Auth(s_nexus.Pusher):
             info = await role.genGateInfo(gateiden)
 
         await info.set(name, valu)
+
+        if mesg is None:
+            mesg = {
+                'iden': iden,
+                'name': name,
+                'valu': valu,
+            }
+        await self.feedBeholder('role:info', mesg, gateiden=gateiden, logged=logged)
+
         role.clearAuthCache()
 
     async def _addRoleNode(self, node):
@@ -355,6 +454,32 @@ class Auth(s_nexus.Pusher):
             raise s_exc.NoSuchAuthGate(iden=iden, mesg=mesg)
         return gate
 
+    def checkUserLimit(self):
+        '''
+        Check if we're at the specified user limit.
+
+        This should be called right before adding/unlocking/unarchiving a user.
+
+        Raises: s_exc.HitLimit if the number of active users is at the maximum.
+        '''
+        if self.maxusers == 0:
+            return
+
+        numusers = 0
+
+        for user in self.users():
+            if user.name == 'root':
+                continue
+
+            if user.isLocked() or user.isArchived():
+                continue
+
+            numusers += 1
+
+        if numusers >= self.maxusers:
+            mesg = f'Cell at maximum number of users ({self.maxusers}).'
+            raise s_exc.HitLimit(mesg=mesg)
+
     async def addUser(self, name, passwd=None, email=None, iden=None):
         '''
         Add a User to the Hive.
@@ -369,6 +494,8 @@ class Auth(s_nexus.Pusher):
             HiveUser: A Hive User.
         '''
 
+        self.checkUserLimit()
+
         if self.usersbyname.get(name) is not None:
             raise s_exc.DupUserName(name=name)
 
@@ -397,18 +524,20 @@ class Auth(s_nexus.Pusher):
 
         return user
 
-    @s_nexus.Pusher.onPush('user:add')
     async def _addUser(self, iden, name):
 
         user = self.usersbyname.get(name)
         if user is not None:
-            return user
+            return
 
         node = await self.node.open(('users', iden))
         await node.set(name)
 
         await self._addUserNode(node)
 
+        user = self.usersbyname.get(name)
+        await self.feedBeholder('user:add', user.pack())
+
     async def addRole(self, name, iden=None):
         if self.rolesbyname.get(name) is not None:
             raise s_exc.DupRoleName(name=name)
@@ -420,24 +549,25 @@ class Auth(s_nexus.Pusher):
 
         return self.role(iden)
 
-    @s_nexus.Pusher.onPush('role:add')
     async def _addRole(self, iden, name):
 
         role = self.rolesbyname.get(name)
         if role is not None:
-            return role
+            return
 
         node = await self.node.open(('roles', iden))
         await node.set(name)
 
         await self._addRoleNode(node)
 
+        role = self.rolesbyname.get(name)
+        await self.feedBeholder('role:add', role.pack())
+
     async def delUser(self, iden):
 
         await self.reqUser(iden)
         return await self._push('user:del', iden)
 
-    @s_nexus.Pusher.onPush('user:del')
     async def _delUser(self, iden):
 
         if iden == self.rootuser.iden:
@@ -448,6 +578,7 @@ class Auth(s_nexus.Pusher):
         if user is None:
             return
 
+        udef = user.pack()
         self.usersbyiden.pop(user.iden)
         self.usersbyname.pop(user.name)
 
@@ -458,6 +589,8 @@ class Auth(s_nexus.Pusher):
 
         await user.fini()
         await self.node.hive.pop(path)
+        await self.fire('user:del', udef=udef)
+        await self.feedBeholder('user:del', {'iden': iden})
 
     def _getUsersInRole(self, role):
         for user in self.users():
@@ -468,7 +601,6 @@ class Auth(s_nexus.Pusher):
         await self.reqRole(iden)
         return await self._push('role:del', iden)
 
-    @s_nexus.Pusher.onPush('role:del')
     async def _delRole(self, iden):
 
         if iden == self.allrole.iden:
@@ -493,6 +625,7 @@ class Auth(s_nexus.Pusher):
         # directly set the node's value and let events prop
         path = self.node.full + ('roles', role.iden)
         await self.node.hive.pop(path)
+        await self.feedBeholder('role:del', {'iden': iden})
 
 class AuthGate(s_base.Base):
     '''
@@ -630,7 +763,7 @@ class HiveRuler(s_base.Base):
 
         self.authgates = {}
 
-    async def _setRulrInfo(self, name, valu, gateiden=None):  # pragma: no cover
+    async def _setRulrInfo(self, name, valu, gateiden=None, nexs=True, mesg=None):  # pragma: no cover
         raise s_exc.NoSuchImpl(mesg='Subclass must implement _setRulrInfo')
 
     def getRules(self, gateiden=None):
@@ -644,20 +777,26 @@ class HiveRuler(s_base.Base):
 
         return list(gateinfo.get('rules', ()))
 
-    async def setRules(self, rules, gateiden=None, nexs=True):
+    async def setRules(self, rules, gateiden=None, nexs=True, mesg=None):
         reqValidRules(rules)
-        return await self._setRulrInfo('rules', rules, gateiden=gateiden, nexs=nexs)
+        return await self._setRulrInfo('rules', rules, gateiden=gateiden, nexs=nexs, mesg=mesg)
 
     async def addRule(self, rule, indx=None, gateiden=None, nexs=True):
         reqValidRules((rule,))
         rules = self.getRules(gateiden=gateiden)
 
+        mesg = {
+            'name': 'rule:add',
+            'iden': self.iden,
+            'valu': rule,
+        }
         if indx is None:
             rules.append(rule)
         else:
             rules.insert(indx, rule)
+            mesg['indx'] = indx
 
-        await self.setRules(rules, gateiden=gateiden, nexs=nexs)
+        await self.setRules(rules, gateiden=gateiden, nexs=nexs, mesg=mesg)
 
     async def delRule(self, rule, gateiden=None):
         reqValidRules((rule,))
@@ -665,8 +804,13 @@ class HiveRuler(s_base.Base):
         if rule not in rules:
             return False
 
+        mesg = {
+            'name': 'rule:del',
+            'iden': self.iden,
+            'valu': rule,
+        }
         rules.remove(rule)
-        await self.setRules(rules, gateiden=gateiden)
+        await self.setRules(rules, gateiden=gateiden, mesg=mesg)
         return True
 
 class HiveRole(HiveRuler):
@@ -685,11 +829,8 @@ class HiveRole(HiveRuler):
             'authgates': {name: info.pack() for (name, info) in self.authgates.items()},
         }
 
-    async def _setRulrInfo(self, name, valu, gateiden=None, nexs=True):
-        if nexs:
-            return await self.auth.setRoleInfo(self.iden, name, valu, gateiden=gateiden)
-        else:
-            return await self.auth._hndlsetRoleInfo(self.iden, name, valu, gateiden=gateiden)
+    async def _setRulrInfo(self, name, valu, gateiden=None, nexs=True, mesg=None):
+        return await self.auth.setRoleInfo(self.iden, name, valu, gateiden=gateiden, logged=nexs, mesg=mesg)
 
     async def setName(self, name):
         return await self.auth.setRoleName(self.iden, name)
@@ -717,7 +858,7 @@ class HiveRole(HiveRuler):
                         return allow
             return default
 
-        # 2. check user rules
+        # 2. check role rules
         for allow, path in self.info.get('rules', ()):
             if perm[:len(path)] == path:
                 return allow
@@ -748,12 +889,20 @@ class HiveUser(HiveRuler):
         self.vars = await varz.dict(nexs=True)
 
         self.permcache = s_cache.FixedCache(self._allowed)
+        self.allowedcache = s_cache.FixedCache(self._getAllowedReason)
 
     def pack(self, packroles=False):
 
         roles = self.info.get('roles', ())
         if packroles:
-            roles = [self.auth.role(r).pack() for r in roles]
+            _roles = []
+            for r in roles:
+                role = self.auth.role(r)
+                if role is None:
+                    logger.error(f'User {self.iden} ({self.name}) contains a missing role: {r}')
+                    continue
+                _roles.append(role.pack())
+            roles = _roles
 
         return {
             'type': 'user',
@@ -768,11 +917,8 @@ class HiveUser(HiveRuler):
             'authgates': {name: info.pack() for (name, info) in self.authgates.items()},
         }
 
-    async def _setRulrInfo(self, name, valu, gateiden=None, nexs=True):
-        if nexs:
-            return await self.auth.setUserInfo(self.iden, name, valu, gateiden=gateiden)
-        else:
-            return await self.auth._hndlsetUserInfo(self.iden, name, valu, gateiden=gateiden)
+    async def _setRulrInfo(self, name, valu, gateiden=None, nexs=True, mesg=None):
+        return await self.auth.setUserInfo(self.iden, name, valu, gateiden=gateiden, logged=nexs, mesg=mesg)
 
     async def setName(self, name):
         return await self.auth.setUserName(self.iden, name)
@@ -781,13 +927,38 @@ class HiveUser(HiveRuler):
         if not self.allowed(perm):
             await self.addRule((True, perm), indx=0)
 
-    def allowed(self, perm, default=None, gateiden=None):
+    def allowed(self,
+                perm: tuple[str, ...],
+                default: Optional[str] = None,
+                gateiden: Optional[str] = None,
+                deepdeny: bool = False) -> Union[bool, None]:
+        '''
+        Check if a user is allowed a given permission.
+
+        Args:
+            perm: The permission tuple to check.
+            default: The default rule value if there is no match.
+            gateiden: The gate iden to check against.
+            deepdeny: If True, give precedence for checking deny rules which are more specific than the requested
+                      permission.
+
+        Notes:
+            The use of the deepdeny argument is intended for checking a less-specific part of a permissions tree, in
+            order to know about possible short circuit options. Using it to check a more specific part may have
+            unintended results.
+
+        Returns:
+            The allowed value of the permission.
+        '''
         perm = tuple(perm)
-        return self.permcache.get((perm, default, gateiden))
+        return self.permcache.get((perm, default, gateiden, deepdeny))
 
     def _allowed(self, pkey):
+        '''
+        NOTE: This must remain in sync with any changes to _getAllowedReason()!
+        '''
 
-        perm, default, gateiden = pkey
+        perm, default, gateiden, deepdeny = pkey
 
         if self.info.get('locked'):
             return False
@@ -795,6 +966,9 @@ class HiveUser(HiveRuler):
         if self.info.get('admin'):
             return True
 
+        if deepdeny and self._hasDeepDeny(perm, gateiden):
+            return False
+
         # 1. check authgate user rules
         if gateiden is not None:
 
@@ -834,8 +1008,119 @@ class HiveUser(HiveRuler):
 
         return default
 
+    def getAllowedReason(self, perm, default=None, gateiden=None):
+        '''
+        A routine which will return a tuple of (allowed, info).
+        '''
+        perm = tuple(perm)
+        return self.allowedcache.get((perm, default, gateiden))
+
+    def _getAllowedReason(self, pkey):
+        '''
+        NOTE: This must remain in sync with any changes to _allowed()!
+        '''
+        perm, default, gateiden = pkey
+        if self.info.get('locked'):
+            return _allowedReason(False, islocked=True)
+
+        if self.info.get('admin'):
+            return _allowedReason(True, isadmin=True)
+
+        # 1. check authgate user rules
+        if gateiden is not None:
+
+            info = self.authgates.get(gateiden)
+            if info is not None:
+
+                if info.get('admin'):
+                    return _allowedReason(True, isadmin=True, gateiden=gateiden)
+
+                for allow, path in info.get('rules', ()):
+                    if perm[:len(path)] == path:
+                        return _allowedReason(allow, gateiden=gateiden, rule=path)
+
+        # 2. check user rules
+        for allow, path in self.info.get('rules', ()):
+            if perm[:len(path)] == path:
+                return _allowedReason(allow, rule=path)
+
+        # 3. check authgate role rules
+        if gateiden is not None:
+
+            for role in self.getRoles():
+
+                info = role.authgates.get(gateiden)
+                if info is None:
+                    continue
+
+                for allow, path in info.get('rules', ()):
+                    if perm[:len(path)] == path:
+                        return _allowedReason(allow, gateiden=gateiden, roleiden=role.iden, rolename=role.name,
+                                              rule=path)
+
+        # 4. check role rules
+        for role in self.getRoles():
+            for allow, path in role.info.get('rules', ()):
+                if perm[:len(path)] == path:
+                    return _allowedReason(allow, roleiden=role.iden, rolename=role.name, rule=path)
+
+        return _allowedReason(default, default=True)
+
+    def _hasDeepDeny(self, perm, gateiden):
+
+        permlen = len(perm)
+
+        # 1. check authgate user rules
+        if gateiden is not None:
+
+            info = self.authgates.get(gateiden)
+            if info is not None:
+
+                if info.get('admin'):
+                    return False
+
+                for allow, path in info.get('rules', ()):
+                    if allow:
+                        continue
+                    if path[:permlen] == perm and len(path) > permlen:
+                        return True
+
+        # 2. check user rules
+        for allow, path in self.info.get('rules', ()):
+            if allow:
+                continue
+
+            if path[:permlen] == perm and len(path) > permlen:
+                return True
+
+        # 3. check authgate role rules
+        if gateiden is not None:
+
+            for role in self.getRoles():
+
+                info = role.authgates.get(gateiden)
+                if info is None:
+                    continue
+
+                for allow, path in info.get('rules', ()):
+                    if allow:
+                        continue
+                    if path[:permlen] == perm and len(path) > permlen:
+                        return True
+
+        # 4. check role rules
+        for role in self.getRoles():
+            for allow, path in role.info.get('rules', ()):
+                if allow:
+                    continue
+                if path[:permlen] == perm and len(path) > permlen:
+                    return True
+
+        return False
+
     def clearAuthCache(self):
         self.permcache.clear()
+        self.allowedcache.clear()
 
     async def genGateInfo(self, gateiden):
         info = self.authgates.get(gateiden)
@@ -853,17 +1138,17 @@ class HiveUser(HiveRuler):
         perm = '.'.join(perm)
         if gateiden is None:
             mesg = f'User {self.name!r} ({self.iden}) must have permission {perm}'
-            raise s_exc.AuthDeny(mesg=mesg, perm=perm, user=self.name)
+            raise s_exc.AuthDeny(mesg=mesg, perm=perm, user=self.iden, username=self.name)
 
         gate = self.auth.reqAuthGate(gateiden)
         mesg = f'User {self.name!r} ({self.iden}) must have permission {perm} on object {gate.iden} ({gate.type}).'
-        raise s_exc.AuthDeny(mesg=mesg, perm=perm, user=self.name)
+        raise s_exc.AuthDeny(mesg=mesg, perm=perm, user=self.iden, username=self.name)
 
     def getRoles(self):
         for iden in self.info.get('roles', ()):
             role = self.auth.role(iden)
             if role is None:
-                logger.warn('user {self.iden} has non-existent role: {iden}')
+                logger.warning(f'user {self.iden} has non-existent role: {iden}')
                 continue
             yield role
 
@@ -883,7 +1168,8 @@ class HiveUser(HiveRuler):
         else:
             roles.insert(indx, role.iden)
 
-        await self.auth.setUserInfo(self.iden, 'roles', roles)
+        mesg = {'name': 'role:grant', 'iden': self.iden, 'role': role.pack()}
+        await self.auth.setUserInfo(self.iden, 'roles', roles, mesg=mesg)
 
     async def setRoles(self, roleidens):
         '''
@@ -910,10 +1196,13 @@ class HiveUser(HiveRuler):
             mesg = 'Role "all" must be in the list of roles set.'
             raise s_exc.BadArg(mesg=mesg)
 
+        roles = []
         for iden in roleidens:
-            await self.auth.reqRole(iden)
+            r = await self.auth.reqRole(iden)
+            roles.append(r.pack())
 
-        await self.auth.setUserInfo(self.iden, 'roles', roleidens)
+        mesg = {'name': 'role:set', 'iden': self.iden, 'roles': roles}
+        await self.auth.setUserInfo(self.iden, 'roles', roleidens, mesg=mesg)
 
     async def revoke(self, iden, nexs=True):
 
@@ -928,10 +1217,8 @@ class HiveUser(HiveRuler):
             return
 
         roles.remove(role.iden)
-        if nexs:
-            await self.auth.setUserInfo(self.iden, 'roles', roles)
-        else:
-            await self.auth._hndlsetUserInfo(self.iden, 'roles', roles)
+        mesg = {'name': 'role:revoke', 'iden': self.iden, 'role': role.pack()}
+        await self.auth.setUserInfo(self.iden, 'roles', roles, logged=nexs, mesg=mesg)
 
     def isLocked(self):
         return self.info.get('locked')
@@ -949,31 +1236,39 @@ class HiveUser(HiveRuler):
 
         return gateinfo.get('admin', False)
 
+    def reqAdmin(self, gateiden=None, mesg=None):
+
+        if self.isAdmin(gateiden=gateiden):
+            return
+
+        if mesg is None:
+            mesg = 'This action requires global admin permissions.'
+            if gateiden is not None:
+                mesg = f'This action requires admin permissions on gate: {gateiden}'
+
+        raise s_exc.AuthDeny(mesg=mesg, user=self.iden, username=self.name)
+
+    def isArchived(self):
+        return self.info.get('archived')
+
     async def setAdmin(self, admin, gateiden=None, logged=True):
         if not isinstance(admin, bool):
             raise s_exc.BadArg(mesg='setAdmin requires a boolean')
-        if logged:
-            await self.auth.setUserInfo(self.iden, 'admin', admin, gateiden=gateiden)
-        else:
-            await self.auth._hndlsetUserInfo(self.iden, 'admin', admin, gateiden=gateiden)
+        await self.auth.setUserInfo(self.iden, 'admin', admin, gateiden=gateiden, logged=logged)
 
     async def setLocked(self, locked, logged=True):
         if not isinstance(locked, bool):
             raise s_exc.BadArg(mesg='setLocked requires a boolean')
-        if logged:
-            await self.auth.setUserInfo(self.iden, 'locked', locked)
-        else:
-            await self.auth._hndlsetUserInfo(self.iden, 'locked', locked)
+        await self.auth.setUserInfo(self.iden, 'locked', locked, logged=logged)
 
     async def setArchived(self, archived):
         if not isinstance(archived, bool):
             raise s_exc.BadArg(mesg='setArchived requires a boolean')
-        archived = bool(archived)
         await self.auth.setUserInfo(self.iden, 'archived', archived)
         if archived:
             await self.setLocked(True)
 
-    async def tryPasswd(self, passwd):
+    async def tryPasswd(self, passwd, nexs=True):
 
         if self.info.get('locked', False):
             return False
@@ -983,21 +1278,40 @@ class HiveUser(HiveRuler):
 
         onepass = self.info.get('onepass')
         if onepass is not None:
-            expires, salt, hashed = onepass
-            if expires >= s_common.now():
-                if s_common.guid((salt, passwd)) == hashed:
-                    await self.auth.setUserInfo(self.iden, 'onepass', None)
-                    return True
+            if isinstance(onepass, dict):
+                shadow = onepass.get('shadow')
+                expires = onepass.get('expires')
+                if expires >= s_common.now():
+                    if await s_passwd.checkShadowV2(passwd=passwd, shadow=shadow):
+                        await self.auth.setUserInfo(self.iden, 'onepass', None)
+                        logger.debug(f'Used one time password for {self.name}',
+                                     extra={'synapse': {'user': self.iden, 'username': self.name}})
+                        return True
             else:
-                await self.auth.setUserInfo(self.iden, 'onepass', None)
+                # Backwards compatible password handling
+                expires, params, hashed = onepass
+                if expires >= s_common.now():
+                    if s_common.guid((params, passwd)) == hashed:
+                        await self.auth.setUserInfo(self.iden, 'onepass', None)
+                        logger.debug(f'Used one time password for {self.name}',
+                                     extra={'synapse': {'user': self.iden, 'username': self.name}})
+                        return True
 
         shadow = self.info.get('passwd')
         if shadow is None:
             return False
 
-        salt, hashed = shadow
+        if isinstance(shadow, dict):
+            return await s_passwd.checkShadowV2(passwd=passwd, shadow=shadow)
 
+        # Backwards compatible password handling
+        salt, hashed = shadow
         if s_common.guid((salt, passwd)) == hashed:
+            logger.debug(f'Migrating password to shadowv2 format for user {self.name}',
+                         extra={'synapse': {'user': self.iden, 'username': self.name}})
+            # Update user to new password hashing scheme.
+            await self.setPasswd(passwd=passwd, nexs=nexs)
+
             return True
 
         return False
@@ -1007,10 +1321,7 @@ class HiveUser(HiveRuler):
         if passwd is None:
             shadow = None
         elif passwd and isinstance(passwd, str):
-            shadow = getShadow(passwd)
+            shadow = await s_passwd.getShadowV2(passwd=passwd)
         else:
             raise s_exc.BadArg(mesg='Password must be a string')
-        if nexs:
-            await self.auth.setUserInfo(self.iden, 'passwd', shadow)
-        else:
-            await self.auth._hndlsetUserInfo(self.iden, 'passwd', shadow)
+        await self.auth.setUserInfo(self.iden, 'passwd', shadow, logged=nexs)
