@@ -1,11 +1,11 @@
 import socket
 import asyncio
 import logging
-
-from OpenSSL import crypto
-
 import collections
 
+import cryptography.x509 as c_x509
+import cryptography.hazmat.primitives.hashes as c_hashes
+
 logger = logging.getLogger(__name__)
 
 import synapse.exc as s_exc
@@ -21,7 +21,7 @@ async def connect(host, port, ssl=None, hostname=None, linkinfo=None):
     '''
     Async connect and return a Link().
     '''
-    info = {'host': host, 'port': port, 'ssl': ssl, 'hostname': hostname}
+    info = {'host': host, 'port': port, 'ssl': ssl, 'hostname': hostname, 'tls': bool(ssl)}
     if linkinfo is not None:
         info.update(linkinfo)
 
@@ -78,13 +78,13 @@ async def linkfile(mode='wb'):
 
     return link0, file1
 
-async def linksock():
+async def linksock(forceclose=False):
     '''
     Connect a Link, socket pair.
     '''
     sock0, sock1 = socket.socketpair()
     reader, writer = await asyncio.open_connection(sock=sock0)
-    link0 = await Link.anit(reader, writer, info={'unix': True})
+    link0 = await Link.anit(reader, writer, info={'unix': True}, forceclose=forceclose)
     return link0, sock1
 
 async def fromspawn(spawninfo):
@@ -98,12 +98,14 @@ class Link(s_base.Base):
     '''
     A Link() is created to wrap a socket reader/writer.
     '''
-    async def __anit__(self, reader, writer, info=None):
+    async def __anit__(self, reader, writer, info=None, forceclose=False):
 
         await s_base.Base.__anit__(self)
 
         self.iden = s_common.guid()
 
+        writer.transport.set_write_buffer_limits(high=1)
+
         self.reader = reader
         self.writer = writer
 
@@ -112,6 +114,7 @@ class Link(s_base.Base):
         self.sock = self.writer.get_extra_info('socket')
         self.peercert = self.writer.get_extra_info('peercert')
 
+        self._forceclose = forceclose
         self._drain_lock = asyncio.Lock()
 
         if info is None:
@@ -134,13 +137,34 @@ class Link(s_base.Base):
 
         self.info = info
 
+        self._addrinfo = {}
+        # _addrinfo is populated in this order so that as first hit tls links (prod deployments)
+        # then unix links (unit tests with local sockets, container healthchecks, local tools )
+        # then tcp links ( unit tests and legacy deployments )
+        if self.info.get('tls'):
+            self._addrinfo['family'] = 'tls'
+            self._addrinfo['addr'] = self.sock.getpeername()
+        elif self.info.get('unix'):
+            self._addrinfo['family'] = 'unix'
+            # Unix sockets don't use getpeername
+            self._addrinfo['addr'] = self.sock.getsockname()
+        else:
+            self._addrinfo['family'] = 'tcp'
+            self._addrinfo['addr'] = self.sock.getpeername()
+        if self.sock.family == socket.AF_INET:
+            self._addrinfo['ipver'] = 'ipv4'
+        elif self.sock.family == socket.AF_INET6:
+            self._addrinfo['ipver'] = 'ipv6'
+
         self.unpk = s_msgpack.Unpk()
 
         async def fini():
             self.writer.close()
+            if self._forceclose:
+                self.reader._transport.abort()
             try:
                 await self.writer.wait_closed()
-            except (BrokenPipeError, ConnectionResetError) as e:
+            except Exception as e:
                 logger.debug('Link error waiting on close: %s', str(e))
 
         self.onfini(fini)
@@ -151,8 +175,8 @@ class Link(s_base.Base):
         if self.certhash is not None:
 
             byts = info.get('ssl').telessl.getpeercert(True)
-            cert = crypto.load_certificate(crypto.FILETYPE_ASN1, byts)
-            thishash = cert.digest('SHA256').decode().lower().replace(':', '')
+            cert = c_x509.load_der_x509_certificate(byts)
+            thishash = s_common.ehex(cert.fingerprint(c_hashes.SHA256()))
             if thishash != self.certhash:
                 mesg = f'Server cert does not match pinned certhash={self.certhash}.'
                 raise s_exc.LinkBadCert(mesg=mesg)
@@ -207,26 +231,12 @@ class Link(s_base.Base):
         '''
         Get a summary of address information related to the link.
         '''
-        ret = {'family': 'tcp',
-               'addr': self.sock.getpeername(),
-               }
-        # Set family information
-        if self.info.get('unix'):
-            ret['family'] = 'unix'
-            # Unix sockets don't use getpeername
-            ret['addr'] = self.sock.getsockname()
-        elif self.info.get('tls'):
-            ret['family'] = 'tls'
-        # Set ipver if needed
-        if self.sock.family == socket.AF_INET:
-            ret['ipver'] = 'ipv4'
-        if self.sock.family == socket.AF_INET6:
-            ret['ipver'] = 'ipv6'
-        return ret
+        return dict(self._addrinfo)
 
     async def send(self, byts):
         self.writer.write(byts)
-        # Avoid Python bug.  See https://bugs.python.org/issue29930
+        # Avoid Python bug.  See https://github.com/python/cpython/issues/74116
+        # TODO Remove drain lock in 3.10+
         async with self._drain_lock:
             await self.writer.drain()
 
@@ -241,8 +251,8 @@ class Link(s_base.Base):
         try:
 
             self.writer.write(byts)
-
-            # Avoid Python bug.  See https://bugs.python.org/issue29930
+            # Avoid Python bug.  See https://github.com/python/cpython/issues/74116
+            # TODO Remove drain lock in 3.10+
             async with self._drain_lock:
                 await self.writer.drain()
 
@@ -255,6 +265,9 @@ class Link(s_base.Base):
 
             raise
 
+    def txfini(self):
+        self.sock.shutdown(1)
+
     async def recv(self, size):
         return await self.reader.read(size)
 
@@ -294,10 +307,11 @@ class Link(s_base.Base):
                 raise
 
             except Exception as e:
+                mesg = f'rx error {e} link={self.getAddrInfo()}'
                 if isinstance(e, (BrokenPipeError, ConnectionResetError)):
-                    logger.warning('rx error: %s', e)
+                    logger.warning(mesg)
                 else:
-                    logger.exception('rx error')
+                    logger.exception(mesg)
                 await self.fini()
                 return None
 
