@@ -15,6 +15,8 @@ import synapse.lib.base as s_base
 
 logger = logging.getLogger(__name__)
 
+leaderversion = 'Leader is a higher version than we are.'
+
 # As a mirror follower, amount of time before giving up on a write request
 FOLLOWER_WRITE_WAIT_S = 30.0
 
@@ -85,11 +87,17 @@ class NexsRoot(s_base.Base):
         self.client = None
         self.started = False
         self.celliden = self.cell.iden
-        self.applylock = asyncio.Lock()
+        self.readonly = False
+        self.writeholds = set()
+
+        self.applytask = None
 
         self.ready = asyncio.Event()
         self.donexslog = self.cell.conf.get('nexslog:en')
 
+        self.miruplink = asyncio.Event()
+        self._mirready = asyncio.Event()  # for testing
+
         self._mirrors: List[ChangeDist] = []
         self._nexskids: Dict[str, 'Pusher'] = {}
 
@@ -101,8 +109,8 @@ class NexsRoot(s_base.Base):
 
         logpath = s_common.genpath(self.dirn, 'slabs', 'nexuslog')
 
-        self.map_async = self.cell.conf.get('nexslog:async')
-        self.nexsslab = await s_lmdbslab.Slab.anit(path, map_async=self.map_async)
+        self.nexsslab = await cell._initSlabFile(path)
+
         self.nexshot = await self.nexsslab.getHotCount('nexs:indx')
 
         if fresh:
@@ -115,8 +123,7 @@ class NexsRoot(s_base.Base):
         elif vers != 2:
             raise s_exc.BadStorageVersion(mesg=f'Got nexus log version {vers}.  Expected 2.  Accidental downgrade?')
 
-        slabopts = {'map_async': self.map_async}
-        self.nexslog = await s_multislabseqn.MultiSlabSeqn.anit(logpath, slabopts=slabopts)
+        self.nexslog = await s_multislabseqn.MultiSlabSeqn.anit(logpath, cell=cell)
 
         # just in case were previously configured differently
         logindx = self.nexslog.index()
@@ -136,6 +143,9 @@ class NexsRoot(s_base.Base):
 
         self.onfini(fini)
 
+    def getNexsKids(self):
+        return list(self._nexskids.values())
+
     async def _migrateV1toV2(self, nexspath, logpath):
         '''
         Close the slab, move it to the new multislab location, then copy out the nexshot
@@ -184,7 +194,8 @@ class NexsRoot(s_base.Base):
 
         # Open a fresh slab where the old one used to be
         logger.warning(f'Re-opening fresh nexslog slab at {nexspath} for nexshot')
-        self.nexsslab = await s_lmdbslab.Slab.anit(nexspath, map_async=self.map_async)
+        self.nexsslab = await self.cell._initSlabFile(nexspath)
+
         self.nexshot = await self.nexsslab.getHotCount('nexs:indx')
 
         logger.warning('Copying nexs:indx data from migrated slab to the fresh nexslog')
@@ -193,7 +204,7 @@ class NexsRoot(s_base.Base):
             olddb = self.nexsslab.initdb('nexs:indx')
             self.nexsslab.dropdb(olddb)
             db = newslab.initdb('nexs:indx')
-            newslab.copydb('nexs:indx', self.nexsslab, destdbname='nexs:indx')
+            await newslab.copydb('nexs:indx', self.nexsslab, destdbname='nexs:indx')
             newslab.dropdb(db)
 
         self.nexshot.set('version', 2)
@@ -217,7 +228,7 @@ class NexsRoot(s_base.Base):
 
     async def enNexsLog(self):
 
-        async with self.applylock:
+        async with self.cell.nexslock:
 
             if self.donexslog:
                 return
@@ -253,21 +264,59 @@ class NexsRoot(s_base.Base):
             raise
 
         except Exception:
-            logger.exception('Exception while replaying log')
+            logger.exception(f'Exception while replaying log: {s_common.trimText(repr(indxitem))}')
+
+    async def addWriteHold(self, reason):
+
+        self.readonly = True
+
+        if reason not in self.writeholds:
+            logger.error(f'Entering Read-Only Mode: {reason}')
+            self.writeholds.add(reason)
+            return True
+
+        return False
+
+    async def delWriteHold(self, reason):
+
+        if reason in self.writeholds:
+
+            self.writeholds.remove(reason)
+
+            if not self.writeholds:
+                self.readonly = False
+
+            return True
+
+        return False
+
+    def reqNotReadOnly(self):
+
+        if not self.readonly:
+            return
+
+        # sets have stable order like dicts, so use the first message...
+        for reason in self.writeholds:
+            raise s_exc.IsReadOnly(mesg=reason)
+
+        mesg = 'Unable to issue Nexus events when readonly is set.'
+        raise s_exc.IsReadOnly(mesg=mesg)
 
     async def issue(self, nexsiden, event, args, kwargs, meta=None):
         '''
         If I'm not a follower, mutate, otherwise, ask the leader to make the change and wait for the follower loop
         to hand me the result through a future.
         '''
-        assert self.started, 'Attempt to issue before nexsroot is started'
-
         # pick up a reference to avoid race when we eventually can promote
         client = self.client
 
         if client is None:
             return await self.eat(nexsiden, event, args, kwargs, meta)
 
+        # check here because we shouldn't be sending an edit upstream if we
+        # are in readonly mode because the mirror sync will never complete.
+        self.reqNotReadOnly()
+
         try:
             await client.waitready(timeout=FOLLOWER_WRITE_WAIT_S)
         except asyncio.TimeoutError:
@@ -283,7 +332,7 @@ class NexsRoot(s_base.Base):
         with self._getResponseFuture(iden=meta.get('resp')) as (iden, futu):
             meta['resp'] = iden
             await client.issue(nexsiden, event, args, kwargs, meta)
-            return await asyncio.wait_for(futu, timeout=FOLLOWER_WRITE_WAIT_S)
+            return await s_common.wait_for(futu, timeout=FOLLOWER_WRITE_WAIT_S)
 
     async def eat(self, nexsiden, event, args, kwargs, meta):
         '''
@@ -292,7 +341,17 @@ class NexsRoot(s_base.Base):
         if meta is None:
             meta = {}
 
-        return await self._eat((nexsiden, event, args, kwargs, meta))
+        if (nexus := self._nexskids.get(nexsiden)) is None:
+            raise s_exc.NoSuchIden(mesg=f'No Nexus Pusher with iden {nexsiden}.', iden=nexsiden)
+
+        if event not in nexus._nexshands:
+            raise s_exc.NoSuchName(mesg=f'No Nexus handler for event {event}.', name=event)
+
+        async with self.cell.nexslock:
+            self.reqNotReadOnly()
+            # Keep a reference to the shielded task to ensure it isn't GC'd
+            self.applytask = asyncio.create_task(self._eat((nexsiden, event, args, kwargs, meta)))
+            return await asyncio.shield(self.applytask)
 
     async def index(self):
         if self.donexslog:
@@ -345,11 +404,10 @@ class NexsRoot(s_base.Base):
         nexus = self._nexskids[nexsiden]
         func, passitem = nexus._nexshands[event]
 
-        async with self.applylock:
-            if passitem:
-                return await func(nexus, *args, nexsitem=(indx, mesg), **kwargs)
+        if passitem:
+            return await func(nexus, *args, nexsitem=(indx, mesg), **kwargs)
 
-            return await func(nexus, *args, **kwargs)
+        return await func(nexus, *args, **kwargs)
 
     async def iter(self, offs: int, tellready=False) -> AsyncIterator[Any]:
         '''
@@ -397,6 +455,8 @@ class NexsRoot(s_base.Base):
         if self.client is not None:
             await self.client.fini()
 
+        self._mirready.clear()
+
         self.client = None
 
         mirurl = self.cell.conf.get('mirror')
@@ -409,7 +469,11 @@ class NexsRoot(s_base.Base):
 
         self.started = True
 
+    async def isNexsReady(self):
+        return self.ready.is_set()
+
     async def setNexsReady(self, status):
+
         if status:
             self.ready.set()
         else:
@@ -429,27 +493,47 @@ class NexsRoot(s_base.Base):
         await self.startup()
 
     async def _onTeleLink(self, proxy):
-        self.client.schedCoro(self.runMirrorLoop(proxy))
+        self.miruplink.set()
+        proxy.onfini(self.miruplink.clear)
+        proxy.schedCoro(self.runMirrorLoop(proxy))
 
     async def runMirrorLoop(self, proxy):
 
-        cellinfo = await proxy.getCellInfo()
-        features = cellinfo.get('features', {})
-        if features.get('dynmirror'):
-            await proxy.readyToMirror()
+        try:
+            cellinfo = await proxy.getCellInfo()
+            features = cellinfo.get('features', {})
+            if features.get('dynmirror'):
+                await proxy.readyToMirror()
 
-        cellvers = cellinfo['synapse']['version']
-
-        if self.celliden is not None:
-            if self.celliden != await proxy.getCellIden():
-                logger.error('remote cell has different iden!  Aborting mirror sync')
-                await proxy.fini()
-                await self.fini()
+            cellvers = cellinfo['cell']['version']
+            if cellvers > self.cell.VERSION:
+                logger.error('Leader is a higher version than we are. Mirrors must be updated first. Entering read-only mode.')
+                await self.addWriteHold(leaderversion)
+                # this will fire again on reconnect...
                 return
 
+            # When we reconnect and the leader version has become ok...
+            await self.delWriteHold(leaderversion)
+
+            if self.celliden is not None:
+                if self.celliden != await proxy.getCellIden():
+                    logger.error('remote cell has different iden!  Aborting mirror sync')
+                    await proxy.fini()
+                    await self.fini()
+                    return
+
+        except Exception as exc:
+            logger.exception(f'Unknown error during mirror loop startup: {exc}')
+            await proxy.fini()
+
         while not proxy.isfini:
 
             try:
+
+                if self.readonly:
+                    await self.waitfini(timeout=2)
+                    continue
+
                 offs = self.nexslog.index()
 
                 opts = {}
@@ -462,14 +546,19 @@ class NexsRoot(s_base.Base):
                     if proxy.isfini:  # pragma: no cover
                         break
 
+                    if self.readonly:
+                        break
+
                     # with tellready we move to ready=true when we get a None
                     if item is None:
+                        logger.info(f'Entering realtime change window for syncing data at offset={offs}')
                         await self.setNexsReady(True)
+                        self._mirready.set()
                         continue
 
                     offs, args = item
-                    if offs != self.nexslog.index():  # pragma: no cover
-                        logger.error('mirror desync')
+                    if offs != self.nexslog.index():
+                        logger.error('Local Nexus offset is out of sync from remote cell! Aborting mirror sync')
                         await self.fini()
                         return
 
@@ -480,9 +569,6 @@ class NexsRoot(s_base.Base):
                     try:
                         retn = await self.eat(*args)
 
-                    except asyncio.CancelledError:  # pragma: no cover  TODO:  remove once >= py 3.8 only
-                        raise
-
                     except Exception as e:
                         if respfutu is not None:
                             assert not respfutu.done()
@@ -494,11 +580,17 @@ class NexsRoot(s_base.Base):
                         if respfutu is not None:
                             respfutu.set_result(retn)
 
-            except asyncio.CancelledError:  # pragma: no cover  TODO:  remove once >= py 3.8 only
-                raise
+            except s_exc.LinkShutDown:
+                logger.warning(f'mirror loop: leader closed the connection.')
 
-            except Exception:  # pragma: no cover
-                logger.exception('error in mirror loop')
+            except Exception as exc:  # pragma: no cover
+                logger.exception(f'error in mirror loop: {exc}')
+
+        # If we've left the mirror loop for some reason, we no longer know if we
+        # will be in the realtime window or not. So we should try to set the ready
+        # value to false and clear our internal flag.
+        if not self.isfini:
+            await self.setNexsReady(not self.cell.conf.get('mirror'))
 
     async def _tellAhaReady(self, status):
 
@@ -507,10 +599,11 @@ class NexsRoot(s_base.Base):
 
         try:
             await self.cell.ahaclient.waitready(timeout=5)
-            ahainfo = await self.cell.ahaclient.getCellInfo()
+            proxy = await self.cell.ahaclient.proxy(timeout=5)
+            ahainfo = await proxy.getCellInfo()
             ahavers = ahainfo['synapse']['version']
             if self.cell.ahasvcname is not None and ahavers >= (2, 95, 0):
-                await self.cell.ahaclient.modAhaSvcInfo(self.cell.ahasvcname, {'ready': True})
+                await proxy.modAhaSvcInfo(self.cell.ahasvcname, {'ready': status})
 
         except asyncio.CancelledError:  # pragma: no cover  TODO:  remove once >= py 3.8 only
             raise
@@ -547,9 +640,21 @@ class Pusher(s_base.Base, metaclass=RegMethType):
             assert prev is not None, f'Failed removing {self.nexsiden}'
 
         self.onfini(onfini)
-
         self.nexsroot = nexsroot
 
+    async def modNexsRoot(self, ctor):
+
+        kids = [self]
+        if self.nexsroot is not None:
+            kids = self.nexsroot.getNexsKids()
+            await self.nexsroot.fini()
+
+        nexsroot = await ctor()
+
+        [kid.setNexsRoot(nexsroot) for kid in kids]
+
+        await nexsroot.startup()
+
     @classmethod
     def onPush(cls, event: str, passitem=False) -> Callable:
         '''
