@@ -1,4 +1,6 @@
 import ast
+import hashlib
+import collections
 
 import lark  # type: ignore
 import regex  # type: ignore
@@ -18,10 +20,9 @@ import synapse.lib.datfile as s_datfile
 # For easier-to-understand syntax errors
 terminalEnglishMap = {
     'AS': 'as',
-    'ABSPROP': 'absolute or universal property',
-    'ABSPROPNOUNIV': 'absolute property',
     'ALLTAGS': '#',
     'AND': 'and',
+    'BACKQUOTE': '`',
     'BASEPROP': 'base property name',
     'BOOL': 'boolean',
     'BREAK': 'break',
@@ -45,14 +46,21 @@ terminalEnglishMap = {
     'DOT': '.',
     'DOUBLEQUOTEDSTRING': 'double-quoted string',
     'ELIF': 'elif',
+    'EMBEDPROPS': 'absolute property name with embed properties',
     'EQNOSPACE': '=',
     'EQSPACE': '=',
     'EQUAL': '=',
     'EXPRDIVIDE': '/',
     'EXPRMINUS': '-',
+    'EXPRMODULO': '%',
+    'EXPRNEG': '-',
     'EXPRPLUS': '+',
+    'EXPRPOW': '**',
+    'EXPRTAGSEGNOVAR': 'non-variable tag segment',
     'EXPRTIMES': '*',
     'FOR': 'for',
+    'FORMATSTRING': 'backtick-quoted format string',
+    'FORMATTEXT': 'text within a format string',
     'FUNCTION': 'function',
     'HASH': '#',
     'HEXNUMBER': 'number',
@@ -62,35 +70,44 @@ terminalEnglishMap = {
     'LISTTOKN': 'unquoted list value',
     'LPAR': '(',
     'LSQB': '[',
+    'MCASEBARE': 'case multi-value',
     'MODSET': '+= or -=',
     'NONQUOTEWORD': 'unquoted value',
     'NOT': 'not',
+    'NULL': 'null',
     'NUMBER': 'number',
+    'OCTNUMBER': 'number',
     'OR': 'or',
     'PROPS': 'absolute property name',
     'RBRACE': '}',
     'RELNAME': 'relative property name',
+    'EXPRRELNAME': 'relative property name',
     'RPAR': ')',
     'RSQB': ']',
     'RSQBNOSPACE': ']',
     'SETTAGOPER': '?',
     'SINGLEQUOTEDSTRING': 'single-quoted string',
     'SWITCH': 'switch',
+    'TAGSEGNOVAR': 'non-variable tag segment',
     'TRY': 'try',
-    'TAGMATCH': 'tag name potentially with asterisks',
     'TRIPLEQUOTEDSTRING': 'triple-quoted string',
     'TRYSET': '?=',
     'TRYSETPLUS': '?+=',
     'TRYSETMINUS': '?-=',
     'UNIVNAME': 'universal property',
+    'EXPRUNIVNAME': 'universal property',
     'VARTOKN': 'variable',
+    'EXPRVARTOKN': 'variable',
     'VBAR': '|',
     'WHILE': 'while',
     'WHITETOKN': 'An unquoted string terminated by whitespace',
     'WILDCARD': '*',
+    'WILDPROPS': 'property name potentially with wildcards',
+    'WILDTAGSEGNOVAR': 'tag segment potentially with asterisks',
     'YIELD': 'yield',
     '_ARRAYCONDSTART': '*[',
     '_COLONDOLLAR': ':$',
+    '_COLONNOSPACE': ':',
     '_DEREF': '*',
     '_EDGEADDN1INIT': '+(',
     '_EDGEADDN2FINI': ')+',
@@ -98,9 +115,13 @@ terminalEnglishMap = {
     '_EDGEN1INIT': '-(',
     '_EDGEN2INIT': '<(',
     '_EDGEN2FINI': ')-',
+    '_EDGEN1JOINFINI': ')+>',
+    '_EDGEN2JOININIT': '<+(',
     '_ELSE': 'else',
     '_EMBEDQUERYSTART': '${',
+    '_EXPRCOLONNOSPACE': ':',
     '_EMIT': 'emit',
+    '_EMPTY': 'empty',
     '_FINI': 'fini',
     '_HASH': '#',
     '_HASHSPACE': '#',
@@ -108,16 +129,22 @@ terminalEnglishMap = {
     '_LEFTJOIN': '<+-',
     '_LEFTPIVOT': '<-',
     '_LPARNOSPACE': '(',
+    '_MATCHHASH': '#',
+    '_MATCHHASHWILD': '#',
     '_RETURN': 'return',
+    '_REVERSE': 'reverse',
     '_RIGHTJOIN': '-+>',
     '_RIGHTPIVOT': '->',
     '_STOP': 'stop',
-    '_TAGSEGNOVAR': 'tag segment potentially with asterisks',
+    '_WALKNJOINN1': '--+>',
+    '_WALKNJOINN2': '<+--',
     '_WALKNPIVON1': '-->',
     '_WALKNPIVON2': '<--',
     '$END': 'end of input',
 }
 
+AstInfo = collections.namedtuple('AstInfo', ('text', 'soff', 'eoff', 'sline', 'eline', 'scol', 'ecol', 'isterm'))
+
 class AstConverter(lark.Transformer):
     '''
     Convert AST from parser into synapse AST, depth first.
@@ -130,24 +157,45 @@ class AstConverter(lark.Transformer):
 
         # Keep the original text for error printing and weird subquery argv parsing
         self.text = text
+        self.texthash = hashlib.md5(text.encode(errors='surrogatepass'), usedforsecurity=False).hexdigest()
 
-    @classmethod
-    def _convert_children(cls, children):
-        return [cls._convert_child(k) for k in children]
+    def metaToAstInfo(self, meta, isterm=False):
+        if isinstance(meta, lark.tree.Meta) and meta.empty:
+            return AstInfo(self.text, -1, -1, -1, -1, -1, -1, isterm)
+        return AstInfo(self.text,
+            meta.start_pos, meta.end_pos,
+            meta.line, meta.end_line,
+            meta.column, meta.end_column, isterm)
 
-    @classmethod
-    def _convert_child(cls, child):
+    def raiseBadSyntax(self, mesg, astinfo):
+        raise s_exc.BadSyntax(mesg=mesg,
+            # keep around for backward compatiblity
+            at=astinfo.soff,
+            line=astinfo.sline,
+            column=astinfo.scol,
+            highlight={
+                'hash': self.texthash,
+                'lines': (astinfo.sline, astinfo.eline),
+                'columns': (astinfo.scol, astinfo.ecol),
+                'offsets': (astinfo.soff, astinfo.eoff),
+            })
+
+    def _convert_children(self, children):
+        return [self._convert_child(k) for k in children]
+
+    def _convert_child(self, child):
         if not isinstance(child, lark.lexer.Token):
             return child
+
         tokencls = terminalClassMap.get(child.type, s_ast.Const)
-        newkid = tokencls(child.value)
-        return newkid
+
+        # Tokens have similar fields to meta
+        astinfo = self.metaToAstInfo(child, isterm=True)
+        return tokencls(astinfo, child.value)
 
     def __default__(self, treedata, children, treemeta):
         assert treedata in ruleClassMap, f'Unknown grammar rule: {treedata}'
-        cls = ruleClassMap[treedata]
-        newkids = self._convert_children(children)
-        return cls(newkids)
+        return ruleClassMap[treedata](self.metaToAstInfo(treemeta), self._convert_children(children))
 
     @lark.v_args(meta=True)
     def subquery(self, meta, kids):
@@ -158,171 +206,124 @@ class AstConverter(lark.Transformer):
 
         return kid
 
-    def _parseJsonToken(self, meta, tokn):
+    def _parseJsonToken(self, tokn):
+
         if isinstance(tokn, lark.lexer.Token) and tokn.type == 'VARTOKN' and not tokn.value[0] in ('"', "'"):
+
             valu = tokn.value
+            astinfo = self.metaToAstInfo(tokn)
+
             try:
                 valu = float(valu) if '.' in valu else int(valu, 0)
             except ValueError as e:
-                mesg = f"Unexpected unquoted string in JSON expression at line {meta.line} col {meta.column}"
-                raise s_exc.BadSyntax(mesg=mesg, at=meta.start_pos, line=meta.line, column=meta.column)
+                self.raiseBadSyntax('Unexpected unquoted string in JSON expression', astinfo)
 
-            return s_ast.Const(valu)
+            return s_ast.Const(astinfo, valu)
         else:
             return self._convert_child(tokn)
 
     @lark.v_args(meta=True)
     def exprlist(self, meta, kids):
-        kids = [self._parseJsonToken(meta, k) for k in kids]
-        return s_ast.ExprList(kids=kids)
+        kids = [self._parseJsonToken(k) for k in kids]
+        astinfo = self.metaToAstInfo(meta, isterm=True)
+        return s_ast.ExprList(astinfo, kids=kids)
 
     @lark.v_args(meta=True)
     def exprdict(self, meta, kids):
-        kids = [self._parseJsonToken(meta, k) for k in kids]
-        return s_ast.ExprDict(kids=kids)
+        kids = [self._parseJsonToken(k) for k in kids]
+        astinfo = self.metaToAstInfo(meta, isterm=True)
+        return s_ast.ExprDict(astinfo, kids=kids)
 
     @lark.v_args(meta=True)
-    def trycatch(self, meta, kids):
-        kids = self._convert_children(kids)
-        return s_ast.TryCatch(kids=kids)
-
-    @lark.v_args(meta=True)
-    def catchblock(self, meta, kids):
-        kids = self._convert_children(kids)
-        return s_ast.CatchBlock(kids=kids)
-
-    @lark.v_args(meta=True)
-    def baresubquery(self, meta, kids):
-        assert len(kids) == 1
-
-        epos = meta.end_pos
-        spos = meta.start_pos
-
-        subq = s_ast.SubQuery(kids)
-        subq.text = self.text[spos:epos]
-
-        return subq
-
-    @lark.v_args(meta=True)
-    def argvquery(self, meta, kids):
-        assert len(kids) == 1
-
-        epos = meta.end_pos
-        spos = meta.start_pos
-
-        argq = s_ast.ArgvQuery(kids)
-        argq.text = self.text[spos:epos]
-
-        return argq
-
-    def yieldvalu(self, kids):
+    def yieldvalu(self, meta, kids):
         kid = self._convert_child(kids[-1])
-        return s_ast.YieldValu(kids=[kid])
+        astinfo = self.metaToAstInfo(meta)
+        return s_ast.YieldValu(astinfo, kids=[kid])
 
     @lark.v_args(meta=True)
     def evalvalu(self, meta, kids):
         return self._convert_child(kids[0])
 
-    @lark.v_args(meta=True)
-    def lookup(self, meta, kids):
-        kids = self._convert_children(kids)
-        look = s_ast.Lookup(kids=kids)
-        return look
-
-    @lark.v_args(meta=True)
-    def search(self, meta, kids):
-        kids = self._convert_children(kids)
-        look = s_ast.Search(kids=kids)
-        return look
-
-    @lark.v_args(meta=True)
-    def query(self, meta, kids):
-        kids = self._convert_children(kids)
-
-        if kids:
-            epos = meta.end_pos
-            spos = meta.start_pos
-        else:
-            epos = spos = 0
-
-        quer = s_ast.Query(kids=kids)
-        quer.text = self.text[spos:epos]
-
-        return quer
-
     @lark.v_args(meta=True)
     def embedquery(self, meta, kids):
         assert len(kids) == 1
-        text = kids[0].text
-        ast = s_ast.EmbedQuery(text, kids)
-        return ast
-
-    @lark.v_args(meta=True)
-    def emit(self, meta, kids):
-        kids = self._convert_children(kids)
-        return s_ast.Emit(kids)
-
-    @lark.v_args(meta=True)
-    def stop(self, meta, kids):
-        return s_ast.Stop()
+        astinfo = self.metaToAstInfo(meta)
+        text = kids[0].getAstText()
+        return s_ast.EmbedQuery(kids[0].astinfo, text, kids=kids)
 
     @lark.v_args(meta=True)
     def funccall(self, meta, kids):
+
+        astinfo = self.metaToAstInfo(meta)
+
         argkids = []
         kwargkids = []
         kwnames = set()
         indx = 1
         kcnt = len(kids)
-        while indx < kcnt:
 
-            kid = self._convert_child(kids[indx])
+        todo = collections.deque(kids)
 
-            if indx + 2 < kcnt and isinstance(kids[indx + 1], lark.lexer.Token) and kids[indx + 1].type in ('EQNOSPACE', 'EQUAL', 'EQSPACE'):
-                kid = s_ast.CallKwarg((kid, self._convert_child(kids[indx + 2])))
-                indx += 3
-            else:
-                indx += 1
+        namekid = todo.popleft()
 
-            if isinstance(kid, s_ast.CallKwarg):
-                name = kid.kids[0].valu
-                if name in kwnames:
-                    mesg = f"Duplicate keyword argument '{name}' in function call at line {meta.line} col {meta.column}"
-                    raise s_exc.BadSyntax(mesg=mesg, at=meta.start_pos, line=meta.line, column=meta.column)
+        while todo:
 
-                kwnames.add(name)
-                kwargkids.append(kid)
-            else:
-                if kwargkids:
-                    mesg = f'Positional argument follows keyword argument in function call at line {meta.line} col {meta.column}'
-                    raise s_exc.BadSyntax(mesg=mesg, at=meta.start_pos, line=meta.line, column=meta.column)
-                argkids.append(kid)
+            kid = self._convert_child(todo.popleft())
 
-        args = s_ast.CallArgs(kids=argkids)
-        kwargs = s_ast.CallKwargs(kids=kwargkids)
-        return s_ast.FuncCall(kids=[kids[0], args, kwargs])
+            # look ahead for name = <valu> kwargs...
+            if len(todo) >= 2:
 
-    def varlist(self, kids):
+                nextkid = todo[0]
+                if isinstance(nextkid, lark.lexer.Token) and nextkid.type in ('EQNOSPACE', 'EQUAL', 'EQSPACE'):
+                    todo.popleft()
+                    valukid = self._convert_child(todo.popleft())
+
+                    if kid.valu in kwnames:
+                        self.raiseBadSyntax(f'Duplicate keyword argument "{kid.valu}" in function call', kid.astinfo)
+
+                    kwnames.add(kid.valu)
+                    kwargkids.append(s_ast.CallKwarg(kid.astinfo, (kid, valukid)))
+
+                    continue
+
+            if kwargkids:
+                self.raiseBadSyntax('Positional argument follows keyword argument in function call', kid.astinfo)
+
+            argkids.append(kid)
+
+        args = s_ast.CallArgs(astinfo, kids=argkids)
+        kwargs = s_ast.CallKwargs(astinfo, kids=kwargkids)
+        return s_ast.FuncCall(astinfo, kids=[namekid, args, kwargs])
+
+    @lark.v_args(meta=True)
+    def varlist(self, meta, kids):
         kids = self._convert_children(kids)
-        return s_ast.VarList([k.valu for k in kids])
+        astinfo = self.metaToAstInfo(meta)
+        return s_ast.VarList(astinfo, [k.valu for k in kids])
 
-    def operrelprop_pivot(self, kids, isjoin=False):
+    @lark.v_args(meta=True)
+    def operrelprop_pivot(self, meta, kids, isjoin=False):
         kids = self._convert_children(kids)
+        astinfo = self.metaToAstInfo(meta)
         relprop, rest = kids[0], kids[1:]
         if not rest:
-            return s_ast.PropPivotOut(kids=kids, isjoin=isjoin)
-        pval = s_ast.RelPropValue(kids=(relprop,))
-        return s_ast.PropPivot(kids=(pval, *kids[1:]), isjoin=isjoin)
+            return s_ast.PropPivotOut(astinfo, kids=kids, isjoin=isjoin)
+        pval = s_ast.RelPropValue(astinfo, kids=(relprop,))
+        return s_ast.PropPivot(astinfo, kids=(pval, *kids[1:]), isjoin=isjoin)
 
-    def operrelprop_join(self, kids):
-        return self.operrelprop_pivot(kids, isjoin=True)
+    @lark.v_args(meta=True)
+    def operrelprop_join(self, meta, kids):
+        return self.operrelprop_pivot(meta, kids, isjoin=True)
 
-    def stormcmdargs(self, kids):
+    @lark.v_args(meta=True)
+    def stormcmdargs(self, meta, kids):
         newkids = []
         for kid in kids:
             if isinstance(kid, lark.lexer.Token) and kid.type == 'EQNOSPACE':
                 continue
             newkids.append(self._convert_child(kid))
-        return s_ast.List(kids=newkids)
+        astinfo = self.metaToAstInfo(meta)
+        return s_ast.List(astinfo, kids=newkids)
 
     @lark.v_args(meta=True)
     def funcargs(self, meta, kids):
@@ -330,44 +331,47 @@ class AstConverter(lark.Transformer):
         A list of function parameters (as part of a function definition)
         '''
         kids = self._convert_children(kids)
-        newkids = []
+        astinfo = self.metaToAstInfo(meta)
 
-        names = set()
+        newkids = []
+        kwnames = set()
         kwfound = False
 
-        indx = 0
-        kcnt = len(kids)
+        todo = collections.deque(kids)
 
-        while indx < kcnt:
-            if indx + 2 < kcnt and isinstance(kids[indx + 1], s_ast.Const) and kids[indx + 1].valu == '=':
-                kid = s_ast.CallKwarg((kids[indx], kids[indx + 2]))
-                indx += 3
-            else:
-                kid = kids[indx]
-                indx += 1
+        while todo:
+
+            kid = todo.popleft()
+
+            if kid.valu in kwnames:
+                mesg = f'Duplicate parameter "{kid.valu}" in function definition'
+                self.raiseBadSyntax(mesg, kid.astinfo)
+
+            kwnames.add(kid.valu)
+
+            # look ahead for name = <default> kwarg decls
+            if len(todo) >= 2:
+
+                nextkid = todo[0]
+                if isinstance(nextkid, s_ast.Const) and nextkid.valu == '=':
+                    todo.popleft()
+                    valukid = todo.popleft()
+
+                    kwfound = True
+
+                    newkids.append(s_ast.CallKwarg(kid.astinfo, (kid, valukid)))
+                    continue
+
+            if kwfound:
+                mesg = f'Positional parameter "{kid.valu}" follows keyword parameter in definition'
+                self.raiseBadSyntax(mesg, astinfo)
 
             newkids.append(kid)
 
-            if isinstance(kid, s_ast.CallKwarg):
-                name = kid.kids[0].valu
-                kwfound = True
-                # Make sure no repeated kwarg
-            else:
-                name = kid.valu
-                # Make sure no positional follows a kwarg
-                if kwfound:
-                    mesg = f"Positional parameter '{name}' follows keyword parameter in definition at line {meta.line} col {meta.column}"
-                    raise s_exc.BadSyntax(mesg=mesg, at=meta.start_pos, line=meta.line, column=meta.column)
+        return s_ast.FuncArgs(astinfo, newkids)
 
-            if name in names:
-                mesg = f"Duplicate parameter '{name}' in function definition at line {meta.line} col {meta.column}"
-                raise s_exc.BadSyntax(mesg=mesg, at=meta.start_pos, line=meta.line, column=meta.column)
-
-            names.add(name)
-
-        return s_ast.FuncArgs(newkids)
-
-    def cmdrargs(self, kids):
+    @lark.v_args(meta=True)
+    def cmdrargs(self, meta, kids):
         argv = []
         indx = 0
 
@@ -408,63 +412,59 @@ class AstConverter(lark.Transformer):
 
             # pragma: no cover
             mesg = f'Unhandled AST node type in cmdrargs: {kid!r}'
-            raise s_exc.BadSyntax(mesg=mesg)
+            self.raiseBadSyntax(mesg, kid.astinfo)
 
         return argv
 
-    @classmethod
-    def _tagsplit(cls, tag):
-        if '$' not in tag:
-            return [s_ast.Const(tag)]
-
-        segs = tag.split('.')
-        kids = [s_ast.VarValue(kids=[s_ast.Const(seg[1:])]) if seg[0] == '$' else s_ast.Const(seg)
-                for seg in segs]
-        return kids
-
-    def varderef(self, kids):
+    @lark.v_args(meta=True)
+    def varderef(self, meta, kids):
         assert kids and len(kids) in (3, 4)
+        astinfo = self.metaToAstInfo(meta)
         if kids[2] == '$':
             tokencls = terminalClassMap.get(kids[3].type, s_ast.Const)
-            newkid = s_ast.VarValue(kids=[tokencls(kids[3])])
+            kidfo = self.metaToAstInfo(kids[3], isterm=True)
+            newkid = s_ast.VarValue(kidfo, kids=[tokencls(kids[3], kids[3])])
         else:
             newkid = self._convert_child(kids[2])
-        return s_ast.VarDeref(kids=(kids[0], newkid))
+        return s_ast.VarDeref(astinfo, kids=(kids[0], newkid))
 
-    def tagname(self, kids):
-        assert kids and len(kids) == 1
-        kid = kids[0]
-        if not isinstance(kid, lark.lexer.Token):
-            return self._convert_child(kid)
+    @lark.v_args(meta=True)
+    def caseentry(self, meta, kids):
+        assert kids and len(kids) >= 2
+        astinfo = self.metaToAstInfo(meta)
+        newkids = self._convert_children(kids)
 
-        valu = kid.value
-        if '*' in valu:
-            mesg = f"Invalid wildcard usage in tag {valu}"
-            raise s_exc.BadSyntax(mesg=mesg, tag=valu)
+        defcase = False
 
-        kids = self._tagsplit(valu)
-        return s_ast.TagName(kids=kids)
+        if len(kids) == 2 and kids[0].type == 'DEFAULTCASE':
+            defcase = True
+            # Strip off the "Const: *" node
+            newkids = [newkids[1]]
 
-    def switchcase(self, kids):
-        newkids = []
+        return s_ast.CaseEntry(astinfo, kids=newkids, defcase=defcase)
 
-        it = iter(kids)
+    @lark.v_args(meta=True)
+    def switchcase(self, meta, kids):
+        kids = self._convert_children(kids)
 
-        varvalu = next(it)
-        newkids.append(varvalu)
+        astinfo = self.metaToAstInfo(meta)
 
-        for casekid, sqkid in zip(it, it):
-            subquery = self._convert_child(sqkid)
-            if casekid.type == 'DEFAULTCASE':
-                caseentry = s_ast.CaseEntry(kids=[subquery])
-            else:
-                casekid = self._convert_child(casekid)
-                caseentry = s_ast.CaseEntry(kids=[casekid, subquery])
+        # Check that we only have one default case
+        defcase = [k for k in kids[1:] if k.defcase]
 
-            newkids.append(caseentry)
+        deflen = len(defcase)
+        if deflen > 1:
+            mesg = f'Switch statements cannot have more than one default case. Found {deflen}.'
+            raise self.raiseBadSyntax(mesg, astinfo)
 
-        return s_ast.SwitchCase(newkids)
+        return s_ast.SwitchCase(astinfo, kids)
 
+    @lark.v_args(meta=True)
+    def liftreverse(self, meta, kids):
+        assert len(kids) == 1
+        astinfo = self.metaToAstInfo(meta)
+        kids[0].reverseLift(astinfo)
+        return kids[0]
 
 with s_datfile.openDatFile('synapse.lib/storm.lark') as larkf:
     _grammar = larkf.read().decode()
@@ -480,7 +480,7 @@ class Parser:
 
         self.offs = offs
         assert text is not None
-        self.text = text.strip()
+        self.text = text
         self.size = len(self.text)
 
     def _larkToSynExc(self, e):
@@ -493,7 +493,7 @@ class Parser:
         column = None
         token = None
         if isinstance(e, lark.exceptions.UnexpectedToken):
-            expected = sorted(terminalEnglishMap[t] for t in e.expected)
+            expected = sorted(set(terminalEnglishMap[t] for t in e.expected))
             at = e.pos_in_stream
             line = e.line
             column = e.column
@@ -511,13 +511,13 @@ class Parser:
             return s_exc.BadSyntax(**origexc.errinfo)
 
         elif isinstance(e, lark.exceptions.UnexpectedCharacters):  # pragma: no cover
-            expected = sorted(terminalEnglishMap[t] for t in e.allowed)
+            expected = sorted(set(terminalEnglishMap[t] for t in e.allowed))
             mesg += f'.  Expecting one of: {", ".join(expected)}'
             at = e.pos_in_stream
             line = e.line
             column = e.column
         elif isinstance(e, lark.exceptions.UnexpectedEOF):  # pragma: no cover
-            expected = sorted(terminalEnglishMap[t] for t in set(e.expected))
+            expected = sorted(set(terminalEnglishMap[t] for t in e.expected))
             mesg += ' ' + ', '.join(expected)
             line = e.line
             column = e.column
@@ -554,18 +554,22 @@ class Parser:
     def lookup(self):
         try:
             tree = LarkParser.parse(self.text, start='lookup')
+            newtree = AstConverter(self.text).transform(tree)
+
         except lark.exceptions.LarkError as e:
             raise self._larkToSynExc(e) from None
-        newtree = AstConverter(self.text).transform(tree)
+
         newtree.text = self.text
         return newtree
 
     def search(self):
         try:
             tree = LarkParser.parse(self.text, start='search')
+            newtree = AstConverter(self.text).transform(tree)
+
         except lark.exceptions.LarkError as e:
             raise self._larkToSynExc(e) from None
-        newtree = AstConverter(self.text).transform(tree)
+
         newtree.text = self.text
         return newtree
 
@@ -575,9 +579,9 @@ class Parser:
         '''
         try:
             tree = LarkParser.parse(self.text, start='cmdrargs')
+            return AstConverter(self.text).transform(tree)
         except lark.exceptions.LarkError as e:
             raise self._larkToSynExc(e) from None
-        return AstConverter(self.text).transform(tree)
 
 def parseQuery(text, mode='storm'):
     '''
@@ -600,79 +604,86 @@ def parseEval(text):
     return Parser(text).eval()
 
 async def _forkedParseQuery(args):
-    return await s_coro.forked(parseQuery, args[0], mode=args[1])
+    return await s_coro._parserforked(parseQuery, args[0], mode=args[1])
 
 async def _forkedParseEval(text):
-    return await s_coro.forked(parseEval, text)
+    return await s_coro._parserforked(parseEval, text)
 
 evalcache = s_cache.FixedCache(_forkedParseEval, size=100)
 querycache = s_cache.FixedCache(_forkedParseQuery, size=100)
 
-def massage_vartokn(x):
-    return s_ast.Const('' if not x else (x[1:-1] if x[0] == "'" else (unescape(x) if x[0] == '"' else x)))
+def massage_vartokn(astinfo, x):
+    return s_ast.Const(astinfo, '' if not x else (x[1:-1] if x[0] == "'" else (unescape(x) if x[0] == '"' else x)))
 
 # For AstConverter, one-to-one replacements from lark to synapse AST
 terminalClassMap = {
-    'ABSPROP': s_ast.AbsProp,
-    'ABSPROPNOUNIV': s_ast.AbsProp,
-    'ALLTAGS': lambda _: s_ast.TagMatch(''),
-    'BREAK': lambda _: s_ast.BreakOper(),
-    'CONTINUE': lambda _: s_ast.ContinueOper(),
-    'DOUBLEQUOTEDSTRING': lambda x: s_ast.Const(unescape(x)),  # drop quotes and handle escape characters
-    'TRIPLEQUOTEDSTRING': lambda x: s_ast.Const(x[3:-3]), # drop the triple 's
-    'NUMBER': lambda x: s_ast.Const(s_ast.parseNumber(x)),
-    'HEXNUMBER': lambda x: s_ast.Const(s_ast.parseNumber(x)),
-    'BOOL': lambda x: s_ast.Bool(x == 'true'),
-    'SINGLEQUOTEDSTRING': lambda x: s_ast.Const(x[1:-1]),  # drop quotes
-    'TAGMATCH': lambda x: s_ast.TagMatch(kids=AstConverter._tagsplit(x)),
+    'ALLTAGS': lambda astinfo, _: s_ast.TagMatch(astinfo, ''),
+    'BREAK': lambda astinfo, _: s_ast.BreakOper(astinfo),
+    'CONTINUE': lambda astinfo, _: s_ast.ContinueOper(astinfo),
+    'DOUBLEQUOTEDSTRING': lambda astinfo, x: s_ast.Const(astinfo, unescape(x)),  # drop quotes and handle escape characters
+    'FORMATTEXT': lambda astinfo, x: s_ast.Const(astinfo, format_unescape(x)),  # handle escape characters
+    'TRIPLEQUOTEDSTRING': lambda astinfo, x: s_ast.Const(astinfo, x[3:-3]), # drop the triple 's
+    'NUMBER': lambda astinfo, x: s_ast.Const(astinfo, s_ast.parseNumber(x)),
+    'HEXNUMBER': lambda astinfo, x: s_ast.Const(astinfo, s_ast.parseNumber(x)),
+    'OCTNUMBER': lambda astinfo, x: s_ast.Const(astinfo, s_ast.parseNumber(x)),
+    'BOOL': lambda astinfo, x: s_ast.Bool(astinfo, x == 'true'),
+    'NULL': lambda astinfo, x: s_ast.Const(astinfo, None),
+    'SINGLEQUOTEDSTRING': lambda astinfo, x: s_ast.Const(astinfo, x[1:-1]),  # drop quotes
     'NONQUOTEWORD': massage_vartokn,
     'VARTOKN': massage_vartokn,
+    'EXPRVARTOKN': massage_vartokn,
 }
 
 # For AstConverter, one-to-one replacements from lark to synapse AST
 ruleClassMap = {
     'abspropcond': s_ast.AbsPropCond,
+    'argvquery': s_ast.ArgvQuery,
     'arraycond': s_ast.ArrayCond,
     'andexpr': s_ast.AndCond,
+    'baresubquery': s_ast.SubQuery,
+    'catchblock': s_ast.CatchBlock,
     'condsubq': s_ast.SubqCond,
     'dollarexpr': s_ast.DollarExpr,
-    'reqdollarexpr': s_ast.DollarExpr,
     'edgeaddn1': s_ast.EditEdgeAdd,
     'edgedeln1': s_ast.EditEdgeDel,
-    'edgeaddn2': lambda kids: s_ast.EditEdgeAdd(kids, n2=True),
-    'edgedeln2': lambda kids: s_ast.EditEdgeDel(kids, n2=True),
+    'edgeaddn2': lambda astinfo, kids: s_ast.EditEdgeAdd(astinfo, kids, n2=True),
+    'edgedeln2': lambda astinfo, kids: s_ast.EditEdgeDel(astinfo, kids, n2=True),
     'editnodeadd': s_ast.EditNodeAdd,
     'editparens': s_ast.EditParens,
+    'emit': s_ast.Emit,
     'initblock': s_ast.InitBlock,
+    'emptyblock': s_ast.EmptyBlock,
     'finiblock': s_ast.FiniBlock,
     'formname': s_ast.FormName,
-    'editpropdel': s_ast.EditPropDel,
+    'editpropdel': lambda astinfo, kids: s_ast.EditPropDel(astinfo, kids[1:]),
     'editpropset': s_ast.EditPropSet,
     'edittagadd': s_ast.EditTagAdd,
-    'edittagdel': s_ast.EditTagDel,
+    'edittagdel': lambda astinfo, kids: s_ast.EditTagDel(astinfo, kids[1:]),
     'edittagpropset': s_ast.EditTagPropSet,
-    'edittagpropdel': s_ast.EditTagPropDel,
-    'editunivdel': s_ast.EditUnivDel,
+    'edittagpropdel': lambda astinfo, kids: s_ast.EditTagPropDel(astinfo, kids[1:]),
+    'editunivdel': lambda astinfo, kids: s_ast.EditUnivDel(astinfo, kids[1:]),
     'editunivset': s_ast.EditPropSet,
     'expror': s_ast.ExprOrNode,
     'exprand': s_ast.ExprAndNode,
     'exprnot': s_ast.UnaryExprNode,
+    'exprunary': s_ast.UnaryExprNode,
     'exprcmp': s_ast.ExprNode,
+    'exprpow': s_ast.ExprNode,
     'exprproduct': s_ast.ExprNode,
     'exprsum': s_ast.ExprNode,
     'filtoper': s_ast.FiltOper,
-    'filtopermust': lambda kids: s_ast.FiltOper([s_ast.Const('+')] + kids),
-    'filtopernot': lambda kids: s_ast.FiltOper([s_ast.Const('-')] + kids),
+    'filtopermust': lambda astinfo, kids: s_ast.FiltOper(astinfo, [s_ast.Const(astinfo, '+')] + kids),
+    'filtopernot': lambda astinfo, kids: s_ast.FiltOper(astinfo, [s_ast.Const(astinfo, '-')] + kids),
     'forloop': s_ast.ForLoop,
-    'whileloop': s_ast.WhileLoop,
-    'formjoin_formpivot': lambda kids: s_ast.FormPivot(kids, isjoin=True),
-    'formjoin_pivotout': lambda _: s_ast.PivotOut(isjoin=True),
-    'formjoinin_pivotin': lambda kids: s_ast.PivotIn(kids, isjoin=True),
-    'formjoinin_pivotinfrom': lambda kids: s_ast.PivotInFrom(kids, isjoin=True),
+    'formatstring': s_ast.FormatString,
+    'formjoin_formpivot': lambda astinfo, kids: s_ast.FormPivot(astinfo, kids, isjoin=True),
+    'formjoin_pivotout': lambda astinfo, _: s_ast.PivotOut(astinfo, isjoin=True),
+    'formjoinin_pivotin': lambda astinfo, kids: s_ast.PivotIn(astinfo, kids, isjoin=True),
+    'formjoinin_pivotinfrom': lambda astinfo, kids: s_ast.PivotInFrom(astinfo, kids, isjoin=True),
     'formpivot_': s_ast.FormPivot,
     'formpivot_pivotout': s_ast.PivotOut,
     'formpivot_pivottotags': s_ast.PivotToTags,
-    'formpivot_jointags': lambda kids: s_ast.PivotToTags(kids, isjoin=True),
+    'formpivot_jointags': lambda astinfo, kids: s_ast.PivotToTags(astinfo, kids, isjoin=True),
     'formpivotin_': s_ast.PivotIn,
     'formpivotin_pivotinfrom': s_ast.PivotInFrom,
     'formtagprop': s_ast.FormTagProp,
@@ -681,7 +692,7 @@ ruleClassMap = {
     'hastagpropcond': s_ast.HasTagPropCond,
     'ifstmt': s_ast.IfStmt,
     'ifclause': s_ast.IfClause,
-    'kwarg': lambda kids: s_ast.CallKwarg(kids=tuple(kids)),
+    'kwarg': lambda astinfo, kids: s_ast.CallKwarg(astinfo, kids=tuple(kids)),
     'liftbytag': s_ast.LiftTag,
     'liftformtag': s_ast.LiftFormTag,
     'liftprop': s_ast.LiftProp,
@@ -691,37 +702,56 @@ ruleClassMap = {
     'liftbytagprop': s_ast.LiftTagProp,
     'liftbyformtagprop': s_ast.LiftFormTagProp,
     'looklist': s_ast.LookList,
+    'lookup': s_ast.Lookup,
+    'n1join': lambda astinfo, kids: s_ast.N1Walk(astinfo, kids, isjoin=True),
+    'n2join': lambda astinfo, kids: s_ast.N2Walk(astinfo, kids, isjoin=True),
     'n1walk': s_ast.N1Walk,
     'n2walk': s_ast.N2Walk,
+    'n1walknjoin': lambda astinfo, kids: s_ast.N1WalkNPivo(astinfo, kids, isjoin=True),
+    'n2walknjoin': lambda astinfo, kids: s_ast.N2WalkNPivo(astinfo, kids, isjoin=True),
     'n1walknpivo': s_ast.N1WalkNPivo,
     'n2walknpivo': s_ast.N2WalkNPivo,
     'notcond': s_ast.NotCond,
     'opervarlist': s_ast.VarListSetOper,
     'orexpr': s_ast.OrCond,
+    'query': s_ast.Query,
     'rawpivot': s_ast.RawPivot,
     'return': s_ast.Return,
-    'relprop': lambda kids: s_ast.RelProp([s_ast.Const(k.valu.lstrip(':')) if isinstance(k, s_ast.Const) else k for k in kids]),
+    'relprop': lambda astinfo, kids: s_ast.RelProp(astinfo, [s_ast.Const(k.astinfo, k.valu.lstrip(':')) if isinstance(k, s_ast.Const) else k for k in kids]),
     'relpropcond': s_ast.RelPropCond,
-    'relpropvalu': lambda kids: s_ast.RelPropValue([s_ast.Const(k.valu.lstrip(':')) if isinstance(k, s_ast.Const) else k for k in kids]),
+    'relpropvalu': lambda astinfo, kids: s_ast.RelPropValue(astinfo, [s_ast.Const(k.astinfo, k.valu.lstrip(':')) if isinstance(k, s_ast.Const) else k for k in kids]),
     'relpropvalue': s_ast.RelPropValue,
-    'setitem': s_ast.SetItemOper,
+    'search': s_ast.Search,
+    'setitem': lambda astinfo, kids: s_ast.SetItemOper(astinfo, [kids[0], kids[1], kids[3]]),
     'setvar': s_ast.SetVarOper,
-    'stormcmd': lambda kids: s_ast.CmdOper(kids=kids if len(kids) == 2 else (kids[0], s_ast.Const(tuple()))),
+    'stop': s_ast.Stop,
+    'stormcmd': lambda astinfo, kids: s_ast.CmdOper(astinfo, kids=kids if len(kids) == 2 else (kids[0], s_ast.Const(astinfo, tuple()))),
     'stormfunc': s_ast.Function,
     'tagcond': s_ast.TagCond,
+    'tagname': s_ast.TagName,
+    'tagmatch': s_ast.TagMatch,
     'tagprop': s_ast.TagProp,
     'tagvalu': s_ast.TagValue,
     'tagpropvalu': s_ast.TagPropValue,
     'tagvalucond': s_ast.TagValuCond,
     'tagpropcond': s_ast.TagPropCond,
+    'trycatch': s_ast.TryCatch,
+    'univprop': s_ast.UnivProp,
+    'univpropvalu': s_ast.UnivPropValue,
     'valulist': s_ast.List,
     'vareval': s_ast.VarEvalOper,
     'varvalue': s_ast.VarValue,
-    'univprop': s_ast.UnivProp,
-    'univpropvalu': s_ast.UnivPropValue,
-    'wordtokn': lambda kids: s_ast.Const(''.join([str(k.valu) for k in kids]))
+    'whileloop': s_ast.WhileLoop,
+    'wordtokn': lambda astinfo, kids: s_ast.Const(astinfo, ''.join([str(k.valu) for k in kids]))
 }
 
+escape_re = regex.compile(r"(?<!\\)((\\\\)*)'")
+
+def format_unescape(valu):
+    repl = valu.replace('\\`', '`').replace('\\{', '{')
+    repl = escape_re.sub(r"\1\'", repl)
+    return unescape(f"'''{repl}'''")
+
 def unescape(valu):
     '''
     Parse a string for backslash-escaped characters and omit them.
@@ -730,7 +760,7 @@ def unescape(valu):
     '''
     try:
         ret = ast.literal_eval(valu)
-    except ValueError as e:
+    except (SyntaxError, ValueError) as e:
         mesg = f"Invalid character in string {repr(valu)}: {e}"
         raise s_exc.BadSyntax(mesg=mesg, valu=repr(valu)) from None
 
