@@ -29,6 +29,7 @@ re_directive = regex.compile(r'^\.\.\s(storm.*|[^:])::(?:\s(.*)$|$)')
 
 logger = logging.getLogger(__name__)
 
+ONLOAD_TIMEOUT = int(os.getenv('SYNDEV_PKG_LOAD_TIMEOUT', 30))  # seconds
 
 class OutPutRst(s_output.OutPutStr):
     '''
@@ -67,7 +68,7 @@ class StormOutput(s_cmds_cortex.StormCmd):
         self.stormopts = stormopts or {}
 
         # hide a few mesg types by default
-        for mtype in ('init', 'fini', 'node:edits', 'node:edits:count', 'prov:new'):
+        for mtype in ('init', 'fini', 'node:edits', 'node:edits:count', ):
             self.cmdmeths[mtype] = self._silence
 
         self.core = core
@@ -207,7 +208,7 @@ class StormCliOutput(s_storm.StormCli):
         self.ctx = {}
         self._print_skips.append('init')
         self._print_skips.append('fini')
-        self._print_skips.append('prov:new')
+        self._print_skips.append('prov:new')  # TODO: Remove in v3.0.0
         self._print_skips.append('node:edits')
         self._print_skips.append('node:edits:count')
 
@@ -413,8 +414,17 @@ class StormRst(s_base.Base):
         core = self._reqCore()
 
         pkg = s_genpkg.loadPkgProto(text)
+
+        if pkg.get('onload') is not None:
+            waiter = core.waiter(1, 'core:pkg:onload:complete')
+        else:
+            waiter = None
+
         await core.addStormPkg(pkg)
 
+        if waiter is not None and not await waiter.wait(timeout=ONLOAD_TIMEOUT):
+            raise s_exc.SynErr(mesg=f'Package onload failed to run for {pkg.get("name")}')
+
     async def _handleStormPre(self, text):
         '''
         Run a Storm query to prepare the Cortex without output.
@@ -453,6 +463,9 @@ class StormRst(s_base.Base):
 
         svc = await self._getCell(ctor, conf=svcconf)
 
+        onloadcnt = len([p for p in svc.cellapi._storm_svc_pkgs if p.get('onload') is not None])
+        waiter = core.waiter(onloadcnt, 'core:pkg:onload:complete') if onloadcnt else None
+
         svc.dmon.share('svc', svc)
         root = await svc.auth.getUserByName('root')
         await root.setPasswd('root')
@@ -463,6 +476,9 @@ class StormRst(s_base.Base):
         await core.nodes(f'service.add {svcname} {surl}')
         await core.nodes(f'$lib.service.wait({svcname})')
 
+        if waiter is not None and not await waiter.wait(timeout=ONLOAD_TIMEOUT):
+            raise s_exc.SynErr(mesg=f'Package onload failed to run for service {svcname}')
+
     async def _handleStormFail(self, text):
         valu = json.loads(text)
         assert valu in (True, False), f'storm-fail must be a boolean: {text}'
