@@ -1,4 +1,7 @@
+import string
+import asyncio
 import logging
+import pathlib
 import functools
 import collections
 
@@ -8,17 +11,23 @@ import unicodedata
 
 import synapse.exc as s_exc
 import synapse.data as s_data
+import synapse.common as s_common
 
 import synapse.lib.chop as s_chop
+import synapse.lib.coro as s_coro
+import synapse.lib.link as s_link
+import synapse.lib.msgpack as s_msgpack
 
 import synapse.lib.crypto.coin as s_coin
 
+ipaddress = s_common.ipaddress
 
 logger = logging.getLogger(__name__)
 
 tldlist = list(s_data.get('iana.tlds'))
 tldlist.extend([
     'bit',
+    'bazar',
     'onion',
 ])
 
@@ -73,6 +82,35 @@ def fqdn_check(match: regex.Match):
             return None, {}
     return valu, {}
 
+def inet_server_check(match: regex.Match):
+    mnfo = match.groupdict()
+    valu = mnfo.get('valu')
+    port = mnfo.get('port')
+    ipv6 = mnfo.get('v6addr')
+
+    port = int(port)
+    if port < 1 or port > 2**16 - 1:
+        return None, {}
+
+    if ipv6 is not None:
+        try:
+            addr = ipaddress.IPv6Address(ipv6)
+        except ipaddress.AddressValueError:
+            return None, {}
+
+    return valu, {}
+
+def ipv6_check(match: regex.Match):
+    mnfo = match.groupdict()
+    valu = mnfo.get('valu')
+
+    try:
+        addr = ipaddress.IPv6Address(valu)
+    except ipaddress.AddressValueError:
+        return None, {}
+
+    return valu, {}
+
 def cve_check(match: regex.Match):
     mnfo = match.groupdict()
     valu = mnfo.get('valu')  # type: str
@@ -81,18 +119,150 @@ def cve_check(match: regex.Match):
     valu = s_chop.replaceUnicodeDashes(valu)
     return valu, cbfo
 
+# Reference NIST 7695
+# Common Platform Enumeration: Naming Specification Version 2.3
+# Figure 6-3. ABNF for Formatted String Binding
+_cpe23_regex = r'''(?P<valu>cpe:2\.3:[aho\*-]
+(?::(([?]+|\*)?([a-z0-9-._]|\\[\\?*!"#$%&\'()+,/:;<=>@\[\]^`{|}~])+([?]+|\*)?|[*-])){5}
+:([*-]|(([a-z]{2,3}){1}(-([0-9]{3}|[a-z]{2}))?))
+(?::(([?]+|\*)?([a-z0-9-._]|\\[\\?*!"#$%&\'()+,/:;<=>@\[\]^`{|}~])+([?]+|\*)?|[*-])){4})
+'''
+
+linux_path_regex = r'''
+(?<![\w\d]+)
+(?P<valu>
+    (?:/[^\x00\n]+)+
+)
+'''
+
+linux_path_rootdirs = (
+    'bin', 'boot', 'cdrom', 'dev', 'etc', 'home',
+    'lib', 'lib64', 'lib32', 'libx32', 'media', 'mnt',
+    'opt', 'proc', 'root', 'run', 'sbin', 'srv',
+    'sys', 'tmp', 'usr', 'var'
+)
+
+def linux_path_check(match: regex.Match):
+    mnfo = match.groupdict()
+    valu = mnfo.get('valu')
+
+    path = pathlib.PurePosixPath(valu)
+    parts = path.parts
+
+    if parts[0] != '/':
+        return None, {}
+
+    if len(parts) < 2:
+        return None, {}
+
+    if parts[1] not in linux_path_rootdirs:
+        return None, {}
+
+    return valu, {}
+
+windows_path_regex = r'''
+(?P<valu>
+    [a-zA-Z]+:\\
+    (?:[^<>:"/|\?\*\n\x00]+)+
+)
+'''
+
+windows_path_reserved = (
+    'CON', 'PRN', 'AUX', 'NUL',
+    'COM0', 'COM1', 'COM2', 'COM3', 'COM4',
+    'COM5', 'COM6', 'COM7', 'COM8', 'COM9',
+    'LPT0', 'LPT1', 'LPT2', 'LPT3', 'LPT4',
+    'LPT5', 'LPT6', 'LPT7', 'LPT8', 'LPT9'
+)
+
+windows_drive_paths = [f'{letter}:\\' for letter in string.ascii_lowercase]
+
+# https://learn.microsoft.com/en-us/windows/win32/fileio/naming-a-file#naming-conventions
+def windows_path_check(match: regex.Match):
+    mnfo = match.groupdict()
+    valu = mnfo.get('valu')
+
+    path = pathlib.PureWindowsPath(valu)
+    parts = path.parts
+
+    if parts[0].lower() not in windows_drive_paths:
+        return None, {}
+
+    for part in parts:
+        if part in windows_path_reserved:
+            return None, {}
+
+        if part.endswith('.'):
+            return None, {}
+
+    return valu, {}
+
+ipv4_match = r'(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?)'
+ipv4_regex = fr'''
+(?P<valu>
+    (?<!\d|\d\.|[0-9a-f:]:)
+    ({ipv4_match})
+    (?!\d|\.\d)
+)
+'''
+
+# Simplified IPv6 regex based on RFC3986, will have false positives and
+# requires validating matches.
+H16 = r'[0-9a-f]{1,4}'
+ipv6_match = fr'''
+(?: (?:{H16}:){{1,7}}
+    (?:(?:
+            :?
+            (?:{H16}:){{0,5}}
+            (?:{ipv4_match}|{H16})
+        ) |
+        :
+    )
+) |
+(?: ::
+    ({H16}:){{0,6}}
+    (?:{ipv4_match}|{H16})
+)
+'''
+ipv6_regex = fr'''
+(?P<valu>
+    (?<![0-9a-f]:|:[0-9a-f]|::|\d\.)
+    ({ipv6_match})
+    (?![0-9a-f:]|\.\d)
+)
+'''
+
+# https://learn.microsoft.com/en-us/openspecs/windows_protocols/ms-dtyp/62e862f4-2a51-452e-8eeb-dc4ff5ee33cc
+def unc_path_check(match: regex.Match):
+    mnfo = match.groupdict()
+    valu = mnfo.get('valu')
+
+    try:
+        valu = s_chop.uncnorm(valu)
+    except s_exc.BadTypeValu:
+        return None, {}
+
+    return valu, {}
+
 # these must be ordered from most specific to least specific to allow first=True to work
 scrape_types = [  # type: ignore
+    ('file:path', linux_path_regex, {'callback': linux_path_check, 'flags': regex.VERBOSE}),
+    ('file:path', windows_path_regex, {'callback': windows_path_check, 'flags': regex.VERBOSE}),
     ('inet:url', r'(?P<prefix>[\\{<\(\[]?)(?P<valu>[a-zA-Z][a-zA-Z0-9]*://(?(?=[,.]+[ \'\"\t\n\r\f\v])|[^ \'\"\t\n\r\f\v])+)',
      {'callback': fqdn_prefix_check}),
+    ('inet:url', r'(["\'])?(?P<valu>\\[^\n]+?)(?(1)\1|\s)', {'callback': unc_path_check}),
     ('inet:email', r'(?=(?:[^a-z0-9_.+-]|^)(?P<valu>[a-z0-9_\.\-+]{1,256}@(?:[a-z0-9_-]{1,63}\.){1,10}(?:%s))(?:[^a-z0-9_.-]|[.\s]|$))' % tldcat, {}),
-    ('inet:server', r'(?P<valu>(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?):[0-9]{1,5})', {}),
-    ('inet:ipv4', r'(?P<valu>(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?))', {}),
+    ('inet:server', fr'(?P<valu>(?:(?<!\d|\d\.|[0-9a-f:]:)((?P<addr>{ipv4_match})|\[(?P<v6addr>{ipv6_match})\]):(?P<port>\d{{1,5}})(?!\d|\.\d)))',
+     {'callback': inet_server_check, 'flags': regex.VERBOSE}),
+    ('inet:ipv4', ipv4_regex, {'flags': regex.VERBOSE}),
+    ('inet:ipv6', ipv6_regex, {'callback': ipv6_check, 'flags': regex.VERBOSE}),
     ('inet:fqdn', r'(?=(?:[^\p{L}\p{M}\p{N}\p{S}\u3002\uff0e\uff61_.-]|^|[' + idna_disallowed + '])(?P<valu>(?:((?![' + idna_disallowed + r'])[\p{L}\p{M}\p{N}\p{S}_-]){1,63}[\u3002\uff0e\uff61\.]){1,10}(?:' + tldcat + r'))(?:[^\p{L}\p{M}\p{N}\p{S}\u3002\uff0e\uff61_.-]|[\u3002\uff0e\uff61.]([\p{Z}\p{Cc}]|$)|$|[' + idna_disallowed + r']))', {'callback': fqdn_check}),
     ('hash:md5', r'(?=(?:[^A-Za-z0-9]|^)(?P<valu>[A-Fa-f0-9]{32})(?:[^A-Za-z0-9]|$))', {}),
     ('hash:sha1', r'(?=(?:[^A-Za-z0-9]|^)(?P<valu>[A-Fa-f0-9]{40})(?:[^A-Za-z0-9]|$))', {}),
     ('hash:sha256', r'(?=(?:[^A-Za-z0-9]|^)(?P<valu>[A-Fa-f0-9]{64})(?:[^A-Za-z0-9]|$))', {}),
     ('it:sec:cve', fr'(?:[^a-z0-9]|^)(?P<valu>CVE[{cve_dashes}][0-9]{{4}}[{cve_dashes}][0-9]{{4,}})(?:[^a-z0-9]|$)', {'callback': cve_check}),
+    ('it:sec:cwe', r'(?=(?:[^A-Za-z0-9]|^)(?P<valu>CWE-[0-9]{1,8})(?:[^A-Za-z0-9]|$))', {}),
+    ('it:sec:cpe', _cpe23_regex, {'flags': regex.VERBOSE}),
     ('crypto:currency:address', r'(?=(?:[^A-Za-z0-9]|^)(?P<valu>[1][a-zA-HJ-NP-Z0-9]{25,39})(?:[^A-Za-z0-9]|$))',
      {'callback': s_coin.btc_base58_check}),
     ('crypto:currency:address', r'(?=(?:[^A-Za-z0-9]|^)(?P<valu>3[a-zA-HJ-NP-Z0-9]{33})(?:[^A-Za-z0-9]|$))',
@@ -109,13 +279,13 @@ scrape_types = [  # type: ignore
      {'callback': s_coin.substrate_check}),
     ('crypto:currency:address', r'(?=(?:[^A-Za-z0-9]|^)(?P<valu>(DdzFF|Ae2td)[a-zA-HJ-NP-Z0-9]{54,99})(?:[^A-Za-z0-9]|$))',
      {'callback': s_coin.cardano_byron_check}),
-    ('crypto:currency:address', r'(?=(?:[^A-Za-z0-9]|^)(?P<valu>addr1[qpzry9x8gf2tvdw0s3jn54khce6mua7l]{53})(?:[^A-Za-z0-9]|$))',
+    ('crypto:currency:address', r'(?=(?:[^A-Za-z0-9]|^)(?P<valu>addr1[qpzry9x8gf2tvdw0s3jn54khce6mua7l]{53,})(?:[^A-Za-z0-9]|$))',
      {'callback': s_coin.cardano_shelly_check}),
 ]
 
 _regexes = collections.defaultdict(list)
 for (name, rule, opts) in scrape_types:
-    blob = (regex.compile(rule, regex.IGNORECASE), opts)
+    blob = (regex.compile(rule, regex.IGNORECASE | opts.get('flags', 0)), opts)
     _regexes[name].append(blob)
 
 def getForms():
@@ -128,15 +298,24 @@ def getForms():
     return sorted(_regexes.keys())
 
 FANGS = {
+    'fxp:': 'ftp:',
+    'fxps:': 'ftps:',
     'hxxp:': 'http:',
     'hxxps:': 'https:',
-    'hxxp[:]': 'http:',
-    'hxxps[:]': 'https:',
-    'hxxp[://]': 'http://',
-    'hxxps[://]': 'https://',
-    'hxxp(:)': 'http:',
-    'hxxps(:)': 'https:',
+    'fxp[s]:': 'ftps:',
+    'hxxp[s]:': 'https:'
+}
+
+for fang in ('[:]', '[://]', '[:', '(:)'):
+    for scheme in ('ftp', 'fxp', 'ftps', 'fxps', 'http', 'hxxp', 'https', 'hxxps'):
+        fanged = f'{scheme}{fang}'
+        defanged = regex.sub(r'[\[\]\(\)]', '', fanged.replace('x', 't'))
+        FANGS[fanged] = defanged
+
+FANGS.update({
     '[.]': '.',
+    '.]': '.',
+    '[.': '.',
     '[．]': '．',
     '[。]': '。',
     '[｡]': '｡',
@@ -144,13 +323,12 @@ FANGS = {
     '(．)': '．',
     '(。)': '。',
     '(｡)': '｡',
+    '[dot]': '.',
     '[:]': ':',
-    'fxp': 'ftp',
-    'fxps': 'ftps',
     '[at]': '@',
     '[@]': '@',
     '\\.': '.',
-}
+})
 
 def genFangRegex(fangs, flags=regex.IGNORECASE):
     # Fangs must be matches of equal or smaller length in order for the
@@ -265,6 +443,36 @@ def _rewriteRawValu(text: str, offsets: dict, info: dict):
     info['match'] = match
     info['offset'] = baseoff + offset
 
+def _genMatchList(text: str, regx: regex.Regex, opts: dict):
+    return [info for info in _genMatches(text, regx, opts)]
+
+def _genMatches(text: str, regx: regex.Regex, opts: dict):
+
+    cb = opts.get('callback')
+
+    for valu in regx.finditer(text):  # type: regex.Match
+        raw_span = valu.span('valu')
+        raw_valu = valu.group('valu')
+
+        info = {
+            'match': raw_valu,
+            'offset': raw_span[0]
+        }
+
+        if cb:
+            # CB is expected to return a tufo of <new valu, info>
+            valu, cbfo = cb(valu)
+            if valu is None:
+                continue
+            # Smash cbfo into our info dict
+            info.update(**cbfo)
+        else:
+            valu = raw_valu
+
+        info['valu'] = valu
+
+        yield info
+
 def genMatches(text: str, regx: regex.Regex, opts: dict):
     '''
     Generate regular expression matches for a blob of text.
@@ -294,31 +502,75 @@ def genMatches(text: str, regx: regex.Regex, opts: dict):
     Yields:
         dict: A dictionary of match results.
     '''
-    cb = opts.get('callback')
+    for match in _genMatches(text, regx, opts):
+        yield match
 
-    for valu in regx.finditer(text):  # type: regex.Match
-        raw_span = valu.span('valu')
-        raw_valu = valu.group('valu')
+async def genMatchesAsync(text: str, regx: regex.Regex, opts: dict):
+    '''
+    Generate regular expression matches for a blob of text, using the shared forked process pool.
 
-        info = {
-            'match': raw_valu,
-            'offset': raw_span[0]
-        }
+    Args:
+        text (str): The text to generate matches for.
+        regx (regex.Regex): A compiled regex object. The regex must contained a named match group for ``valu``.
+        opts (dict): An options dictionary.
 
-        if cb:
-            # CB is expected to return a tufo of <new valu, info>
-            valu, cbfo = cb(valu)
-            if valu is None:
-                continue
-            # Smash cbfo into our info dict
-            info.update(**cbfo)
-        else:
-            valu = raw_valu
+    Notes:
+        The dictionaries yielded by this function contains the following keys:
 
-        info['valu'] = valu
+            raw_valu
+                The raw matching text found in the input text.
 
+            offset
+                The offset into the text where the match was found.
+
+            valu
+                The resulting value - this may be altered by callbacks.
+
+        The options dictionary can contain a ``callback`` key. This function is expected to take a single argument,
+        a regex.Match object, and return a tuple of the new valu and info dictionary. The new valu is used as the
+        ``valu`` key in the returned dictionary, and any other information in the info dictionary is pushed into
+        the return dictionary as well.
+
+    Yields:
+        dict: A dictionary of match results.
+    '''
+    matches = await s_coro.semafork(_genMatchList, text, regx, opts)
+    for info in matches:
         yield info
 
+def _contextMatches(scrape_text, text, ruletype, refang, offsets):
+
+        for (regx, opts) in _regexes[ruletype]:
+
+            for info in genMatches(scrape_text, regx, opts):
+
+                info['form'] = ruletype
+
+                if refang and offsets:
+                    _rewriteRawValu(text, offsets, info)
+
+                yield info
+
+def _contextScrapeList(text, form=None, refang=True, first=False):
+    return [info for info in _contextScrape(text, form=form, refang=refang, first=first)]
+
+def _contextScrape(text, form=None, refang=True, first=False):
+    scrape_text = text
+    offsets = {}
+    if refang:
+        scrape_text, offsets = refang_text2(text)
+
+    for ruletype, blobs in _regexes.items():
+        if form and form != ruletype:
+            continue
+
+        for info in _contextMatches(scrape_text, text, ruletype, refang, offsets):
+
+            yield info
+
+            if first:
+                return
+
 def contextScrape(text, form=None, refang=True, first=False):
     '''
     Scrape types from a blob of text and yield info dictionaries.
@@ -347,28 +599,8 @@ def contextScrape(text, form=None, refang=True, first=False):
     Returns:
         (dict): Yield info dicts of results.
     '''
-    scrape_text = text
-    offsets = {}
-    if refang:
-        scrape_text, offsets = refang_text2(text)
-
-    for ruletype, blobs in _regexes.items():
-        if form and form != ruletype:
-            continue
-
-        for (regx, opts) in blobs:
-
-            for info in genMatches(scrape_text, regx, opts):
-
-                info['form'] = ruletype
-
-                if refang and offsets:
-                    _rewriteRawValu(text, offsets, info)
-
-                yield info
-
-                if first:
-                    return
+    for info in _contextScrape(text, form=form, refang=refang, first=first):
+        yield info
 
 def scrape(text, ptype=None, refang=True, first=False):
     '''
@@ -383,6 +615,54 @@ def scrape(text, ptype=None, refang=True, first=False):
     Returns:
         (str, object): Yield tuples of node ndef values.
     '''
+    for info in _contextScrape(text, form=ptype, refang=refang, first=first):
+        yield info.get('form'), info.get('valu')
 
-    for info in contextScrape(text, form=ptype, refang=refang, first=first):
+async def contextScrapeAsync(text, form=None, refang=True, first=False):
+    '''
+    Scrape types from a blob of text and yield info dictionaries, using the shared forked process pool.
+
+    Args:
+        text (str): Text to scrape.
+        form (str): Optional form to scrape. If present, only scrape items which match the provided form.
+        refang (bool): Whether to remove de-fanging schemes from text before scraping.
+        first (bool): If true, only yield the first item scraped.
+
+    Notes:
+        The dictionaries yielded by this function contains the following keys:
+
+            match
+                The raw matching text found in the input text.
+
+            offset
+                The offset into the text where the match was found.
+
+            valu
+                The resulting value.
+
+            form
+                The corresponding form for the valu.
+
+    Returns:
+        (dict): Yield info dicts of results.
+    '''
+    matches = await s_coro.semafork(_contextScrapeList, text, form=form, refang=refang, first=first)
+    for info in matches:
+        yield info
+
+async def scrapeAsync(text, ptype=None, refang=True, first=False):
+    '''
+    Scrape types from a blob of text and return node tuples, using the shared forked process pool.
+
+    Args:
+        text (str): Text to scrape.
+        ptype (str): Optional ptype to scrape. If present, only scrape items which match the provided type.
+        refang (bool): Whether to remove de-fanging schemes from text before scraping.
+        first (bool): If true, only yield the first item scraped.
+
+    Returns:
+        (str, object): Yield tuples of node ndef values.
+    '''
+    matches = await s_coro.semafork(_contextScrapeList, text, form=ptype, refang=refang, first=first)
+    for info in matches:
         yield info.get('form'), info.get('valu')
