@@ -108,7 +108,7 @@ class SlabSeqn:
     def stat(self):
         return self.slab.stat(db=self.db)
 
-    def save(self, items):
+    async def save(self, items):
         '''
         Save a series of items to a sequence.
 
@@ -123,6 +123,7 @@ class SlabSeqn:
 
         size = 0
         tick = s_common.now()
+        abstick = s_common.mononow()
 
         for item in items:
 
@@ -135,8 +136,8 @@ class SlabSeqn:
 
             rows.append((lkey, byts))
 
-        retn = self.slab.putmulti(rows, append=True, db=self.db)
-        took = s_common.now() - tick
+        retn = await self.slab.putmulti(rows, append=True, db=self.db)
+        took = s_common.mononow() - abstick
 
         assert retn, "Not adding the largest indices"
 
@@ -168,7 +169,7 @@ class SlabSeqn:
 
         return s_common.int64un(byts) + 1
 
-    def iter(self, offs):
+    def iter(self, offs, reverse=False):
         '''
         Iterate over items in a sequence from a given offset.
 
@@ -179,10 +180,16 @@ class SlabSeqn:
             (indx, valu): The index and valu of the item.
         '''
         startkey = s_common.int64en(offs)
-        for lkey, lval in self.slab.scanByRange(startkey, db=self.db):
-            offs = s_common.int64un(lkey)
-            valu = s_msgpack.un(lval)
-            yield offs, valu
+        if reverse:
+            for lkey, lval in self.slab.scanByRangeBack(startkey, db=self.db):
+                offs = s_common.int64un(lkey)
+                valu = s_msgpack.un(lval)
+                yield offs, valu
+        else:
+            for lkey, lval in self.slab.scanByRange(startkey, db=self.db):
+                offs = s_common.int64un(lkey)
+                valu = s_msgpack.un(lval)
+                yield offs, valu
 
     async def aiter(self, offs, wait=False, timeout=None):
         '''
@@ -197,13 +204,20 @@ class SlabSeqn:
             (indx, valu): The index and valu of the item.
         '''
         startkey = s_common.int64en(offs)
+        scanoffs = None
         for lkey, lval in self.slab.scanByRange(startkey, db=self.db):
-            offs = s_common.int64un(lkey)
+            scanoffs = s_common.int64un(lkey)
             valu = s_msgpack.un(lval)
-            yield offs, valu
+            yield scanoffs, valu
 
         # no awaiting between here and evnt.timewait()
         if wait:
+
+            if scanoffs is None:
+                offs -= 1
+            else:
+                offs = scanoffs
+
             evnt = s_coro.Event()
             try:
                 self.addevents.append(evnt)
