@@ -7,6 +7,8 @@ import synapse.lib.const as s_const
 import synapse.lib.msgpack as s_msgpack
 import synapse.lib.lmdbslab as s_lmdbslab
 
+MAX_SPOOL_SIZE = 10000
+
 class Spooled(s_base.Base):
     '''
     A Base class that can be used to implement objects which fallback to lmdb.
@@ -15,7 +17,7 @@ class Spooled(s_base.Base):
     together. Under memory pressure, these objects have a better shot of getting paged out.
     '''
 
-    async def __anit__(self, dirn=None, size=10000):
+    async def __anit__(self, dirn=None, size=MAX_SPOOL_SIZE, cell=None):
         '''
         Args:
             dirn(Optional[str]): base directory used for backing slab.  If None, system temporary directory is used
@@ -23,6 +25,7 @@ class Spooled(s_base.Base):
         '''
         await s_base.Base.__anit__(self)
 
+        self.cell = cell
         self.size = size
         self.dirn = dirn
         self.slab = None
@@ -46,17 +49,29 @@ class Spooled(s_base.Base):
         slabpath = tempfile.mkdtemp(dir=dirn, prefix='spooled_', suffix='.lmdb')
 
         self.slab = await s_lmdbslab.Slab.anit(slabpath, map_size=s_const.mebibyte * 32)
+        if self.cell is not None:
+            self.slab.addResizeCallback(self.cell.checkFreeSpace)
 
 class Set(Spooled):
     '''
     A minimal set-like implementation that will spool to a slab on large growth.
     '''
 
-    async def __anit__(self, dirn=None, size=10000):
-        await Spooled.__anit__(self, dirn=dirn, size=size)
+    async def __anit__(self, dirn=None, size=MAX_SPOOL_SIZE, cell=None):
+        await Spooled.__anit__(self, dirn=dirn, size=size, cell=cell)
         self.realset = set()
         self.len = 0
 
+    async def __aiter__(self):
+
+        if not self.fallback:
+            for item in self.realset:
+                yield item
+            return
+
+        for byts in self.slab.scanKeys():
+            yield s_msgpack.un(byts)
+
     def __contains__(self, valu):
         if self.fallback:
             return self.slab.get(s_msgpack.en(valu)) is not None
@@ -71,6 +86,27 @@ class Set(Spooled):
 
         return len(self.realset)
 
+    async def copy(self):
+        newset = await Set.anit(dirn=self.dirn, size=self.size, cell=self.cell)
+
+        if self.fallback:
+            await newset._initFallBack()
+            await self.slab.copydb(None, newset.slab)
+            newset.len = self.len
+
+        else:
+            newset.realset = self.realset.copy()
+
+        return newset
+
+    async def clear(self):
+        if self.fallback:
+            self.len = 0
+            await self.slab.trash()
+            await self._initFallBack()
+        else:
+            self.realset.clear()
+
     async def add(self, valu):
 
         if self.fallback:
@@ -86,6 +122,11 @@ class Set(Spooled):
             self.len = len(self.realset)
             self.realset.clear()
 
+    def has(self, key):
+        if self.fallback:
+            return self.slab.has(s_msgpack.en(key))
+        return key in self.realset
+
     def discard(self, valu):
 
         if self.fallback:
@@ -99,9 +140,9 @@ class Set(Spooled):
 
 class Dict(Spooled):
 
-    async def __anit__(self, dirn=None, size=10000):
+    async def __anit__(self, dirn=None, size=MAX_SPOOL_SIZE, cell=None):
 
-        await Spooled.__anit__(self, dirn=dirn, size=size)
+        await Spooled.__anit__(self, dirn=dirn, size=size, cell=cell)
         self.realdict = {}
         self.len = 0
 
@@ -125,6 +166,15 @@ class Dict(Spooled):
             self.len = len(self.realdict)
             self.realdict.clear()
 
+    def pop(self, key, defv=None):
+        if self.fallback:
+            ret = self.slab.pop(s_msgpack.en(key))
+            if ret is None:
+                return defv
+            self.len -= 1
+            return s_msgpack.un(ret)
+        return self.realdict.pop(key, defv)
+
     def has(self, key):
         if self.fallback:
             return self.slab.has(s_msgpack.en(key))
