@@ -4,11 +4,14 @@ import asyncio
 import logging
 import datetime
 
+import regex
+
 import synapse.exc as s_exc
 import synapse.common as s_common
 import synapse.lib.coro as s_coro
 import synapse.lib.node as s_node
 import synapse.lib.msgpack as s_msgpack
+import synapse.lib.schemas as s_schemas
 import synapse.lib.stormctrl as s_stormctrl
 import synapse.lib.stormtypes as s_stormtypes
 
@@ -24,12 +27,19 @@ def uuid4(valu=None):
     guid = s_common.guid(valu=valu)
     return str(uuid.UUID(guid, version=4))
 
+stixtype_re = regex.compile('^[a-z0-9-]{3,250}$')
+
 SYN_STIX_EXTENSION_ID = f'extension-definition--{uuid4("bf1c0a4d90ee557ac05055385971f17c")}'
 
 _DefaultConfig = {
 
     'maxsize': 10000,
 
+    'custom': {
+        'objects': {
+        },
+    },
+
     'forms': {
 
         'ou:campaign': {
@@ -238,7 +248,7 @@ _DefaultConfig = {
                         'name': '{+:name return(:name)} return($node.repr())',
                         'size': '+:size return(:size)',
                         'hashes': '''
-                            init { $dict = $lib.dict() }
+                            init { $dict = ({}) }
                             { +:md5 $dict.MD5 = :md5 }
                             { +:sha1 $dict."SHA-1" = :sha1 }
                             { +:sha256 $dict."SHA-256" = :sha256 }
@@ -383,7 +393,7 @@ _DefaultConfig = {
                         'description': 'if (:desc) { return (:desc) }',
                         'created': 'return($lib.stix.export.timestamp(.created))',
                         'modified': 'return($lib.stix.export.timestamp(.created))',
-                        'external_references': 'if :cve { $cve=:cve $cve=$cve.upper() $list=$lib.list($lib.dict(source_name=cve, external_id=$cve)) return($list) }'
+                        'external_references': 'if :cve { $cve=:cve $cve=$cve.upper() $list=$lib.list(({"source_name": "cve", "external_id": $cve})) return($list) }'
                     },
                     'rels': (
 
@@ -392,6 +402,33 @@ _DefaultConfig = {
             }
         },
 
+        'ou:technique': {
+            'default': 'attack-pattern',
+            'stix': {
+                'attack-pattern': {
+                    'props': {
+                        'name': '{ +:name return(:name) } return($node.repr())',
+                        'description': '+:desc return(:desc)',
+                    },
+                    'revs': (
+                        ('mitigates', 'course-of-action', '<(addresses)- risk:mitigation'),
+                    ),
+                },
+            },
+        },
+
+        'risk:mitigation': {
+            'default': 'course-of-action',
+            'stix': {
+                'course-of-action': {
+                    'props': {
+                        'name': '{ +:name return(:name) } return($node.repr())',
+                        'description': '+:desc return(:desc)',
+                    },
+                },
+            },
+        },
+
         'media:news': {
             'default': 'report',
             'stix': {
@@ -449,17 +486,33 @@ _DefaultConfig = {
     },
 }
 
-def _validateConfig(core, config):
+perm_maxsize = ('storm', 'lib', 'stix', 'export', 'maxsize')
+def _validateConfig(runt, config):
+
+    core = runt.snap.core
 
     maxsize = config.get('maxsize', 10000)
 
+    alltypes = stix_all.copy()
+
+    custom = config.get('custom')
+    if custom is not None:
+        objdefs = custom.get('objects')
+        if objdefs is not None:
+            for name, info in objdefs.items():
+                if not stixtype_re.match(name):
+                    mesg = 'Invalid custom STIX type name. Must match regex: ^[a-z0-9-]{3,250}$'
+                    raise s_exc.BadConfValu(mesg=mesg)
+                alltypes.add(name)
+
     if not isinstance(maxsize, int):
         mesg = f'STIX Bundle config maxsize option must be an integer.'
         raise s_exc.BadConfValu(mesg=mesg)
 
-    if maxsize > 10000:
-        mesg = f'STIX Bundle config maxsize option must be <= 10000.'
-        raise s_exc.BadConfValu(mesg=mesg)
+    if maxsize > 10000 and not runt.allowed(perm_maxsize):
+        permstr = '.'.join(perm_maxsize)
+        mesg = f'Setting STIX export maxsize > 10,000 requires permission: {permstr}'
+        raise s_exc.AuthDeny(mesg=mesg, perm=permstr)
 
     formmaps = config.get('forms')
     if formmaps is None:
@@ -470,14 +523,14 @@ def _validateConfig(core, config):
         form = core.model.form(formname)
         if form is None:
             mesg = f'STIX Bundle config contains invalid form name {formname}.'
-            raise s_exc.NoSuchForm(mesg=mesg)
+            raise s_exc.NoSuchForm.init(formname, mesg=mesg)
 
         stixdef = formconf.get('default')
         if stixdef is None:
             mesg = f'STIX Bundle config is missing default mapping for form {formname}.'
             raise s_exc.NeedConfValu(mesg=mesg)
 
-        if stixdef not in stix_all:
+        if stixdef not in alltypes:
             mesg = f'STIX Bundle default mapping ({stixdef}) for {formname} is not a STIX type.'
             raise s_exc.BadConfValu(mesg=mesg)
 
@@ -492,7 +545,7 @@ def _validateConfig(core, config):
 
         for stixtype, stixinfo in stixmaps.items():
 
-            if stixtype not in stix_all:
+            if stixtype not in alltypes:
                 mesg = f'STIX Bundle config has unknown STIX type {stixtype} for form {formname}.'
                 raise s_exc.BadConfValu(mesg=mesg)
 
@@ -510,6 +563,26 @@ def _validateConfig(core, config):
                         mesg = f'STIX Bundle config has invalid rel entry {formname} {stixtype} {stixrel}.'
                         raise s_exc.BadConfValu(mesg=mesg)
 
+            stixrevs = stixinfo.get('revs')
+            if stixrevs is not None:
+                for stixrev in stixrevs:
+                    if len(stixrev) != 3:
+                        mesg = f'STIX Bundle config has invalid rev entry {formname} {stixtype} {stixrev}.'
+                        raise s_exc.BadConfValu(mesg=mesg)
+
+            stixpivs = stixinfo.get('pivots')
+            if stixpivs is not None:
+                for stixpiv in stixpivs:
+
+                    if not isinstance(stixpiv.get('storm'), str):
+                        raise s_exc.BadConfValu(mesg=f'Pivot for {formname} (as {stixtype}) has invalid/missing storm field.')
+
+                    pivtype = stixpiv.get('stixtype')
+                    if isinstance(pivtype, str):
+                        if pivtype not in alltypes:
+                            mesg = f'STIX Bundle config has unknown pivot STIX type {pivtype} for form {formname}.'
+                            raise s_exc.BadConfValu(mesg=mesg)
+
 def validateStix(bundle, version='2.1'):
     ret = {
         'ok': False,
@@ -573,7 +646,7 @@ class LibStix(s_stormtypes.Lib):
                 'args': (
                     {'type': 'dict', 'name': 'bundle', 'desc': 'The STIX bundle to lift nodes from.'},
                 ),
-                'returns': {'name': 'Yields', 'type': 'storm:node', 'desc': 'Yields nodes'}
+                'returns': {'name': 'Yields', 'type': 'node', 'desc': 'Yields nodes'}
             }
         },
     )
@@ -585,10 +658,12 @@ class LibStix(s_stormtypes.Lib):
             'validate': self.validateBundle,
         }
 
+    @s_stormtypes.stormfunc(readonly=True)
     async def validateBundle(self, bundle):
         bundle = await s_stormtypes.toprim(bundle)
-        return await s_coro.spawn((validateStix, (bundle,), {}))
+        return await s_coro.semafork(validateStix, bundle)
 
+    @s_stormtypes.stormfunc(readonly=True)
     async def liftBundle(self, bundle):
         bundle = await s_stormtypes.toprim(bundle)
 
@@ -757,7 +832,9 @@ stixingest = {
 
 @s_stormtypes.registry.registerLib
 class LibStixImport(s_stormtypes.Lib):
-
+    '''
+    A Storm Library for importing Stix Version 2.1 data.
+    '''
     _storm_lib_path = ('stix', 'import')
 
     _storm_locals = (  # type: ignore
@@ -780,7 +857,7 @@ class LibStixImport(s_stormtypes.Lib):
                     {'type': 'dict', 'name': 'bundle', 'desc': 'The STIX bundle to ingest.'},
                     {'type': 'dict', 'name': 'config', 'default': None, 'desc': 'An optional STIX ingest configuration.'},
                 ),
-                'returns': {'name': 'Yields', 'type': 'storm:node', 'desc': 'Yields nodes'}
+                'returns': {'name': 'Yields', 'type': 'node', 'desc': 'Yields nodes'}
             },
         },
     )
@@ -791,6 +868,7 @@ class LibStixImport(s_stormtypes.Lib):
             'ingest': self.ingest,
         }
 
+    @s_stormtypes.stormfunc(readonly=True)
     async def config(self):
         return s_msgpack.deepcopy(stixingest, use_list=True)
 
@@ -799,16 +877,22 @@ class LibStixImport(s_stormtypes.Lib):
         if config is None:
             config = stixingest
 
+        bundle = await s_stormtypes.toprim(bundle)
         config = await s_stormtypes.toprim(config)
 
+        if not isinstance(config, dict):
+            mesg = 'STIX ingest config must be a dictionary.'
+            raise s_exc.BadArg(mesg=mesg)
+
         config.setdefault('bundle', {})
         config.setdefault('objects', {})
         config.setdefault('relationships', [])
 
-        rellook = {r['type']: r for r in config.get('relationships', ())}
-
-        nodesbyid = {}
-        relationships = []
+        try:
+            rellook = {r['type']: r for r in config.get('relationships', ())}
+        except Exception as e:
+            mesg = f'Error processing relationships in STIX bundle ingest config: {e}.'
+            raise s_exc.BadArg(mesg=mesg)
 
         bundlenode = None
 
@@ -816,13 +900,44 @@ class LibStixImport(s_stormtypes.Lib):
         if bundleconf is None:
             bundleconf = {}
 
+        if not isinstance(bundleconf, dict):
+            mesg = 'STIX ingest config bundle value must be a dictionary.'
+            raise s_exc.BadArg(mesg=mesg)
+
         bundlestorm = bundleconf.get('storm')
         if bundlestorm:
+            if not isinstance(bundlestorm, str):
+                mesg = 'STIX ingest config storm query must be a string.'
+                raise s_exc.BadArg(mesg=mesg)
+
             bundlenode = await self._callStorm(bundlestorm, {'bundle': bundle})
 
         self.runt.layerConfirm(('node', 'edge', 'add', 'refs'))
 
-        for obj in bundle.get('objects'):
+        config = s_schemas.reqValidStixIngestConfig(config)
+        bundle = s_schemas.reqValidStixIngestBundle(bundle)
+
+        try:
+            nodesbyid = await self._ingestObjects(bundle, config, rellook)
+        except Exception as e:
+            mesg = f'Error processing objects in STIX bundle ingest: {e}.'
+            raise s_exc.BadArg(mesg=mesg)
+
+        if bundlenode is not None:
+            for node in nodesbyid.values():
+                await bundlenode.addEdge('refs', node.iden())
+                await asyncio.sleep(0)
+            yield bundlenode
+
+        for node in nodesbyid.values():
+            yield node
+
+    async def _ingestObjects(self, bundle, config, rellook):
+
+        nodesbyid = {}
+        relationships = []
+
+        for obj in bundle.get('objects', ()):
 
             objtype = obj.get('type')
 
@@ -897,7 +1012,7 @@ class LibStixImport(s_stormtypes.Lib):
                 await self.runt.snap.warnonce(f'STIX bundle ingest has no relationship definition for: {reltype}.')
 
         # attempt to resolve object_refs
-        for obj in bundle.get('objects'):
+        for obj in bundle.get('objects', ()):
 
             node = nodesbyid.get(obj.get('id'))
             if node is None:
@@ -910,22 +1025,12 @@ class LibStixImport(s_stormtypes.Lib):
 
                 await node.addEdge('refs', refsnode.iden())
 
-        if bundlenode is not None:
-            for node in nodesbyid.values():
-                await bundlenode.addEdge('refs', node.iden())
-                await asyncio.sleep(0)
-            yield bundlenode
-
-        for node in nodesbyid.values():
-            yield node
-
-        if bundlenode:
-            yield bundlenode
+        return nodesbyid
 
     async def _callStorm(self, text, varz):
 
         query = await self.runt.snap.core.getStormQuery(text)
-        async with self.runt.getSubRuntime(query, opts={'vars': varz}) as runt:
+        async with self.runt.getCmdRuntime(query, opts={'vars': varz}) as runt:
             try:
                 async for _ in runt.execute():
                     await asyncio.sleep(0)
@@ -939,6 +1044,11 @@ class LibStixExport(s_stormtypes.Lib):
     '''
     A Storm Library for exporting to STIX version 2.1 CS02.
     '''
+    _storm_lib_perms = (
+        {'perm': ('storm', 'lib', 'stix', 'export', 'maxsize'), 'gate': 'cortex',
+         'desc': 'Controls the ability to specify a STIX export bundle maxsize of greater than 10,000.'},
+    )
+
     _storm_locals = (  # type: ignore
         {
             'name': 'bundle',
@@ -963,6 +1073,10 @@ class LibStixExport(s_stormtypes.Lib):
                                         "rels": (
                                             ( <relname>, <target_stixtype>, <storm> ),
                                             ...
+                                        ),
+                                        "revs": (
+                                            ( <revname>, <source_stixtype>, <storm> ),
+                                            ...
                                         )
                                     },
                                     <stixtype1>: ...
@@ -995,6 +1109,21 @@ class LibStixExport(s_stormtypes.Lib):
                             },
                     }},
 
+                You may also specify pivots on a per form+stixtype basis to automate pivoting to additional nodes
+                to include in the bundle::
+
+                    {"forms": {
+                        "inet:fqdn":
+                            ...
+                            "domain-name": {
+                                ...
+                                "pivots": [
+                                    {"storm": "-> inet:dns:a -> inet:ipv4", "stixtype": "ipv4-addr"}
+                                ]
+                            {
+                        }
+                    }
+
                 Note:
                     The default config is an evolving set of mappings.  If you need to guarantee stable output please
                     specify a config.
@@ -1004,7 +1133,7 @@ class LibStixExport(s_stormtypes.Lib):
                 'args': (
                     {'type': 'dict', 'name': 'config', 'default': None, 'desc': 'The STIX bundle export config to use.'},
                 ),
-                'returns': {'type': 'storm:stix:bundle', 'desc': 'A new ``storm:stix:bundle`` instance.'},
+                'returns': {'type': 'stix:bundle', 'desc': 'A new ``stix:bundle`` instance.'},
             },
         },
 
@@ -1040,23 +1169,26 @@ class LibStixExport(s_stormtypes.Lib):
             'timestamp': self.timestamp,
         }
 
+    @s_stormtypes.stormfunc(readonly=True)
     async def config(self):
         # make a new mutable config
         return json.loads(json.dumps(_DefaultConfig))
 
+    @s_stormtypes.stormfunc(readonly=True)
     async def bundle(self, config=None):
 
         if config is None:
             config = _DefaultConfig
 
         config = await s_stormtypes.toprim(config)
-        _validateConfig(self.runt.snap.core, config)
+        _validateConfig(self.runt, config)
 
         return StixBundle(self, self.runt, config)
 
     def timestamp(self, tick):
-        dt = datetime.datetime.utcfromtimestamp(tick / 1000.0)
-        return dt.isoformat(timespec='milliseconds') + 'Z'
+        dt = datetime.datetime.fromtimestamp(tick / 1000.0, datetime.UTC)
+        millis = int(dt.microsecond / 1000)
+        return f'{dt.strftime("%Y-%m-%dT%H:%M:%S")}.{millis:03d}Z'
 
 stix_sdos = {
     'attack-pattern',
@@ -1114,7 +1246,7 @@ class StixBundle(s_stormtypes.Prim):
     Implements the Storm API for creating and packing a STIX bundle for v2.1
     '''
 
-    _storm_typename = 'storm:stix:bundle'
+    _storm_typename = 'stix:bundle'
     _storm_locals = (
 
         {'name': 'add', 'desc': '''
@@ -1142,6 +1274,11 @@ class StixBundle(s_stormtypes.Prim):
                   'args': (),
                   'returns': {'type': 'dict', }}},
 
+        {'name': 'size', 'desc': 'Return the number of STIX objects currently in the bundle.',
+         'type': {'type': 'function', '_funcname': 'size',
+                  'args': (),
+                  'returns': {'type': 'int', }}},
+
     )
 
     def __init__(self, libstix, runt, config, path=None):
@@ -1153,6 +1290,7 @@ class StixBundle(s_stormtypes.Prim):
         self.config = config
         self.locls.update(self.getObjLocals())
         self.objs = {}  # id -> STIX obj(dict)
+        self.synextension = config.get('synapse_extension', True)
         self.maxsize = config.get('maxsize', 10000)
 
     async def value(self):
@@ -1162,12 +1300,14 @@ class StixBundle(s_stormtypes.Prim):
         return {
             'add': self.add,
             'pack': self.pack,
+            'size': self.size,
         }
 
     # TODO config modification helpers
     # async def addPropMap(self, formname, stixtype, propname, stormtext):
     # async def addRelsMap(self, formname, stixtype, relname, targtype,  stormtext):
 
+    @s_stormtypes.stormfunc(readonly=True)
     async def add(self, node, stixtype=None):
 
         if len(self.objs) >= self.maxsize:
@@ -1219,6 +1359,16 @@ class StixBundle(s_stormtypes.Prim):
                 n2id = await self.add(relnode, stixtype=reltype)
                 await self._addRel(stixid, relname, n2id)
 
+        for (revname, revtype, revstorm) in stixconf.get('revs', ()):
+            async for revnode, revpath in node.storm(self.runt, revstorm):
+                n1id = await self.add(revnode, stixtype=revtype)
+                await self._addRel(n1id, revname, stixid)
+
+        for pivdef in stixconf.get('pivots', ()):
+            pivtype = pivdef.get('stixtype')
+            async for pivnode, pivpath in node.storm(self.runt, pivdef.get('storm')):
+                await self.add(pivnode, stixtype=pivtype)
+
         return stixid
 
     def _initStixItem(self, stixid, stixtype, node):
@@ -1227,14 +1377,14 @@ class StixBundle(s_stormtypes.Prim):
             'id': stixid,
             'type': stixtype,
             'spec_version': '2.1',
-            'extensions': {
+        }
+        if self.synextension:
+            retn['extensions'] = {
                 SYN_STIX_EXTENSION_ID: {
                     "extension_type": "property-extension",
                     'synapse_ndef': ndef
                 }
             }
-
-        }
         return retn
 
     async def _addRel(self, srcid, reltype, targid):
@@ -1273,9 +1423,11 @@ class StixBundle(s_stormtypes.Prim):
         }
         return ret
 
+    @s_stormtypes.stormfunc(readonly=True)
     def pack(self):
         objects = list(self.objs.values())
-        objects.insert(0, self._getSynapseExtensionDefinition())
+        if self.synextension:
+            objects.insert(0, self._getSynapseExtensionDefinition())
         bundle = {
             'type': 'bundle',
             'id': f'bundle--{uuid4()}',
@@ -1283,11 +1435,18 @@ class StixBundle(s_stormtypes.Prim):
         }
         return bundle
 
+    @s_stormtypes.stormfunc(readonly=True)
+    def size(self):
+        return len(self.objs)
+
     async def _callStorm(self, text, node):
 
-        opts = {'vars': {'bundle': self}}
+        varz = self.runt.getScopeVars()
+        varz['bundle'] = self
+
+        opts = {'vars': varz}
         query = await self.runt.snap.core.getStormQuery(text)
-        async with self.runt.getSubRuntime(query, opts=opts) as runt:
+        async with self.runt.getCmdRuntime(query, opts=opts) as runt:
 
             async def genr():
                 yield node, runt.initPath(node)
