@@ -1,3 +1,4 @@
+import sys
 import asyncio
 import decimal
 import logging
@@ -17,6 +18,7 @@ import synapse.lib.layer as s_layer
 import synapse.lib.config as s_config
 import synapse.lib.msgpack as s_msgpack
 import synapse.lib.grammar as s_grammar
+import synapse.lib.stormtypes as s_stormtypes
 
 logger = logging.getLogger(__name__)
 
@@ -46,7 +48,7 @@ class Type:
         self.form = None  # this will reference a Form() if the type is a form
         self.subof = None  # This references the name that a type was extended from.
 
-        self.info.setdefault('bases', ())
+        self.info.setdefault('bases', ('base',))
 
         self.opts = dict(self._opt_defs)
         self.opts.update(opts)
@@ -77,6 +79,14 @@ class Type:
 
         self.postTypeInit()
 
+        normopts = dict(self.opts)
+        for optn, valu in normopts.items():
+            if isinstance(valu, float):
+                normopts[optn] = str(valu)
+
+        ctor = '.'.join([self.__class__.__module__, self.__class__.__qualname__])
+        self.typehash = sys.intern(s_common.guid((ctor, s_common.flatten(normopts))))
+
     def _storLiftSafe(self, cmpr, valu):
         try:
             return self.storlifts['=']('=', valu)
@@ -208,9 +218,12 @@ class Type:
             raise s_exc.NoSuchCmpr(cmpr=name, name=self.name)
 
         norm1 = self.norm(val1)[0]
-        norm2 = self.norm(val2)[0]
 
-        return ctor(norm2)(norm1)
+        if name != '~=':
+            # Don't norm regex patterns
+            val2 = self.norm(val2)[0]
+
+        return ctor(val2)(norm1)
 
     def _ctorCmprEq(self, text):
         norm, info = self.norm(text)
@@ -236,7 +249,7 @@ class Type:
         return cmpr
 
     def _ctorCmprRe(self, text):
-        regx = regex.compile(text)
+        regx = regex.compile(text, flags=regex.I)
 
         def cmpr(valu):
             vtxt = self.repr(valu)
@@ -355,6 +368,13 @@ class Type:
         topt.update(opts)
         return self.__class__(self.modl, self.name, self.info, topt)
 
+    def __eq__(self, othr):
+        if self.name != othr.name:
+            return False
+        if self.opts != othr.opts:
+            return False
+        return True
+
 class Bool(Type):
 
     stortype = s_layer.STOR_TYPE_U8
@@ -363,6 +383,8 @@ class Bool(Type):
         self.setNormFunc(str, self._normPyStr)
         self.setNormFunc(int, self._normPyInt)
         self.setNormFunc(bool, self._normPyInt)
+        self.setNormFunc(decimal.Decimal, self._normPyInt)
+        self.setNormFunc(s_stormtypes.Number, self._normNumber)
 
     def _normPyStr(self, valu):
 
@@ -383,8 +405,11 @@ class Bool(Type):
     def _normPyInt(self, valu):
         return int(bool(valu)), {}
 
+    def _normNumber(self, valu):
+        return int(bool(valu.valu)), {}
+
     def repr(self, valu):
-        return repr(bool(valu))
+        return repr(bool(valu)).lower()
 
 class Array(Type):
 
@@ -497,8 +522,8 @@ class Comp(Type):
 
         fields = self.opts.get('fields')
         if len(fields) != len(valu):
-            raise s_exc.BadTypeValu(name=self.name, valu=valu,
-                                    mesg='invalid number of fields given for norming')
+            raise s_exc.BadTypeValu(name=self.name, fields=fields, numitems=len(valu),
+                                    mesg=f'invalid number of fields given for norming: {s_common.trimText(repr(valu))}')
 
         subs = {}
         adds = []
@@ -620,17 +645,38 @@ class Hex(Type):
 
     _opt_defs = (
         ('size', 0),  # type: ignore
+        ('zeropad', 0),
     )
 
     def postTypeInit(self):
         self._size = self.opts.get('size')
+
+        # This is for backward compat with v2.142.x where zeropad was a bool
+        self._zeropad = self.opts.get('zeropad')
+        if isinstance(self._zeropad, bool):
+            if self._zeropad:
+                self._zeropad = self._size
+            else:
+                self._zeropad = 0
+
         if self._size < 0:
             # zero means no width check
             raise s_exc.BadConfValu(name='size', valu=self._size,
-                                    mesg='Size must be > 0')
+                                    mesg='Size must be >= 0')
         if self._size % 2 != 0:
             raise s_exc.BadConfValu(name='size', valu=self._size,
                                     mesg='Size must be a multiple of 2')
+
+        if self._zeropad < 0:
+            raise s_exc.BadConfValu(name='zeropad', valu=self._zeropad,
+                                    mesg='Zeropad must be >= 0')
+        if self._zeropad % 2 != 0:
+            raise s_exc.BadConfValu(name='zeropad', valu=self._zeropad,
+                                    mesg='Zeropad must be a multiple of 2')
+
+        if self._size:
+            self._zeropad = min(self._zeropad, self._size)
+
         self.setNormFunc(str, self._normPyStr)
         self.setNormFunc(bytes, self._normPyBytes)
         self.storlifts.update({
@@ -638,13 +684,16 @@ class Hex(Type):
             '^=': self._storLiftPref,
         })
 
+    def _preNormHex(self, text):
+        text = text.strip().lower()
+        if text.startswith('0x'):
+            text = text[2:]
+        return text.replace(' ', '').replace(':', '')
+
     def _storLiftEq(self, cmpr, valu):
 
-        if type(valu) == str:
-            valu = valu.strip().lower()
-            if valu.startswith('0x'):
-                valu = valu[2:]
-
+        if isinstance(valu, str):
+            valu = self._preNormHex(valu)
             if valu.endswith('*'):
                 return (
                     ('^=', valu[:-1], self.stortype),
@@ -653,16 +702,33 @@ class Hex(Type):
         return self._storLiftNorm(cmpr, valu)
 
     def _storLiftPref(self, cmpr, valu):
-        valu = valu.strip().lower()
-        if valu.startswith('0x'):
-            valu = valu[2:]
-
+        valu = self._preNormHex(valu)
         return (
             ('^=', valu, self.stortype),
         )
 
     def _normPyStr(self, valu):
-        valu = s_chop.hexstr(valu)
+        valu = valu.strip().lower()
+        if valu.startswith('0x'):
+            valu = valu[2:]
+
+        valu = valu.replace(' ', '').replace(':', '')
+
+        if not valu:
+            raise s_exc.BadTypeValu(valu=valu, name='hex',
+                                    mesg='No string left after stripping')
+
+        if self._zeropad and len(valu) < self._zeropad:
+            padlen = self._zeropad - len(valu)
+            valu = ('0' * padlen) + valu
+
+        try:
+            # checks for valid hex width and does character
+            # checking in C without using regex
+            s_common.uhex(valu)
+        except (binascii.Error, ValueError) as e:
+            raise s_exc.BadTypeValu(valu=valu, name='hex', mesg=str(e)) from None
+
         if self._size and len(valu) != self._size:
             raise s_exc.BadTypeValu(valu=valu, reqwidth=self._size, name=self.name,
                                     mesg='invalid width')
@@ -690,6 +756,7 @@ class HugeNum(Type):
     stortype = s_layer.STOR_TYPE_HUGENUM
 
     _opt_defs = (
+        ('units', None),   # type: ignore
         ('modulo', None),  # type: ignore
     )
 
@@ -712,10 +779,39 @@ class HugeNum(Type):
 
         self.modulo = None
 
+        self.units = {}
+        units = self.opts.get('units')
+        if units is not None:
+            for name, mult in units.items():
+                self.units[name] = s_common.hugenum(mult)
+
         modulo = self.opts.get('modulo')
         if modulo is not None:
             self.modulo = s_common.hugenum(modulo)
 
+    def _normHugeText(self, rawtext):
+
+        text = rawtext.lower().strip()
+        text = text.replace(',', '').replace(' ', '')
+
+        try:
+            valu, off = s_grammar.chop_float(text, 0)
+        except Exception:
+            mesg = f'Value does not start with a number: "{rawtext}"'
+            raise s_exc.BadTypeValu(mesg=mesg)
+
+        huge = s_common.hugenum(valu)
+
+        unit, off = s_grammar.nom(text, off, s_grammar.unitset)
+        if unit:
+            mult = self.units.get(unit)
+            if mult is None:
+                mesg = f'Unknown units for value: "{rawtext}"'
+                raise s_exc.BadTypeValu(mesg=mesg)
+            huge = s_common.hugemul(huge, mult)
+
+        return huge
+
     def norm(self, valu):
 
         if valu is None:
@@ -724,7 +820,10 @@ class HugeNum(Type):
 
         try:
 
-            huge = s_common.hugenum(valu)
+            if isinstance(valu, str):
+                huge = self._normHugeText(valu)
+            else:
+                huge = s_common.hugenum(valu)
 
             # behave modulo like int/float
             if self.modulo is not None:
@@ -808,6 +907,9 @@ class IntBase(Type):
             'range=': self._storLiftRange,
         })
 
+        self.setNormFunc(decimal.Decimal, self._normPyDecimal)
+        self.setNormFunc(s_stormtypes.Number, self._normNumber)
+
     def _storLiftRange(self, cmpr, valu):
         minv, minfo = self.norm(valu[0])
         maxv, maxfo = self.norm(valu[1])
@@ -841,11 +943,18 @@ class IntBase(Type):
             return valu < norm
         return cmpr
 
+    def _normPyDecimal(self, valu):
+        return self._normPyInt(int(valu))
+
+    def _normNumber(self, valu):
+        return self._normPyInt(int(valu.valu))
+
 class Int(IntBase):
 
     _opt_defs = (
         ('size', 8),  # type: ignore # Set the storage size of the integer type in bytes.
         ('signed', True),
+        ('enums:strict', True),
 
         # Note: currently unused
         ('fmt', '%d'),  # Set to an integer compatible format string to control repr.
@@ -869,6 +978,8 @@ class Int(IntBase):
         self.enumnorm = {}
         self.enumrepr = {}
 
+        self.enumstrict = self.opts.get('enums:strict')
+
         enums = self.opts.get('enums')
         if enums is not None:
             self.enumrepr.update(dict(enums))
@@ -910,6 +1021,7 @@ class Int(IntBase):
         self.setNormFunc(str, self._normPyStr)
         self.setNormFunc(int, self._normPyInt)
         self.setNormFunc(bool, self._normPyBool)
+        self.setNormFunc(float, self._normPyFloat)
 
     def merge(self, oldv, newv):
 
@@ -948,12 +1060,15 @@ class Int(IntBase):
             mesg = f'value is above max={self.maxval}'
             raise s_exc.BadTypeValu(valu=repr(valu), name=self.name, mesg=mesg)
 
-        if self.enumrepr and valu not in self.enumrepr:
+        if self.enumrepr and self.enumstrict and valu not in self.enumrepr:
             mesg = 'Value is not a valid enum value.'
             raise s_exc.BadTypeValu(valu=valu, name=self.name, mesg=mesg)
 
         return valu, {}
 
+    def _normPyFloat(self, valu):
+        return self._normPyInt(int(valu))
+
     def repr(self, norm):
 
         text = self.enumrepr.get(norm)
@@ -1045,6 +1160,8 @@ class Float(Type):
         self.setNormFunc(str, self._normPyStr)
         self.setNormFunc(int, self._normPyInt)
         self.setNormFunc(float, self._normPyFloat)
+        self.setNormFunc(decimal.Decimal, self._normPyInt)
+        self.setNormFunc(s_stormtypes.Number, self._normNumber)
 
     def _normPyStr(self, valu):
 
@@ -1059,6 +1176,9 @@ class Float(Type):
         valu = float(valu)
         return self._normPyFloat(valu)
 
+    def _normNumber(self, valu):
+        return self._normPyFloat(float(valu.valu))
+
     def _normPyFloat(self, valu):
 
         if self.minval is not None and not self.mincmp(valu, self.minval):
@@ -1097,6 +1217,8 @@ class Ival(Type):
         self.setNormFunc(str, self._normPyStr)
         self.setNormFunc(list, self._normPyIter)
         self.setNormFunc(tuple, self._normPyIter)
+        self.setNormFunc(decimal.Decimal, self._normPyInt)
+        self.setNormFunc(s_stormtypes.Number, self._normNumber)
         self.storlifts.update({
             '@=': self._storLiftAt,
         })
@@ -1157,6 +1279,11 @@ class Ival(Type):
         maxv, info = self.timetype._normPyInt(minv + 1)
         return (minv, maxv), info
 
+    def _normNumber(self, valu):
+        minv, _ = self.timetype._normPyInt(valu.valu)
+        maxv, info = self.timetype._normPyInt(minv + 1)
+        return (minv, maxv), info
+
     def _normRelStr(self, valu, relto=None):
         valu = valu.strip().lower()
         # assumes the relative string starts with a - or +
@@ -1291,12 +1418,37 @@ class Loc(Type):
 
 class Ndef(Type):
 
-    stortype = s_layer.STOR_TYPE_MSGP
+    stortype = s_layer.STOR_TYPE_NDEF
 
     def postTypeInit(self):
         self.setNormFunc(list, self._normPyTuple)
         self.setNormFunc(tuple, self._normPyTuple)
 
+        self.formfilter = None
+
+        self.forms = self.opts.get('forms')
+        self.ifaces = self.opts.get('interfaces')
+
+        if self.forms or self.ifaces:
+            if self.forms is not None:
+                forms = set(self.forms)
+
+            if self.ifaces is not None:
+                ifaces = set(self.ifaces)
+
+            def filtfunc(form):
+                if self.forms is not None and form.name in forms:
+                    return False
+
+                if self.ifaces is not None:
+                    for iface in form.ifaces.keys():
+                        if iface in ifaces:
+                            return False
+
+                return True
+
+            self.formfilter = filtfunc
+
     def _normStormNode(self, valu):
         return self._normPyTuple(valu.ndef)
 
@@ -1308,7 +1460,17 @@ class Ndef(Type):
 
         form = self.modl.form(formname)
         if form is None:
-            raise s_exc.NoSuchForm(name=self.name, form=formname)
+            raise s_exc.NoSuchForm.init(formname)
+
+        if self.formfilter is not None and self.formfilter(form):
+            mesg = f'Ndef of form {formname} is not allowed as a value for {self.name} with form filter'
+            if self.forms is not None:
+                mesg += f' forms={self.forms}'
+
+            if self.ifaces is not None:
+                mesg += f' interfaces={self.ifaces}'
+
+            raise s_exc.BadTypeValu(valu=formname, name=self.name, mesg=mesg, forms=self.forms, interfaces=self.ifaces)
 
         formnorm, forminfo = form.type.norm(formvalu)
         norm = (form.name, formnorm)
@@ -1322,7 +1484,7 @@ class Ndef(Type):
         formname, formvalu = norm
         form = self.modl.form(formname)
         if form is None:
-            raise s_exc.NoSuchForm(name=self.name, form=formname)
+            raise s_exc.NoSuchForm.init(formname)
 
         repv = form.type.repr(formvalu)
         return (formname, repv)
@@ -1336,6 +1498,8 @@ class Edge(Type):
 
     def postTypeInit(self):
 
+        self.deprecated = True
+
         self.fieldoffs = {'n1': 0, 'n2': 1}
 
         self.ndeftype = self.modl.types.get('ndef')  # type: Ndef
@@ -1402,7 +1566,7 @@ class TimeEdge(Edge):
     def _normPyTuple(self, valu):
 
         if len(valu) != 3:
-            mesg = 'timeedge requires (ndef, ndef, time)'
+            mesg = f'timeedge requires (ndef, ndef, time), got {valu}'
             raise s_exc.BadTypeValu(mesg=mesg, name=self.name, valu=valu)
 
         n1, n2, tick = valu
@@ -1441,7 +1605,7 @@ class Data(Type):
             if self.validator is not None:
                 self.validator(valu)
         except (s_exc.MustBeJsonSafe, s_exc.SchemaViolation) as e:
-            raise s_exc.BadTypeValu(name=self.name, valu=valu, mesg=str(e)) from None
+            raise s_exc.BadTypeValu(name=self.name, mesg=f'{e}: {s_common.trimText(repr(valu))}') from None
         byts = s_msgpack.en(valu)
         return s_msgpack.un(byts), {}
 
@@ -1459,10 +1623,11 @@ class NodeProp(Type):
         return self._normPyTuple(valu)
 
     def _normPyTuple(self, valu):
-        try:
-            propname, propvalu = valu
-        except Exception as e:
-            raise s_exc.BadTypeValu(name=self.name, valu=valu, mesg=str(e)) from None
+        if len(valu) != 2:
+            mesg = f'Must be a 2-tuple: {s_common.trimText(repr(valu))}'
+            raise s_exc.BadTypeValu(name=self.name, numitems=len(valu), mesg=mesg) from None
+
+        propname, propvalu = valu
 
         prop = self.modl.prop(propname)
         if prop is None:
@@ -1501,8 +1666,8 @@ class Range(Type):
 
     def _normPyTuple(self, valu):
         if len(valu) != 2:
-            raise s_exc.BadTypeValu(valu=valu, name=self.name,
-                                    mesg=f'Must be a 2-tuple of type {self.subtype.name}')
+            mesg = f'Must be a 2-tuple of type {self.subtype.name}: {s_common.trimText(repr(valu))}'
+            raise s_exc.BadTypeValu(numitems=len(valu), name=self.name, mesg=mesg)
 
         minv = self.subtype.norm(valu[0])[0]
         maxv = self.subtype.norm(valu[1])[0]
@@ -1540,6 +1705,9 @@ class Str(Type):
         self.setNormFunc(str, self._normPyStr)
         self.setNormFunc(int, self._normPyInt)
         self.setNormFunc(bool, self._normPyBool)
+        self.setNormFunc(float, self._normPyFloat)
+        self.setNormFunc(decimal.Decimal, self._normPyInt)
+        self.setNormFunc(s_stormtypes.Number, self._normNumber)
 
         self.storlifts.update({
             '=': self._storLiftEq,
@@ -1605,6 +1773,13 @@ class Str(Type):
     def _normPyInt(self, valu):
         return self._normPyStr(str(valu))
 
+    def _normNumber(self, valu):
+        return self._normPyStr(str(valu.valu))
+
+    def _normPyFloat(self, valu):
+        deci = s_common.hugectx.create_decimal(str(valu))
+        return self._normPyStr(format(deci, 'f'))
+
     def _normPyStr(self, valu):
 
         info = {}
@@ -1669,8 +1844,19 @@ class Taxonomy(Str):
         self.setNormFunc(tuple, self._normPyList)
         self.taxon = self.modl.type('taxon')
 
+    def _ctorCmprPref(self, valu):
+        norm = self._normForLift(valu)
+
+        def cmpr(valu):
+            return valu.startswith(norm)
+
+        return cmpr
+
     def _normForLift(self, valu):
-        return self.norm(valu)[0]
+        norm = self.norm(valu)[0]
+        if isinstance(valu, str) and not valu.strip().endswith('.'):
+            return norm.rstrip('.')
+        return norm
 
     def _normPyList(self, valu):
 
@@ -1689,6 +1875,9 @@ class Taxonomy(Str):
     def _normPyStr(self, text):
         return self._normPyList(text.strip().strip('.').split('.'))
 
+    def repr(self, norm):
+        return norm.rstrip('.')
+
 class Tag(Str):
 
     def postTypeInit(self):
@@ -1714,7 +1903,13 @@ class Tag(Str):
             mesg = f'Tag does not match tagre: [{s_grammar.tagre.pattern}]'
             raise s_exc.BadTypeValu(valu=norm, name=self.name, mesg=mesg)
 
-        return norm, {'subs': subs}
+        core = self.modl.core
+        if core is not None:
+            (ok, mesg) = core.isTagValid(norm)
+            if not ok:
+                raise s_exc.BadTypeValu(valu=norm, name=self.name, mesg=mesg)
+
+        return norm, {'subs': subs, 'toks': toks}
 
     def _normPyStr(self, text):
         toks = text.strip('#').split('.')
@@ -1792,7 +1987,7 @@ class Velocity(IntBase):
 
         unit = valu[offs:].strip()
         if not unit:
-            return base, {}
+            return int(base), {}
 
         if unit.find('/') != -1:
             dist, dura = unit.split('/', 1)
@@ -1928,6 +2123,8 @@ class Time(IntBase):
 
         if self.ismin:
             self.stortype = s_layer.STOR_TYPE_MINTIME
+        elif self.ismax:
+            self.stortype = s_layer.STOR_TYPE_MAXTIME
 
     def _liftByIval(self, cmpr, valu):
 
@@ -1987,12 +2184,12 @@ class Time(IntBase):
 
             return self._normPyInt(delt + bgn)
 
-        valu = s_time.parse(valu, base=base)
+        valu = s_time.parse(valu, base=base, chop=True)
         return self._normPyInt(valu)
 
     def _normPyInt(self, valu):
         if valu > self.maxsize and valu != self.futsize:
-            mesg = f'Time exceeds max size [{self.maxsize}] allowed for a non-future marker.'
+            mesg = f'Time exceeds max size [{self.maxsize}] allowed for a non-future marker, got {valu}'
             raise s_exc.BadTypeValu(mesg=mesg, valu=valu, name=self.name)
         return valu, {}
 
@@ -2019,7 +2216,8 @@ class Time(IntBase):
 
             lowr = valu.strip().lower()
             if not lowr:
-                raise s_exc.BadTypeValu(name=self.name, valu=valu)
+                mesg = f'Invalid time provided, got [{valu}]'
+                raise s_exc.BadTypeValu(mesg=mesg, name=self.name, valu=valu)
 
             if lowr == 'now':
                 return s_common.now()
@@ -2053,7 +2251,7 @@ class Time(IntBase):
         try:
             _tick = self._getLiftValu(val0)
         except ValueError:
-            mesg = 'Unable to process the value for val0 in _getLiftValu.'
+            mesg = f'Unable to process the value for val0 in _getLiftValu, got {val0}'
             raise s_exc.BadTypeValu(name=self.name, valu=val0,
                                     mesg=mesg) from None
 
@@ -2086,10 +2284,12 @@ class Time(IntBase):
         '''
 
         if not isinstance(vals, (list, tuple)):
-            raise s_exc.BadCmprValu(valu=vals, cmpr='range=')
+            mesg = f'Must be a 2-tuple: {s_common.trimText(repr(vals))}'
+            raise s_exc.BadCmprValu(itemtype=type(vals), cmpr='range=', mesg=mesg)
 
         if len(vals) != 2:
-            raise s_exc.BadCmprValu(valu=vals, cmpr='range=')
+            mesg = f'Must be a 2-tuple: {s_common.trimText(repr(vals))}'
+            raise s_exc.BadCmprValu(itemtype=type(vals), cmpr='range=', mesg=mesg)
 
         tick, tock = self.getTickTock(vals)
 
