@@ -39,10 +39,22 @@ class PsModule(s_module.CoreModule):
                 ('ps:persona:has', ('comp', {'fields': (('persona', 'ps:persona'), ('node', 'ndef'))}), {
                     'deprecated': True,
                     'doc': 'A persona owns, controls, or has exclusive use of an object or'
-                           ' resource, potentially during a specific period of time.'
-                }),
+                           ' resource, potentially during a specific period of time.'}),
+
                 ('ps:contact', ('guid', {}), {
                     'doc': 'A GUID for a contact info record.',
+                    'display': {
+                        'columns': (
+                            {'type': 'prop', 'opts': {'name': 'name'}},
+                            {'type': 'prop', 'opts': {'name': 'type'}},
+                            {'type': 'prop', 'opts': {'name': 'orgname'}},
+                            {'type': 'prop', 'opts': {'name': 'email'}},
+                        ),
+                    }}),
+
+                ('ps:contact:type:taxonomy', ('taxonomy', {}), {
+                    'interfaces': ('meta:taxonomy',),
+                    'doc': 'A taxonomy of contact types.',
                 }),
                 ('ps:contactlist', ('guid', {}), {
                     'doc': 'A GUID for a list of associated contacts.',
@@ -50,6 +62,40 @@ class PsModule(s_module.CoreModule):
                 ('ps:workhist', ('guid', {}), {
                     'doc': "A GUID representing entry in a contact's work history.",
                 }),
+                ('ps:vitals', ('guid', {}), {
+                    'doc': 'Statistics and demographic data about a person or contact.'}),
+
+                ('ps:skill', ('guid', {}), {
+                    'doc': 'A specific skill which a person or organization may have.',
+                    'display': {
+                        'columns': (
+                            {'type': 'prop', 'opts': {'name': 'name'}},
+                            {'type': 'prop', 'opts': {'name': 'type'}},
+                        ),
+                    }}),
+
+                ('ps:skill:type:taxonomy', ('taxonomy', {}), {
+                    'interfaces': ('meta:taxonomy',),
+                    'doc': 'A taxonomy of skill types.'}),
+
+                ('ps:proficiency', ('guid', {}), {
+                    'doc': 'The assessment that a given contact possesses a specific skill.',
+                    'display': {
+                        'columns': (
+                            {'type': 'prop', 'opts': {'name': 'contact::name'}},
+                            {'type': 'prop', 'opts': {'name': 'skill::name'}},
+                        ),
+                    }}),
+            ),
+            'edges': (
+                (('ps:contact', 'has', None), {
+                    'doc': 'The contact is or was in possession of the target node.'}),
+                (('ps:person', 'has', None), {
+                    'doc': 'The person is or was in possession of the target node.'}),
+                (('ps:contact', 'owns', None), {
+                    'doc': 'The contact owns or owned the target node.'}),
+                (('ps:person', 'owns', None), {
+                    'doc': 'The person owns or owned the target node.'}),
             ),
             'forms': (
                 ('ps:workhist', {}, (
@@ -204,6 +250,9 @@ class PsModule(s_module.CoreModule):
                     ('nick', ('inet:user', {}), {
                         'doc': 'A username commonly used by the person.',
                     }),
+                    ('vitals', ('ps:vitals', {}), {
+                        'doc': 'The most recent known vitals for the person.',
+                    }),
                     ('name', ('ps:name', {}), {
                         'doc': 'The localized name for the person.',
                     }),
@@ -283,25 +332,35 @@ class PsModule(s_module.CoreModule):
                         'doc': 'The form of the object or resource that is owned or controlled by the persona.',
                     }),
                 )),
+                ('ps:contact:type:taxonomy', {}, ()),
                 ('ps:contact', {}, (
                     ('org', ('ou:org', {}), {
                         'doc': 'The org which this contact represents.',
                     }),
+                    ('type', ('ps:contact:type:taxonomy', {}), {
+                        'doc': 'The type of contact which may be used for entity resolution.',
+                    }),
                     ('asof', ('time', {}), {
                         'date': 'The time this contact was created or modified.',
                     }),
                     ('person', ('ps:person', {}), {
                         'doc': 'The ps:person GUID which owns this contact.',
                     }),
+                    ('vitals', ('ps:vitals', {}), {
+                        'doc': 'The most recent known vitals for the contact.',
+                    }),
                     ('name', ('ps:name', {}), {
                         'doc': 'The person name listed for the contact.',
                     }),
                     ('desc', ('str', {}), {
                         'doc': 'A description of this contact.',
                     }),
-                    ('title', ('str', {'lower': True, 'strip': True}), {
-                        'doc': 'The job/org title listed for this contact.',
-                    }),
+                    ('title', ('ou:jobtitle', {}), {
+                        'doc': 'The job/org title listed for this contact.'}),
+
+                    ('titles', ('array', {'type': 'ou:jobtitle', 'sorted': True, 'uniq': True}), {
+                        'doc': 'An array of alternate titles for the contact.'}),
+
                     ('photo', ('file:bytes', {}), {
                         'doc': 'The photo listed for this contact.',
                     }),
@@ -312,13 +371,18 @@ class PsModule(s_module.CoreModule):
                         'doc': 'The listed org/company FQDN for this contact.',
                     }),
                     ('user', ('inet:user', {}), {
-                        'doc': 'The username or handle for this contact.',
-                    }),
+                        'doc': 'The username or handle for this contact.'}),
+
+                    ('service:accounts', ('array', {'type': 'inet:service:account', 'sorted': True, 'uniq': True}), {
+                        'doc': 'The service accounts associated with this contact.'}),
+
                     ('web:acct', ('inet:web:acct', {}), {
-                        'doc': 'The social media account for this contact.',
+                        'deprecated': True,
+                        'doc': 'Deprecated. Use :service:accounts.',
                     }),
                     ('web:group', ('inet:web:group', {}), {
-                        'doc': 'A web group representing this contact.',
+                        'deprecated': True,
+                        'doc': 'Deprecated. Use inet:service:group:profile to link to a group.',
                     }),
                     ('birth:place', ('geo:place', {}), {
                         'doc': 'A fully resolved place of birth for this contact.',
@@ -358,10 +422,14 @@ class PsModule(s_module.CoreModule):
                     }),
                     ('address', ('geo:address', {}), {
                         'doc': 'The street address listed for the contact.',
+                        'disp': {'hint': 'text'}
                     }),
                     ('place', ('geo:place', {}), {
                         'doc': 'The place associated with this contact.',
                     }),
+                    ('place:name', ('geo:name', {}), {
+                        'doc': 'The reported name of the place associated with this contact.',
+                    }),
                     ('phone', ('tel:phone', {}), {
                         'doc': 'The main phone number for this contact.',
                     }),
@@ -388,13 +456,17 @@ class PsModule(s_module.CoreModule):
                     }),
                     # A few probable multi-fields for entity resolution
                     ('names', ('array', {'type': 'ps:name', 'uniq': True, 'sorted': True}), {
-                        'doc': 'The person name listed for the contact.',
+                        'doc': 'An array of associated names/aliases for the person.',
+                    }),
+                    ('orgnames', ('array', {'type': 'ou:name', 'uniq': True, 'sorted': True}), {
+                        'doc': 'An array of associated names/aliases for the organization.',
                     }),
                     ('emails', ('array', {'type': 'inet:email', 'uniq': True, 'sorted': True}), {
                         'doc': 'An array of secondary/associated email addresses.',
                     }),
                     ('web:accts', ('array', {'type': 'inet:web:acct', 'uniq': True, 'sorted': True}), {
-                        'doc': 'An array of secondary/associated web accounts.',
+                        'deprecated': True,
+                        'doc': 'Deprecated. Use :service:accounts.',
                     }),
                     ('id:numbers', ('array', {'type': 'ou:id:number', 'uniq': True, 'sorted': True}), {
                         'doc': 'An array of secondary/associated IDs.',
@@ -405,6 +477,31 @@ class PsModule(s_module.CoreModule):
                     ('crypto:address', ('crypto:currency:address', {}), {
                         'doc': 'A crypto currency address associated with the contact.'
                     }),
+
+                    ('lang', ('lang:language', {}), {
+                        'doc': 'The language specified for the contact.'}),
+
+                    ('langs', ('array', {'type': 'lang:language'}), {
+                        'doc': 'An array of alternative languages specified for the contact.'}),
+                )),
+                ('ps:vitals', {}, (
+                    ('asof', ('time', {}), {
+                        'doc': 'The time the vitals were gathered or computed.'}),
+                    ('contact', ('ps:contact', {}), {
+                        'doc': 'The contact that the vitals are about.'}),
+                    ('person', ('ps:person', {}), {
+                        'doc': 'The person that the vitals are about.'}),
+                    ('height', ('geo:dist', {}), {
+                        'doc': 'The height of the person or contact.'}),
+                    ('weight', ('mass', {}), {
+                        'doc': 'The weight of the person or contact.'}),
+                    ('econ:currency', ('econ:currency', {}), {
+                        'doc': 'The currency that the price values are recorded using.'}),
+                    ('econ:net:worth', ('econ:price', {}), {
+                        'doc': 'The net worth of the contact.'}),
+                    ('econ:annual:income', ('econ:price', {}), {
+                        'doc': 'The yearly income of the contact.'}),
+                    # TODO: eye color etc. color names / rgb values?
                 )),
                 ('ps:contactlist', {}, (
                     ('contacts', ('array', {'type': 'ps:contact', 'uniq': True, 'split': ',', 'sorted': True}), {
@@ -417,8 +514,31 @@ class PsModule(s_module.CoreModule):
                         'doc': 'The file from which the contact list was extracted.',
                     }),
                     ('source:acct', ('inet:web:acct', {}), {
-                        'doc': 'The web account from which the contact list was extracted.',
+                        'deprecated': True,
+                        'doc': 'Deprecated. Use :source:account.',
                     }),
+                    ('source:account', ('inet:service:account', {}), {
+                        'doc': 'The service account from which the contact list was extracted.',
+                    }),
+                )),
+
+                ('ps:skill:type:taxonomy', {}, ()),
+                ('ps:skill', {}, (
+
+                    ('name', ('str', {'lower': True, 'onespace': True}), {
+                        'doc': 'The name of the skill.'}),
+
+                    ('type', ('ps:skill:type:taxonomy', {}), {
+                        'doc': 'The type of skill as a taxonomy.'})
+                )),
+
+                ('ps:proficiency', {}, (
+
+                    ('skill', ('ps:skill', {}), {
+                        'doc': 'The skill in which the contact is proficient.'}),
+
+                    ('contact', ('ps:contact', {}), {
+                        'doc': 'The contact which is proficient in the skill.'}),
                 )),
             )
         }
