@@ -376,7 +376,12 @@ class TelcoModule(s_module.CoreModule):
                     # User related data
                     ('name', ('ps:name', {}), {}),
                     ('email', ('inet:email', {}), {}),
-                    ('acct', ('inet:web:acct', {}), {}),
+                    ('acct', ('inet:web:acct', {}), {
+                        'doc': 'Deprecated, use :account.',
+                        'deprecated': True}),
+
+                    ('account', ('inet:service:account', {}), {
+                        'doc': 'The service account which is associated with the tracked device.'}),
 
                     # reporting related data
                     ('app', ('it:prod:softver', {}), {}),
