@@ -12,8 +12,6 @@ import logging
 import contextlib
 import collections
 
-import aiohttp
-
 import synapse.exc as s_exc
 import synapse.glob as s_glob
 import synapse.common as s_common
@@ -24,6 +22,7 @@ import synapse.lib.queue as s_queue
 import synapse.lib.certdir as s_certdir
 import synapse.lib.threads as s_threads
 import synapse.lib.urlhelp as s_urlhelp
+import synapse.lib.version as s_version
 import synapse.lib.hashitem as s_hashitem
 
 logger = logging.getLogger(__name__)
@@ -136,40 +135,58 @@ def mergeAhaInfo(info0, info1):
 
     return info0
 
-async def getAhaProxy(urlinfo):
+async def open(url, onlink=None):
     '''
-    Return a telepath proxy by looking up a host from an aha registry.
+    Open a new telepath ClientV2 object based on the given URL.
+
+    Args:
+        url (str): The URL to connect to.
+        onlink: An optional async callback function to run when connections are made.
+
+    Notes:
+        The onlink callback function has the call signature ``(proxy, urlinfo)``.
+        The proxy is the Telepath Proxy object.
+        The urlinfo is the parsed URL information used to create the proxy object.
+        The urlinfo structure may change between versions of Synapse.
+
+    Returns:
+        ClientV2: A ClientV2 object.
     '''
+    return await ClientV2.anit(url, onlink=onlink)
+
+async def _getAhaSvc(urlinfo, timeout=None):
+
     host = urlinfo.get('host')
     if host is None:
-        mesg = f'getAhaProxy urlinfo has no host: {urlinfo}'
+        mesg = f'AHA urlinfo has no host: {urlinfo}'
         raise s_exc.NoSuchName(mesg=mesg)
 
     if not aha_clients:
         mesg = f'No aha servers registered to lookup {host}'
         raise s_exc.NotReady(mesg=mesg)
 
-    laste = None
+    errs = []
     for ahaurl, cnfo in list(aha_clients.items()):
         client = cnfo.get('client')
         if client is None:
-            client = await Client.anit(cnfo.get('url'))
+            client = await Client.anit(ahaurl)
             client._fini_atexit = True
             cnfo['client'] = client
 
         try:
-            proxy = await client.proxy(timeout=5)
+            proxy = await client.proxy(timeout=timeout)
 
-            cellinfo = await asyncio.wait_for(proxy.getCellInfo(), timeout=5)
+            cellinfo = await s_common.wait_for(proxy.getCellInfo(), timeout=5)
 
-            if cellinfo['synapse']['version'] >= (2, 95, 0):
-                filters = {
-                    'mirror': bool(yaml.safe_load(urlinfo.get('mirror', 'false')))
+            kwargs = {}
+            synvers = cellinfo['synapse']['version']
+
+            if synvers >= (2, 95, 0):
+                kwargs['filters'] = {
+                    'mirror': bool(s_common.yamlloads(urlinfo.get('mirror', 'false'))),
                 }
-                ahasvc = await asyncio.wait_for(proxy.getAhaSvc(host, filters=filters), timeout=5)
-            else:
-                ahasvc = await asyncio.wait_for(proxy.getAhaSvc(host), timeout=5)
 
+            ahasvc = await s_common.wait_for(proxy.getAhaSvc(host, **kwargs), timeout=5)
             if ahasvc is None:
                 continue
 
@@ -177,12 +194,7 @@ async def getAhaProxy(urlinfo):
             if not svcinfo.get('online'):
                 continue
 
-            info = mergeAhaInfo(urlinfo, svcinfo.get('urlinfo', {}))
-
-            return await openinfo(info)
-
-        except asyncio.CancelledError:  # pragma: no cover
-            raise
+            return client, ahasvc
 
         except Exception as e:
             if isinstance(ahaurl, str):
@@ -190,15 +202,25 @@ async def getAhaProxy(urlinfo):
             else:
                 surl = tuple([s_urlhelp.sanitizeUrl(u) for u in ahaurl])
             logger.exception(f'Unable to get aha client ({surl})')
+            errs.append(e)
 
-            laste = e
-
-    if laste is not None:
-        raise laste
+    if errs:
+        raise errs[-1]
 
     mesg = f'aha lookup failed: {host}'
     raise s_exc.NoSuchName(mesg=mesg)
 
+async def getAhaProxy(urlinfo):
+    '''
+    Return a telepath proxy by looking up a host from an AHA registry.
+    '''
+    ahaclient, ahasvc = await _getAhaSvc(urlinfo, timeout=5)
+
+    svcinfo = ahasvc.get('svcinfo', {})
+    svcurlinfo = mergeAhaInfo(urlinfo, svcinfo.get('urlinfo', {}))
+
+    return await openinfo(svcurlinfo)
+
 @contextlib.asynccontextmanager
 async def withTeleEnv():
 
@@ -263,7 +285,8 @@ async def loadTeleCell(dirn):
     ahaurl = None
     if os.path.isfile(confpath):
         conf = s_common.yamlload(confpath)
-        ahaurl = conf.get('aha:registry')
+        if conf is not None:
+            ahaurl = conf.get('aha:registry')
 
     if usecerts:
         s_certdir.addCertPath(certpath)
@@ -477,6 +500,7 @@ class GenrMethod(Method):
 class Pipeline(s_base.Base):
 
     async def __anit__(self, proxy, genr, name=None):
+        s_common.deprecated('Telepath.Pipeline', curv='2.167.0')
 
         await s_base.Base.__anit__(self)
 
@@ -574,6 +598,7 @@ class Proxy(s_base.Base):
         self.tasks = {}
         self.shares = {}
 
+        self._ahainfo = {}
         self.sharinfo = {}
         self.methinfo = {}
 
@@ -807,7 +832,7 @@ class Proxy(s_base.Base):
 
                         mesg = await link.rx()
                         if mesg is None:
-                            raise s_exc.LinkShutDown(mesg=mesg)
+                            raise s_exc.LinkShutDown(mesg='Remote peer disconnected')
 
                         if mesg[0] != 't2:yield':  # pragma: no cover
                             info = 'Telepath protocol violation:  unexpected message received'
@@ -844,6 +869,8 @@ class Proxy(s_base.Base):
         if self.sess is not None:
             return await self.taskv2(todo, name=name)
 
+        s_common.deprecated('Telepath task with no session', curv='2.166.0')
+
         task = Task()
 
         mesg = ('task:init', {
@@ -878,6 +905,7 @@ class Proxy(s_base.Base):
             raise s_exc.LinkShutDown(mesg=mesg)
 
         self.sess = self.synack[1].get('sess')
+        self._ahainfo = self.synack[1].get('ahainfo', {})
         self.sharinfo = self.synack[1].get('sharinfo', {})
         self.methinfo = self.sharinfo.get('meths', {})
 
@@ -954,6 +982,219 @@ class Proxy(s_base.Base):
         setattr(self, name, meth)
         return meth
 
+class ClientV2(s_base.Base):
+    '''
+    A telepath client which:
+        * connects to multiple services
+        * distributes API calls across them
+        * receives topology updates from AHA
+
+    NOTE: This must co-exist with Client until we eliminate uses that
+          attempt to call telepath APIs directly from the Client rather
+          than awaiting a proxy()
+    '''
+    async def __anit__(self, urlinfo, onlink=None):
+
+        await s_base.Base.__anit__(self)
+
+        self.aha = None
+
+        self.clients = {}
+        self.proxies = set()
+
+        self.poolname = None
+
+        self.onlink = onlink
+
+        self.bootdeque = collections.deque()
+
+        self.setBootUrls(urlinfo)
+
+        self.ready = asyncio.Event()
+        self.deque = collections.deque()
+
+        self.mesghands = {
+            'svc:add': self._onPoolSvcAdd,
+            'svc:del': self._onPoolSvcDel,
+        }
+
+        async def fini():
+            await self._shutDownPool()
+            # wake the sleepers
+            self.ready.set()
+
+        self.onfini(fini)
+
+        self.schedCoro(self._initBootProxy())
+
+    def setBootUrls(self, urlinfo):
+
+        if not isinstance(urlinfo, (list, tuple)):
+            urlinfo = (urlinfo,)
+
+        self.booturls = [chopurl(u) for u in urlinfo]
+
+        self.bootdeque.clear()
+        self.bootdeque.extend(self.booturls)
+
+    def getNextBootUrl(self):
+        if not self.bootdeque:
+            self.bootdeque.extend(self.booturls)
+        return self.bootdeque.popleft()
+
+    async def _initBootProxy(self):
+
+        lastlog = 0.0
+        while not self.isfini:
+
+            urlinfo = self.getNextBootUrl()
+
+            try:
+
+                if urlinfo.get('scheme') == 'aha':
+
+                    self.aha, svcinfo = await _getAhaSvc(urlinfo)
+
+                    # if the service is a pool, enter pool mode and fire
+                    # the topography sync task to manage pool members.
+                    services = svcinfo.get('services')
+                    if services is not None:
+                        # we are an AHA pool!
+                        if self.poolname is None:
+                            self.poolname = svcinfo.get('name')
+                            self.schedCoro(self._toposync())
+                        return
+
+                # regular telepath client behavior
+                proxy = await openinfo(urlinfo)
+
+                async def reconnect():
+                    if not self.isfini:
+                        self.schedCoro(self._initBootProxy())
+
+                proxy.onfini(reconnect)
+
+                await self._onPoolLink(proxy, urlinfo)
+                return
+
+            except Exception as e:
+
+                now = time.monotonic()
+                if now > lastlog + 60.0:  # don't logspam the disconnect message more than 1/min
+                    url = s_urlhelp.sanitizeUrl(zipurl(urlinfo))
+                    logger.exception(f'telepath clientv2 ({url}) encountered an error: {e}')
+                    lastlog = now
+
+                retrysleep = float(urlinfo.get('retrysleep', 0.2))
+                await self.waitfini(timeout=retrysleep)
+
+    async def waitready(self, timeout=None):
+        await s_common.wait_for(self.ready.wait(), timeout=timeout)
+
+    def size(self):
+        return len(self.proxies)
+
+    async def _onPoolSvcAdd(self, mesg):
+        svcname = mesg[1].get('name')
+
+        if (oldc := self.clients.pop(svcname, None)) is not None:
+            await oldc.fini()
+
+        urlinfo = {'scheme': 'aha', 'host': svcname, 'path': ''}
+        self.clients[svcname] = await ClientV2.anit(urlinfo, onlink=self._onPoolLink)
+        await self.fire('svc:add', **mesg[1])
+
+    async def _onPoolSvcDel(self, mesg):
+        svcname = mesg[1].get('name')
+        client = self.clients.pop(svcname, None)
+        if client is not None:
+            await client.fini()
+        self.deque.clear()
+        await self.fire('svc:del', **mesg[1])
+
+    async def _onPoolLink(self, proxy, urlinfo):
+
+        async def onfini():
+            if proxy in self.proxies:
+                self.proxies.remove(proxy)
+            if proxy in self.deque:
+                self.deque.remove(proxy)
+            if not len(self.proxies):
+                self.ready.clear()
+
+        proxy.onfini(onfini)
+        self.proxies.add(proxy)
+        self.ready.set()
+
+        if self.onlink is not None:
+            try:
+                await self.onlink(proxy, urlinfo)
+            except Exception as e:
+                logger.exception(f'onlink: {self.onlink}')
+
+    async def _shutDownPool(self):
+        # when we reconnect to our AHA service, we need to dump the current
+        # topology state and gather it again.
+        for client in list(self.clients.values()):
+            await client.fini()
+
+        for proxy in list(self.proxies):
+            await proxy.fini()
+
+        self.deque.clear()
+        self.ready.clear()
+        self.clients.clear()
+        self.proxies.clear()
+
+    async def _toposync(self):
+
+        async def reset():
+            self.ready.clear()
+            await self._shutDownPool()
+            await self.fire('pool:reset')
+
+        while not self.isfini:
+
+            try:
+                ahaproxy = await self.aha.proxy()
+
+                await reset()
+
+                async for mesg in ahaproxy.iterPoolTopo(self.poolname):
+                    hand = self.mesghands.get(mesg[0])
+                    if hand is None:  # pragma: no cover
+                        logger.warning(f'Unknown AHA pool topography message: {mesg}')
+                        continue
+
+                    await hand(mesg)
+
+            except Exception as e:
+                logger.warning(f'AHA pool topology task restarting: {e}')
+                await self.waitfini(timeout=1)
+
+    async def proxy(self, timeout=None):
+
+        async def getNextProxy():
+
+            while not self.isfini:
+
+                await self.ready.wait()
+
+                if self.isfini:  # pragma: no cover
+                    raise s_exc.IsFini()
+
+                if not self.deque:
+                    self.deque.extend(self.proxies)
+
+                if not self.deque:
+                    self.ready.clear()
+                    continue
+
+                return self.deque.popleft()
+
+        # use an inner function so we can wait overall...
+        return await s_common.wait_for(getNextProxy(), timeout)
+
 class Client(s_base.Base):
     '''
     A Telepath client object which reconnects and allows waiting for link up.
@@ -970,7 +1211,7 @@ class Client(s_base.Base):
             }
 
     '''
-    async def __anit__(self, url, opts=None, conf=None, onlink=None):
+    async def __anit__(self, urlinfo, opts=None, conf=None, onlink=None):
 
         await s_base.Base.__anit__(self)
 
@@ -980,8 +1221,9 @@ class Client(s_base.Base):
         if opts is None:
             opts = {}
 
-        self._t_url = url
-        self._t_urls = collections.deque()
+        self._t_urldeque = collections.deque()
+
+        self.setBootUrls(urlinfo)
 
         self._t_opts = opts
         self._t_conf = conf
@@ -1006,21 +1248,20 @@ class Client(s_base.Base):
 
         await self._fireLinkLoop()
 
-    def _initUrlDeque(self):
-        self._t_urls.clear()
-        if isinstance(self._t_url, str):
-            self._t_urls.append(self._t_url)
-            return
-        self._t_urls.extend(self._t_url)
+    def setBootUrls(self, urlinfo):
+
+        if not isinstance(urlinfo, (list, tuple)):
+            urlinfo = (urlinfo,)
+
+        self._t_urlinfo = [chopurl(u) for u in urlinfo]
+
+        self._t_urldeque.clear()
+        self._t_urldeque.extend(self._t_urlinfo)
 
     def _getNextUrl(self):
-        # TODO url list in deque
-        if not self._t_urls:
-            self._initUrlDeque()
-        return self._t_urls.popleft()
-
-    def _setNextUrl(self, url):
-        self._t_urls.appendleft(url)
+        if not self._t_urldeque:
+            self._t_urldeque.extend(self._t_urlinfo)
+        return self._t_urldeque.popleft()
 
     async def onlink(self, func):
         self._t_onlinks.append(func)
@@ -1033,6 +1274,7 @@ class Client(s_base.Base):
     async def _fireLinkLoop(self):
         self._t_proxy = None
         self._t_ready.clear()
+        await self.fire('tele:client:linkloop')
         self.schedCoro(self._teleLinkLoop())
 
     async def _teleLinkLoop(self):
@@ -1040,36 +1282,32 @@ class Client(s_base.Base):
 
         while not self.isfini:
 
-            url = self._getNextUrl()
+            urlinfo = self._getNextUrl()
 
             try:
-                await self._initTeleLink(url)
+                await self._initTeleLink(urlinfo)
                 self._t_ready.set()
                 return
 
-            except s_exc.TeleRedir as e:
-                self._setNextUrl(e.errinfo.get('url'))
-                continue
-
-            except asyncio.CancelledError:  # pragma: no cover  TODO:  remove once >= py 3.8 only
-                raise
-
             except Exception as e:
                 now = time.monotonic()
                 if now > lastlog + 60.0:  # don't logspam the disconnect message more than 1/min
-                    logger.exception(f'telepath client ({s_urlhelp.sanitizeUrl(url)}) encountered an error: {e}')
+                    url = s_urlhelp.sanitizeUrl(zipurl(urlinfo))
+                    logger.warning(f'telepath client ({url}) encountered an error: {e}', exc_info=e)
                     lastlog = now
                 await self.waitfini(timeout=self._t_conf.get('retrysleep', 0.2))
 
     async def proxy(self, timeout=10):
         await self.waitready(timeout=timeout)
         ret = self._t_proxy
-        if ret is None or ret.isfini is True:
+        if ret is None or ret.isfini is True: # pragma: no cover
             raise s_exc.IsFini(mesg='Telepath Client Proxy is not available.')
         return ret
 
-    async def _initTeleLink(self, url):
-        self._t_proxy = await openurl(url, **self._t_opts)
+    async def _initTeleLink(self, urlinfo):
+        info = urlinfo.copy()
+        info.update(self._t_opts)
+        self._t_proxy = await openinfo(info)
         self._t_methinfo = self._t_proxy.methinfo
         self._t_proxy._link_poolsize = self._t_conf.get('link_poolsize', 4)
 
@@ -1089,36 +1327,17 @@ class Client(s_base.Base):
                 # in case the callback fini()s the proxy
                 if self._t_proxy is None:
                     break
-            except asyncio.CancelledError:  # pragma: no cover
-                raise
             except Exception as e:
                 logger.exception(f'onlink: {onlink}')
 
     async def task(self, todo, name=None):
         # implement the main workhorse method for a proxy to allow Method
         # objects to use us as the proxy.
-        while not self.isfini:
-            try:
-                await self.waitready()
-
-                # there is a small race where the daemon may fini the proxy
-                # account for that here...
-                if self._t_proxy is None or self._t_proxy.isfini:
-                    self._t_ready.clear()
-                    continue
-
-                return await self._t_proxy.task(todo, name=name)
-
-            except s_exc.TeleRedir as e:
-                url = e.errinfo.get('url')
-                self._setNextUrl(url)
-                logger.warning(f'telepath task redirected: ({s_urlhelp.sanitizeUrl(url)})')
-                await self._t_proxy.fini()
-
-        raise s_exc.IsFini(mesg='Telepath Client isfini')
+        proxy = await self.proxy()
+        return await proxy.task(todo, name=name)
 
     async def waitready(self, timeout=10):
-        await asyncio.wait_for(self._t_ready.wait(), self._t_conf.get('timeout', timeout))
+        await s_common.wait_for(self._t_ready.wait(), self._t_conf.get('timeout', timeout))
 
     def __getattr__(self, name):
         if self._t_methinfo is None:
@@ -1177,80 +1396,6 @@ class Client(s_base.Base):
         '''
         return self._t_proxy._getClasses()
 
-async def disc_consul(info):
-    '''
-    Support for updating a URL info dictionary which came from a protocol+consul:// URL.
-
-    Notes:
-        This updates the info-dictionary in place, placing the ``host`` value into
-        an ``original_host`` key, and updating ``host`` and ``port``.
-
-        By default we pull the ``host`` value from the catalog ``Address`` value,
-        and the ``port`` from the ``ServicePort`` value.
-
-        The following HTTP parameters are supported:
-
-        - consul: This is the consul host (schema, fqdn and port) to connect to.
-        - consul_tag: If set, iterate through the catalog results until a result
-          is found which matches the tag value. This is a case sensitive match.
-        - consul_tag_address: If set, prefer the ``TaggedAddresses`` from the catalog.
-        - consul_service_tag_address: If set, prefer the associated value from the
-          ``ServiceTaggedAddresses`` field.
-        - consul_nosslverify: If set, disables SSL verification.
-
-    '''
-    info.setdefault('original_host', info.get('host'))
-    service = info.get('original_host')
-    host = info.get('consul')
-    tag = info.get('consul_tag')  # iterate through entries until a match for tag is present.
-    ctag_addr = info.get('consul_tag_address')  # Prefer a taggedAddress if set
-    csvc_tag_addr = info.get('consul_service_tag_address')  # Prefer a serviceTaggedAddress if set
-    gkwargs = {'raise_for_status': True}
-    if info.get('consul_nosslverify'):
-        gkwargs['ssl'] = False
-
-    if ctag_addr and csvc_tag_addr:
-        mesg = 'Cannot resolve consul values with both consul_tag_address and consul_service_tag_address'
-        raise s_exc.BadUrl(mesg=mesg, consul_tag_address=ctag_addr,
-                           consul_service_tag_address=csvc_tag_addr)
-
-    url = f'{host}/v1/catalog/service/{service}'
-    try:
-        async with aiohttp.ClientSession() as session:
-            async with session.get(url, **gkwargs) as resp:
-                if resp.status == 200:
-                    found = await resp.json()
-                    for entry in found:
-
-                        # If the consul_tag parameter is passed, we will iterate over
-                        # the Consul results until we find the service which matches
-                        # our requested tag. Otherwise, we will grab data from the
-                        # first record from the service catalog.
-                        if tag and tag not in entry.get('ServiceTags'):
-                            continue
-
-                        if csvc_tag_addr:
-                            # Use the ServiceTaggedAddresses
-                            info['host'] = entry['ServiceTaggedAddresses'][csvc_tag_addr]['address']
-                            info['port'] = entry['ServiceTaggedAddresses'][csvc_tag_addr]['port']
-                        elif ctag_addr:
-                            # Use the TaggedAddresses values.
-                            info['host'] = entry['TaggedAddresses'][ctag_addr]
-                            info['port'] = entry['ServicePort']
-                        else:
-                            # Use the generic service address/port
-                            info['host'] = entry['Address']
-                            info['port'] = entry['ServicePort']
-                        return
-
-    except asyncio.CancelledError:  # pragma: no cover  TODO:  remove once >= py 3.8 only
-        raise
-    except Exception as e:
-        raise s_exc.BadUrl(mesg=f'Unknown error while resolving service name [{service}] via consul [{str(e)}].',
-                           name=service, consul=host) from e
-    raise s_exc.BadUrl(mesg=f'Unable to resolve service information from consul for [{service}].',
-                       name=service, consul=host, tag=tag)
-
 def alias(name):
     '''
     Resolve a telepath alias via ~/.syn/aliases.yaml
@@ -1366,10 +1511,8 @@ async def openinfo(info):
 
     if '+' in scheme:
         scheme, disc = scheme.split('+', 1)
-        # Discovery protocols modify info dict inband?
-        if disc == 'consul':
-            await disc_consul(info)
-
+        if disc == 'consul':  # pragma: no cover
+            raise s_exc.FeatureNotSupported(mesg='Consul is no longer supported.')
         else:
             raise s_exc.BadUrl(mesg=f'Unknown discovery protocol [{disc}].',
                                disc=disc)
@@ -1412,7 +1555,7 @@ async def openinfo(info):
             path, name = path.split(':')
         link = await s_link.unixconnect(path)
 
-    else:
+    elif scheme in ('tcp', 'ssl'):
 
         path = info.get('path')
         name = info.get('name', path[1:])
@@ -1461,6 +1604,9 @@ async def openinfo(info):
 
         link = await s_link.connect(host, port, linkinfo=linkinfo)
 
+    else:
+        raise s_exc.BadUrl(mesg=f'Invalid URL scheme: {scheme}')
+
     prox = await Proxy.anit(link, name)
     prox.onfini(link)
 
