@@ -41,6 +41,34 @@ class DeprecatedModel(s_module.CoreModule):
 
 class DataModelTest(s_t_utils.SynTest):
 
+    async def test_datamodel_basics(self):
+        async with self.getTestCore() as core:
+            core.model.addType('woot:one', 'guid', {}, {
+                'display': {
+                    'columns': (
+                        {'type': 'newp', 'opts': {}},
+                    ),
+                },
+            })
+            with self.raises(s_exc.BadFormDef):
+                core.model.addForm('woot:one', {}, ())
+
+            core.model.addType('woot:two', 'guid', {}, {
+                'display': {
+                    'columns': (
+                        {'type': 'prop', 'opts': {'name': 'hehe'}},
+                    ),
+                },
+            })
+            with self.raises(s_exc.BadFormDef):
+                core.model.addForm('woot:two', {}, ())
+
+            with self.raises(s_exc.NoSuchForm):
+                core.model.reqForm('newp:newp')
+
+            with self.raises(s_exc.NoSuchProp):
+                core.model.reqForm('inet:asn').reqProp('newp')
+
     async def test_datamodel_formname(self):
         modl = s_datamodel.Model()
         mods = (
@@ -112,11 +140,27 @@ class DataModelTest(s_t_utils.SynTest):
         modl.addForm('foo:foo', {}, ())
         modl.addFormProp('foo:foo', 'bar', ('bar', {}), {})
 
+        with self.raises(s_exc.NoSuchForm):
+            modl.addFormProp('foo:newp', 'bar', ('bar', {}), {})
+
         with self.raises(s_exc.CantDelType):
             modl.delType('bar')
 
+        with self.raises(s_exc.CantDelForm):
+            modl.delForm('foo:foo')
+
+        modl.delFormProp('foo:foo', 'bar')
         modl.delForm('foo:foo')
 
+        modl.addIface('depr:iface', {'deprecated': True})
+
+        with self.getAsyncLoggerStream('synapse.datamodel') as dstream:
+            modl.addType('foo:bar', 'int', {}, {'interfaces': ('depr:iface',)})
+            modl.addForm('foo:bar', {}, ())
+
+        dstream.seek(0)
+        self.isin('Form foo:bar depends on deprecated interface depr:iface', dstream.read())
+
     async def test_datamodel_del_prop(self):
 
         modl = s_datamodel.Model()
@@ -168,6 +212,8 @@ class DataModelTest(s_t_utils.SynTest):
             refs = core.model.form('test:comp').getRefsOut()
             self.len(1, refs['prop'])
 
+            self.len(1, [prop for prop in core.model.getPropsByType('time') if prop.full == 'it:exec:url:time'])
+
     async def test_model_deprecation(self):
         # Note: Inverting these currently causes model loading to fail (20200831)
         mods = ['synapse.tests.utils.TestModule',
@@ -259,3 +305,32 @@ class DataModelTest(s_t_utils.SynTest):
             mesg = f'Comp forms with secondary properties that are not read-only ' \
                    f'are present in the model: {[n.ndef[1] for n in nodes]}'
             self.len(0, nodes, mesg)
+
+    async def test_datamodel_edges(self):
+
+        async with self.getTestCore() as core:
+
+            with self.raises(s_exc.NoSuchForm):
+                core.model.addEdge(('hehe', 'woot', 'newp'), {})
+
+            with self.raises(s_exc.NoSuchForm):
+                core.model.addEdge(('inet:ipv4', 'woot', 'newp'), {})
+
+            with self.raises(s_exc.BadArg):
+                core.model.addEdge(('inet:ipv4', 10, 'inet:ipv4'), {})
+
+            with self.raises(s_exc.BadArg):
+                core.model.addEdge(('meta:rule', 'matches', None), {})
+
+            model = await core.getModelDict()
+            self.isin(('meta:rule', 'matches', None), [e[0] for e in model['edges']])
+
+            model = (await core.getModelDefs())[0][1]
+            self.isin(('meta:rule', 'matches', None), [e[0] for e in model['edges']])
+
+            self.nn(core.model.edge(('meta:rule', 'matches', None)))
+
+            core.model.delEdge(('meta:rule', 'matches', None))
+            self.none(core.model.edge(('meta:rule', 'matches', None)))
+
+            core.model.delEdge(('meta:rule', 'matches', None))
