@@ -8,8 +8,7 @@ import synapse.exc as s_exc
 import synapse.common as s_common
 import synapse.tests.utils as s_t_utils
 
-import synapse.lib.hive as s_hive
-import synapse.lib.lmdbslab as s_lmdbslab
+import synapse.tools.backup as s_tools_backup
 
 import synapse.lib.agenda as s_agenda
 from synapse.lib.agenda import TimeUnit as s_tu
@@ -155,7 +154,7 @@ class AgendaTest(s_t_utils.SynTest):
         newts = ar.nexttime(now)
         self.eq(newts, datetime.datetime(year=2018, month=12, day=5, hour=7, minute=2, tzinfo=tz.utc).timestamp())
 
-    async def test_agenda(self):
+    async def test_agenda_base(self):
         MONO_DELT = 1543827303.0
         unixtime = datetime.datetime(year=2018, month=12, day=5, hour=7, minute=0, tzinfo=tz.utc).timestamp()
 
@@ -174,7 +173,6 @@ class AgendaTest(s_t_utils.SynTest):
                 await visi.setAdmin(True)
 
                 agenda = core.agenda
-                await agenda.start()  # make sure it doesn't blow up
 
                 self.eq([], agenda.list())
 
@@ -207,6 +205,12 @@ class AgendaTest(s_t_utils.SynTest):
                 await self.asyncraises(ValueError, agenda.add(cdef))
                 await self.asyncraises(s_exc.NoSuchIden, agenda.get('DOIT'))
 
+                # Require valid storm
+                cdef = {'creator': core.auth.rootuser.iden, 'iden': 'DOIT', 'storm': ' | | | ',
+                        'reqs': {s_agenda.TimeUnit.MINUTE: 1}}
+                await self.asyncraises(s_exc.BadSyntax, agenda.add(cdef))
+                await self.asyncraises(s_exc.NoSuchIden, agenda.get('DOIT'))
+
                 # Schedule a one-shot to run immediately
                 doit = s_common.guid()
                 cdef = {'creator': core.auth.rootuser.iden, 'iden': doit,
@@ -323,7 +327,18 @@ class AgendaTest(s_t_utils.SynTest):
 
                 appt = await agenda.get(guid)
                 self.eq(appt.isrunning, True)
-                await appt.task.kill()
+                self.eq(core.view.iden, appt.task.info.get('view'))
+
+                self.true(await core._killCronTask(guid))
+
+                events = [
+                    {'event': 'cron:stop', 'info': {'iden': appt.iden}},
+                ]
+
+                task = core.schedCoro(s_t_utils.waitForBehold(core, events))
+                await asyncio.wait_for(task, timeout=5)
+
+                self.false(await core._killCronTask(guid))
 
                 appt = await agenda.get(guid)
                 self.eq(appt.isrunning, False)
@@ -338,35 +353,33 @@ class AgendaTest(s_t_utils.SynTest):
                 adef = await agenda.add(cdef)
                 guid = adef.get('iden')
 
+                strt = await core.nexsroot.index()
                 # bypass the API because it would actually syntax check
                 unixtime += 60
                 self.eq((11, 'boom'), await asyncio.wait_for(core.callStorm('return($lib.queue.gen(visi).pop(wait=$lib.true))'), timeout=5))
+                await core.nexsroot.waitOffs(strt + 5)
 
                 appt = await agenda.get(guid)
                 self.eq(appt.isrunning, False)
                 self.eq(appt.lastresult, "raised exception StormRaise: errname='OmgWtfBbq' mesg='boom'")
 
                 # Test setting the global enable/disable flag
-                # reset
                 await agenda.delete(guid)
                 self.len(0, agenda.apptheap)
 
-                agenda._schedtask.cancel()
-                agenda._schedtask = agenda.schedCoro(agenda._scheduleLoop())
-
                 # schedule a query to run every Wednesday and Friday at 10:15am
                 cdef = {'creator': visi.iden, 'iden': s_common.guid(), 'storm': '$lib.queue.gen(visi).put(bar)',
+                        'pool': True,
                         'reqs': {s_tu.HOUR: 10, s_tu.MINUTE: 15},
                         'incunit': s_agenda.TimeUnit.DAYOFWEEK,
                         'incvals': (2, 4)}
                 adef = await agenda.add(cdef)
+
+                self.true(adef['pool'])
                 guid = adef.get('iden')
 
                 self.len(1, agenda.apptheap)
 
-                # disable crons and advance time
-                agenda.enabled = False
-
                 unixtime = datetime.datetime(year=2019, month=2, day=6, hour=10, minute=16, tzinfo=tz.utc).timestamp()
                 nexttime = datetime.datetime(year=2019, month=2, day=8, hour=10, minute=15, tzinfo=tz.utc).timestamp()
 
@@ -377,11 +390,19 @@ class AgendaTest(s_t_utils.SynTest):
                 self.true(appt.enabled)
                 self.eq(0, appt.startcount)
 
-                # enable crons and advance time
-                agenda.enabled = True
+                # Ensure structured logging captures the cron iden value
+                core.stormlog = True
+                with self.getStructuredAsyncLoggerStream('synapse.storm') as stream:
+                    unixtime = datetime.datetime(year=2019, month=2, day=13, hour=10, minute=16,
+                                                 tzinfo=tz.utc).timestamp()
+                    self.eq((12, 'bar'), await asyncio.wait_for(core.callStorm('return($lib.queue.gen(visi).pop(wait=$lib.true))'), timeout=5))
+                core.stormlog = False
 
-                unixtime = datetime.datetime(year=2019, month=2, day=13, hour=10, minute=16, tzinfo=tz.utc).timestamp()
-                self.eq((12, 'bar'), await asyncio.wait_for(core.callStorm('return($lib.queue.gen(visi).pop(wait=$lib.true))'), timeout=5))
+                msgs = stream.jsonlines()
+                msgs = [m for m in msgs if m['text'] == '$lib.queue.gen(visi).put(bar)']
+                self.gt(len(msgs), 0)
+                for m in msgs:
+                    self.eq(m.get('cron'), appt.iden)
 
                 self.eq(1, appt.startcount)
 
@@ -391,6 +412,8 @@ class AgendaTest(s_t_utils.SynTest):
                 await agenda.add(cdef)
 
                 # Lock user and advance time
+                strt = await core.nexsroot.index()
+
                 await visi.setLocked(True)
 
                 with self.getLoggerStream('synapse.lib.agenda', 'locked') as stream:
@@ -400,162 +423,91 @@ class AgendaTest(s_t_utils.SynTest):
                     while not stream.wait(0.1):
                         await asyncio.sleep(0)
 
+                    await core.nexsroot.waitOffs(strt + 4)
+
                     self.eq(2, appt.startcount)
 
+                # Can't use an existing authgate iden
+                viewiden = core.getView().iden
+                cdef = {'creator': core.auth.rootuser.iden,
+                        'storm': '[test:str=bar]',
+                        'reqs': {'hour': 10},
+                        'incunit': 'dayofweek',
+                        'incvals': (2, 4),
+                        'iden': viewiden}
+                await self.asyncraises(s_exc.DupIden, core.addCronJob(cdef))
+                await core.delCronJob(viewiden)
+
+                self.nn(core.getAuthGate(viewiden))
+
     async def test_agenda_persistence(self):
         ''' Test we can make/change/delete appointments and they are persisted to storage '''
-        with self.getTestDir() as fdir:
 
-            core = mock.Mock()
+        with self.getTestDir() as dirn:
 
-            def raiseOnBadStorm(q):
-                ''' Just enough storm parsing for this test '''
-                # TODO: Async this and use AsyncMock when Python 3.8+ only
-                f = asyncio.Future()
-                if (q[0] == '[') != (q[-1] == ']'):
-                    f.set_exception(s_exc.BadSyntax(mesg='mismatched braces'))
-                else:
-                    f.set_result('all good')
-                return f
+            async with self.getTestCore(dirn=dirn) as core:
 
-            core.getStormQuery = raiseOnBadStorm
+                cdef = {'creator': core.auth.rootuser.iden,
+                        'storm': '[test:str=bar]',
+                        'reqs': {'hour': 10, 'minute': 15},
+                        'incunit': 'dayofweek',
+                        'incvals': (2, 4)}
+                adef = await core.addCronJob(cdef)
+                guid1 = adef.get('iden')
 
-            async with await s_lmdbslab.Slab.anit(fdir, map_size=100000) as slab:
-                db = slab.initdb('hive')
-                hive = await s_hive.SlabHive.anit(slab, db=db)
-                core.hive = hive
-
-                async with await s_agenda.Agenda.anit(core) as agenda:
-                    await agenda.start()
-                    # Schedule a query to run every Wednesday and Friday at 10:15am
-                    cdef = {'creator': 'visi', 'iden': 'IDEN1', 'storm': '[test:str=bar]',
-                            'reqs': {s_tu.HOUR: 10, s_tu.MINUTE: 15},
-                            'incunit': s_agenda.TimeUnit.DAYOFWEEK,
-                            'incvals': (2, 4)}
-                    adef = await agenda.add(cdef)
-                    guid1 = adef.get('iden')
-
-                    # every 6th of the month at 7am and 8am (the 6th is a Thursday)
-                    cdef = {'creator': 'visi', 'iden': 'IDEN2', 'storm': '[test:str=baz]',
-                            'reqs': {s_tu.HOUR: (7, 8), s_tu.MINUTE: 0, s_tu.DAYOFMONTH: 6},
-                            'incunit': s_agenda.TimeUnit.MONTH,
-                            'incvals': 1}
-                    adef = await agenda.add(cdef)
-
-                    guid2 = adef.get('iden')
-                    appt = agenda.appts[guid2]
-                    appt.lasterrs.append('error happened')
-                    await agenda._storeAppt(appt)
-
-                    # Add an appt with an invalid query
-                    cdef = {'creator': 'visi', 'iden': 'BAD1', 'storm': '[test:str=',
-                            'reqs': {s_tu.HOUR: (7, 8)},
-                            'incunit': s_agenda.TimeUnit.MONTH,
-                            'incvals': 1}
-                    adef = await agenda.add(cdef)
-                    badguid1 = adef.get('iden')
-
-                    # Add an appt with a bad version in storage
-                    cdef = {'creator': 'visi', 'iden': 'BAD2', 'storm': '[test:str=foo]',
-                            'reqs': {s_tu.HOUR: (7, 8)},
-                            'incunit': s_agenda.TimeUnit.MONTH,
-                            'incvals': 1}
-                    adef = await agenda.add(cdef)
-                    badguid2 = adef.get('iden')
-
-                    adef['ver'] = 1337
-                    full = agenda._hivenode.full + (badguid2,)
-                    await agenda.core.hive.set(full, adef)
-
-                    xmas = {s_tu.DAYOFMONTH: 25, s_tu.MONTH: 12, s_tu.YEAR: 2099}
-                    lasthanu = {s_tu.DAYOFMONTH: 10, s_tu.MONTH: 12, s_tu.YEAR: 2099}
-
-                    await agenda.delete(guid1)
-
-                    # And one-shots for Christmas and last day of Hanukkah of 2018
-                    cdef = {'creator': 'visi', 'iden': 'IDEN3', 'storm': '#happyholidays',
-                            'reqs': (xmas, lasthanu)}
-
-                    adef = await agenda.add(cdef)
-                    guid3 = adef.get('iden')
-
-                    await agenda.mod(guid3, '#bahhumbug')
-
-                async with await s_agenda.Agenda.anit(core) as agenda:
-                    await agenda.start()
-                    agenda.enabled = True
-
-                    appts = agenda.list()
-                    self.len(3, appts)
-
-                    last_appt = [appt for (iden, appt) in appts if iden == guid3][0]
-                    self.eq(last_appt.query, '#bahhumbug')
-
-                    bad_appt = [appt for (iden, appt) in appts if iden == badguid1][0]
-                    self.false(bad_appt.enabled)
-
-                    self.len(0, [appt for (iden, appt) in appts if iden == badguid2])
-
-    async def test_cron_perms(self):
-
-        async with self.getTestCore() as core:
-
-            visi = await core.auth.addUser('visi')
-            newb = await core.auth.addUser('newb')
-            async with core.getLocalProxy(user='visi') as proxy:
-
-                cdef = {'storm': 'inet:ipv4', 'reqs': {'hour': 2}}
-                with self.raises(s_exc.AuthDeny):
-                    await proxy.addCronJob(cdef)
-
-                await visi.addRule((True, ('cron', 'add')))
-                cron0 = await proxy.addCronJob(cdef)
-                cron0_iden = cron0.get('iden')
-
-                cdef = {'storm': 'inet:ipv6', 'reqs': {'hour': 2}}
-                cron1 = await proxy.addCronJob(cdef)
-                cron1_iden = cron1.get('iden')
-
-                await proxy.delCronJob(cron0_iden)
-
-                cdef = {'storm': '[test:str=foo]', 'reqs': {'now': True},
+                # every 6th of the month at 7am and 8am (the 6th is a Thursday)
+                cdef = {'creator': core.auth.rootuser.iden,
+                        'storm': '[test:str=baz]',
+                        'reqs': {'hour': (7, 8), 'minute': 0, 'dayofmonth': 6},
                         'incunit': 'month',
                         'incvals': 1}
-                await self.asyncraises(s_exc.BadConfValu, proxy.addCronJob(cdef))
+                adef = await core.addCronJob(cdef)
 
-            async with core.getLocalProxy(user='newb') as proxy:
+                guid2 = adef.get('iden')
+                appt = core.agenda.appts[guid2]
+                appt.lasterrs.append('error happened')
+                await appt.save()
 
-                with self.raises(s_exc.AuthDeny):
-                    await proxy.delCronJob(cron1_iden)
+                # Add an appt with an invalid query
+                cdef = {'creator': core.auth.rootuser.iden,
+                        'storm': '[test:str=',
+                        'reqs': {'hour': (7, 8)},
+                        'incunit': 'month',
+                        'incvals': 1}
 
-                self.eq(await proxy.listCronJobs(), ())
-                await newb.addRule((True, ('cron', 'get')))
-                self.len(1, await proxy.listCronJobs())
+                with self.raises(s_exc.BadSyntax):
+                    await core.addCronJob(cdef)
 
-                with self.raises(s_exc.AuthDeny):
-                    await proxy.disableCronJob(cron1_iden)
+                xmas = {'dayofmonth': 25, 'month': 12, 'year': 2099}
+                lasthanu = {'dayofmonth': 10, 'month': 12, 'year': 2099}
 
-                await newb.addRule((True, ('cron', 'set')))
-                self.none(await proxy.disableCronJob(cron1_iden))
+                await core.delCronJob(guid1)
 
-                await newb.addRule((True, ('cron', 'del')))
-                await proxy.delCronJob(cron1_iden)
+                # And one-shots for Christmas and last day of Hanukkah of 2018
+                cdef = {'creator': core.auth.rootuser.iden,
+                        'storm': '#happyholidays',
+                        'reqs': (xmas, lasthanu)}
 
-    async def test_agenda_stop(self):
+                adef = await core.addCronJob(cdef)
+                guid3 = adef.get('iden')
 
-        async with self.getTestCore() as core:
-            await core.callStorm('$lib.queue.add(foo)')
-            await core.callStorm('cron.add --minute +1 { $lib.queue.get(foo).put((99)) $lib.time.sleep(10) }')
-            appts = core.agenda.list()
-            await core.agenda._execute(appts[0][1])
-            self.eq((0, 99), await core.callStorm('return($lib.queue.get(foo).get())'))
+                await core.updateCronJob(guid3, '#bahhumbug')
 
-            appts[0][1].creator = 'fakeuser'
-            await core.agenda._execute(appts[0][1])
-            self.eq(appts[0][1].lastresult, 'Failed due to unknown user')
+                # Add a job with invalid storage version
+                cdef = (await core.listCronJobs())[0]
+                guid = s_common.guid()
+                cdef['ver'] = 0
+                cdef['iden'] = guid
+                core.agenda.apptdefs.set(guid, cdef)
 
-            await core.agenda.stop()
-            self.false(core.agenda.enabled)
+            async with self.getTestCore(dirn=dirn) as core:
+
+                appts = await core.listCronJobs()
+
+                self.len(2, appts)
+
+                last_appt = [appt for appt in appts if appt.get('iden') == guid3][0]
+                self.eq(last_appt.get('query'), '#bahhumbug')
 
     async def test_agenda_custom_view(self):
 
@@ -563,7 +515,7 @@ class AgendaTest(s_t_utils.SynTest):
             # no existing view
             await core.callStorm('$lib.queue.add(testq)')
             defview = core.getView()
-            fakeiden = hashlib.md5(defview.iden.encode('utf-8')).hexdigest()
+            fakeiden = hashlib.md5(defview.iden.encode('utf-8'), usedforsecurity=False).hexdigest()
             opts = {'vars': {'fakeiden': fakeiden}}
             with self.raises(s_exc.NoSuchView):
                 await prox.callStorm('cron.add --view $fakeiden --minute +2 { $lib.queue.get(testq).put((43)) }', opts=opts)
@@ -624,6 +576,7 @@ class AgendaTest(s_t_utils.SynTest):
             jobs = await core.callStorm('return($lib.cron.list())')
             self.len(1, jobs)
             self.eq(defview.iden, jobs[0]['view'])
+            self.nn(jobs[0].get('created'))
 
             core.agenda._addTickOff(60)
             retn = await core.callStorm('return($lib.queue.get(testq).get())', opts=asfail)
@@ -722,3 +675,269 @@ class AgendaTest(s_t_utils.SynTest):
             self.len(1, nodes)
             nodes = await core.nodes('test:int=97', opts={'view': newview})
             self.len(0, nodes)
+
+    async def test_agenda_edit(self):
+
+        async with self.getTestCore() as core:
+
+            lowuser = await core.addUser('lowuser')
+            lowuser = lowuser.get('iden')
+
+            msgs = await core.stormlist('cron.add --hourly 32 { $lib.print(woot) }')
+            self.stormHasNoWarnErr(msgs)
+
+            cdef = await core.callStorm('for $cron in $lib.cron.list() { return($cron) }')
+            self.false(cdef['pool'])
+            self.eq(cdef['creator'], core.auth.rootuser.iden)
+
+            cdef = await core.callStorm('for $cron in $lib.cron.list() { $cron.set(pool, (true)) return($cron) }')
+            self.true(cdef['pool'])
+
+            opts = {'vars': {'lowuser': lowuser}}
+            cdef = await core.callStorm('for $cron in $lib.cron.list() { return($cron.set(creator, $lowuser)) }',
+                                        opts=opts)
+            self.eq(cdef['creator'], lowuser)
+
+            opts = {'user': lowuser, 'vars': {'iden': cdef.get('iden'), 'lowuser': lowuser}}
+            q = '$cron = $lib.cron.get($iden) return ( $cron.set(creator, $lowuser) )'
+            msgs = await core.stormlist(q, opts=opts)
+            # XXX FIXME - This is an odd message since the new creator does not implicitly have
+            # access to the cronjob that is running as them.
+            self.stormIsInErr('Provided iden does not match any valid authorized cron job.', msgs)
+
+            await core.addUserRule(lowuser, (True, ('cron', 'get')))
+            opts = {'user': lowuser, 'vars': {'iden': cdef.get('iden'), 'lowuser': lowuser}}
+            q = '$cron = $lib.cron.get($iden) return ( $cron.set(creator, $lowuser) )'
+            msgs = await core.stormlist(q, opts=opts)
+            self.stormIsInErr('must have permission cron.set.creator', msgs)
+
+    async def test_agenda_fatal_run(self):
+
+        # Create a scenario where an agenda appointment is "correct"
+        # but encounters a fatal error when attempting to be created.
+        # This error is then logged, and corrected.
+
+        async with self.getTestCore() as core:
+
+            udef = await core.addUser('user')
+            user = udef.get('iden')
+
+            fork = await core.callStorm('$fork = $lib.view.get().fork().iden return ( $fork )')
+
+            q = '$lib.log.info(`I am a cron job run by {$lib.user.name()} in {$lib.view.get().iden}`)'
+            msgs = await core.stormlist('cron.add --minute +1 $q', opts={'vars': {'q': q}, 'view': fork})
+            self.stormHasNoWarnErr(msgs)
+
+            cdef = await core.callStorm('for $cron in $lib.cron.list() { return($cron.set(creator, $user)) }',
+                                        opts={'vars': {'user': user}})
+            self.eq(cdef['creator'], user)
+
+            # Force the cron to run.
+
+            with self.getAsyncLoggerStream('synapse.lib.agenda', 'Agenda error running appointment ') as stream:
+                core.agenda._addTickOff(55)
+                self.true(await stream.wait(timeout=12))
+
+            await core.addUserRule(user, (True, ('storm',)))
+            await core.addUserRule(user, (True, ('view', 'read')), gateiden=fork)
+
+            with self.getAsyncLoggerStream('synapse.storm.log', 'I am a cron job') as stream:
+                core.agenda._addTickOff(60)
+                self.true(await stream.wait(timeout=12))
+
+    async def test_agenda_mirror_realtime(self):
+        with self.getTestDir() as dirn:
+
+            path00 = s_common.gendir(dirn, 'core00')
+            path01 = s_common.gendir(dirn, 'core01')
+
+            async with self.getTestCore(dirn=path00) as core00:
+                await core00.nodes('[ inet:ipv4=1.2.3.4 ]')
+
+            s_tools_backup.backup(path00, path01)
+
+            async with self.getTestCore(dirn=path00) as core00:
+                self.false(core00.conf.get('mirror'))
+
+                await core00.callStorm('cron.add --minute +1 { $lib.time.sleep(5) }')
+
+                url = core00.getLocalUrl()
+
+                core01conf = {'mirror': url}
+
+                async with self.getTestCore(dirn=path01, conf=core01conf) as core01:
+                    core00.agenda._addTickOff(55)
+
+                    mesgs = []
+                    async for mesg in core00.behold():
+                        mesgs.append(mesg)
+                        core00.agenda._addTickOff(30)
+                        if len(mesgs) == 2:
+                            break
+
+                    await core01.sync()
+
+                    cron00 = await core00.callStorm('return($lib.cron.list())')
+                    cron01 = await core01.callStorm('return($lib.cron.list())')
+
+                    self.eq(cron00, cron01)
+
+                    start = mesgs[0]
+                    stop = mesgs[1]
+
+                    self.eq(start['event'], 'cron:start')
+                    self.eq(stop['event'], 'cron:stop')
+
+                    self.eq(start['info']['iden'], cron00[0]['iden'])
+                    self.eq(stop['info']['iden'], cron00[0]['iden'])
+
+                async with self.getTestCore(dirn=path01, conf=core01conf) as core01:
+                    nodes = await core00.nodes('syn:cron')
+                    self.len(1, nodes)
+
+                    msgs = await core00.stormlist('syn:cron [ :name=foo :doc=bar ]')
+                    self.stormHasNoWarnErr(msgs)
+                    await core01.sync()
+
+                    nodes = await core01.nodes('syn:cron')
+                    self.len(1, nodes)
+                    self.nn(nodes[0].props.get('.created'))
+                    self.eq(nodes[0].props.get('name'), 'foo')
+                    self.eq(nodes[0].props.get('doc'), 'bar')
+
+                    appt = await core01.agenda.get(nodes[0].ndef[1])
+                    self.eq(appt.name, 'foo')
+                    self.eq(appt.doc, 'bar')
+
+            with self.getLoggerStream('synapse.lib.agenda') as stream:
+                async with self.getTestCore(dirn=path00) as core00:
+                    appts = core00.agenda.list()
+                    self.len(1, appts)
+                    appt = appts[0][1]
+
+                    edits = {
+                        'invalid': 'newp',
+                    }
+                    await core00.addCronEdits(appt.iden, edits)
+
+                stream.seek(0)
+                data = stream.read()
+                self.isin("_Appt.edits() Invalid attribute received: invalid = 'newp'", data)
+
+    async def test_cron_kill(self):
+        async with self.getTestCore() as core:
+
+            data = []
+            evt = asyncio.Event()
+
+            async def task():
+                async for mesg in core.behold():
+                    data.append(mesg)
+                    if mesg.get('event') == 'cron:stop':
+                        evt.set()
+
+            core.schedCoro(task())
+
+            q = '$q=$lib.queue.gen(test) for $i in $lib.range(60) { $lib.time.sleep(0.1) $q.put($i) }'
+            guid = s_common.guid()
+            cdef = {
+                'creator': core.auth.rootuser.iden, 'iden': guid,
+                'storm': q,
+                'reqs': {'now': True}
+            }
+            await core.addCronJob(cdef)
+
+            q = '$q=$lib.queue.gen(test) for $valu in $q.get((0), wait=(true)) { return ($valu) }'
+            valu = await core.callStorm(q)
+            self.eq(valu, 0)
+
+            opts = {'vars': {'iden': guid}}
+            get_cron = 'return($lib.cron.get($iden).pack())'
+            cdef = await core.callStorm(get_cron, opts=opts)
+            self.true(cdef.get('isrunning'))
+
+            self.true(await core.callStorm('return($lib.cron.get($iden).kill())', opts=opts))
+
+            self.true(await asyncio.wait_for(evt.wait(), timeout=12))
+
+            cdef = await core.callStorm(get_cron, opts=opts)
+            self.false(cdef.get('isrunning'))
+
+    async def test_cron_kill_pool(self):
+
+        async with self.getTestAha() as aha:
+
+            import synapse.cortex as s_cortex
+            import synapse.lib.base as s_base
+
+            async with await s_base.Base.anit() as base:
+
+                with self.getTestDir() as dirn:
+
+                    dirn00 = s_common.genpath(dirn, 'cell00')
+                    dirn01 = s_common.genpath(dirn, 'cell01')
+
+                    core00 = await base.enter_context(self.addSvcToAha(aha, '00.core', s_cortex.Cortex, dirn=dirn00))
+                    provinfo = {'mirror': '00.core'}
+                    core01 = await base.enter_context(self.addSvcToAha(aha, '01.core', s_cortex.Cortex, dirn=dirn01, provinfo=provinfo))
+
+                    self.len(1, await core00.nodes('[inet:asn=0]'))
+                    await core01.sync()
+                    self.len(1, await core01.nodes('inet:asn=0'))
+
+                    msgs = await core00.stormlist('aha.pool.add pool00...')
+                    self.stormHasNoWarnErr(msgs)
+                    self.stormIsInPrint('Created AHA service pool: pool00.synapse', msgs)
+
+                    msgs = await core00.stormlist('aha.pool.svc.add pool00... 01.core...')
+                    self.stormHasNoWarnErr(msgs)
+                    self.stormIsInPrint('AHA service (01.core...) added to service pool (pool00.synapse)', msgs)
+
+                    msgs = await core00.stormlist('cortex.storm.pool.set --connection-timeout 1 --sync-timeout 1 aha://pool00...')
+                    self.stormHasNoWarnErr(msgs)
+                    self.stormIsInPrint('Storm pool configuration set.', msgs)
+
+                    await core00.stormpool.waitready(timeout=12)
+
+                    data = []
+                    evt = asyncio.Event()
+
+                    async def task():
+                        async for mesg in core00.behold():
+                            data.append(mesg)
+                            if mesg.get('event') == 'cron:stop':
+                                evt.set()
+
+                    core00.schedCoro(task())
+
+                    q = '$q=$lib.queue.gen(test) for $i in $lib.range(60) { $lib.time.sleep(0.1) $q.put($i) }'
+                    guid = s_common.guid()
+                    cdef = {
+                        'creator': core00.auth.rootuser.iden, 'iden': guid,
+                        'storm': q,
+                        'reqs': {'NOW': True},
+                        'pool': True,
+                    }
+                    await core00.addCronJob(cdef)
+
+                    q = '$q=$lib.queue.gen(test) for $valu in $q.get((0), wait=(true)) { return ($valu) }'
+                    valu = await core00.callStorm(q, opts={'mirror': False})
+                    self.eq(valu, 0)
+
+                    opts = {'vars': {'iden': guid}, 'mirror': False}
+                    get_cron = 'return($lib.cron.get($iden).pack())'
+                    cdef = await core00.callStorm(get_cron, opts=opts)
+                    self.true(cdef.get('isrunning'))
+
+                    self.true(await core00.callStorm('return($lib.cron.get($iden).kill())', opts=opts))
+
+                    self.true(await asyncio.wait_for(evt.wait(), timeout=12))
+
+                    cdef00 = await core00.callStorm(get_cron, opts=opts)
+                    self.false(cdef00.get('isrunning'))
+
+                    cdef01 = await core01.callStorm(get_cron, opts=opts)
+                    self.false(cdef01.get('isrunning'))
+                    self.eq(cdef01.get('lastresult'), 'cancelled')
+                    self.gt(cdef00['laststarttime'], 0)
+                    self.eq(cdef00['laststarttime'], cdef01['laststarttime'])
