@@ -9,6 +9,7 @@ import synapse.exc as s_exc
 
 import synapse.lib.base as s_base
 import synapse.lib.coro as s_coro
+import synapse.lib.scope as s_scope
 
 import synapse.tests.utils as s_t_utils
 
@@ -59,6 +60,21 @@ class BaseTest(s_t_utils.SynTest):
         event = await base.fire('woot', x=3, y=5, ret=[])
         self.eq(event[1]['ret'], 8)
 
+        base00 = await s_base.Base.anit()
+        base01 = await s_base.Base.anit()
+        data = {}
+        async def onfini():
+            data['woot'] = True
+
+        await base00.fini()
+        base00.onfini(onfini)
+        await asyncio.sleep(0)
+        self.true(data.get('woot'))
+
+        base00.onfini(base01)
+        await asyncio.sleep(0)
+        self.true(base01.isfini)
+
     async def test_base_anit(self):
 
         afoo = await Hehe.anit(20)
@@ -200,6 +216,7 @@ class BaseTest(s_t_utils.SynTest):
         self.eq(data['count'], 1)
 
     async def test_base_waiter(self):
+
         base0 = await s_base.Base.anit()
 
         wait0 = base0.waiter(3, 'foo:bar')
@@ -223,6 +240,10 @@ class BaseTest(s_t_utils.SynTest):
         evts = await wait2.wait(1)
         self.len(2, evts)
 
+        with self.raises(s_exc.TimeOut):
+            async with base0.waiter(1, 'newp', 'nuuh', timeout=0.01):
+                pass
+
     async def test_baseref(self):
 
         bref = await s_base.BaseRef.anit()
@@ -357,7 +378,7 @@ class BaseTest(s_t_utils.SynTest):
         proc = ctx.Process(target=block_processing, args=(evt1,))
         proc.start()
 
-        self.true(evt1.wait(timeout=10))
+        self.true(evt1.wait(timeout=30))
         os.kill(proc.pid, signal.SIGTERM)
         proc.join(timeout=10)
         self.eq(proc.exitcode, 137)
@@ -375,7 +396,7 @@ class BaseTest(s_t_utils.SynTest):
         proc = ctx.Process(target=block_processing, args=(evt1,))
         proc.start()
 
-        self.true(evt1.wait(timeout=10))
+        self.true(evt1.wait(timeout=30))
         os.kill(proc.pid, signal.SIGINT)
 
         proc.join(timeout=10)
@@ -487,3 +508,56 @@ class BaseTest(s_t_utils.SynTest):
             self.true(await s_coro.event_wait(event, 3.0))
 
         await self.asyncraises(asyncio.CancelledError, task)
+
+    async def test_lib_base_scope(self):
+
+        # Simple test with a single scope stack.
+
+        data = {}
+
+        async def func1(key):
+            valu = s_scope.get(key, defval=None)
+            data[key] = valu
+            # Get / set in our scope works fine
+            s_scope.set('hehe', 'newp')
+            self.eq(s_scope.get('hehe'), 'newp')
+
+        async with await s_base.Base.anit() as base:
+            vals = {'foo': 'bar'}
+            with s_scope.enter(vals=vals):  # Ensure we have a known value in the scope
+                task = base.schedCoro(func1('foo'))
+                await task
+
+        # The scoped data was accessible in the task
+        self.eq(data.get('foo'), 'bar')
+
+        # The scope data set in the task is not present outside of it.
+        self.none(s_scope.get('hehe'))
+
+        # Nested scopes across multiple tasks
+        data = {}
+
+        async def func2(key):
+            valu = s_scope.get(key, defval=None)
+            data[key] = valu
+            # Get / set in our scope works fine
+            s_scope.set('hehe', 'newp')
+            self.eq(s_scope.get('hehe'), 'newp')
+
+        async def func3(bobj, key, valu):
+            s_scope.set(key, valu)
+            task = bobj.schedCoro(func2(key))
+            await task
+            self.none(s_scope.get('hehe'))
+
+        async with await s_base.Base.anit() as base:
+            vals = {'foo': 'bar'}
+            with s_scope.enter(vals=vals):  # Ensure we have a known value in the scope
+                task = base.schedCoro(func3(base, 'beep', 'boop'))
+                await task
+
+        # The scoped data was accessible in the task
+        self.eq(data.get('beep'), 'boop')
+
+        # The scope data set in the task is not present outside of it.
+        self.none(s_scope.get('hehe'))
