@@ -1,4 +1,4 @@
-
+import regex
 
 import synapse.common as s_common
 import synapse.lib.crypto.coin as s_coin
@@ -10,3 +10,6 @@ class CryptoCoinTest(s_t_utils.SynTest):
         # Test bad input on eip55
         v = s_common.guid() + 'X'
         self.none(s_coin.ether_eip55(v))
+
+        valu = regex.search(r'(?P<valu>[a-z]+)', 'foobar')
+        self.eq(s_coin.eth_check(valu), (None, {}))
