@@ -14,7 +14,6 @@ import synapse.lib.lmdbslab as s_lmdbslab
 import synapse.lib.thisplat as s_thisplat
 
 import synapse.tests.utils as s_t_utils
-from synapse.tests.utils import alist
 
 def getFileMapCount(filename):
     filename = str(filename)
@@ -57,9 +56,11 @@ class LmdbSlabTest(s_t_utils.SynTest):
 
                 testgenr = slab.scanKeys(db=testdb)
                 dupsgenr = slab.scanKeys(db=dupsdb)
+                nodupsgenr = slab.scanKeys(db=dupsdb, nodup=True)
 
                 testlist = [next(testgenr)]
                 dupslist = [next(dupsgenr)]
+                nodupslist = [next(nodupsgenr)]
 
                 slab.put(b'derp', b'derp', db=editdb)
 
@@ -68,16 +69,25 @@ class LmdbSlabTest(s_t_utils.SynTest):
 
                 testlist.extend(testgenr)
                 dupslist.extend(dupsgenr)
+                nodupslist.extend(nodupsgenr)
 
                 self.eq(testlist, (b'hehe', b'hoho'))
                 self.eq(dupslist, (b'hehe', b'hehe', b'hoho'))
+                self.eq(nodupslist, (b'hehe', b'hoho'))
+
+                self.eq([b'hehe', b'hehe', b'hoho'], list(slab.scanKeysByPref(b'h', db=dupsdb)))
+                self.eq([b'hehe', b'hoho'], list(slab.scanKeysByPref(b'h', db=dupsdb, nodup=True)))
+
+                self.eq(3, await slab.countByPref(b'h', db=dupsdb))
 
                 # now lets delete the key we're on
                 testgenr = slab.scanKeys(db=testdb)
                 dupsgenr = slab.scanKeys(db=testdb)
+                nodupsgenr = slab.scanKeys(db=testdb, nodup=True)
 
                 testlist = [next(testgenr)]
                 dupslist = [next(dupsgenr)]
+                nodupslist = [next(nodupsgenr)]
 
                 slab.delete(b'hehe', db=testdb)
                 for lkey, lval in slab.scanByDups(b'hehe', db=dupsdb):
@@ -87,9 +97,11 @@ class LmdbSlabTest(s_t_utils.SynTest):
 
                 testlist.extend(testgenr)
                 dupslist.extend(dupsgenr)
+                nodupslist.extend(nodupsgenr)
 
                 self.eq(testlist, (b'hehe', b'hoho'))
                 self.eq(dupslist, (b'hehe', b'hoho'))
+                self.eq(nodupslist, (b'hehe', b'hoho'))
 
                 # by pref
                 self.eq([b'hoho'], list(slab.scanKeysByPref(b'h', db=dupsdb)))
@@ -281,8 +293,9 @@ class LmdbSlabTest(s_t_utils.SynTest):
             self.eq(items, ((b'\x00\x02', b'haha'), (b'\x00\x01', b'hehe')))
 
             # Copy a database inside the same slab
-            self.raises(s_exc.DataAlreadyExists, slab.copydb, foo, slab, 'bar')
-            self.eq(3, slab.copydb(foo, slab, 'foo2'))
+            with self.raises(s_exc.DataAlreadyExists):
+                await slab.copydb(foo, slab, 'bar')
+            self.eq(3, await slab.copydb(foo, slab, 'foo2'))
 
             # Increase the size of the new source DB to trigger a resize on the next copydb
             foo2 = slab.initdb('foo2')
@@ -298,7 +311,7 @@ class LmdbSlabTest(s_t_utils.SynTest):
             async with await s_lmdbslab.Slab.anit(path2, map_size=512 * 1024) as slab2:
                 with patch('synapse.lib.lmdbslab.PROGRESS_PERIOD', 2):
 
-                    self.eq(4, slab.copydb(foo2, slab2, destdbname='foo2', progresscb=progfunc))
+                    self.eq(4, await slab.copydb(foo2, slab2, destdbname='foo2', progresscb=progfunc))
                     self.gt(vardict.get('prog', 0), 0)
 
             # Test slab.drop and slab.dbexists
@@ -337,7 +350,7 @@ class LmdbSlabTest(s_t_utils.SynTest):
         with self.getTestDir() as dirn, patch('synapse.lib.lmdbslab.Slab.WARN_COMMIT_TIME_MS', 1), \
                 patch('synapse.common.now', self.simplenow):
             path = os.path.join(dirn, 'test.lmdb')
-            with self.getAsyncLoggerStream('synapse.lib.lmdbslab', 'Commit with') as stream:
+            with self.getStructuredAsyncLoggerStream('synapse.lib.lmdbslab', 'Commit with') as stream:
                 async with await s_lmdbslab.Slab.anit(path, map_size=100000) as slab:
                     foo = slab.initdb('foo', dupsort=True)
                     byts = b'\x00' * 256
@@ -345,6 +358,19 @@ class LmdbSlabTest(s_t_utils.SynTest):
                         slab.put(b'\xff\xff\xff\xff' + s_common.guid(i).encode('utf8'), byts, db=foo)
                 self.true(await stream.wait(timeout=1))
 
+            msgs = stream.jsonlines()
+            self.gt(len(msgs), 0)
+            self.nn(msgs[0].get('delta'))
+            self.nn(msgs[0].get('path'))
+            self.nn(msgs[0].get('xactopslen'))
+            self.sorteq([
+                'vm.swappiness',
+                'vm.dirty_expire_centisecs',
+                'vm.dirty_writeback_centisecs',
+                'vm.dirty_background_ratio',
+                'vm.dirty_ratio',
+            ], msgs[0].get('sysctls', {}).keys())
+
     async def test_lmdbslab_max_replay(self):
         with self.getTestDir() as dirn:
             path = os.path.join(dirn, 'test.lmdb')
@@ -649,6 +675,42 @@ class LmdbSlabTest(s_t_utils.SynTest):
                 self.eq((b'1', b'1'), next(it))
                 self.raises(StopIteration, next, it)
 
+    async def test_lmdbslab_scanback(self):
+
+        with self.getTestDir() as dirn:
+
+            path = os.path.join(dirn, 'test.lmdb')
+
+            async with await s_lmdbslab.Slab.anit(path, map_size=100000, growsize=10000) as slab:
+
+                foodup = slab.initdb('foodup', dupsort=True)
+                foonodup = slab.initdb('foonodup', dupsort=False)
+
+                for db in (foodup, foonodup):
+                    slab.put(b'\x01', b'foo', db=db)
+                    slab.put(b'\x01\x01', b'bar', db=db)
+                    slab.put(b'\x01\x03', b'baz', db=db)
+                    slab.put(b'\x02', b'faz', db=db)
+
+                items = list(slab.scanByPrefBack(b'\x01', db=foonodup))
+                self.eq(items, (
+                    (b'\x01\x03', b'baz'),
+                    (b'\x01\x01', b'bar'),
+                    (b'\x01', b'foo')
+                ))
+
+                self.eq((), list(slab.scanByPrefBack(b'\x00', db=foonodup)))
+
+                slab.put(b'\x01\x03', b'waz', db=foodup)
+
+                items = list(slab.scanByPrefBack(b'\x01', db=foodup))
+                self.eq(items, (
+                    (b'\x01\x03', b'waz'),
+                    (b'\x01\x03', b'baz'),
+                    (b'\x01\x01', b'bar'),
+                    (b'\x01', b'foo')
+                ))
+
     async def test_lmdbslab_count_empty(self):
 
         with self.getTestDir() as dirn:
@@ -744,7 +806,8 @@ class LmdbSlabTest(s_t_utils.SynTest):
                 self.raises(s_exc.IsReadOnly, newdb.replace, b'1234', b'3456')
                 self.raises(s_exc.IsReadOnly, newdb.pop, b'1234')
                 self.raises(s_exc.IsReadOnly, newdb.delete, b'1234')
-                self.raises(s_exc.IsReadOnly, newdb.putmulti, ((b'1234', b'3456'),))
+                with self.raises(s_exc.IsReadOnly):
+                    await newdb.putmulti((b'1234', b'3456'))
 
                 # While we have the DB open in readonly, have another process write a bunch of data to cause the
                 # map size to be increased
@@ -772,7 +835,7 @@ class LmdbSlabTest(s_t_utils.SynTest):
                 # A putmulti across a grow
                 before_mapsize = slab.mapsize
                 kvpairs = [(x, x) for x in data]
-                retn = slab.putmulti(kvpairs)
+                retn = await slab.putmulti(kvpairs)
                 self.eq(retn, (1000, 1000))
 
                 after_mapsize1 = slab.mapsize
@@ -780,7 +843,7 @@ class LmdbSlabTest(s_t_utils.SynTest):
 
                 # A putmulti across a grow with a generator passed in
                 kvpairs = ((b' ' + x, x) for x in data)
-                retn = slab.putmulti(kvpairs)
+                retn = await slab.putmulti(kvpairs)
                 self.eq(retn, (1000, 1000))
                 after_mapsize2 = slab.mapsize
                 self.gt(after_mapsize2, after_mapsize1)
@@ -831,8 +894,11 @@ class LmdbSlabTest(s_t_utils.SynTest):
             async with await s_lmdbslab.Slab.anit(path) as slab:
                 guidstor = s_lmdbslab.GuidStor(slab, 'guids')
 
+                self.false(guidstor.has('aaaa'))
                 info0 = guidstor.gen('aaaa')
+                self.false(guidstor.has('aaaa'))
                 info0.set('hehe', 20)
+                self.true(guidstor.has('aaaa'))
                 self.eq(20, info0.get('hehe'))
                 self.none(info0.get('haha'))
 
@@ -845,6 +911,14 @@ class LmdbSlabTest(s_t_utils.SynTest):
                 self.none(info0.pop('woot'))
                 self.true(info0.pop('woot', s_common.novalu) is s_common.novalu)
 
+                iden1 = s_common.guid()
+                info1 = guidstor.gen(iden1)
+                self.false(guidstor.has(iden1))
+                info1.set('weeee', 'wowie')
+                self.true(guidstor.has(iden1))
+                await guidstor.del_(iden1)
+                self.false(guidstor.has(iden1))
+
                 # Sad path case
                 self.raises(s_exc.NotMsgpackSafe, info0.set, 'newp', {1, 2, 3})
 
@@ -855,6 +929,8 @@ class LmdbSlabTest(s_t_utils.SynTest):
                 self.none(info1.pop('woot'))
                 self.len(1, info1.items())
                 self.eq((('hehe', 20), ), info1.items())
+                self.true(guidstor.has('aaaa'))
+                self.false(guidstor.has(iden1))
 
     async def test_slab_initdb_grow(self):
         self.thisHostMust(platform='linux')
@@ -915,11 +991,9 @@ class LmdbSlabTest(s_t_utils.SynTest):
             async with self.getTestCore() as core:
                 before_mapsize = core.view.layers[0].layrslab.mapsize
                 for i in range(numbatches):
-                    async with await core.snap() as snap:
-                        ips = ((('test:int', i * 1000000 + x), {'props': {'loc': 'us'}}) for x in range(batchsize))
-                        await alist(snap.addNodes(ips))
-                        # Wait for the syncloop to run
-                        await asyncio.sleep(1.1)
+                    q = 'for $i in $vals { [test:int=$i :loc=us] } | spin'
+                    await core.nodes(q, opts={'vars': {'vals': list(range(batchsize))}})
+                    await asyncio.sleep(1.1)
 
                 # Verify that it hit
                 self.gt(core.view.layers[0].layrslab.mapsize, before_mapsize)
@@ -936,7 +1010,7 @@ class LmdbSlabTest(s_t_utils.SynTest):
             async with await s_lmdbslab.Slab.anit(path, map_size=32000, growsize=5000) as slab:
                 slab.initdb('foo')
                 kvpairs = [(x, x) for x in data]
-                slab.putmulti(kvpairs)
+                await slab.putmulti(kvpairs)
                 slab.forcecommit()
                 before_mapsize = slab.mapsize
                 slab.dropdb('foo')
@@ -954,7 +1028,7 @@ class LmdbSlabTest(s_t_utils.SynTest):
             async with await s_lmdbslab.Slab.anit(path, map_size=32000, growsize=5000) as slab:
                 slab.initdb('foo')
                 kvpairs = [(x, x) for x in data]
-                slab.putmulti(kvpairs)
+                await slab.putmulti(kvpairs)
                 slab.forcecommit()
 
         asyncio.run(workloop())
@@ -972,7 +1046,7 @@ class LmdbSlabTest(s_t_utils.SynTest):
 
                 proc = mpctx.Process(target=self.make_slab, args=(path,))
                 proc.start()
-                proc.join(10)
+                proc.join(20)
                 self.nn(proc.exitcode)
                 slab.initdb('foo')
                 self.true(True)
@@ -1430,6 +1504,96 @@ class LmdbSlabTest(s_t_utils.SynTest):
                 self.len(6, items)
                 self.eq(items, ['bot', 'timewarp', 'foo', 'bar', 'baz', 'eot'])
 
+    async def test_lmdbslab_count(self):
+
+        with self.getTestDir() as dirn:
+
+            path = os.path.join(dirn, 'test.lmdb')
+            async with await s_lmdbslab.Slab.anit(path) as slab:
+
+                testdb = slab.initdb('test')
+                dupsdb = slab.initdb('dups', dupsort=True)
+
+                self.eq(0, slab.count(b'newp', db=testdb))
+                self.eq(0, slab.count(b'newp', db=dupsdb))
+
+                slab.put(b'foo', b'bar', db=testdb)
+
+                self.eq(1, slab.count(b'foo', db=testdb))
+
+                slab.put(b'foo', b'bar', db=dupsdb)
+                slab.put(b'foo', b'baz', db=dupsdb)
+                slab.put(b'foo', b'faz', db=dupsdb)
+
+                self.eq(3, slab.count(b'foo', db=dupsdb))
+
+    async def test_safekeyval(self):
+        with self.getTestDir() as dirn:
+
+            path = os.path.join(dirn, 'test.lmdb')
+
+            async with await s_lmdbslab.Slab.anit(path) as slab:
+
+                safekv = slab.getSafeKeyVal('test')
+
+                subkv1 = safekv.getSubKeyVal('pref1')
+                subkv2 = subkv1.getSubKeyVal('pref2')
+
+                self.eq(subkv2.prefix, 'pref1pref2')
+
+                subkv2.set('foo', 'bar')
+                subkv2.set('baz', 'faz')
+
+                self.eq(list(subkv2.keys()), ['baz', 'foo'])
+                self.eq(list(safekv.keys()), ['pref1pref2baz', 'pref1pref2foo'])
+                self.eq(list(safekv.values()), ['faz', 'bar'])
+
+                await safekv.truncate()
+                self.len(0, safekv.items())
+
+                subkv2.set('wow', 'guys')
+
+                newp = 'a' * 512
+
+                with self.raises(s_exc.BadArg):
+                    safekv.getSubKeyVal(newp)
+
+                with self.raises(s_exc.BadArg):
+                    safekv.get(newp)
+
+                with self.raises(s_exc.BadArg):
+                    safekv.set(newp, 'newp')
+
+                with self.raises(s_exc.BadArg):
+                    safekv.pop(newp)
+
+                with self.raises(s_exc.BadArg):
+                    safekv.delete(newp)
+
+                with self.raises(s_exc.BadArg):
+                    list(safekv.keys(pref=newp))
+
+                with self.raises(s_exc.BadArg):
+                    list(safekv.items(pref=newp))
+
+                with self.raises(s_exc.BadArg):
+                    list(safekv.values(pref=newp))
+
+                with self.raises(s_exc.BadArg):
+                    await safekv.truncate(pref=newp)
+
+                with self.raises(s_exc.BadArg):
+                    safekv.getSubKeyVal('')
+
+                with self.raises(s_exc.BadArg):
+                    slab.getSafeKeyVal('newp', create=False)
+
+            async with await s_lmdbslab.Slab.anit(path) as slab:
+                safekv = slab.getSafeKeyVal('test', create=False)
+                subkv1 = safekv.getSubKeyVal('pref1')
+                subkv2 = subkv1.getSubKeyVal('pref2')
+                self.eq(list(subkv2.keys()), ['wow'])
+
 
 class LmdbSlabMemLockTest(s_t_utils.SynTest):
 
