@@ -21,7 +21,7 @@ HI
 .. storm-pre:: [ inet:ipv6=0 ]
 .. storm-pkg:: synapse/tests/files/stormpkg/testpkg.yaml
 .. storm:: --hide-props testpkgcmd foo
-.. storm:: --hide-query $lib.print(secret)
+.. storm:: --hide-query $lib.print(secret) $lib.print($lib.globals.get(testpkg))
 .. storm:: --hide-query file:bytes
 .. storm-svc:: synapse.tests.files.rstorm.testsvc.Testsvc test {"secret": "jupiter"}
 .. storm:: testsvc.test
@@ -44,6 +44,7 @@ HI
 ::
 
     secret
+    testpkg-done
 
 ::
 
@@ -53,6 +54,7 @@ HI
 
     > testsvc.test
     jupiter
+    testsvc-done
 
 '''
 
@@ -104,7 +106,7 @@ multi_rst_in_http_opts = '''
 .. storm-vcr-opts:: {"record_mode": "none"}
 .. storm-vcr-callback:: synapse.tests.test_lib_rstorm.vcrcallbacktst
 .. storm-mock-http:: /this/path/does/not/exist.yaml
-.. storm:: $resp = $lib.inet.http.get("http://example.com") if $resp.body { $lib.print('unexpected results') } else { $lib.print($lib.str.concat('this', ' test', ' passed')) }
+.. storm:: $resp = $lib.inet.http.get("http://example.com") if $resp.body { $lib.print('unexpected results') } else { $lib.print('this test passed') }
 '''
 
 storm_http_sadpath_callback = '''
@@ -204,7 +206,7 @@ multiline00_input = '''
 A multiline secondary property.
 .. storm-cortex:: default
 .. storm-opts:: {"vars": {"name": "Node\\nWith a\\nweird name"}}
-.. storm:: [graph:node=(n1,) :name=$name]
+.. storm:: [meta:note=(n1,) :text=$name]
 Bye!
 '''
 
@@ -212,9 +214,9 @@ multiline00_output = '''
 A multiline secondary property.
 ::
 
-    > [graph:node=(n1,) :name=$name]
-    graph:node=85d3511b97098d7fd9e07be21f6390de
-            :name = Node
+    > [meta:note=(n1,) :text=$name]
+    meta:note=85d3511b97098d7fd9e07be21f6390de
+            :text = Node
                     With a
                     weird name
 
@@ -259,6 +261,17 @@ HI
 .. storm-cortex:: path.to.NewpCell
 '''
 
+pkg_onload_timeout = '''
+.. storm-cortex:: default
+.. storm-pre:: $lib.globals.set(onload_sleep, 2)
+.. storm-pkg:: synapse/tests/files/stormpkg/testpkg.yaml
+'''
+
+svc_onload_timeout = '''
+.. storm-cortex:: default
+.. storm-pre:: $lib.globals.set(onload_sleep, 2)
+.. storm-svc:: synapse.tests.files.rstorm.testsvc.Testsvc test {"secret": "jupiter"}
+'''
 
 async def get_rst_text(rstfile):
     async with await s_rstorm.StormRst.anit(rstfile) as rstorm:
@@ -339,7 +352,7 @@ class RStormLibTest(s_test.SynTest):
             self.isin('inet:ipv4=5.6.7.8', text)  # one mock at a time
             self.isin('it:dev:str=notjson', text)  # one mock at a time
 
-            # multi reqest in 1 rstorm command
+            # multi request in 1 rstorm command
             path = s_common.genpath(dirn, 'http_multi.rst')
             with s_common.genfile(path) as fd:
                 fd.write(multi_rst_in_http.encode())
@@ -496,6 +509,29 @@ class RStormLibTest(s_test.SynTest):
             with self.raises(s_exc.NoSuchCtor):
                 await get_rst_text(path)
 
+            # onload failures
+
+            try:
+                oldv = s_rstorm.ONLOAD_TIMEOUT
+                s_rstorm.ONLOAD_TIMEOUT = 0.1
+
+                path = s_common.genpath(dirn, 'pkg_onload_timeout.rst')
+                with s_common.genfile(path) as fd:
+                    fd.write(pkg_onload_timeout.encode())
+                with self.raises(s_exc.SynErr) as ectx:
+                    await get_rst_text(path)
+                self.eq('Package onload failed to run for testpkg', ectx.exception.errinfo['mesg'])
+
+                path = s_common.genpath(dirn, 'svc_onload_timeout.rst')
+                with s_common.genfile(path) as fd:
+                    fd.write(svc_onload_timeout.encode())
+                with self.raises(s_exc.SynErr) as ectx:
+                    await get_rst_text(path)
+                self.eq('Package onload failed to run for service test', ectx.exception.errinfo['mesg'])
+
+            finally:
+                s_rstorm.ONLOAD_TIMEOUT = oldv
+
     async def test_rstorm_cli(self):
 
         with self.getTestDir() as dirn:
