@@ -1,3 +1,5 @@
+from unittest import mock
+
 import synapse.tests.utils as s_test
 
 
@@ -83,8 +85,7 @@ class StormScrapeTest(s_test.SynTest):
                         $lib.print('{f}={v} {i}', f=$form, v=$valu, i=$info)
                     }'''
 
-        conf = {'provenance:en': False}
-        async with self.getTestCore(conf=conf) as core:
+        async with self.getTestCore() as core:
             self.none(core.modsbyiface.get('scrape'))
 
             mods = await core.getStormIfaces('scrape')
@@ -127,7 +128,7 @@ class StormScrapeTest(s_test.SynTest):
                             ('ps:name', 'mallory'),
                             ('inet:url', 'https://giggles.com/mallory.html')))
 
-        conf = {'provenance:en': False, 'storm:interface:scrape': False, }
+        conf = {'storm:interface:scrape': False, }
         async with self.getTestCore(conf=conf) as core:
 
             await core.loadStormPkg(pkgdef)
