@@ -55,6 +55,7 @@ class StormLibStixTest(s_test.SynTest):
     async def test_stormlib_libstix(self, conf=None):
 
         async with self.getTestCore(conf=conf) as core:
+            visi = await core.auth.addUser('visi')
             opts = {'vars': {
                 'ind': '6ba7d8500964902bf2e03126ed0f6cb1',
                 'news': '840b9b003a765020705ea8d203a7659c',
@@ -73,7 +74,6 @@ class StormLibStixTest(s_test.SynTest):
                 'prodsoft': 'a120b6d58329662bc5cabec03ef72ffa',
 
                 'sha256': '00001c4644c1d607a6ff6fbf883873d88fe8770714893263e2dfb27f291a6c4e',
-                'sha256': '00001c4644c1d607a6ff6fbf883873d88fe8770714893263e2dfb27f291a6c4e',
             }}
 
             self.len(22, await core.nodes('''[
@@ -167,6 +167,13 @@ class StormLibStixTest(s_test.SynTest):
 
             self.bundeq(self.getTestBundle('custom0.json'), bund)
 
+            self.eq(2, await core.callStorm('''
+                $bund = $lib.stix.export.bundle()
+                [ inet:asn=42 inet:asn=31337 ]
+                $bund.add($node)
+                fini{ return($bund.size()) }
+            '''))
+
             resp = await core.callStorm('return($lib.stix.validate($bundle))', {'vars': {'bundle': bund}})
             self.true(resp.get('ok'))
             result = resp.get('result')
@@ -180,9 +187,38 @@ class StormLibStixTest(s_test.SynTest):
             self.false(resp.get('ok'))
             self.isin('Error validating bundle', resp.get('mesg'))
 
+            self.len(14, bund.get('objects'))
+            self.isin(s_stix.SYN_STIX_EXTENSION_ID, json.dumps(bund))
             nodes = await core.nodes('yield $lib.stix.lift($bundle)', {'vars': {'bundle': bund}})
             self.len(10, nodes)
 
+            # Bundle made without the synapse extension cannot be lifted
+            bund_noext = await core.callStorm('''
+            init {
+                $config = $lib.stix.export.config()
+                $config.synapse_extension=$lib.false  // Disable synapse extension
+                $config.forms."syn:tag".stix.malware.rels.append(
+                    (communicates-with, url, ${-> file:bytes -> it:exec:url:exe -> inet:url})
+                )
+                $config.forms."syn:tag".stix.malware.props.name = ${return(redtree)}
+                $bundle = $lib.stix.export.bundle(config=$config)
+            }
+
+            [ syn:tag=cno.mal.redtree ]
+
+            {[( file:bytes=$file +#cno.mal.redtree )]}
+            {[( it:exec:url=$execurl :exe=$file :url=http://vertex.link/ )]}
+
+            $bundle.add($node, stixtype=malware)
+
+            fini { return($bundle) }
+            ''', opts=opts)
+            self.len(13, bund_noext.get('objects'))
+            self.reqValidStix(bund_noext)
+            nodes = await core.nodes('yield $lib.stix.lift($bundle)', {'vars': {'bundle': bund_noext}})
+            self.len(0, nodes)
+            self.notin(s_stix.SYN_STIX_EXTENSION_ID, json.dumps(bund_noext))
+
             # test some sad paths...
             self.none(await core.callStorm('return($lib.stix.export.bundle().add($lib.true))'))
             self.none(await core.callStorm('[ ou:conference=* ] return($lib.stix.export.bundle().add($node))'))
@@ -193,9 +229,9 @@ class StormLibStixTest(s_test.SynTest):
                 opts = {'vars': {'config': config}}
                 await core.callStorm('$lib.stix.export.bundle(config=$config)', opts=opts)
 
-            with self.raises(s_exc.BadConfValu):
+            with self.raises(s_exc.AuthDeny):
                 config = {'maxsize': 10000000}
-                opts = {'vars': {'config': config}}
+                opts = {'user': visi.iden, 'vars': {'config': config}}
                 await core.callStorm('$lib.stix.export.bundle(config=$config)', opts=opts)
 
             with self.raises(s_exc.NoSuchForm):
@@ -298,6 +334,16 @@ class StormLibStixTest(s_test.SynTest):
             viewiden = await core.callStorm('return($lib.view.get().fork().iden)')
             stix = s_common.yamlload(self.getTestFilePath('stix_import', 'oasis-example-00.json'))
             msgs = await core.stormlist('yield $lib.stix.import.ingest($stix)', opts={'view': viewiden, 'vars': {'stix': stix}})
+            # self.stormHasNoWarnErr(msgs)
+            self.len(1, await core.nodes('ps:contact:name="adversary bravo"', opts={'view': viewiden}))
+            self.len(1, await core.nodes('it:prod:soft', opts={'view': viewiden}))
+
+            # Pass in a heavy dict object
+            viewiden = await core.callStorm('return($lib.view.get().fork().iden)')
+            stix = s_common.yamlload(self.getTestFilePath('stix_import', 'oasis-example-00.json'))
+            q = '''init { $data = ({"id": $stix.id, "type": $stix.type, "objects": $stix.objects}) }
+            yield $lib.stix.import.ingest($data)'''
+            msgs = await core.stormlist(q, opts={'view': viewiden, 'vars': {'stix': stix}})
             self.len(1, await core.nodes('ps:contact:name="adversary bravo"', opts={'view': viewiden}))
             self.len(1, await core.nodes('it:prod:soft', opts={'view': viewiden}))
 
@@ -364,3 +410,137 @@ class StormLibStixTest(s_test.SynTest):
             nodes = [mesg[1] for mesg in msgs if mesg[0] == 'node']
             self.len(1, [n for n in nodes if n[0][0] == 'it:cmd'])
             self.stormIsInWarn("STIX bundle ingest has no relationship definition for: ('threat-actor', 'gronks', 'threat-actor')", msgs)
+
+            msgs = await core.stormlist('yield $lib.stix.import.ingest(({}), newp)')
+            self.stormIsInErr('config must be a dictionary', msgs)
+
+            msgs = await core.stormlist('yield $lib.stix.import.ingest(({}), ({"relationships": 5}))')
+            self.stormIsInErr('Error processing relationships', msgs)
+
+            msgs = await core.stormlist('yield $lib.stix.import.ingest(({}), ({"bundle": 3}))')
+            self.stormIsInErr('bundle value must be a dictionary', msgs)
+
+            msgs = await core.stormlist('yield $lib.stix.import.ingest(({}), ({"bundle": {"storm": 3}}))')
+            self.stormIsInErr('storm query must be a string', msgs)
+
+            msgs = await core.stormlist('yield $lib.stix.import.ingest(({"objects": 3}), ({}))')
+            self.stormIsInErr('data.objects must be array', msgs)
+
+    async def test_stix_export_custom(self):
+
+        async with self.getTestCore() as core:
+
+            bund = await core.callStorm('''
+                init {
+                    $config = $lib.stix.export.config()
+
+                    // register a custom object type so we pass validation
+                    // (dictionary contents are reserved for future use )
+
+                    $config.custom.objects."vtx-mitigation" = ({})
+
+                    $config.forms."risk:mitigation" = ({
+                        "default": "vtx-mitigation",
+                        "stix": {
+                            "vtx-mitigation": {
+                                "props": {
+                                    "desc": "return($desc)",
+                                    "name": "{+:name return(:name)} return($node.repr())",
+                                    "created": "return($lib.stix.export.timestamp(.created))",
+                                    "modified": "return($lib.stix.export.timestamp(.created))",
+                                },
+                            },
+                        },
+                    })
+
+                    $bundle = $lib.stix.export.bundle(config=$config)
+                }
+
+                [ risk:mitigation=c4f6dc09f1e1e6b7e7b05c9ce4186ce8 :name="patch stuff and do things" ]
+                $desc = "scopevar"
+                $bundle.add($node)
+
+                fini { return($bundle) }
+            ''')
+
+            self.eq('vtx-mitigation--2df2a437-e372-468b-b989-d01753603659', bund['objects'][1]['id'])
+            self.eq('scopevar', bund['objects'][1]['desc'])
+            self.eq('patch stuff and do things', bund['objects'][1]['name'])
+            self.nn(bund['objects'][1]['created'])
+            self.nn(bund['objects'][1]['modified'])
+
+            with self.raises(s_exc.BadConfValu):
+                await core.callStorm('''
+                    init {
+                        $config = $lib.stix.export.config()
+                        $config.custom.objects.NEWP = ({})
+                        $bundle = $lib.stix.export.bundle(config=$config)
+                    }
+                    fini { return($bundle) }
+                ''')
+
+    async def test_stix_export_pivots(self):
+
+        async with self.getTestCore() as core:
+            await core.nodes('[ inet:dns:a=(vertex.link, 1.2.3.4) ]')
+
+            with self.raises(s_exc.BadConfValu):
+                await core.callStorm('''
+                    $config = $lib.stix.export.config()
+                    $config.forms."inet:fqdn".stix."domain-name".pivots = ([
+                        {"storm": 10}
+                    ])
+                    $bundle = $lib.stix.export.bundle(config=$config)
+                ''')
+
+            with self.raises(s_exc.BadConfValu):
+                await core.callStorm('''
+                    $config = $lib.stix.export.config()
+                    $config.forms."inet:fqdn".stix."domain-name".pivots = ([
+                        {"storm": "woot", "stixtype": "newp"}
+                    ])
+                    $bundle = $lib.stix.export.bundle(config=$config)
+                ''')
+
+            bund = await core.callStorm('''
+                init {
+                    $config = $lib.stix.export.config()
+                    $config.forms."inet:fqdn".stix."domain-name".pivots = ([
+                        {"storm": "-> inet:dns:a -> inet:ipv4", "stixtype": "ipv4-addr"}
+                    ])
+                    $bundle = $lib.stix.export.bundle(config=$config)
+                }
+
+                inet:fqdn=vertex.link
+                $bundle.add($node)
+
+                fini { return($bundle) }
+            ''')
+            stixids = [obj['id'] for obj in bund['objects']]
+            self.isin('ipv4-addr--cbc65d5e-3732-55b3-9b9b-e06155c186db', stixids)
+
+    async def test_stix_revs(self):
+
+        async with self.getTestCore() as core:
+            await core.nodes('[risk:mitigation=* :name=bar +(addresses)> {[ ou:technique=* :name=foo ]} ]')
+
+            with self.raises(s_exc.BadConfValu):
+                bund = await core.callStorm('''
+                    $config = $lib.stix.export.config()
+                    $config.forms."ou:technique".stix."attack-pattern".revs = (["a"])
+                    $bundle = $lib.stix.export.bundle(config=$config)
+                    ou:technique
+                    $bundle.add($node, "attack-pattern")
+                    fini { return($bundle.pack()) }
+                ''')
+
+            bund = await core.callStorm('''
+                $bundle = $lib.stix.export.bundle()
+                ou:technique
+                $bundle.add($node, "attack-pattern")
+                fini { return($bundle.pack()) }
+            ''')
+            rels = [sobj for sobj in bund['objects'] if sobj.get('relationship_type') == 'mitigates']
+            self.len(1, rels)
+            self.true(rels[0]['target_ref'].startswith('attack-pattern--'))
+            self.true(rels[0]['source_ref'].startswith('course-of-action--'))
