@@ -11,18 +11,21 @@ from synapse.tests.utils import alist
 
 class ViewTest(s_t_utils.SynTest):
 
-    async def test_view_nomerge(self):
+    async def test_view_protected(self):
         async with self.getTestCore() as core:
             view = await core.callStorm('return($lib.view.get().fork().iden)')
             opts = {'view': view}
 
             await core.nodes('[ ou:org=* ]', opts=opts)
-            await core.nodes('$lib.view.get().set(nomerge, $lib.true)', opts=opts)
+            await core.nodes('$lib.view.get().set(protected, $lib.true)', opts=opts)
 
             with self.raises(s_exc.CantMergeView):
                 await core.nodes('$lib.view.get().merge()', opts=opts)
 
-            await core.nodes('$lib.view.get().set(nomerge, $lib.false)', opts=opts)
+            with self.raises(s_exc.CantDelView):
+                await core.nodes('$lib.view.del($lib.view.get().iden)', opts=opts)
+
+            await core.nodes('$lib.view.get().set(protected, $lib.false)', opts=opts)
             await core.nodes('$lib.view.get().merge()', opts=opts)
 
             self.len(1, await core.nodes('ou:org'))
@@ -31,6 +34,50 @@ class ViewTest(s_t_utils.SynTest):
             with self.raises(s_exc.BadOptValu):
                 await core.view.setViewInfo('hehe', 10)
 
+        async with self.getTestCore() as core:
+            # Delete this block when nomerge is removed
+            view = await core.callStorm('return($lib.view.get().fork().iden)')
+            opts = {'view': view}
+
+            # Setting/getting nomerge should be redirected to protected
+            getnomerge = 'return($lib.view.get().get(nomerge))'
+            setnomerge = '$lib.view.get().set(nomerge, $valu)'
+
+            getprotected = 'return($lib.view.get().get(protected))'
+            setprotected = '$lib.view.get().set(protected, $valu)'
+
+            nomerge = await core.callStorm(getnomerge, opts=opts)
+            protected = await core.callStorm(getprotected, opts=opts)
+            self.false(nomerge)
+            self.false(protected)
+
+            opts['vars'] = {'valu': True}
+            await core.callStorm(setnomerge, opts=opts)
+
+            nomerge = await core.callStorm(getnomerge, opts=opts)
+            protected = await core.callStorm(getprotected, opts=opts)
+            self.true(nomerge)
+            self.true(protected)
+
+            opts['vars'] = {'valu': False}
+            await core.callStorm(setprotected, opts=opts)
+
+            nomerge = await core.callStorm(getnomerge, opts=opts)
+            protected = await core.callStorm(getprotected, opts=opts)
+            self.false(nomerge)
+            self.false(protected)
+
+    async def test_view_nomerge_migration(self):
+        async with self.getRegrCore('cortex-defaults-v2') as core:
+            view = core.getView('0df16dd693c74109da0d58ab87ba768a')
+            self.none(view.info.get('nomerge'))
+            self.true(view.info.get('protected'))
+
+            with self.raises(s_exc.CantMergeView):
+                await core.callStorm('return($lib.view.get(0df16dd693c74109da0d58ab87ba768a).merge())')
+            with self.raises(s_exc.CantDelView):
+                await core.callStorm('return($lib.view.del(0df16dd693c74109da0d58ab87ba768a))')
+
     async def test_view_set_parent(self):
 
         async with self.getTestCore() as core:
@@ -45,10 +92,13 @@ class ViewTest(s_t_utils.SynTest):
             layr02 = await core.addLayer()
             vdef02 = {'layers': [layr02['iden']]}
             view02 = core.getView((await core.addView(vdef=vdef02))['iden'])
+            view03 = await core.callStorm('$view=$lib.view.get($view02).fork() return ( $view.iden )',
+                                          opts={'vars': {'view02': view02.iden}})
 
             # test the storm APIs for setting view parent
             opts = {'vars': {'base': view02.iden, 'fork': view00.iden}}
-            await core.stormlist('$lib.view.get($fork).set(parent, $base)', opts=opts)
+            msgs = await core.stormlist('$lib.view.get($fork).set(parent, $base)', opts=opts)
+            self.stormHasNoWarnErr(msgs)
 
             # test that merging selected nodes works correctly
             self.len(0, await view02.nodes('inet:fqdn=vertex.link'))
@@ -61,9 +111,14 @@ class ViewTest(s_t_utils.SynTest):
             self.len(1, await view00.nodes('#bar'))
             self.len(1, await view01.nodes('#bar'))
 
-            # test that the API prevents you from setting view parent that's already set
+            # setting the parent value the the existing value is okay
             opts = {'vars': {'base': view02.iden, 'fork': view00.iden}}
             msgs = await core.stormlist('$lib.view.get($fork).set(parent, $base)', opts=opts)
+            self.stormHasNoWarnErr(msgs)
+
+            # test that the API prevents you from setting view parent that's already set
+            opts = {'vars': {'base': view03, 'fork': view00.iden}}
+            msgs = await core.stormlist('$lib.view.get($fork).set(parent, $base)', opts=opts)
             self.stormIsInErr('You may not set parent on a view which already has one', msgs)
 
             opts = {'vars': {'fork': view00.iden}}
@@ -95,8 +150,8 @@ class ViewTest(s_t_utils.SynTest):
             await core.auth.addUser('visi')
             await core.addTagProp('score', ('int', {}), {})
 
-            self.len(1, await alist(core.eval('test:int=8 +#faz')))
-            self.len(2, await alist(core.eval('test:int=9 test:int=10')))
+            self.eq(1, await core.count('test:int=8 +#faz'))
+            self.eq(2, await core.count('test:int=9 test:int=10'))
             self.eq(3, (await core.getFormCounts()).get('test:int'))
 
             # Fork the main view
@@ -139,8 +194,8 @@ class ViewTest(s_t_utils.SynTest):
             # Add a node back
             await self.agenlen(1, view2.eval('[ test:int=12 ]'))
 
-            # Add a bunch of nodes to require chunking of splices when merging
-            for i in range(1000):
+            # Add a bunch of test nodes to the view.
+            for i in range(20):
                 await self.agenlen(1, view2.eval('[test:int=$val]', opts={'vars': {'val': i + 1000}}))
 
             # Add prop that will only exist in the child
@@ -174,8 +229,7 @@ class ViewTest(s_t_utils.SynTest):
             tmpiden = tmplayr['iden']
             await self.asyncraises(s_exc.ReadOnlyLayer, core.view.addLayer(tmpiden))
             await self.asyncraises(s_exc.ReadOnlyLayer, view2.addLayer(tmpiden))
-            await self.asyncraises(s_exc.ReadOnlyLayer, core.view.setLayers([tmpiden]))
-            await self.asyncraises(s_exc.ReadOnlyLayer, view2.setLayers([tmpiden]))
+            await self.asyncraises(s_exc.BadArg, view2.setLayers([tmpiden]))
 
             # You can't merge a non-forked view
             await self.asyncraises(s_exc.SynErr, view2.core.view.merge())
@@ -200,7 +254,7 @@ class ViewTest(s_t_utils.SynTest):
 
             async with core.getLocalProxy(user='visi') as prox:
                 with self.raises(s_exc.AuthDeny):
-                    await prox.eval('test:int=12', opts={'view': view2.iden}).list()
+                    await prox.count('test:int=12', opts={'view': view2.iden})
 
             # The parent count is correct
             self.eq(4, (await core.view.getFormCounts()).get('test:int'))
@@ -210,7 +264,7 @@ class ViewTest(s_t_utils.SynTest):
             await view2.wipeLayer()
 
             # The parent counts includes all the nodes that were merged
-            self.eq(1005, (await core.view.getFormCounts()).get('test:int'))
+            self.eq(25, (await core.view.getFormCounts()).get('test:int'))
 
             # A node added to the child is now present in the parent
             nodes = await core.nodes('test:int=12')
@@ -240,13 +294,15 @@ class ViewTest(s_t_utils.SynTest):
             self.len(2, await core.nodes('test:int -(refs)> *'))
 
             # The child count includes all the nodes in the view
-            self.eq(1005, (await view2.getFormCounts()).get('test:int'))
+            self.eq(25, (await view2.getFormCounts()).get('test:int'))
 
             # The child can see nodes that got merged
             nodes = await view2.nodes('test:int=12')
             self.len(1, nodes)
             nodes = await view2.nodes('test:int=1000')
             self.len(1, nodes)
+            nodes = await view2.nodes('test:int=1019')
+            self.len(1, nodes)
 
             await core.delView(view2.iden)
             await core.view.addLayer(layriden)
@@ -328,7 +384,7 @@ class ViewTest(s_t_utils.SynTest):
                 'storm': '[ test:str=mainhit ]'
             })
 
-            nodes = await alist(core.eval('[ test:int=11 ]', opts={'view': view2.iden}))
+            nodes = await core.nodes('[ test:int=11 ]', opts={'view': view2.iden})
             self.len(1, nodes)
 
             self.len(0, await core.view.nodes('test:str=mainhit'))
@@ -404,15 +460,6 @@ class ViewTest(s_t_utils.SynTest):
             self.eq(2, count['node:edits'])
             self.eq(0, count['node:add'])
 
-            mesgs = await core.stormlist('[test:str=foo2 :hehe=bar]', opts={'editformat': 'splices'})
-            count = collections.Counter(m[0] for m in mesgs)
-            self.eq(1, count['init'])
-            self.eq(1, count['node:add'])
-            self.eq(2, count['prop:set'])  # .created and .hehe
-            self.eq(0, count['node:edits'])
-            self.eq(1, count['node'])
-            self.eq(1, count['fini'])
-
             mesgs = await core.stormlist('[test:str=foo3 :hehe=bar]', opts={'editformat': 'count'})
             count = collections.Counter(m[0] for m in mesgs)
             self.eq(1, count['init'])
@@ -670,7 +717,7 @@ class ViewTest(s_t_utils.SynTest):
 
                 mirror_catchup = await core2.getNexsIndx() - 1 + 2 + layr.nodeeditlog.size
                 mirror_view, mirror_layr = await core2.callStorm('''
-                    $ldef = $lib.dict(mirror=$lib.str.concat($baseurl, "/", $baseiden))
+                    $ldef = ({'mirror':$lib.str.concat($baseurl, "/", $baseiden)})
                     $lyr = $lib.layer.add(ldef=$ldef)
                     $view = $lib.view.add(($lyr.iden,))
                     return(($view.iden, $lyr.iden))
@@ -722,21 +769,15 @@ class ViewTest(s_t_utils.SynTest):
 
             with self.raises(s_exc.AuthDeny) as cm:
                 await core.nodes('$lib.view.get().merge()', opts=viewopts)
-            self.eq('node.add.test:str', cm.exception.errinfo['perm'])
-
-            await core.addUserRule(useriden, (True, ('node', 'add')), gateiden=baselayr)
-
-            with self.raises(s_exc.AuthDeny) as cm:
-                await core.nodes('$lib.view.get().merge()', opts=viewopts)
-            self.eq('node.prop.set.test:str:.created', cm.exception.errinfo['perm'])
+            self.eq('node.prop.set.syn:tag.base', cm.exception.errinfo['perm'])
 
             await core.addUserRule(useriden, (True, ('node', 'prop', 'set')), gateiden=baselayr)
 
             with self.raises(s_exc.AuthDeny) as cm:
                 await core.nodes('$lib.view.get().merge()', opts=viewopts)
-            self.eq('node.edge.add.refs', cm.exception.errinfo['perm'])
+            self.eq('node.add.syn:tag', cm.exception.errinfo['perm'])
 
-            await core.addUserRule(useriden, (True, ('node', 'edge', 'add')), gateiden=baselayr)
+            await core.addUserRule(useriden, (True, ('node', 'add')), gateiden=baselayr)
 
             with self.raises(s_exc.AuthDeny) as cm:
                 await core.nodes('$lib.view.get().merge()', opts=viewopts)
@@ -750,6 +791,12 @@ class ViewTest(s_t_utils.SynTest):
 
             await core.addUserRule(useriden, (True, ('node', 'data', 'set')), gateiden=baselayr)
 
+            with self.raises(s_exc.AuthDeny) as cm:
+                await core.nodes('$lib.view.get().merge()', opts=viewopts)
+            self.eq('node.edge.add.refs', cm.exception.errinfo['perm'])
+
+            await core.addUserRule(useriden, (True, ('node', 'edge', 'add')), gateiden=baselayr)
+
             await core.nodes('$lib.view.get().merge()', opts=viewopts)
 
             nodes = await core.nodes('test:str=foo $node.data.load(foo)')
@@ -759,3 +806,126 @@ class ViewTest(s_t_utils.SynTest):
             self.nn(nodes[0].tagprops.get('seen'))
             self.nn(nodes[0].tagprops['seen'].get('score'))
             self.nn(nodes[0].nodedata.get('foo'))
+
+            await core.delUserRule(useriden, (True, ('node', 'tag', 'add')), gateiden=baselayr)
+
+            await core.addUserRule(useriden, (True, ('node', 'tag', 'add', 'rep', 'foo')), gateiden=baselayr)
+
+            await core.nodes('test:str=foo [ -#seen +#rep.foo ]', opts=viewopts)
+
+            await core.nodes('$lib.view.get().merge()', opts=viewopts)
+            nodes = await core.nodes('test:str=foo')
+            self.nn(nodes[0].get('#rep.foo'))
+
+            await core.nodes('test:str=foo [ -#rep ]')
+
+            await core.nodes('test:str=foo | merge --apply', opts=viewopts)
+            nodes = await core.nodes('test:str=foo')
+            self.nn(nodes[0].get('#rep.foo'))
+
+            await core.nodes('test:str=foo [ -#rep ]')
+            await core.nodes('test:str=foo [ +#rep=now ]', opts=viewopts)
+
+            with self.raises(s_exc.AuthDeny) as cm:
+                await core.nodes('$lib.view.get().merge()', opts=viewopts)
+
+    async def test_view_insert_parent_fork(self):
+
+        async with self.getTestCore() as core:
+
+            role = await core.auth.addRole('ninjas')
+            visi = await core.auth.addUser('visi')
+            asvisi = {'user': visi.iden}
+
+            view00 = core.getView()
+            view02 = core.getView((await view00.fork())['iden'])
+            view03 = core.getView((await view02.fork())['iden'])
+
+            self.eq(view02.parent, view00)
+
+            self.len(2, view02.layers)
+            self.len(3, view03.layers)
+
+            await view00.nodes('[ inet:fqdn=vertex.link ]')
+            await view02.nodes('inet:fqdn=vertex.link [ +#foo ]')
+
+            await view02.nodes('auth.user.addrule visi node --gate $lib.view.get().iden')
+            await view02.nodes('auth.user.mod visi --admin $lib.true --gate $lib.view.get().iden')
+            userrules = visi.getRules(gateiden=view02.iden)
+
+            await view02.nodes('auth.role.addrule ninjas node.add --gate $lib.view.get().iden')
+            rolerules = role.getRules(gateiden=view02.iden)
+
+            msgs = await core.stormlist('auth.user.addrule visi node --gate $lib.view.get().layers.0.iden')
+            self.stormHasNoWarnErr(msgs)
+
+            opts = {'vars': {'role': role.iden}}
+            quorum = await core.callStorm('return($lib.view.get().set(quorum, ({"count": 1, "roles": [$role]})))', opts=opts)
+
+            forkopts = {'view': view02.iden}
+            await core.callStorm('return($lib.view.get().setMergeRequest(comment=woot))', opts=forkopts)
+
+            merging = 'return($lib.view.get().getMergingViews()) '
+            self.eq([view02.iden], await core.callStorm(merging))
+
+            q = 'return($lib.view.get().insertParentFork(name=staging).iden)'
+            newiden = await core.callStorm(q, opts=forkopts)
+
+            self.eq([], await core.callStorm(merging))
+
+            view01 = core.getView(newiden)
+
+            self.ne(view02.parent, view00)
+            self.eq(view03.parent, view02)
+            self.eq(view02.parent, view01)
+            self.eq(view01.parent, view00)
+
+            self.len(2, view01.layers)
+            self.len(3, view02.layers)
+            self.len(4, view03.layers)
+            self.isin(view01.layers[0], view02.layers)
+            self.isin(view01.layers[0], view03.layers)
+
+            self.eq(userrules, visi.getRules(gateiden=view01.iden))
+            self.eq(rolerules, role.getRules(gateiden=view01.iden))
+
+            nodes = await view01.nodes('inet:fqdn=vertex.link')
+            self.none(nodes[0].getTag('foo'))
+
+            await core.nodes('merge --diff --apply', opts=forkopts)
+
+            nodes = await core.nodes('inet:fqdn=vertex.link')
+            self.none(nodes[0].getTag('foo'))
+
+            nodes = await view01.nodes('inet:fqdn=vertex.link')
+            self.nn(nodes[0].getTag('foo'))
+
+            with self.raises(s_exc.BadState):
+                await view00.insertParentFork(visi.iden)
+
+            with self.raises(s_exc.BadState):
+                await core.callStorm('return($lib.view.get().insertParentFork().iden)')
+
+    async def test_view_children(self):
+
+        async with self.getTestCore() as core:
+
+            view00 = core.getView()
+            view01 = core.getView((await view00.fork())['iden'])
+            view02 = core.getView((await view01.fork())['iden'])
+            view03 = core.getView((await view01.fork())['iden'])
+
+            q = '''
+            $kids = ([])
+            for $child in $lib.view.get($iden).children() { $kids.append($child.iden) }
+            return($kids)
+            '''
+
+            opts = {'vars': {'iden': view00.iden}}
+            self.eq([view01.iden], await core.callStorm(q, opts=opts))
+
+            opts['vars']['iden'] = view01.iden
+            self.eq([view02.iden, view03.iden], await core.callStorm(q, opts=opts))
+
+            opts['vars']['iden'] = view02.iden
+            self.eq([], await core.callStorm(q, opts=opts))
