@@ -27,8 +27,8 @@ HEXSTR_PRIVATE_PRIME_Q = 'c53b9c8dfb3dda04d16c7f779a02b3b8c7b44bf876dc88ad562778
                          '1bef7207fa489e398041787cfbd155f1034a207d517f06bc76a044262484f82f0c6a887f776b1dce837408999d8' \
                          '8dd33a96c7f80e23719e77a11075d337bf9cc47d7dbf98e341b81c23f165dd15ccfd2973ab'
 
-TEST_MD5 = hashlib.md5(b'test').hexdigest()
-TEST_SHA1 = hashlib.sha1(b'test').hexdigest()
+TEST_MD5 = hashlib.md5(b'test', usedforsecurity=False).hexdigest()
+TEST_SHA1 = hashlib.sha1(b'test', usedforsecurity=False).hexdigest()
 TEST_SHA256 = hashlib.sha256(b'test').hexdigest()
 TEST_SHA384 = hashlib.sha384(b'test').hexdigest()
 TEST_SHA512 = hashlib.sha512(b'test').hexdigest()
@@ -45,14 +45,45 @@ class CryptoModelTest(s_t_utils.SynTest):
             nodes = await core.nodes('''
                 crypto:currency:address=btc/1BvBMSEYstWetqTFn5Au4m4GFg7xJaNVN2
                 [ :seed={
-                    [ crypto:key=* :algorithm=aes256 :private=00000000 :public=ffffffff :seed:passwd=s3cret :seed:algorithm=pbkdf2 ]
+                    [ crypto:key=*
+                        :algorithm=aes256
+                        :mode=CBC
+                        :iv=41414141
+                        :iv:text=AAAA
+                        :private=00000000
+                        :private:text=hehe
+                        :private:md5=$md5
+                        :private:sha1=$sha1
+                        :private:sha256=$sha256
+                        :public=ffffffff
+                        :public:md5=$md5
+                        :public:sha1=$sha1
+                        :public:sha256=$sha256
+                        :public:text=haha
+                        :seed:passwd=s3cret
+                        :seed:algorithm=pbkdf2 ]
                 }]
-            ''')
+            ''', opts={'vars': {'md5': TEST_MD5, 'sha1': TEST_SHA1, 'sha256': TEST_SHA256}})
 
             self.len(1, await core.nodes('crypto:algorithm=aes256'))
-            self.len(1, await core.nodes('crypto:key:algorithm=aes256 +:private=00000000 +:public=ffffffff +:seed:algorithm=pbkdf2 +:seed:passwd=s3cret'))
+            self.len(1, await core.nodes('''
+                    crypto:key:algorithm=aes256
+                        +:private=00000000
+                        +:public=ffffffff
+                        +:seed:algorithm=pbkdf2
+                        +:seed:passwd=s3cret
+                        +:mode=cbc
+                        +:iv=41414141
+            '''))
+            self.len(1, await core.nodes('it:dev:str=AAAA -> crypto:key'))
+            self.len(1, await core.nodes('it:dev:str=hehe -> crypto:key'))
+            self.len(1, await core.nodes('it:dev:str=haha -> crypto:key'))
             self.len(1, await core.nodes('inet:passwd=s3cret -> crypto:key -> crypto:currency:address'))
 
+            self.len(2, await core.nodes('crypto:key -> hash:md5'))
+            self.len(2, await core.nodes('crypto:key -> hash:sha1'))
+            self.len(2, await core.nodes('crypto:key -> hash:sha256'))
+
             nodes = await core.nodes('inet:client=1.2.3.4 -> crypto:currency:client -> crypto:currency:address')
             self.eq(nodes[0].get('coin'), 'btc')
             self.eq(nodes[0].get('iden'), '1BvBMSEYstWetqTFn5Au4m4GFg7xJaNVN2')
@@ -224,12 +255,102 @@ class CryptoModelTest(s_t_utils.SynTest):
             self.len(1, await core.nodes('crypto:smart:effect:edittokensupply -> crypto:smart:contract'))
             self.len(1, await core.nodes('crypto:smart:effect:edittokensupply -> crypto:currency:transaction'))
 
+            nodes = await core.nodes('''
+                [
+                    crypto:smart:effect:minttoken=*
+                        :index=0
+                        :token=(2bdea834252a220b61aadf592cc0de66, 30)
+                        :transaction=*
+                ]''')
+            self.len(1, nodes)
+            node = nodes[0]
+            self.eq(node.get('index'), 0)
+            self.nn(node.get('token'))
+            self.nn(node.get('transaction'))
+            self.len(1, await core.nodes('crypto:smart:effect:minttoken -> crypto:smart:token'))
+            self.len(1, await core.nodes('crypto:smart:effect:minttoken -> crypto:currency:transaction'))
+
+            nodes = await core.nodes('''
+                [
+                    crypto:smart:effect:burntoken=*
+                        :index=0
+                        :token=(2bdea834252a220b61aadf592cc0de66, 30)
+                        :transaction=*
+                ]''')
+            self.len(1, nodes)
+            node = nodes[0]
+            self.eq(node.get('index'), 0)
+            self.nn(node.get('token'))
+            self.nn(node.get('transaction'))
+            self.len(1, await core.nodes('crypto:smart:effect:burntoken -> crypto:smart:token'))
+            self.len(1, await core.nodes('crypto:smart:effect:burntoken -> crypto:currency:transaction'))
+
+            nodes = await core.nodes('''
+                [
+                    crypto:smart:effect:proxytoken=*
+                        :index=0
+                        :token=(2bdea834252a220b61aadf592cc0de66, 30)
+                        :transaction=*
+                        :owner=(btc, 1BvBMSEYstWetqTFn5Au4m4GFg7xJaNVN2)
+                        :proxy=(btc, 1BvBMSEYstWetqTFn5Au4m4GFg7xJaNVN2)
+                ]''')
+            self.len(1, nodes)
+            node = nodes[0]
+            self.eq(node.get('index'), 0)
+            self.nn(node.get('token'))
+            self.nn(node.get('owner'))
+            self.nn(node.get('proxy'))
+            self.len(1, await core.nodes('crypto:smart:effect:minttoken -> crypto:smart:token'))
+            self.len(1, await core.nodes('crypto:smart:effect:minttoken -> crypto:currency:transaction'))
+
+            nodes = await core.nodes('''
+                [
+                    crypto:smart:effect:proxytokenall=*
+                        :index=0
+                        :transaction=*
+                        :contract=*
+                        :owner=(btc, 1BvBMSEYstWetqTFn5Au4m4GFg7xJaNVN2)
+                        :proxy=(btc, 1BvBMSEYstWetqTFn5Au4m4GFg7xJaNVN2)
+                        :approval=$lib.true
+                ]''')
+            self.len(1, nodes)
+            node = nodes[0]
+            self.eq(node.get('index'), 0)
+            self.nn(node.get('owner'))
+            self.nn(node.get('proxy'))
+            self.nn(node.get('contract'))
+            self.true(node.get('approval'))
+            self.len(2, await core.nodes('crypto:smart:effect:proxytokenall -> crypto:currency:address'))
+            self.len(1, await core.nodes('crypto:smart:effect:proxytokenall -> crypto:currency:transaction'))
+            self.len(1, await core.nodes('crypto:smart:effect:proxytokenall -> crypto:smart:contract'))
+
+            nodes = await core.nodes('''
+                [
+                    crypto:smart:effect:proxytokens=*
+                        :index=0
+                        :transaction=*
+                        :contract=*
+                        :owner=(btc, 1BvBMSEYstWetqTFn5Au4m4GFg7xJaNVN2)
+                        :proxy=(btc, 1BvBMSEYstWetqTFn5Au4m4GFg7xJaNVN2)
+                        :amount=0xff
+                ]''')
+            self.len(1, nodes)
+            node = nodes[0]
+            self.eq(node.get('index'), 0)
+            self.nn(node.get('owner'))
+            self.nn(node.get('proxy'))
+            self.nn(node.get('contract'))
+            self.eq(node.get('amount'), 'ff')
+            self.len(2, await core.nodes('crypto:smart:effect:proxytokens -> crypto:currency:address'))
+            self.len(1, await core.nodes('crypto:smart:effect:proxytokens -> crypto:currency:transaction'))
+            self.len(1, await core.nodes('crypto:smart:effect:proxytokens -> crypto:smart:contract'))
+
             nodes = await core.nodes('''
                 [
                     crypto:smart:token=(2bdea834252a220b61aadf592cc0de66, 30)
                         :owner=eth/aaaa
                         :nft:url = https://coin.vertex.link/nfts/30
-                        :nft:meta = $lib.dict(name=WootWoot)
+                        :nft:meta = ({'name':'WootWoot'})
                         :nft:meta:name = WootWoot
                         :nft:meta:description = LoLoL
                         :nft:meta:image = https://vertex.link/favicon.ico
@@ -260,13 +381,19 @@ class CryptoModelTest(s_t_utils.SynTest):
             self.len(1, await core.nodes('crypto:currency:transaction:value=1e-24'))
             self.len(1, await core.nodes('crypto:currency:transaction:value=0.000000000000000000000001'))
 
-            huge = '730750818665451459101841.00000000000000000002'
-            huge2 = '730750818665451459101841.000000000000000000015'
+            huge = '730750818665451459101841.000000000000000000000002'
+            huge2 = '730750818665451459101841.0000000000000000000000015'
+            huge3 = '730750818665451459101841.000000000000000000000001'
 
             self.len(1, await core.nodes(f'[ crypto:currency:transaction=* :value={huge} ]'))
             self.len(1, await core.nodes(f'[ crypto:currency:transaction=* :value={huge2} ]'))
             self.len(2, await core.nodes(f'crypto:currency:transaction:value={huge}'))
 
+            self.len(1, await core.nodes(f'[ crypto:currency:transaction=* :value={huge3} ]'))
+            self.len(2, await core.nodes(f'crypto:currency:transaction:value={huge}'))
+            self.len(2, await core.nodes(f'crypto:currency:transaction:value={huge2}'))
+            self.len(1, await core.nodes(f'crypto:currency:transaction:value={huge3}'))
+
     async def test_norm_lm_ntlm(self):
         async with self.getTestCore() as core:  # type: s_cortex.Cortex
             lm = core.model.type('hash:lm')
@@ -283,30 +410,38 @@ class CryptoModelTest(s_t_utils.SynTest):
 
     async def test_forms_crypto_simple(self):
         async with self.getTestCore() as core:  # type: s_cortex.Cortex
-            async with await core.snap() as snap:
-                # md5
-                node = await snap.addNode('hash:md5', TEST_MD5.upper())
-                self.eq(node.ndef, ('hash:md5', TEST_MD5))
-                await self.asyncraises(s_exc.BadTypeValu, snap.addNode('hash:md5', TEST_SHA1))
-                # sha1
-                node = await snap.addNode('hash:sha1', TEST_SHA1.upper())
-                self.eq(node.ndef, ('hash:sha1', TEST_SHA1))
-                await self.asyncraises(s_exc.BadTypeValu, snap.addNode('hash:sha1', TEST_SHA256))
-                # sha256
-                node = await snap.addNode('hash:sha256', TEST_SHA256.upper())
-                self.eq(node.ndef, ('hash:sha256', TEST_SHA256))
-                await self.asyncraises(s_exc.BadTypeValu, snap.addNode('hash:sha256', TEST_SHA384))
-                # sha384
-                node = await snap.addNode('hash:sha384', TEST_SHA384.upper())
-                self.eq(node.ndef, ('hash:sha384', TEST_SHA384))
-                await self.asyncraises(s_exc.BadTypeValu, snap.addNode('hash:sha384', TEST_SHA512))
-                # sha512
-                node = await snap.addNode('hash:sha512', TEST_SHA512.upper())
-                self.eq(node.ndef, ('hash:sha512', TEST_SHA512))
-                await self.asyncraises(s_exc.BadTypeValu, snap.addNode('hash:sha512', TEST_MD5))
+
+            nodes = await core.nodes('[(hash:md5=$valu)]', opts={'vars': {'valu': TEST_MD5.upper()}})
+            self.len(1, nodes)
+            self.eq(nodes[0].ndef, ('hash:md5', TEST_MD5))
+            with self.raises(s_exc.BadTypeValu):
+                await core.nodes('[(hash:md5=$valu)]', opts={'vars': {'valu': TEST_SHA1}})
+
+            nodes = await core.nodes('[(hash:sha1=$valu)]', opts={'vars': {'valu': TEST_SHA1.upper()}})
+            self.len(1, nodes)
+            self.eq(nodes[0].ndef, ('hash:sha1', TEST_SHA1))
+            with self.raises(s_exc.BadTypeValu):
+                await core.nodes('[(hash:sha1=$valu)]', opts={'vars': {'valu': TEST_SHA256}})
+
+            nodes = await core.nodes('[(hash:sha256=$valu)]', opts={'vars': {'valu': TEST_SHA256.upper()}})
+            self.len(1, nodes)
+            self.eq(nodes[0].ndef, ('hash:sha256', TEST_SHA256))
+            with self.raises(s_exc.BadTypeValu):
+                await core.nodes('[(hash:sha256=$valu)]', opts={'vars': {'valu': TEST_SHA384}})
+
+            nodes = await core.nodes('[(hash:sha384=$valu)]', opts={'vars': {'valu': TEST_SHA384.upper()}})
+            self.len(1, nodes)
+            self.eq(nodes[0].ndef, ('hash:sha384', TEST_SHA384))
+            with self.raises(s_exc.BadTypeValu):
+                await core.nodes('[(hash:sha384=$valu)]', opts={'vars': {'valu': TEST_SHA512}})
+
+            nodes = await core.nodes('[(hash:sha512=$valu)]', opts={'vars': {'valu': TEST_SHA512.upper()}})
+            self.len(1, nodes)
+            self.eq(nodes[0].ndef, ('hash:sha512', TEST_SHA512))
+            with self.raises(s_exc.BadTypeValu):
+                await core.nodes('[(hash:sha512=$valu)]', opts={'vars': {'valu': TEST_MD5}})
 
     async def test_form_rsakey(self):
-        prop = 'rsa:key'
         props = {
             'bits': BITS,
             'priv:exp': HEXSTR_PRIVATE_EXPONENT,
@@ -317,18 +452,19 @@ class CryptoModelTest(s_t_utils.SynTest):
 
         async with self.getTestCore() as core:  # type: s_cortex.Cortex
 
-            async with await core.snap() as snap:
+            opts = {'vars': {'valu': valu, 'p': props}}
+            q = '[(rsa:key=$valu :bits=$p.bits :priv:exp=$p."priv:exp" :priv:p=$p."priv:p" :priv:q=$p."priv:q")]'
+            nodes = await core.nodes(q, opts=opts)
+            self.len(1, nodes)
+            node = nodes[0]
 
-                node = await snap.addNode(prop, valu, props)
-
-                self.eq(node.ndef[1], (HEXSTR_MODULUS, HEXSTR_PUBLIC_EXPONENT))
-
-                self.eq(node.get('mod'), HEXSTR_MODULUS)
-                self.eq(node.get('bits'), BITS)
-                self.eq(node.get('pub:exp'), HEXSTR_PUBLIC_EXPONENT)
-                self.eq(node.get('priv:exp'), HEXSTR_PRIVATE_EXPONENT)
-                self.eq(node.get('priv:p'), HEXSTR_PRIVATE_PRIME_P)
-                self.eq(node.get('priv:q'), HEXSTR_PRIVATE_PRIME_Q)
+            self.eq(node.ndef[1], (HEXSTR_MODULUS, HEXSTR_PUBLIC_EXPONENT))
+            self.eq(node.get('mod'), HEXSTR_MODULUS)
+            self.eq(node.get('bits'), BITS)
+            self.eq(node.get('pub:exp'), HEXSTR_PUBLIC_EXPONENT)
+            self.eq(node.get('priv:exp'), HEXSTR_PRIVATE_EXPONENT)
+            self.eq(node.get('priv:p'), HEXSTR_PRIVATE_PRIME_P)
+            self.eq(node.get('priv:q'), HEXSTR_PRIVATE_PRIME_Q)
 
     async def test_model_x509(self):
 
@@ -337,7 +473,7 @@ class CryptoModelTest(s_t_utils.SynTest):
             crl = s_common.guid()
             cert = s_common.guid()
             icert = s_common.guid()
-            fileguid = 'guid:' + s_common.guid()
+            fileguid = f'guid:{s_common.guid()}'
 
             nodes = await core.nodes('''
                 [ crypto:x509:cert=$icert
@@ -358,7 +494,7 @@ class CryptoModelTest(s_t_utils.SynTest):
                     :issuer="DN FOO THING"
                     :issuer:cert=$icert
 
-                    :serial=12345
+                    :serial=0000000000000000000000000000000000003039
                     :version=v3
 
                     :validity:notafter=2019
@@ -388,7 +524,7 @@ class CryptoModelTest(s_t_utils.SynTest):
             self.eq(nodes[0].get('subject'), "CN=vertex.link")
             self.eq(nodes[0].get('issuer'), "DN FOO THING")
             self.eq(nodes[0].get('issuer:cert'), icert)
-            self.eq(nodes[0].get('serial'), "12345")
+            self.eq(nodes[0].get('serial'), "0000000000000000000000000000000000003039")
             self.eq(nodes[0].get('version'), 2)
 
             self.eq(nodes[0].get('validity:notafter'), 1546300800000)
@@ -434,3 +570,18 @@ class CryptoModelTest(s_t_utils.SynTest):
             self.eq(nodes[0].ndef, ('crypto:x509:revoked', (crl, cert)))
             self.eq(nodes[0].get('crl'), crl)
             self.nn(nodes[0].get('cert'), cert)
+
+            # odd-length serials
+            serials = [
+                '1' * 7,
+                '2' * 9,
+                '3' * 15,
+                '4' * 17,
+                '5' * 31,
+                '6' * 33,
+                '7' * 39,
+            ]
+
+            for serial in serials:
+                msgs = await core.stormlist(f'[crypto:x509:cert=* :serial={serial}]')
+                self.stormHasNoErr(msgs)
