@@ -19,6 +19,12 @@ class ProjModelTest(s_test.SynTest):
             self.nn(proj)
             self.len(1, await core.nodes('proj:project:desc=bar'))
 
+            nodes = await core.nodes('proj:project [ :type=foo.bar ]')
+            self.len(1, nodes)
+            self.eq('foo.bar.', nodes[0].get('type'))
+
+            self.len(1, await core.nodes('proj:project -> proj:project:type:taxonomy'))
+
             opts = {'user': visi.iden, 'vars': {'proj': proj}}
             with self.raises(s_exc.AuthDeny):
                 await core.callStorm('return($lib.projects.get($proj).epics.add(bar))', opts=opts)
@@ -236,12 +242,14 @@ class ProjModelTest(s_test.SynTest):
             self.len(1, nodes)
             self.eq(proj, nodes[0].get('project'))
 
-            nodes = await core.nodes('proj:ticket')
+            nodes = await core.nodes('proj:ticket [ :ext:creator={[ps:contact=* :name=visi ]} :ext:assignee={[ps:contact=* :name=bob ]} ]')
             self.len(1, nodes)
             self.nn(nodes[0].get('creator'))
             self.nn(nodes[0].get('created'))
             self.nn(nodes[0].get('updated'))
             self.nn(nodes[0].get('assignee'))
+            self.nn(nodes[0].get('ext:creator'))
+            self.nn(nodes[0].get('ext:assignee'))
             self.eq(70, nodes[0].get('status'))
             self.eq(50, nodes[0].get('priority'))
             self.eq('done', nodes[0].repr('status'))
@@ -326,7 +334,7 @@ class ProjModelTest(s_test.SynTest):
             self.true(await core.callStorm('return($lib.projects.del($proj))', opts=opts))
             self.false(await core.callStorm('return($lib.projects.del(newp))', opts=opts))
 
-            self.none(core.auth.getAuthGate(proj))
+            self.nn(core.auth.getAuthGate(proj))
 
             self.len(1, await core.nodes('yield $lib.projects.add(proj)'))
             self.len(1, await core.nodes('yield $lib.projects.get(proj).epics.add(epic)'))
@@ -337,6 +345,16 @@ class ProjModelTest(s_test.SynTest):
             name = await core.callStorm('$p=$lib.projects.get(proj) $p.name=newproj return ( $p.name )')
             self.eq(name, 'newproj')
 
+            viewiden = core.getView().iden
+            self.len(1, await core.nodes(f'[ proj:project={viewiden}]'))
+            gate = core.auth.getAuthGate(viewiden)
+            self.eq(gate.type, 'view')
+
+            await core.nodes(f'proj:project={viewiden} | delnode')
+            gate = core.auth.getAuthGate(viewiden)
+            self.nn(gate)
+            self.eq(gate.type, 'view')
+
     async def test_model_proj_attachment(self):
 
         async with self.getTestCore() as core:
