@@ -8,7 +8,7 @@ import threading
 
 from unittest import mock
 
-logger = logging.getLogger(__name__)
+import cryptography.hazmat.primitives.hashes as c_hashes
 
 import synapse.exc as s_exc
 import synapse.glob as s_glob
@@ -18,13 +18,14 @@ import synapse.telepath as s_telepath
 
 import synapse.lib.cell as s_cell
 import synapse.lib.coro as s_coro
+import synapse.lib.link as s_link
 import synapse.lib.share as s_share
 import synapse.lib.certdir as s_certdir
-import synapse.lib.httpapi as s_httpapi
 import synapse.lib.version as s_version
 
 import synapse.tests.utils as s_t_utils
-from synapse.tests.utils import alist
+
+logger = logging.getLogger(__name__)
 
 class Boom:
     pass
@@ -173,16 +174,6 @@ class TeleAuth(s_telepath.Aware):
     def getFooBar(self, x, y):
         return x + y
 
-class ConsulV1Handler(s_httpapi.Handler):
-    '''Fake Hashicorp Consul /v1/catalog/service/:service: handler'''
-    async def get(self, servicename):
-        data = self.cell.consul_data
-        if servicename != data[0].get('ServiceName'):
-            self.set_header('Content-Type', 'application/json')
-            return self.write(json.dumps([]))
-        self.set_header('Content-Type', 'application/json')
-        return self.write(json.dumps(data))
-
 class TeleTest(s_t_utils.SynTest):
 
     async def test_telepath_basics(self):
@@ -257,7 +248,7 @@ class TeleTest(s_t_utils.SynTest):
             # check an async generator return channel
             genr = prox.corogenr(3)
             self.true(isinstance(genr, s_telepath.GenrIter))
-            self.eq((0, 1, 2), await alist(genr))
+            self.eq((0, 1, 2), await s_t_utils.alist(genr))
 
             # check async generator explodes channel
             genr = prox.agenrboom()
@@ -299,8 +290,8 @@ class TeleTest(s_t_utils.SynTest):
             q = '[' + ' '.join([f'{form}={i}' for i in range(10)]) + ' ]'
 
             # This puts a link into the link pool
-            podes = list(proxy.eval(q))
-            self.len(10, podes)
+            msgs = list(proxy.storm(q, opts={'show': ('node',)}))
+            self.len(12, msgs)
 
             evt = threading.Event()
 
@@ -313,11 +304,11 @@ class TeleTest(s_t_utils.SynTest):
 
             # Break from the generator right away, causing a
             # GeneratorExit in the GenrHelp object __iter__ method.
-            pode = None
-            for pode in proxy.eval(q):
+            mesg = None
+            for mesg in proxy.storm(q):
                 break
             # Ensure the query did yield an object
-            self.nn(pode)
+            self.nn(mesg)
 
             # Ensure the link we have a reference too was torn down
             self.true(evt.wait(4))
@@ -352,11 +343,11 @@ class TeleTest(s_t_utils.SynTest):
 
                 # check a generator return channel
                 genr = await prox.genr()
-                self.eq((10, 20, 30), await alist(genr))
+                self.eq((10, 20, 30), await s_t_utils.alist(genr))
 
                 # check an async generator return channel
                 genr = prox.corogenr(3)
-                self.eq((0, 1, 2), await alist(genr))
+                self.eq((0, 1, 2), await s_t_utils.alist(genr))
 
                 await self.asyncraises(s_exc.NoSuchMeth, prox.raze())
 
@@ -407,7 +398,7 @@ class TeleTest(s_t_utils.SynTest):
                 hostkey, hostcert = certdir.genHostCert(hostname, signas='loopy')
                 self.none(certdir.getHostCertHash('newp.newp.newp'))
 
-                certhash = hostcert.digest('sha256').decode().lower().replace(':', '')
+                certhash = s_common.ehex(hostcert.fingerprint(c_hashes.SHA256()))
 
                 host, port = await dmon.listen(f'ssl://{hostname}:0')
                 dmon.share('foo', foo)
@@ -671,7 +662,7 @@ class TeleTest(s_t_utils.SynTest):
                          key: 'synapse.tests.test_telepath.CustomShare'})
 
                 # and we can still use the first obj we made
-                ret = await alist(obj.custgenr(3))
+                ret = await s_t_utils.alist(obj.custgenr(3))
                 self.eq(ret, [0, 1, 2])
 
             # check that a dynamic share works
@@ -809,79 +800,6 @@ class TeleTest(s_t_utils.SynTest):
                 # check a standard return value
                 self.eq(30, await prox.bar(10, 20))
 
-    async def test_telepath_client_redir(self):
-
-        class TestRedir(s_telepath.Aware):
-
-            def __init__(self, valu, redir=None):
-                self.valu = valu
-                self.redir = redir
-
-            def getTeleApi(self, link, mesg, path):
-                if self.redir is not None:
-                    raise s_exc.TeleRedir(url=self.redir)
-                return self
-
-            async def dostuff(self, x):
-
-                if self.redir:
-                    raise s_exc.TeleRedir(url=self.redir)
-
-                return x + self.valu
-
-        dmon0 = await s_daemon.Daemon.anit()
-        dmon1 = await s_daemon.Daemon.anit()
-
-        addr0 = await dmon0.listen('tcp://127.0.0.1:0/')
-        addr1 = await dmon1.listen('tcp://127.0.0.1:0/')
-
-        url0 = f'tcp://127.0.0.1:{addr0[1]}/foo'
-        url1 = f'tcp://127.0.0.1:{addr1[1]}/foo'
-
-        rdir0 = TestRedir(10)
-        rdir1 = TestRedir(20, redir=url0)
-
-        dmon0.share('foo', rdir0)
-        dmon1.share('foo', rdir1)
-
-        async with await s_telepath.Client.anit(url0) as targ:
-
-            with self.raises(s_exc.NotReady):
-                targ.dostuff(100)
-
-            await targ.waitready()
-            proxy = await targ.proxy()
-            self.eq(proxy._getSynVers(), s_version.version)
-
-            # Client implements some base helpers the proxy does
-            self.eq(targ._getSynVers(), s_version.version)
-            self.eq(targ._getSynCommit(), s_version.commit)
-            self.eq(targ._getClasses(),
-                    ('synapse.tests.test_telepath.TestRedir',
-                     'synapse.telepath.Aware'))
-            # client works as a passthrough to the proxy
-            self.eq(110, await targ.dostuff(100))
-
-        # this should get redirected to url0...
-        async with await s_telepath.Client.anit(url1) as targ:
-            await targ.waitready()
-            self.eq(110, await targ.dostuff(100))
-
-        # fake out the redirect to connect, then redirect on call...
-        rdir1.redir = None
-        async with await s_telepath.Client.anit(url1) as targ:
-            await targ.waitready()
-            self.eq(120, await targ.dostuff(100))
-            rdir1.redir = url0
-            self.eq(110, await targ.dostuff(100))
-
-        # client is now fini and methods on longer work
-        with self.raises(s_exc.IsFini):
-            await targ.dostuff(100)
-
-        await dmon0.fini()
-        await dmon1.fini()
-
     async def test_telepath_client_failover(self):
 
         class TestFail:
@@ -909,21 +827,25 @@ class TeleTest(s_t_utils.SynTest):
 
         urls = (url0, url1)
 
-        async with await s_telepath.Client.anit(urls) as targ:
+        async with await s_telepath.open(urls) as targ:
 
             await targ.waitready()
 
-            self.eq(110, await targ.dostuff(100))
+            prox00 = await targ.proxy(timeout=12)
+            self.eq(110, await prox00.dostuff(100))
+
             self.eq(1, fail0.count)
             self.eq(0, fail1.count)
 
             await dmon0.fini()
+            self.true(await prox00.waitfini(10))
 
-            self.eq(110, await targ.dostuff(100))
+            prox01 = await targ.proxy(timeout=12)
+            self.eq(110, await prox01.dostuff(100))
             self.eq(1, fail0.count)
             self.eq(1, fail1.count)
 
-        async with await s_telepath.Client.anit(urls) as targ:
+        async with await s_telepath.open(urls) as targ:
 
             with self.getAsyncLoggerStream('synapse.telepath', 'Connect call failed') as stream:
 
@@ -935,11 +857,36 @@ class TeleTest(s_t_utils.SynTest):
                 mesgs = stream.read()
                 self.notin('password', mesgs)
 
-            self.eq(110, await targ.dostuff(100))
+            prox00 = await targ.proxy(timeout=12)
+            self.eq(110, await prox00.dostuff(100))
 
             self.eq(1, fail0.count)
             self.eq(2, fail1.count)
 
+        async with await s_telepath.open(url1) as targ:
+            await targ.waitready(timeout=12)
+            prox00 = await targ.proxy(timeout=12)
+            self.eq(110, await prox00.dostuff(100))
+
+        async def onlink(proxy, urlinfo):
+            self.eq(110, await proxy.dostuff(100))
+            _url = s_telepath.zipurl(urlinfo)
+            logger.info(f'Connected to url={_url}')
+
+        with self.getAsyncLoggerStream('synapse.tests.test_telepath',
+                                       f'Connected to url=tcp://127.0.0.1:{addr1[1]}/foo') as stream:
+            async with await s_telepath.open(url1, onlink=onlink) as targ:
+                self.true(await stream.wait(timeout=12))
+
+        # Coverage
+        async def badonlink(proxy, urlinfo):
+            raise ValueError('oopsie')
+
+        with self.getAsyncLoggerStream('synapse.telepath', 'onlink: ') as stream:
+            async with await s_telepath.open(url1, onlink=badonlink) as targ:
+                self.true(await stream.wait(timeout=12))
+
+        await dmon0.fini()
         await dmon1.fini()
 
     async def test_telepath_poolsize(self):
@@ -1058,142 +1005,6 @@ class TeleTest(s_t_utils.SynTest):
 
             await self.asyncraises(s_exc.LinkShutDown, task)
 
-    async def test_discovery_consul(self):
-
-        # Example data from https://www.consul.io/api/catalog.html
-        consul_data = [
-            {
-                "ID": "40e4a748-2192-161a-0510-9bf59fe950b5",
-                "Node": "foobar",
-                "Address": "192.168.10.10",
-                "Datacenter": "dc1",
-                "TaggedAddresses": {
-                    "lan": "192.168.10.10",
-                    "wan": "10.0.10.10"
-                },
-                "NodeMeta": {
-                    "somekey": "somevalue"
-                },
-                "CreateIndex": 51,
-                "ModifyIndex": 51,
-                "ServiceAddress": "172.17.0.3",
-                "ServiceEnableTagOverride": False,
-                "ServiceID": "32a2a47f7992:nodea:5000",
-                "ServiceName": "foobar",
-                "ServicePort": 5000,
-                "ServiceMeta": {
-                    "foobar_meta_value": "baz"
-                },
-                "ServiceTaggedAddresses": {
-                    "lan": {
-                        "address": "172.17.0.3",
-                        "port": 5000
-                    },
-                    "wan": {
-                        "address": "198.18.0.1",
-                        "port": 512
-                    }
-                },
-                "ServiceTags": [
-                    "tacos"
-                ],
-                "ServiceProxy": {
-                    "DestinationServiceName": "",
-                    "DestinationServiceID": "",
-                    "LocalServiceAddress": "",
-                    "LocalServicePort": 0,
-                    "Config": None,
-                    "Upstreams": None
-                },
-                "ServiceConnect": {
-                    "Native": False,
-                    "Proxy": None
-                },
-                "Namespace": "default"
-            }
-        ]
-        with self.getTestDir() as dirn:
-            async with await s_cell.Cell.anit(dirn) as cell:
-                root = await cell.auth.getUserByName('root')
-                await root.setPasswd('root')
-                cell.consul_data = consul_data
-                hhost, hport = await cell.addHttpsPort(0)
-                thost, tport = await cell.dmon.listen('tcp://127.0.0.1:0')
-                cell.addHttpApi('/v1/catalog/service/(.*)', ConsulV1Handler, {'cell': cell})
-
-                consul = f'https://127.0.0.1:{hport}'
-
-                burl = f'tcp+consul://root:root@foobar/*?consul_nosslverify=1&consul={consul}'
-                info = s_telepath.chopurl(burl)
-                self.eq(info.get('host'), 'foobar')
-                self.none(info.get('port'))
-
-                # Discovery functions update info dict in place.
-                await s_telepath.disc_consul(info)
-                self.eq(info.get('host'), '192.168.10.10')
-                self.eq(info.get('port'), 5000)
-                self.eq(info.get('original_host'), 'foobar')
-
-                # Support tag_address and service_tag_address
-                url = burl + '&consul_tag_address=wan'
-                info = s_telepath.chopurl(url)
-                await s_telepath.disc_consul(info)
-                self.eq(info.get('host'), '10.0.10.10')
-                self.eq(info.get('port'), 5000)
-                self.eq(info.get('original_host'), 'foobar')
-
-                url = burl + '&consul_service_tag_address=wan'
-                info = s_telepath.chopurl(url)
-                await s_telepath.disc_consul(info)
-                self.eq(info.get('host'), '198.18.0.1')
-                self.eq(info.get('port'), 512)
-                self.eq(info.get('original_host'), 'foobar')
-
-                # Support servicetag based port finding
-                url = burl + '&consul_tag=tacos'
-                info = s_telepath.chopurl(url)
-                await s_telepath.disc_consul(info)
-                self.eq(info.get('host'), '192.168.10.10')
-                self.eq(info.get('port'), 5000)
-                self.eq(info.get('original_host'), 'foobar')
-
-                # Punch in our host/port info into consul_data so we can do a end to end test
-                consul_data[0]['Address'] = thost
-                consul_data[0]['ServicePort'] = tport
-                async with await s_telepath.openurl(burl) as proxy:
-                    self.isin('synapse.lib.cell.CellApi', proxy._getClasses())
-
-                # Sad path - non-existing service name.
-                with self.raises(s_exc.BadUrl) as cm:
-                    url = f'tcp+consul://root:root@newp/*?consul_nosslverify=1&consul={consul}'
-                    await s_telepath.disc_consul(s_telepath.chopurl(url))
-                self.eq(cm.exception.get('name'), 'newp')
-
-                # Sad path - multiple tag resolution.
-                with self.raises(s_exc.BadUrl) as cm:
-                    url = burl + '&consul_tag_address=wan'
-                    url = url + '&consul_service_tag_address=lan'
-                    await s_telepath.disc_consul(s_telepath.chopurl(url))
-                self.eq(cm.exception.get('consul_tag_address'), 'wan')
-                self.eq(cm.exception.get('consul_service_tag_address'), 'lan')
-
-                # Sad path - ssl verification failure
-                with self.raises(s_exc.BadUrl) as cm:
-                    url = f'tcp+consul://root:root@foobar/*?consul={consul}'
-                    await s_telepath.disc_consul(s_telepath.chopurl(url))
-                self.isin('certificate verify failed', cm.exception.get('mesg'))
-                self.isinstance(cm.exception.__context__, ssl.SSLCertVerificationError)
-
-                # Sad path - iterating through results and having a
-                # non-existent consul_tag value.
-                new_data = consul_data[0].copy()
-                new_data['ServiceTags'] = []
-                cell.consul_data.insert(0, new_data)
-                with self.raises(s_exc.BadUrl) as cm:
-                    url = burl + '&consul_tag=burritos'
-                    await s_telepath.disc_consul(s_telepath.chopurl(url))
-                self.eq('burritos', cm.exception.get('tag'))
-
     async def test_telepath_pipeline(self):
 
         foo = Foo()
@@ -1248,16 +1059,21 @@ class TeleTest(s_t_utils.SynTest):
             'inits': 0
         }
         evnt = asyncio.Event()
+        loopevent = asyncio.Event()
         origfire = s_telepath.Client._fireLinkLoop
         originit = s_telepath.Client._initTeleLink
 
+        tgt = {'n': 3}
+
         async def countLinkLoops(self):
             cnts['loops'] += 1
+            if cnts['loops'] > tgt.get('n'):
+                loopevent.set()
             await origfire(self)
 
         async def countInitLinks(self, url):
             cnts['inits'] += 1
-            if cnts['inits'] > 3:
+            if cnts['inits'] > tgt.get('n'):
                 evnt.set()
 
             await originit(self, url)
@@ -1281,25 +1097,30 @@ class TeleTest(s_t_utils.SynTest):
             async with await s_telepath.Client.anit(url) as targ:
                 proxy = await targ.proxy(timeout=2)
                 await targ.onlink(onLinkOk)
-                self.eq(1, cnts['ok'])
+                self.ge(cnts['ok'], 1)
                 with self.raises(s_exc.SynErr):
                     await targ.onlink(onlinkExc)
-                self.eq(1, cnts['exc'])
+                self.ge(cnts['exc'], 1)
 
             with mock.patch('synapse.telepath.Client._fireLinkLoop', countLinkLoops):
                 with mock.patch('synapse.telepath.Client._initTeleLink', countInitLinks):
                     async with await s_telepath.Client.anit(url, onlink=onlinkFail) as targ:
-
-                        self.true(await asyncio.wait_for(evnt.wait(), timeout=6))
-                        self.eq(4, cnts['loops'])
-                        self.eq(4, cnts['inits'])
+                        fut = asyncio.gather(asyncio.wait_for(evnt.wait(), timeout=6),
+                                             asyncio.wait_for(loopevent.wait(), timeout=6))
+                        self.eq((True, True), await fut)
+                        self.ge(cnts['loops'], 4)
+                        self.ge(cnts['inits'], 4)
 
                     evnt.clear()
-                    async with await s_telepath.Client.anit(url, onlink=onlinkExc) as targ:
+                    loopevent.clear()
+                    tgt['n'] = 4
 
-                        self.true(await asyncio.wait_for(evnt.wait(), timeout=6))
-                        self.eq(5, cnts['loops'])
-                        self.eq(5, cnts['inits'])
+                    async with await s_telepath.Client.anit(url, onlink=onlinkExc) as targ:
+                        fut = asyncio.gather(asyncio.wait_for(evnt.wait(), timeout=30),
+                                             asyncio.wait_for(loopevent.wait(), timeout=30),)
+                        self.eq((True, True), await fut)
+                        self.ge(cnts['loops'], 5)
+                        self.ge(cnts['inits'], 5)
 
     async def test_client_method_reset(self):
         class Foo:
@@ -1405,3 +1226,48 @@ class TeleTest(s_t_utils.SynTest):
             async with await s_telepath.openurl(furl) as proxy:
                 self.isin('synapse.tests.test_telepath.Foo', proxy._getClasses())
                 self.eq(await proxy.echo('oh hi mark!'), 'oh hi mark!')
+
+    async def test_tls_ciphers(self):
+
+        self.thisHostMustNot(platform='darwin')
+
+        foo = Foo()
+
+        async with self.getTestDmon() as dmon:
+            # As a workaround to a Python bug (https://bugs.python.org/issue30945) that prevents localhost:0 from
+            # being connected via TLS, make a certificate for whatever my hostname is and sign it with the test CA
+            # key.
+            hostname = socket.gethostname()
+
+            dmon.certdir.genHostCert(hostname, signas='ca')
+
+            _, port = await dmon.listen(f'ssl://{hostname}:0')
+
+            dmon.share('foo', foo)
+
+            # Ensure tls listener is working before trying downgraded versions
+            async with await s_telepath.openurl(f'ssl://{hostname}/foo', port=port) as prox:
+                self.eq(30, await prox.bar(10, 20))
+
+            sslctx = ssl.SSLContext(protocol=ssl.PROTOCOL_TLSv1)
+            with self.raises((ssl.SSLError, ConnectionResetError)):
+                link = await s_link.connect(hostname, port=port, ssl=sslctx)
+
+            sslctx = ssl.SSLContext(protocol=ssl.PROTOCOL_TLSv1_1)
+            with self.raises((ssl.SSLError, ConnectionResetError)):
+                link = await s_link.connect(hostname, port=port, ssl=sslctx)
+
+            sslctx = ssl.SSLContext(protocol=ssl.PROTOCOL_TLSv1_2)
+            sslctx.set_ciphers('ADH-AES256-SHA')
+            with self.raises(ssl.SSLError):
+                link = await s_link.connect(hostname, port=port, ssl=sslctx)
+
+            sslctx = ssl.SSLContext(protocol=ssl.PROTOCOL_TLSv1_2)
+            sslctx.set_ciphers('AES256-GCM-SHA384')
+            with self.raises(ConnectionResetError):
+                link = await s_link.connect(hostname, port=port, ssl=sslctx)
+
+            sslctx = ssl.SSLContext(protocol=ssl.PROTOCOL_TLSv1_2)
+            sslctx.set_ciphers('DHE-RSA-AES256-SHA256')
+            with self.raises(ConnectionResetError):
+                link = await s_link.connect(hostname, port=port, ssl=sslctx)
