@@ -41,6 +41,10 @@ class GenPkgTest(s_test.SynTest):
             ymlpath = s_common.genpath(dirname, 'files', 'stormpkg', 'nosuchfile.yaml')
             await s_genpkg.main((ymlpath,))
 
+        with self.raises(s_exc.NoSuchFile):
+            ymlpath = s_common.genpath(dirname, 'files', 'stormpkg', 'newpfile.yaml')
+            await s_genpkg.main((ymlpath,))
+
         with self.raises(s_exc.BadPkgDef):
             ymlpath = s_common.genpath(dirname, 'files', 'stormpkg', 'nopath.yaml')
             await s_genpkg.main((ymlpath,))
@@ -57,6 +61,18 @@ class GenPkgTest(s_test.SynTest):
             ymlpath = s_common.genpath(dirname, 'files', 'stormpkg', 'nocontent.yaml')
             await s_genpkg.main((ymlpath,))
 
+        with self.raises(s_exc.SchemaViolation):
+            ymlpath = s_common.genpath(dirname, 'files', 'stormpkg', 'badcmdname.yaml')
+            await s_genpkg.main((ymlpath,))
+
+        with self.raises(s_exc.BadArg):
+            ymlpath = s_common.genpath(dirname, 'files', 'stormpkg', 'badjsonpkg.yaml')
+            await s_genpkg.main((ymlpath,))
+
+        with self.raises(s_exc.SchemaViolation):
+            ymlpath = s_common.genpath(dirname, 'files', 'stormpkg', 'badapidef.yaml')
+            await s_genpkg.main((ymlpath,))
+
         ymlpath = s_common.genpath(dirname, 'files', 'stormpkg', 'testpkg.yaml')
         async with self.getTestCore() as core:
 
@@ -70,24 +86,50 @@ class GenPkgTest(s_test.SynTest):
             await s_genpkg.main(argv)
 
             msgs = await core.stormlist('testpkgcmd')
-            self.stormIsInPrint('argument <foo> is required', msgs)
+            self.stormIsInErr('argument <foo> is required', msgs)
             msgs = await core.stormlist('$mod=$lib.import(testmod) $lib.print($mod)')
             self.stormIsInPrint('Imported Module testmod', msgs)
 
+            gdefs = await core.callStorm('return($lib.graph.list())')
+            self.len(1, gdefs)
+            self.eq(gdefs[0]['name'], 'testgraph')
+            self.eq(gdefs[0]['power-up'], 'testpkg')
+
             pdef = s_common.yamlload(savepath)
             s_common.yamlsave(pdef, yamlpath)
 
             self.eq(pdef['name'], 'testpkg')
-            self.eq(pdef['version'], (0, 0, 1))
+            self.eq(pdef['version'], '0.0.1')
             self.eq(pdef['modules'][0]['name'], 'testmod')
             self.eq(pdef['modules'][0]['storm'], 'inet:ipv4\n')
-            self.eq(pdef['modules'][1]['name'], 'testpkg.testext')
-            self.eq(pdef['modules'][1]['storm'], 'inet:fqdn\n')
-            self.eq(pdef['modules'][2]['name'], 'testpkg.testextfile')
+            self.eq(pdef['modules'][1]['name'], 'apimod')
+            self.isin('function search', pdef['modules'][1]['storm'])
+            self.eq(pdef['modules'][2]['name'], 'testpkg.testext')
             self.eq(pdef['modules'][2]['storm'], 'inet:fqdn\n')
+            self.eq(pdef['modules'][3]['name'], 'testpkg.testextfile')
+            self.eq(pdef['modules'][3]['storm'], 'inet:fqdn\n')
             self.eq(pdef['commands'][0]['name'], 'testpkgcmd')
             self.eq(pdef['commands'][0]['storm'], 'inet:ipv6\n')
 
+            self.eq(pdef['perms'][0]['perm'], ['power-ups', 'testpkg', 'user'])
+            self.eq(pdef['perms'][0]['gate'], 'cortex')
+            self.eq(pdef['perms'][0]['desc'], 'Controls user access to testpkg.')
+            self.eq(pdef['perms'][0]['workflowconfig'], True)
+
+            self.eq(pdef['configvars'][0]['name'], 'API key')
+            self.eq(pdef['configvars'][0]['varname'], 'testpkg:apikey')
+            self.eq(pdef['configvars'][0]['desc'], 'API key to use for querying the testpkg API.')
+            self.eq(pdef['configvars'][0]['scopes'], ['global', 'self'])
+            self.eq(pdef['configvars'][0]['workflowconfig'], True)
+            self.eq(pdef['configvars'][0]['type'], 'hugenum')
+            self.eq(pdef['configvars'][1]['name'], 'Tag Prefix')
+            self.eq(pdef['configvars'][1]['varname'], 'testpkg:tag:prefix')
+            self.eq(pdef['configvars'][1]['desc'], 'Tag prefix to use when recording tags.')
+            self.eq(pdef['configvars'][1]['scopes'], ['global', 'self'])
+            self.eq(pdef['configvars'][1]['default'], 'rep.testpkg')
+            self.eq(pdef['configvars'][1]['workflowconfig'], True)
+            self.eq(pdef['configvars'][1]['type'], ['inet:fqdn', ['str', 'inet:url']])
+
             self.eq(pdef['optic']['files']['index.html']['file'], 'aGkK')
 
             self.eq(pdef['docs'][0]['title'], 'Foo Bar')
@@ -207,3 +249,38 @@ class GenPkgTest(s_test.SynTest):
             self.stormIsInPrint('hello foo', msgs)
             msgs = await core.stormlist('dotstorm.bar')
             self.stormIsInPrint('hello bar', msgs)
+
+class TestStormPkgTest(s_test.StormPkgTest):
+    assetdir = s_common.genpath(dirname, 'files', 'stormpkg', 'dotstorm', 'testassets')
+    pkgprotos = (s_common.genpath(dirname, 'files', 'stormpkg', 'dotstorm', 'dotstorm.yaml'),)
+
+    async def initTestCore(self, core):
+        await core.callStorm('$lib.globals.set(inittestcore, frob)')
+
+    async def test_stormpkg_base(self):
+        async with self.getTestCore() as core:
+            msgs = await core.stormlist('dotstorm.bar')
+            self.stormHasNoWarnErr(msgs)
+            self.eq('frob', await core.callStorm('return($lib.globals.get(inittestcore))'))
+
+    async def stormpkg_preppkghook(self, core):
+        await core.callStorm('$lib.globals.set(stormpkg_preppkghook, boundmethod)')
+
+    async def test_stormpkg_preppkghook(self):
+
+        # inline example
+        async def hook(core):
+            await core.callStorm('$lib.globals.set(inlinehook, haha)')
+
+        async with self.getTestCore(prepkghook=hook) as core:
+            msgs = await core.stormlist('dotstorm.bar')
+            self.stormHasNoWarnErr(msgs)
+            self.eq('haha', await core.callStorm('return($lib.globals.get(inlinehook))'))
+            self.eq('frob', await core.callStorm('return($lib.globals.get(inittestcore))'))
+
+        # bound method example
+        async with self.getTestCore(prepkghook=self.stormpkg_preppkghook) as core:
+            msgs = await core.stormlist('dotstorm.bar')
+            self.stormHasNoWarnErr(msgs)
+            self.eq('boundmethod', await core.callStorm('return($lib.globals.get(stormpkg_preppkghook))'))
+            self.eq('frob', await core.callStorm('return($lib.globals.get(inittestcore))'))
