@@ -2,6 +2,8 @@ import json
 import asyncio
 import logging
 
+import unittest.mock as mock
+
 import synapse.tests.utils as s_t_utils
 
 import synapse.tools.healthcheck as s_t_healthcheck
@@ -10,6 +12,8 @@ logger = logging.getLogger(__name__)
 
 class HealthcheckTest(s_t_utils.SynTest):
 
+    # Patch out the default password handler iterations to a value that runs quickly.
+    @mock.patch('synapse.lib.crypto.passwd.PBKDF2_ITERATIONS', 10_000)
     async def test_healthcheck(self):
 
         # Show a passing / failing healthcheck on a cell
@@ -61,11 +65,12 @@ class HealthcheckTest(s_t_utils.SynTest):
             m = 'Synapse error encountered.'
             self.eq(resp.get('components')[0].get('mesg'), m)
 
-            # dont do this in prod...
+            visi = await core.auth.addUser('visi')
+            await visi.setPasswd('secret')
+
             logger.info('Checking without perms')
-            await root.setAdmin(False)
             outp.clear()
-            retn = await s_t_healthcheck.main(['-c', curl, '-t', '0.2'], outp)
+            retn = await s_t_healthcheck.main(['-c', f'tcp://visi:secret@127.0.0.1:{port}/cortex', '-t', '0.2'], outp)
             self.eq(retn, 1)
             resp = json.loads(str(outp))
             self.eq(resp.get('components')[0].get('name'), 'error')
@@ -82,5 +87,5 @@ class HealthcheckTest(s_t_utils.SynTest):
             self.eq(retn, 1)
             resp = json.loads(str(outp))
             self.eq(resp.get('components')[0].get('name'), 'error')
-            m = 'Unable to connect to cell.'
-            self.eq(resp.get('components')[0].get('mesg'), m)
+            m = 'Unable to connect to cell'
+            self.isin(m, resp.get('components')[0].get('mesg'))
