@@ -1,6 +1,8 @@
 import os
 import time
+import json
 import logging
+import unittest
 
 import synapse.common as s_common
 
@@ -94,6 +96,10 @@ class TestUtils(s_t_utils.SynTest):
         outp = self.getTestOutp()
         self.isinstance(outp, s_output.OutPut)
 
+        with self.raises(unittest.SkipTest) as cm:
+            self.skipIfNoPath('newpDoesNotExist', mesg='hehe')
+        self.isin('newpDoesNotExist mesg=hehe', str(cm.exception))
+
     def test_syntest_logstream(self):
         with self.getLoggerStream('synapse.tests.test_utils') as stream:
             logger.error('ruh roh i am a error message')
@@ -104,13 +110,13 @@ class TestUtils(s_t_utils.SynTest):
     def test_syntest_logstream_event(self):
 
         @s_common.firethread
-        def logathing():
+        def logathing(mesg):
             time.sleep(0.01)
-            logger.error('StreamEvent Test Message')
+            logger.error(mesg)
 
         logger.error('notthere')
         with self.getLoggerStream('synapse.tests.test_utils', 'Test Message') as stream:
-            thr = logathing()
+            thr = logathing('StreamEvent Test Message')
             self.true(stream.wait(10))
             thr.join()
 
@@ -119,16 +125,34 @@ class TestUtils(s_t_utils.SynTest):
         self.isin('StreamEvent Test Message', mesgs)
         self.notin('notthere', mesgs)
 
+        with self.getLoggerStream('synapse.tests.test_utils', 'Test Message') as stream:
+            thr = logathing(json.dumps({'mesg': 'Test Message'}))
+            self.true(stream.wait(10))
+            thr.join()
+
+        msgs = stream.jsonlines()
+        self.len(1, msgs)
+        self.eq(msgs[0], {'mesg': 'Test Message'})
+
     def test_syntest_envars(self):
         os.environ['foo'] = '1'
         os.environ['bar'] = '2'
 
-        with self.setTstEnvars(foo=1, bar='joke', baz=1234) as cm:
+        with self.setTstEnvars(foo=1, bar='joke', baz=1234, FOO_THING=1, BAR_THING=0) as cm:
             self.none(cm)
             self.eq(os.environ.get('foo'), '1')
             self.eq(os.environ.get('bar'), 'joke')
             self.eq(os.environ.get('baz'), '1234')
 
+            self.thisEnvMust('FOO_THING', 'baz')
+            self.thisEnvMustNot('BAR_THING', 'NEWP_THING')
+            with self.raises(unittest.SkipTest):
+                self.thisEnvMust('MEWP_THING')
+            with self.raises(unittest.SkipTest):
+                self.thisEnvMust('BAR_THING')
+            with self.raises(unittest.SkipTest):
+                self.thisEnvMustNot('FOO_THING')
+
         self.eq(os.environ.get('foo'), '1')
         self.eq(os.environ.get('bar'), '2')
         self.none(os.environ.get('baz'))
@@ -194,18 +218,31 @@ class TestUtils(s_t_utils.SynTest):
 
         await self.asyncraises(ZeroDivisionError, araiser())
 
-    async def test_storm_mesgs(self):
+    async def test_storm_msgs(self):
 
         async with self.getTestCore() as core:
 
-            mesgs = await core.stormlist('[test:str=1234] | count')
-            self.stormIsInPrint('Counted 1 nodes.', mesgs)
+            msgs = await core.stormlist('[test:str=1234] | count')
+            self.stormIsInPrint('Counted 1 nodes.', msgs)
 
-            mesgs = await core.stormlist('iden newp')
-            self.stormIsInWarn('Failed to decode iden', mesgs)
+            msgs = await core.stormlist('iden newp')
+            self.stormIsInWarn('Failed to decode iden', msgs)
 
-            mesgs = await core.stormlist('[test:str=')
-            self.stormIsInErr("Unexpected token 'end of input'", mesgs)
+            msgs = await core.stormlist('[test:str=')
+            self.stormIsInErr("Unexpected token 'end of input'", msgs)
+
+            with self.raises(AssertionError):
+                self.stormHasNoErr(msgs)
+
+            with self.raises(AssertionError):
+                self.stormHasNoWarnErr(msgs)
+
+            msgs = await core.stormlist('test:str')
+            self.stormHasNoErr(msgs)
+
+            msgs = await core.stormlist('test:str $lib.warn("oh hi")')
+            with self.raises(AssertionError):
+                self.stormHasNoWarnErr(msgs)
 
     async def test_stable_uids(self):
         with self.withStableUids():
@@ -228,7 +265,8 @@ class TestUtils(s_t_utils.SynTest):
     def test_utils_certdir(self):
         oldcertdirn = s_certdir.getCertDirn()
         oldcertdir = s_certdir.getCertDir()
-        self.eq(oldcertdir.pathrefs, {oldcertdirn: 1})
+
+        self.eq(1, oldcertdir.pathrefs[oldcertdirn])
 
         with self.getTestDir() as dirn:
             path = s_common.genpath(dirn, 'haha')
@@ -237,7 +275,7 @@ class TestUtils(s_t_utils.SynTest):
             with self.getTestCertDir(path) as certdir:
 
                 # The singleton functionality now refers to the patched objects
-                self.eq(certdir.pathrefs, {path: 1})
+                self.eq(1, certdir.pathrefs[path])
                 self.true(certdir is s_certdir.getCertDir())
                 self.false(oldcertdir is s_certdir.getCertDir())
 
@@ -247,12 +285,14 @@ class TestUtils(s_t_utils.SynTest):
                 # Adding / deleting paths does not affect the old singleton
                 newpath = s_common.genpath(dirn, 'hehe')
                 s_certdir.addCertPath(newpath)
-                self.eq(certdir.pathrefs, {path: 1, newpath: 1})
-                self.eq(oldcertdir.pathrefs, {oldcertdirn: 1})
+                self.eq(1, certdir.pathrefs[path])
+                self.eq(1, certdir.pathrefs[newpath])
+                self.eq(1, oldcertdir.pathrefs[oldcertdirn])
 
                 s_certdir.delCertPath(newpath)
-                self.eq(certdir.pathrefs, {path: 1})
-                self.eq(oldcertdir.pathrefs, {oldcertdirn: 1})
+                self.eq(1, certdir.pathrefs[path])
+                self.eq(None, certdir.pathrefs.get(newpath))
+                self.eq(1, oldcertdir.pathrefs[oldcertdirn])
 
         # Patch is removed and singleton behavior is restored
         self.true(oldcertdir is s_certdir.getCertDir())
