@@ -21,9 +21,11 @@ import io
 import os
 import sys
 import copy
+import json
 import math
 import types
 import shutil
+import typing
 import asyncio
 import hashlib
 import inspect
@@ -36,6 +38,8 @@ import collections
 
 import unittest.mock as mock
 
+import vcr
+import regex
 import aiohttp
 
 from prompt_toolkit.formatted_text import FormattedText
@@ -50,6 +54,8 @@ import synapse.cryotank as s_cryotank
 import synapse.telepath as s_telepath
 
 import synapse.lib.aha as s_aha
+import synapse.lib.base as s_base
+import synapse.lib.cell as s_cell
 import synapse.lib.coro as s_coro
 import synapse.lib.cmdr as s_cmdr
 import synapse.lib.hive as s_hive
@@ -66,12 +72,17 @@ import synapse.lib.httpapi as s_httpapi
 import synapse.lib.msgpack as s_msgpack
 import synapse.lib.jsonstor as s_jsonstor
 import synapse.lib.lmdbslab as s_lmdbslab
+import synapse.lib.modelrev as s_modelrev
 import synapse.lib.thishost as s_thishost
 import synapse.lib.structlog as s_structlog
 import synapse.lib.stormtypes as s_stormtypes
 
+import synapse.tools.genpkg as s_genpkg
+
 logger = logging.getLogger(__name__)
 
+logging.getLogger('vcr').setLevel(logging.ERROR)
+
 # Default LMDB map size for tests
 TEST_MAP_SIZE = s_const.gibibyte
 
@@ -85,12 +96,74 @@ def norm(z):
         return {norm(k): norm(v) for (k, v) in z.items()}
     return z
 
+def deguidify(x):
+    return regex.sub('[0-9a-f]{32}', '*' * 32, x)
+
+def jsonlines(text: str):
+    lines = [k for k in text.split('\n') if k]
+    return [json.loads(line) for line in lines]
+
+async def waitForBehold(core, events):
+    async for mesg in core.behold():
+        for event in list(events):
+            for key, valu in event.items():
+                if mesg.get(key) != valu:
+                    break
+            else:
+                events.remove(event)
+
+        if len(events) == 0:
+            break
+
+@contextlib.asynccontextmanager
+async def matchContexts(testself):
+    origenter = s_base.Base.__aenter__
+    origexit = s_base.Base.__aexit__
+    origstorm = s_cortex.Cortex.storm
+    orignodes = s_cortex.Cortex.nodes
+
+    contexts = collections.defaultdict(int)
+
+    async def enter(self):
+        contexts[type(self)] += 1
+        return await origenter(self)
+
+    async def exit(self, exc, cls, tb):
+        contexts[type(self)] -= 1
+        await origexit(self, exc, cls, tb)
+
+    async def storm(self, text, opts=None):
+        async for mesg in origstorm(self, text, opts=opts):
+            yield mesg
+
+        for cont, refs in contexts.items():
+            testself.eq(0, refs)
+
+    async def nodes(self, text, opts=None):
+        nodes = await orignodes(self, text, opts=opts)
+
+        for cont, refs in contexts.items():
+            testself.eq(0, refs)
+
+        return nodes
+
+    with mock.patch('synapse.lib.base.Base.__aenter__', enter):
+        with mock.patch('synapse.lib.base.Base.__aexit__', exit):
+            with mock.patch('synapse.cortex.Cortex.nodes', nodes):
+                with mock.patch('synapse.cortex.Cortex.storm', storm):
+                    yield
+
+class PickleableMagicMock(mock.MagicMock):
+    def __reduce__(self):
+        return mock.MagicMock, ()
+
 class LibTst(s_stormtypes.Lib):
     '''
     LibTst for testing!
     '''
     _storm_locals = (
         {'name': 'beep',
+         'deprecated': {'eoldate': '8080-08-08'},
          'desc': '''
         Example storm func.
 
@@ -101,12 +174,23 @@ class LibTst(s_stormtypes.Lib):
                       {'name': 'valu', 'type': 'str', 'desc': 'The value to beep.', },
                   ),
                   'returns': {'type': 'str', 'desc': 'The beeped string.', }}},
+        {'name': 'someargs',
+         'desc': '''Example storm func with args.''',
+         'deprecated': {'eolvers': 'v3.0.0', 'mesg': 'This is a test library was deprecated from the day it was made.'},
+         'type': {'type': 'function', '_funcname': 'someargs',
+                  'args': (
+                      {'name': 'valu', 'type': 'str', 'desc': 'The value to beep.', },
+                      {'name': 'bar', 'type': 'bool', 'desc': 'The value to beep.', 'default': True, },
+                      {'name': 'faz', 'type': 'str', 'desc': 'The value to beep.', 'default': None, },
+                  ),
+                  'returns': {'type': 'str', 'desc': 'The beeped string.', }}},
     )
     _storm_lib_path = ('test',)
 
     def addLibFuncs(self):
         self.locls.update({
             'beep': self.beep,
+            'someargs': self.someargs,
         })
 
     async def beep(self, valu):
@@ -116,6 +200,39 @@ class LibTst(s_stormtypes.Lib):
         ret = f'A {valu} beep!'
         return ret
 
+    async def someargs(self, valu, bar=True, faz=None):
+        '''
+        Example storm func
+        '''
+        ret = f'A {valu} beep which {bar} the {faz}!'
+        return ret
+
+class LibDepr(s_stormtypes.Lib):
+    '''
+    Deprecate me!
+    '''
+    _storm_locals = (
+        {'name': 'boop',
+         'desc': '''
+         An example storm function that's not deprecated on its own, but the entire library is.
+         ''',
+         'type': {'type': 'function', '_funcname': 'boop',
+                  'args': (
+                      {'name': 'valu', 'type': 'str', 'desc': 'What to boop.', },
+                  ),
+                  'returns': {'type': 'str', 'desc': 'The booped.', }}},
+    )
+    _storm_lib_path = ('depr',)
+    _storm_lib_deprecation = {'eolvers': 'v3.0.0'}
+
+    def addLibFuncs(self):  # pragma: no cover
+        self.locls.update({
+            'boop': self.boop,
+        })
+
+    async def boop(self, valu):  # pragma: no cover
+        return f'You have been booped, {valu}!'
+
 class TestType(s_types.Type):
 
     stortype = s_layer.STOR_TYPE_UTF8
@@ -177,7 +294,16 @@ testmodel = {
         ('test:type', 'synapse.tests.utils.TestType', {}, {}),
         ('test:threetype', 'synapse.tests.utils.ThreeType', {}, {}),
     ),
-
+    'interfaces': (
+        ('test:interface', {
+            'doc': 'test interface',
+            'props': (
+                ('size', ('int', {}), {}),
+                ('names', ('array', {'type': 'str'}), {}),
+            ),
+            'interfaces': ('inet:proto:request',)
+        }),
+    ),
     'types': (
         ('test:type10', ('test:type', {'foo': 10}), {
             'doc': 'A fake type.'}),
@@ -188,6 +314,7 @@ testmodel = {
 
         ('test:ival', ('ival', {}), {}),
 
+        ('test:ro', ('str', {}), {}),
         ('test:int', ('int', {}), {}),
         ('test:float', ('float', {}), {}),
         ('test:str', ('str', {}), {}),
@@ -196,6 +323,8 @@ testmodel = {
         ('test:edge', ('edge', {}), {}),
         ('test:guid', ('guid', {}), {}),
         ('test:data', ('data', {}), {}),
+        ('test:taxonomy', ('taxonomy', {}), {'interfaces': ('meta:taxonomy',)}),
+        ('test:hugenum', ('hugenum', {}), {}),
 
         ('test:arrayprop', ('guid', {}), {}),
         ('test:arrayform', ('array', {'type': 'int'}), {}),
@@ -214,6 +343,8 @@ testmodel = {
         )}), {'doc': 'A complex comp type.'}),
         ('test:hexa', ('hex', {}), {'doc': 'anysize test hex type'}),
         ('test:hex4', ('hex', {'size': 4}), {'doc': 'size 4 test hex type'}),
+        ('test:hexpad', ('hex', {'size': 8, 'zeropad': True}), {'doc': 'size 8 test hex type, zero padded'}),
+        ('test:zeropad', ('hex', {'zeropad': 20}), {'doc': 'test hex type, zero padded to 40 bytes'}),
 
         ('test:pivtarg', ('str', {}), {}),
         ('test:pivcomp', ('comp', {'fields': (('targ', 'test:pivtarg'), ('lulz', 'test:str'))}), {}),
@@ -223,8 +354,20 @@ testmodel = {
         ('test:cycle1', ('str', {}), {}),
 
         ('test:ndef', ('ndef', {}), {}),
-        ('test:runt', ('str', {'lower': True, 'strip': True}), {'doc': 'A Test runt node'}),
+        ('test:ndef:formfilter1', ('ndef', {
+            'forms': ('inet:ipv4', 'inet:ipv6')
+        }), {}),
+        ('test:ndef:formfilter2', ('ndef', {
+            'interfaces': ('meta:taxonomy',)
+        }), {}),
+        ('test:ndef:formfilter3', ('ndef', {
+            'forms': ('inet:ipv4',),
+            'interfaces': ('file:mime:msoffice',)
+        }), {}),
 
+        ('test:runt', ('str', {'lower': True, 'strip': True}), {'doc': 'A Test runt node'}),
+        ('test:hasiface', ('str', {}), {'interfaces': ('test:interface',)}),
+        ('test:hasiface2', ('str', {}), {'interfaces': ('test:interface',)}),
     ),
 
     'univs': (
@@ -241,6 +384,7 @@ testmodel = {
         )),
         ('test:arrayform', {}, (
         )),
+        ('test:taxonomy', {}, ()),
         ('test:type10', {}, (
 
             ('intprop', ('int', {'min': 20, 'max': 30}), {}),
@@ -303,11 +447,16 @@ testmodel = {
             ('data', ('test:data', {}), {}),
         )),
 
+        ('test:hugenum', {}, (
+            ('huge', ('test:hugenum', {}), {}),
+        )),
+
         ('test:str', {}, (
             ('bar', ('ndef', {}), {}),
             ('baz', ('nodeprop', {}), {}),
             ('tick', ('test:time', {}), {}),
             ('hehe', ('str', {}), {}),
+            ('ndefs', ('array', {'type': 'ndef'}), {}),
         )),
 
         ('test:migr', {}, (
@@ -322,6 +471,7 @@ testmodel = {
         ('test:auto', {}, ()),
         ('test:hexa', {}, ()),
         ('test:hex4', {}, ()),
+        ('test:zeropad', {}, ()),
         ('test:ival', {}, (
             ('interval', ('ival', {}), {}),
         )),
@@ -352,6 +502,14 @@ testmodel = {
             ('newp', ('str', {}), {'doc': 'A stray property we never use in nodes.'}),
         )),
 
+        ('test:ro', {}, (
+            ('writeable', ('str', {}), {'doc': 'writeable property'}),
+            ('readable', ('str', {}), {'doc': 'ro property', 'ro': True}),
+        )),
+
+        ('test:hasiface', {}, ()),
+        ('test:hasiface2', {}, ()),
+
     ),
 }
 
@@ -416,10 +574,7 @@ class TestModule(s_module.CoreModule):
 
         self.core.setFeedFunc('com.test.record', self.addTestRecords)
 
-        async with await self.core.snap() as snap:
-            node = await snap.getNodeByNdef(('meta:source', self.testguid))
-            if node is None:
-                await snap.addNode('meta:source', self.testguid, {'name': 'test'})
+        await self.core.addNode(self.core.auth.rootuser, 'meta:source', self.testguid, {'name': 'test'})
 
         self.core.addStormLib(('test',), LibTst)
 
@@ -557,7 +712,7 @@ class TstEnv:
 
 class TstOutPut(s_output.OutPutStr):
 
-    def expect(self, substr, throw=True):
+    def expect(self, substr, throw=True, whitespace=True):
         '''
         Check if a string is present in the messages captured by the OutPutStr object.
 
@@ -569,6 +724,11 @@ class TstOutPut(s_output.OutPutStr):
             bool: True if the string is present; False if the string is not present and throw is False.
         '''
         outs = str(self)
+
+        if not whitespace:
+            outs = ' '.join(outs.split())
+            substr = ' '.join(outs.split())
+
         if outs.find(substr) == -1:
             if throw:
                 mesg = 'TestOutPut.expect(%s) not in %s' % (substr, outs)
@@ -635,6 +795,10 @@ class StreamEvent(io.StringIO, threading.Event):
         if self.mesg and self.mesg in s:
             self.set()
 
+    def jsonlines(self) -> typing.List[dict]:
+        '''Get the messages as jsonlines. May throw Json errors if the captured stream is not jsonlines.'''
+        return jsonlines(self.getvalue())
+
 class AsyncStreamEvent(io.StringIO, asyncio.Event):
     '''
     A combination of a io.StringIO object and an asyncio.Event object.
@@ -667,6 +831,10 @@ class AsyncStreamEvent(io.StringIO, asyncio.Event):
             return await asyncio.Event.wait(self)
         return await s_coro.event_wait(self, timeout=timeout)
 
+    def jsonlines(self) -> typing.List[dict]:
+        '''Get the messages as jsonlines. May throw Json errors if the captured stream is not jsonlines.'''
+        return jsonlines(self.getvalue())
+
 class HttpReflector(s_httpapi.Handler):
     '''Test handler which reflects get/post data back to the caller'''
 
@@ -692,6 +860,12 @@ class HttpReflector(s_httpapi.Handler):
             sleep = int(sleep[0])
             await asyncio.sleep(sleep)
 
+        redirect = params.get('redirect')
+        if redirect:
+            self.add_header('Redirected', '1')
+            self.redirect(redirect[0])
+            return
+
         self.sendRestRetn(resp)
 
     async def post(self):
@@ -797,6 +971,31 @@ test_schema = {
         'type': 'object',
     }
 
+class ReloadCell(s_cell.Cell):
+    async def postAnit(self):
+        self._reloaded = False
+        self._reloadevt = None
+
+    async def getCellInfo(self):
+        info = await s_cell.Cell.getCellInfo(self)
+        info['cell']['reloaded'] = self._reloaded
+        return info
+
+    async def addTestReload(self):
+        async def func():
+            self._reloaded = True
+            if self._reloadevt:
+                self._reloadevt.set()
+            return True
+
+        self.addReloadableSystem('testreload', func)
+
+    async def addTestBadReload(self):
+        async def func():
+            return 1 / 0
+
+        self.addReloadableSystem('badreload', func)
+
 class SynTest(unittest.TestCase):
     '''
     Mark all async test methods as s_glob.synchelp decorated.
@@ -829,17 +1028,6 @@ class SynTest(unittest.TestCase):
             node = await core.nodes('', opts={'ndefs': (ndef,)})
             self.len(1, node)
 
-    def worker(func, *args, **kwargs):
-        '''
-        Fire a worker thread to run the given func(*args,**kwargs)
-        '''
-        def work():
-            return func(*args, **kwargs)
-
-        thr = threading.Thread(target=work)
-        thr.start()
-        return thr
-
     def printed(self, msgs, text):
         # a helper for testing storm print message output
         for mesg in msgs:
@@ -870,10 +1058,30 @@ class SynTest(unittest.TestCase):
             yield regrdir
 
     @contextlib.asynccontextmanager
-    async def getRegrCore(self, vers, conf=None):
-        with self.getRegrDir('cortexes', vers) as dirn:
-            async with await s_cortex.Cortex.anit(dirn, conf=conf) as core:
-                yield core
+    async def getRegrCore(self, vers, conf=None, maxvers=None):
+        with self.withNexusReplay():
+            with self.getRegrDir('cortexes', vers) as dirn:
+                if maxvers is not None:
+
+                    orig = s_modelrev.ModelRev
+                    class ModelRev(orig):
+                        def __init__(self, core):
+                            orig.__init__(self, core)
+                            self.revs = [k for k in self.revs if k[0] <= maxvers]
+
+                    with mock.patch.object(s_modelrev, 'ModelRev', ModelRev):
+                        async with await s_cortex.Cortex.anit(dirn, conf=conf) as core:
+                            yield core
+                else:
+                    async with await s_cortex.Cortex.anit(dirn, conf=conf) as core:
+                        yield core
+
+    @contextlib.asynccontextmanager
+    async def getRegrAxon(self, vers, conf=None):
+        with self.withNexusReplay():
+            with self.getRegrDir('axons', vers) as dirn:
+                async with await s_axon.Axon.anit(dirn, conf=conf) as axon:
+                    yield axon
 
     def skipIfNoInternet(self):  # pragma: no cover
         '''
@@ -895,6 +1103,34 @@ class SynTest(unittest.TestCase):
         if bool(int(os.getenv('SYN_TEST_SKIP_LONG', 0))):
             raise unittest.SkipTest('SYN_TEST_SKIP_LONG envar set')
 
+    def skipIfNexusReplay(self):
+        '''
+        Allow skipping a test if SYNDEV_NEXUS_REPLAY envar is set.
+
+        Raises:
+            unittest.SkipTest if SYNDEV_NEXUS_REPLAY envar is set to true value.
+        '''
+        if s_common.envbool('SYNDEV_NEXUS_REPLAY'):
+            raise unittest.SkipTest('SYNDEV_NEXUS_REPLAY envar set')
+
+    def skipIfNoPath(self, path, mesg=None):
+        '''
+        Allows skipping a test if the test/files path does not exist.
+
+        Args:
+            path (str): Path to check.
+            mesg (str): Optional additional message.
+
+        Raises:
+            unittest.SkipTest if the path does not exist.
+        '''
+        path = self.getTestFilePath(path)
+        if not os.path.exists(path):
+            m = f'Path does not exist: {path}'
+            if mesg:
+                m = f'{m} mesg={mesg}'
+            raise unittest.SkipTest(m)
+
     def getTestOutp(self):
         '''
         Get a Output instance with a expects() function.
@@ -904,6 +1140,28 @@ class SynTest(unittest.TestCase):
         '''
         return TstOutPut()
 
+    def thisEnvMust(self, *envvars):
+        '''
+        Requires a host must have environment variables set to truthy values.
+
+        Args:
+            *envars: Environment variables to require being present.
+        '''
+        for envar in envvars:
+            if not s_common.envbool(envar):
+                self.skip(f'Envar {envar} is not set to a truthy value.')
+
+    def thisEnvMustNot(self, *envvars):
+        '''
+        Requires a host must not have environment variables set to truthy values.
+
+        Args:
+            *envars: Environment variables to require being absent or set to falsey values.
+        '''
+        for envar in envvars:
+            if s_common.envbool(envar):
+                self.skip(f'Envar {envar} is set to a truthy value.')
+
     def thisHostMust(self, **props):  # pragma: no cover
         '''
         Requires a host having a specific property.
@@ -941,15 +1199,21 @@ class SynTest(unittest.TestCase):
         Returns:
             s_axon.Axon: A Axon object.
         '''
-        if dirn is not None:
-            async with await s_axon.Axon.anit(dirn, conf) as axon:
-                yield axon
 
-            return
+        if conf is None:
+            conf = {}
+        conf = copy.deepcopy(conf)
 
-        with self.getTestDir() as dirn:
-            async with await s_axon.Axon.anit(dirn, conf) as axon:
-                yield axon
+        with self.withNexusReplay():
+            if dirn is not None:
+                async with await s_axon.Axon.anit(dirn, conf) as axon:
+                    yield axon
+
+                return
+
+            with self.getTestDir() as dirn:
+                async with await s_axon.Axon.anit(dirn, conf) as axon:
+                    yield axon
 
     @contextlib.contextmanager
     def withTestCmdr(self, cmdg):
@@ -1001,14 +1265,10 @@ class SynTest(unittest.TestCase):
         Context manager to mock calls to the setlogging function to avoid unittests calling logging.basicconfig.
 
         Returns:
-            mock.MagicMock: Yields a mock.MagikMock object.
+            mock.MagicMock: Yields a mock.MagicMock object.
         '''
-        # Since the setlogging routine is used in the forkserver initializer,
-        # we need to make sure we've initialized our worker prior to mocking
-        self.eq(1, await s_coro.forked(int, '1'))
-
         with mock.patch('synapse.common.setlogging',
-                        mock.MagicMock(return_value=dict())) as patch:  # type: mock.MagicMock
+                        PickleableMagicMock(return_value=dict())) as patch:  # type: mock.MagicMock
             yield patch
 
     def getMagicPromptLines(self, patch):
@@ -1093,14 +1353,18 @@ class SynTest(unittest.TestCase):
         '''
         Patch so that the Nexus apply log is applied twice. Useful to verify idempotency.
 
+        Args:
+            replay (bool): Set the default value of resolving the existence of SYNDEV_NEXUS_REPLAY variable.
+                           This can be used to force the apply patch without using the environment variable.
+
         Notes:
             This is applied if the environment variable SYNDEV_NEXUS_REPLAY is set
-            or the replay argument is set to True.
+            to a non zero value or the replay argument is set to True.
 
         Returns:
             contextlib.ExitStack: An exitstack object.
         '''
-        replay = os.environ.get('SYNDEV_NEXUS_REPLAY', default=replay)
+        replay = s_common.envbool('SYNDEV_NEXUS_REPLAY', defval=str(replay))
 
         with contextlib.ExitStack() as stack:
             if replay:
@@ -1115,36 +1379,20 @@ class SynTest(unittest.TestCase):
         Returns:
             s_cortex.Cortex: A Cortex object.
         '''
-        if conf is None:
-            conf = {'layer:lmdb:map_async': True,
-                    'provenance:en': True,
-                    'nexslog:en': True,
-                    'layers:logedits': True,
-                    }
+        conf = self.getCellConf(conf=conf)
 
-        conf = copy.deepcopy(conf)
-
-        mods = conf.get('modules')
-
-        if mods is None:
-            mods = []
-            conf['modules'] = mods
+        mods = list(conf.get('modules', ()))
+        conf['modules'] = mods
 
         mods.insert(0, ('synapse.tests.utils.TestModule', {'key': 'valu'}))
 
         with self.withNexusReplay():
 
-            if dirn is not None:
+            with self.mayTestDir(dirn) as dirn:
 
                 async with await s_cortex.Cortex.anit(dirn, conf=conf) as core:
                     yield core
 
-                return
-
-            with self.getTestDir() as dirn:
-                async with await s_cortex.Cortex.anit(dirn, conf=conf) as core:
-                    yield core
-
     @contextlib.asynccontextmanager
     async def getTestCoreAndProxy(self, conf=None, dirn=None):
         '''
@@ -1160,10 +1408,20 @@ class SynTest(unittest.TestCase):
                 yield core, prox
 
     @contextlib.asynccontextmanager
-    async def getTestJsonStor(self):
-        with self.getTestDir() as dirn:
-            async with await s_jsonstor.JsonStorCell.anit(dirn) as jsonstor:
-                yield jsonstor
+    async def getTestJsonStor(self, dirn=None, conf=None):
+
+        conf = self.getCellConf(conf=conf)
+
+        with self.withNexusReplay():
+            if dirn is not None:
+                async with await s_jsonstor.JsonStorCell.anit(dirn, conf) as jsonstor:
+                    yield jsonstor
+
+                return
+
+            with self.getTestDir() as dirn:
+                async with await s_jsonstor.JsonStorCell.anit(dirn, conf) as jsonstor:
+                    yield jsonstor
 
     @contextlib.asynccontextmanager
     async def getTestCryo(self, dirn=None, conf=None):
@@ -1173,15 +1431,12 @@ class SynTest(unittest.TestCase):
         Returns:
             s_cryotank.CryoCell: Test cryocell.
         '''
-        if dirn is not None:
-            async with await s_cryotank.CryoCell.anit(dirn, conf=conf) as cryo:
-                yield cryo
+        conf = self.getCellConf(conf=conf)
 
-            return
-
-        with self.getTestDir() as dirn:
-            async with await s_cryotank.CryoCell.anit(dirn, conf=conf) as cryo:
-                yield cryo
+        with self.withNexusReplay():
+            with self.mayTestDir(dirn) as dirn:
+                async with await s_cryotank.CryoCell.anit(dirn, conf=conf) as cryo:
+                    yield cryo
 
     @contextlib.asynccontextmanager
     async def getTestCryoAndProxy(self, dirn=None):
@@ -1197,6 +1452,7 @@ class SynTest(unittest.TestCase):
 
     @contextlib.asynccontextmanager
     async def getTestDmon(self):
+        self.skipIfNoPath(path='certdir', mesg='Missing files for test dmon!')
         with self.getTestDir(mirror='certdir') as certpath:
             certdir = s_certdir.CertDir(path=certpath)
             s_certdir.addCertPath(certpath)
@@ -1207,25 +1463,15 @@ class SynTest(unittest.TestCase):
             s_certdir.delCertPath(certpath)
 
     @contextlib.asynccontextmanager
-    async def getTestCell(self, ctor, conf=None, dirn=None):
+    async def getTestCell(self, ctor=s_cell.Cell, conf=None, dirn=None):
         '''
         Get a test Cell.
         '''
-        if conf is None:
-            conf = {}
-
-        conf = copy.deepcopy(conf)
-
-        if dirn is not None:
-
-            async with await ctor.anit(dirn, conf=conf) as cell:
-                yield cell
-
-            return
-
-        with self.getTestDir() as dirn:
-            async with await ctor.anit(dirn, conf=conf) as cell:
-                yield cell
+        conf = self.getCellConf(conf=conf)
+        with self.withNexusReplay():
+            with self.mayTestDir(dirn) as dirn:
+                async with await ctor.anit(dirn, conf=conf) as cell:
+                    yield cell
 
     @contextlib.asynccontextmanager
     async def getTestCoreProxSvc(self, ssvc, ssvc_conf=None, core_conf=None):
@@ -1246,16 +1492,112 @@ class SynTest(unittest.TestCase):
 
                 yield core, prox, testsvc
 
+    @contextlib.contextmanager
+    def mayTestDir(self, dirn: str | None) -> contextlib.AbstractContextManager[str, None, None]:
+        '''
+        Convenience method to make a temporary directory or use an existing one.
+
+        Args:
+            dirn: Directory to use, or None.
+
+        Returns:
+            The directory to use.
+        '''
+        if dirn is not None:
+            yield dirn
+            return
+
+        with self.getTestDir() as dirn:
+            yield dirn
+
     @contextlib.asynccontextmanager
-    async def getTestAha(self, conf=None, dirn=None):
-        if dirn:
-            async with await s_aha.AhaCell.anit(dirn, conf=conf) as aha:
-                yield aha
-        else:
-            with self.getTestDir() as dirn:
-                async with await s_aha.AhaCell.anit(dirn, conf=conf) as aha:
+    async def getTestAha(self, conf=None, dirn=None, ctor=None):
+
+        if conf is None:
+            conf = {}
+
+        if ctor is None:
+            ctor = s_aha.AhaCell.anit
+
+        conf = s_msgpack.deepcopy(conf)
+
+        hasdmonlisn = 'dmon:listen' in conf
+        hasprovlisn = 'provision:listen' in conf
+
+        network = conf.setdefault('aha:network', 'synapse')
+        hostname = conf.setdefault('dns:name', '00.aha.loop.vertex.link')
+
+        if hostname:
+            conf.setdefault('provision:listen', f'ssl://0.0.0.0:0?hostname={hostname}')
+            conf.setdefault('dmon:listen', f'ssl://0.0.0.0:0?hostname={hostname}&ca={network}')
+
+        conf.setdefault('health:sysctl:checks', False)
+
+        with self.withNexusReplay():
+
+            with self.mayTestDir(dirn) as dirn:
+
+                if conf.get('aha:network') == 'synapse':
+                    dstpath = os.path.join(dirn, 'certs')
+                    if not os.path.isdir(dstpath):
+                        srcpath = self.getTestFilePath('aha/certs')
+                        shutil.copytree(srcpath, os.path.join(dirn, 'certs'))
+
+                async with await ctor(dirn, conf=conf) as aha:
+
+                    mods = {}
+                    if not hasdmonlisn and hostname:
+                        mods['dmon:listen'] = f'ssl://0.0.0.0:{aha.sockaddr[1]}?hostname={hostname}&ca={network}'
+
+                    if not hasprovlisn and hostname:
+                        mods['provision:listen'] = f'ssl://0.0.0.0:{aha.provaddr[1]}?hostname={hostname}'
+
+                    if mods:
+                        aha.modCellConf(mods)
+
                     yield aha
 
+    def getCellConf(self, conf=None):
+        if conf is None:
+            conf = {}
+        conf = s_msgpack.deepcopy(conf)
+        conf.setdefault('health:sysctl:checks', False)
+        return conf
+
+    @contextlib.asynccontextmanager
+    async def addSvcToAha(self, aha, svcname, ctor,
+                          conf=None, dirn=None, provinfo=None):
+        '''
+        Creates a service and provisions it in an Aha network via the provisioning API.
+
+        This assumes the Aha cell has a provision:listen and aha:urls set.
+
+        Args:
+            aha (s_aha.AhaCell): Aha cell.
+            svcname (str): Service name.
+            ctor: Service class to add.
+            conf (dict): Optional service conf.
+            dirn (str): Optional directory.
+            provinfo (dict): Optional provisioning info.
+
+        Notes:
+            The config data for the cell is pushed into dirn/cell.yaml.
+            The cells are created with the ``ctor.anit()`` function.
+        '''
+        ahanetw = aha.conf.req('aha:network')
+        svcfull = f'{svcname}.{ahanetw}'
+        onetime = await aha.addAhaSvcProv(svcname, provinfo=provinfo)
+
+        conf = self.getCellConf(conf=conf)
+        conf['aha:provision'] = onetime
+
+        waiter = aha.waiter(1, f'aha:svcadd:{svcfull}')
+        with self.mayTestDir(dirn) as dirn:
+            s_common.yamlsave(conf, dirn, 'cell.yaml')
+            async with await ctor.anit(dirn) as svc:
+                self.true(await waiter.wait(timeout=12))
+                yield svc
+
     async def addSvcToCore(self, svc, core, svcname='svc'):
         '''
         Add a service to a Cortex using telepath over tcp.
@@ -1289,14 +1631,17 @@ class SynTest(unittest.TestCase):
         return s_telepath.openurl(f'tcp:///{name}', **kwargs)
 
     @contextlib.contextmanager
-    def getTestDir(self, mirror=None, copyfrom=None, chdir=False, startdir=None):
+    def getTestDir(self, mirror=None, copyfrom=None, chdir=False, startdir=None) -> contextlib.AbstractContextManager[str, None, None]:
         '''
         Get a temporary directory for test purposes.
         This destroys the directory afterwards.
 
         Args:
-            mirror (str): A directory to mirror into the test directory.
-            startdir (str): The directory under which to place the temporary kdirectory
+            mirror (str): A Synapse test directory to mirror into the test directory.
+            copyfrom (str): An arbitrary directory to copy into the test directory.
+            chdir (boolean): If true, chdir the current process to that directory. This is undone when the context
+                             manager exits.
+            startdir (str): The directory under which to place the temporary directory
 
         Notes:
             The mirror argument is normally used to mirror test directory
@@ -1410,14 +1755,13 @@ class SynTest(unittest.TestCase):
             slogger.setLevel(level)
 
     @contextlib.contextmanager
-    def getAsyncLoggerStream(self, logname, mesg=''):
+    def getAsyncLoggerStream(self, logname, mesg='') -> contextlib.AbstractContextManager[StreamEvent, None, None]:
         '''
         Async version of getLoggerStream.
 
         Args:
             logname (str): Name of the logger to get.
-            mesg (str): A string which, if provided, sets the StreamEvent event if a message
-            containing the string is written to the log.
+            mesg (str): A string which, if provided, sets the StreamEvent event if a message containing the string is written to the log.
 
         Notes:
             The event object mixed in for the AsyncStreamEvent is a asyncio.Event object.
@@ -1456,14 +1800,13 @@ class SynTest(unittest.TestCase):
             slogger.setLevel(level)
 
     @contextlib.contextmanager
-    def getStructuredAsyncLoggerStream(self, logname, mesg=''):
+    def getStructuredAsyncLoggerStream(self, logname, mesg='') -> contextlib.AbstractContextManager[AsyncStreamEvent, None, None]:
         '''
         Async version of getLoggerStream which uses structured logging.
 
         Args:
             logname (str): Name of the logger to get.
-            mesg (str): A string which, if provided, sets the StreamEvent event if a message
-            containing the string is written to the log.
+            mesg (str): A string which, if provided, sets the StreamEvent event if a message containing the string is written to the log.
 
         Notes:
             The event object mixed in for the AsyncStreamEvent is a asyncio.Event object.
@@ -1480,9 +1823,7 @@ class SynTest(unittest.TestCase):
                     # Wait for the mesg to be written to the stream
                     await stream.wait(timeout=10)
 
-                data = stream.getvalue()
-                raw_mesgs = [m for m in data.split('\n') if m]
-                msgs = [json.loads(m) for m in raw_mesgs]
+                msgs = stream.jsonlines()
                 # Do something with messages
 
         Returns:
@@ -1612,15 +1953,18 @@ class SynTest(unittest.TestCase):
             new_stdin(file-like object):  file-like object.
 
         Examples:
-            inp = io.StringIO('stdin stuff\nanother line\n')
-            with self.redirectStdin(inp):
-                main()
+            Patch stdin with a string buffer::
 
-            Here's a way to use this for code that's expecting the stdin buffer to have bytes.
-            inp = Mock()
-            inp.buffer = io.BytesIO(b'input data')
-            with self.redirectStdin(inp):
-                main()
+                inp = io.StringIO('stdin stuff\\nanother line\\n')
+                with self.redirectStdin(inp):
+                    main()
+
+            Here's a way to use this for code that's expecting the stdin buffer to have bytes::
+
+                inp = Mock()
+                inp.buffer = io.BytesIO(b'input data')
+                with self.redirectStdin(inp):
+                    main()
 
         Returns:
             None
@@ -1751,6 +2095,7 @@ class SynTest(unittest.TestCase):
         Assert X is not None
         '''
         self.assertIsNotNone(x, msg=msg)
+        return x
 
     def none(self, x, msg=None):
         '''
@@ -1847,7 +2192,7 @@ class SynTest(unittest.TestCase):
             count += 1
         self.eq(x, count, msg=msg)
 
-    def stormIsInPrint(self, mesg, mesgs):
+    def stormIsInPrint(self, mesg, mesgs, deguid=False, whitespace=True):
         '''
         Check if a string is present in all of the print messages from a stream of storm messages.
 
@@ -1855,7 +2200,17 @@ class SynTest(unittest.TestCase):
             mesg (str): A string to check.
             mesgs (list): A list of storm messages.
         '''
+        if deguid:
+            mesg = deguidify(mesg)
+
         print_str = '\n'.join([m[1].get('mesg') for m in mesgs if m[0] == 'print'])
+        if not whitespace:
+            mesg = ' '.join(mesg.split())
+            print_str = ' '.join(print_str.split())
+
+        if deguid:
+            print_str = deguidify(print_str)
+
         self.isin(mesg, print_str)
 
     def stormNotInPrint(self, mesg, mesgs):
@@ -1880,6 +2235,17 @@ class SynTest(unittest.TestCase):
         print_str = '\n'.join([m[1].get('mesg') for m in mesgs if m[0] == 'warn'])
         self.isin(mesg, print_str)
 
+    def stormNotInWarn(self, mesg, mesgs):
+        '''
+        Assert a string is not present in all of the warn messages from a stream of storm messages.
+
+        Args:
+            mesg (str): A string to check.
+            mesgs (list): A list of storm messages.
+        '''
+        print_str = '\n'.join([m[1].get('mesg') for m in mesgs if m[0] == 'warn'])
+        self.notin(mesg, print_str)
+
     def stormIsInErr(self, mesg, mesgs):
         '''
         Check if a string is present in all of the error messages from a stream of storm messages.
@@ -1891,17 +2257,39 @@ class SynTest(unittest.TestCase):
         print_str = '\n'.join([m[1][1].get('mesg', '') for m in mesgs if m[0] == 'err'])
         self.isin(mesg, print_str)
 
+    def stormHasNoErr(self, mesgs):
+        '''
+        Raise an AssertionError if there is a message of type "err" in the list.
+
+        Args:
+            mesgs (list): A list of storm messages.
+        '''
+        for mesg in mesgs:
+            if mesg[0] == 'err':
+                self.fail(f'storm err mesg found: {mesg}')
+
+    def stormHasNoWarnErr(self, mesgs):
+        '''
+        Raise an AssertionError if there is a message of type "err" or "warn" in the list.
+
+        Args:
+            mesgs (list): A list of storm messages.
+        '''
+        for mesg in mesgs:
+            if mesg[0] in ('err', 'warn'):
+                self.fail(f'storm err/warn mesg found: {mesg}')
+
     def istufo(self, obj):
         '''
         Check to see if an object is a tufo.
 
         Args:
-            obj (object): Object being inspected. This is validated to be a
-            tuple of length two, containing a str or None as the first value,
-            and a dict as the second value.
+            obj (object): Object being inspected.
 
         Notes:
             This does not make any assumptions about the contents of the dictionary.
+            This validates the object to be a tuple of length two, containing a str
+            or None as the first value, and a dict as the second value.
 
         Returns:
             None
@@ -2011,7 +2399,7 @@ class SynTest(unittest.TestCase):
             return retn
 
         byts = s_msgpack.en(valu)
-        return hashlib.md5(byts).hexdigest()
+        return hashlib.md5(byts, usedforsecurity=False).hexdigest()
 
     @contextlib.contextmanager
     def withStableUids(self):
@@ -2029,3 +2417,52 @@ class SynTest(unittest.TestCase):
         async def coro():
             return await core.nodes(query, opts)
         return await core.schedCoro(coro())
+
+ONLOAD_TIMEOUT = int(os.getenv('SYNDEV_PKG_LOAD_TIMEOUT', 30))  # seconds
+
+class StormPkgTest(SynTest):
+
+    vcr = None
+    assetdir = None
+    pkgprotos = ()
+
+    @contextlib.asynccontextmanager
+    async def getTestCore(self, conf=None, dirn=None, prepkghook=None):
+
+        async with SynTest.getTestCore(self, conf=None, dirn=None) as core:
+
+            if prepkghook is not None:
+                await prepkghook(core)
+
+            for pkgproto in self.pkgprotos:
+
+                pkgdef = s_genpkg.loadPkgProto(pkgproto, no_docs=True, readonly=True)
+
+                waiter = None
+                if pkgdef.get('onload') is not None:
+                    waiter = core.waiter(1, 'core:pkg:onload:complete')
+
+                await core.addStormPkg(pkgdef)
+
+                if waiter is not None:
+                    self.nn(await waiter.wait(timeout=ONLOAD_TIMEOUT),
+                            f'Package onload failed to run for {pkgdef.get("name")}')
+
+            if self.assetdir is not None:
+
+                if self.vcr is None:
+                    self.vcr = vcr.VCR()
+
+                assetdir = s_common.gendir(self.assetdir)
+                vcrname = f'{self.__class__.__name__}.{self._testMethodName}.yaml'
+                cass = self.vcr.use_cassette(s_common.genpath(assetdir, vcrname))
+                await core.enter_context(cass)
+
+            await self.initTestCore(core)
+            yield core
+
+    async def initTestCore(self, core):
+        '''
+        This is executed after the package has been loaded and a VCR context has been entered.
+        '''
+        pass
