@@ -14,6 +14,9 @@ reqver = '>=0.2.0,<3.0.0'
 
 async def main(argv, outp=s_output.stdout):
 
+    mesg = s_common.deprecated('synapse.tools.hive.load', curv='2.167.0')
+    outp.printf(f'WARNING: {mesg}')
+
     pars = argparse.ArgumentParser(prog='synapse.tools.hive.load',
                                    description='Load data into a remote hive from a previous hivesave.')
 
@@ -36,23 +39,25 @@ async def main(argv, outp=s_output.stdout):
     if opts.path is not None:
         path = opts.path.split('/')
 
-    async with await s_telepath.openurl(opts.hiveurl) as hive:
-        classes = hive.sharinfo.get('classes', ())
+    async with s_telepath.withTeleEnv():
 
-        try:
-            s_version.reqVersion(hive._getSynVers(), reqver)
-            if 'synapse.lib.hive.HiveApi' in classes:
-                await hive.loadHiveTree(tree, path=path, trim=opts.trim)
-            else:
-                todo = s_common.todo('loadHiveTree', tree, path=path, trim=opts.trim)
-                await hive.dyncall('cell', todo)
+        async with await s_telepath.openurl(opts.hiveurl) as hive:
+            classes = hive.sharinfo.get('classes', ())
 
-        except s_exc.BadVersion as e:
-            valu = s_version.fmtVersion(*e.get('valu'))
-            outp.printf(f'Hive version {valu} is outside of the hive.load supported range ({reqver}).')
-            outp.printf(
-                f'Please use a version of Synapse which supports {valu}; current version is {s_version.verstring}.')
-            return 1
+            try:
+                s_version.reqVersion(hive._getSynVers(), reqver)
+                if 'synapse.lib.hive.HiveApi' in classes:
+                    await hive.loadHiveTree(tree, path=path, trim=opts.trim)
+                else:
+                    todo = s_common.todo('loadHiveTree', tree, path=path, trim=opts.trim)
+                    await hive.dyncall('cell', todo)
+
+            except s_exc.BadVersion as e:
+                valu = s_version.fmtVersion(*e.get('valu'))
+                outp.printf(f'Hive version {valu} is outside of the hive.load supported range ({reqver}).')
+                outp.printf(
+                    f'Please use a version of Synapse which supports {valu}; current version is {s_version.verstring}.')
+                return 1
 
     return 0
 
